@@ -8,10 +8,12 @@
 #define LIFEDLG_H
 
 #include <qtimer.h>
-#include <qpushbt.h>
-#include <qlabel.h>
-#include <qscrbar.h>
-#include <qcombo.h>
+#include <qwidget.h>
+
+class QSlider;
+class QPushButton;
+class QLabel;
+class QComboBox;
 
 #include "life.h"
 
@@ -52,7 +54,7 @@ private:
     QPushButton *pb;
     QComboBox	*cb;
     QLabel	*sp;
-    QScrollBar	*scroll;
+    QSlider	*scroll;
 };
 
 
