@@ -19,6 +19,7 @@ public:
 
 public slots:
     void open();
+    void news();
     void save();
     void closeDoc();
     void undo();
@@ -27,6 +28,7 @@ public slots:
     void bold();
     void underline();
     void about();
+    void aboutQt();
     void printer();
     void file();
     void fax();
