@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qwerty.h,v 1.3 1997/02/07 11:17:18 hanord Exp $
+** $Id: qwerty.h,v 1.5 1997/06/06 10:26:04 eiriken Exp $
 **
 ** QWERTY - little more than a typewriter
 **
@@ -11,7 +11,7 @@
 #include <qwidget.h>
 #include <qmenubar.h>
 #include <qmlined.h>
-
+#include <qprinter.h>
 
 class Editor : public QWidget
 {
@@ -21,19 +21,21 @@ public:
    ~Editor();
 
 public slots:
+    void newDoc();
     void load();
     void load( const char *fileName );
     void save();
-    void create();
     void print();
+    void closeDoc();
 
 protected:
     void resizeEvent( QResizeEvent * );
+    void closeEvent( QCloseEvent * );
 
 private:
     QMenuBar 	   *m;
     QMultiLineEdit *e;
+    QPrinter        printer;
 };
 
-
 #endif // QWERTY_H
