@@ -7,10 +7,15 @@
 #include <qdialog.h>
 #include <qmsgbox.h>
 #include <qpixmap.h>
+#include <qmovie.h>
+#include <qlayout.h>
 #include <qapp.h>
+#include <qkeycode.h>
 
 // Standard Qt widgets
 
+#include <qmenubar.h>
+#include <qpopmenu.h>
 #include <qbttngrp.h>
 #include <qchkbox.h>
 #include <qcombo.h>
@@ -18,6 +23,7 @@
 #include <qgrpbox.h>
 #include <qlabel.h>
 #include <qlcdnum.h>
+#include <qmlined.h>
 #include <qlined.h>
 #include <qlistbox.h>
 #include <qpushbt.h>
@@ -31,19 +37,23 @@
 #include "../dclock/dclock.h"
 
 
+#define MOVIEFILENAME "trolltech.gif"
+
 //
 // WidgetView contains lots of Qt widgets.
 //
 
-class WidgetView : public QDialog
+class WidgetView : public QWidget
 {
     Q_OBJECT
 public:
     WidgetView( QWidget *parent=0, const char *name=0 );
 
+public slots:
+    void	setStatus(const char*);
+
 private slots:
     void	button1Clicked();
-    void	button2Clicked();
     void	checkBoxClicked( int );
     void	radioButtonClicked( int );
     void	sliderValueChanged( int );
@@ -51,11 +61,15 @@ private slots:
     void	comboBoxItemActivated( int );
     void	edComboBoxItemActivated( const char * );
     void	lineEditTextChanged( const char * );
+    void	movieStatus( int );
+    void	movieUpdate( const QRect& );
 
 private:
     bool	eventFilter( QObject *, QEvent * );
     QLabel     *msg;
     QCheckBox  *cb[3];
+    QLabel     *movielabel;
+    QMovie      movie;
 };
 
 
@@ -64,7 +78,7 @@ private:
 //
 
 WidgetView::WidgetView( QWidget *parent, const char *name )
-    : QDialog( parent, name )
+    : QWidget( parent, name )
 {
     QColor col;
 
@@ -76,79 +90,114 @@ WidgetView::WidgetView( QWidget *parent, const char *name )
 
     qApp->installEventFilter( this );
 
+  // Create a layout to position the widgets
+
+    QVBoxLayout *topLayout = new QVBoxLayout( this, 10 );
+
+  // Create a grid layout to hold most of the widgets
+
+    QGridLayout *grid = new QGridLayout( 6, 3 );
+ // This layout will get all of the stretch
+    topLayout->addLayout( grid, 10 );
+
+  // Create a menubar
+    QMenuBar *menubar = new QMenuBar( this );
+    menubar->setSeparator( QMenuBar::InWindowsStyle );
+  // Create an easter egg
+    QToolTip::add( menubar, QRect( 0, 0, 2, 2 ), "easter egg" );
+
+    QPopupMenu* popup;
+    popup = new QPopupMenu;
+    int id;
+    id = popup->insertItem( "&New" );
+    popup->setItemEnabled( id, FALSE );
+    id = popup->insertItem( "&Open" );
+    popup->setItemEnabled( id, FALSE );
+    popup->insertSeparator();
+    popup->insertItem( "&Quit", qApp, SLOT(quit()), CTRL+Key_Q );
+
+    menubar->insertItem( "&File", popup );
+
+    // Must tell the layout about a menubar in a widget
+    topLayout->setMenuBar( menubar );
+
   // Create an analog and a digital clock
 
     AnalogClock  *aclock = new AnalogClock( this );
     DigitalClock *dclock = new DigitalClock( this );
-    aclock->setGeometry( 300,100, 100,100 );
-    dclock->setGeometry( 300,220, 100, 60 );
+    dclock->setMaximumWidth(200);
+    grid->addWidget( aclock, 0, 2 );
+    grid->addWidget( dclock, 1, 2 );
 
   // Give the dclock widget a blue palette
 
     col.setRgb( 0xaa, 0xbe, 0xff );
-    QColorGroup colgrp( black, col, col.light(), col.dark(), col.dark(120),
-			black, white );
-    dclock->setPalette( QPalette(colgrp,colgrp,colgrp) );
+    dclock->setPalette( QPalette( col ) );
 
   // make tool tips for both of them
 
-    QToolTip::add( aclock, "custom widget: analock clock" );
+    QToolTip::add( aclock, "custom widget: analog clock" );
     QToolTip::add( dclock, "custom widget: digital clock" );
 
-  // Create two push buttons, a text button and a pixmap button
+  // Create a push button.
 
     QPushButton *pb;
     pb = new QPushButton( this, "button1" );	// create button 1
     pb->setText( "Push button 1" );
-    pb->setGeometry( 10,30, 120,30 );
+    pb->setFixedHeight( pb->sizeHint().height() );
+    grid->addWidget( pb, 0, 0, AlignVCenter );
     connect( pb, SIGNAL(clicked()), SLOT(button1Clicked()) );
     QToolTip::add( pb, "push button 1" );
-    pb = new QPushButton( this, "button2" );	// create button 2
     QPixmap pm;
-    bool pix = TRUE;
-    if ( !pm.load("qt.bmp") ) {			// load pixmap for button 2
-	QMessageBox::message( "Error", "Could not load qt.bmp pixmap" );
-	pb->setText( "No pixmap" );
-	pix = FALSE;
+    bool pix = pm.load("qt.bmp");		// load pixmap for button 2
+    if ( !pix ) {
+	QMessageBox::information( 0, "Qt Widgets Example",
+				  "Could not load the file \"qt.bmp\", which\n"
+				  "contains an icon used...\n\n"
+				  "The text \"line 42\" will be substituted.",
+				  QMessageBox::Ok + QMessageBox::Default );
     }
-    else
-	pb->setPixmap( pm );
-    pb->setGeometry( 150,10, 60,60 );
-    connect( pb, SIGNAL(clicked()), SLOT(button2Clicked()) );
-    QToolTip::add( pb, "push button 2" );
 
-  // Create a quit button
-  // Pressing the quit button will terminate the application
+    // Create a label containing a QMovie
 
-    QPushButton *quitButton = new QPushButton( this, "quitButton" );
-    quitButton->setText( "Quit" );
-    quitButton->setGeometry( 300,30, 80,30 );
-    connect( quitButton, SIGNAL(clicked()), qApp, SLOT(quit()) );
-
-    QToolTip::add( quitButton, "quit the application" );
-
-  // Make the quit button red and give it another font
-
-    col = QColor( 240, 10, 10 );
-    colgrp = QColorGroup( black, col, col.light(), col.dark(), col.dark(120),
-			  blue, white );
-    quitButton->setPalette( QPalette(colgrp,colgrp,colgrp) );
-    quitButton->setFont( QFont("times",14,QFont::Bold,TRUE) );
+    movielabel = new QLabel( this, "label0" );
+    movie = QMovie( MOVIEFILENAME );
+    movie.connectStatus(this, SLOT(movieStatus(int)));
+    movie.connectUpdate(this, SLOT(movieUpdate(const QRect&)));
+    movielabel->setFrameStyle( QFrame::Box | QFrame::Plain );
+    movielabel->setMovie( movie );
+    movielabel->setMargin( 0 );
+    movielabel->setFixedSize( 128+movielabel->frameWidth()*2,
+			      64+movielabel->frameWidth()*2 );
+    grid->addWidget( movielabel, 0, 1, AlignCenter );
+    QToolTip::add( movielabel, "movie" );
 
   // Create a group of check boxes
 
     QButtonGroup *bg = new QButtonGroup( this, "checkGroup" );
     bg->setTitle( "Check Boxes" );
+    grid->addWidget( bg, 1, 0 );
+
+  // Create a layout for the check boxes
+    QVBoxLayout *vbox = new QVBoxLayout(bg, 10);
+
+    vbox->addSpacing( bg->fontMetrics().height() );
+
     cb[0] = new QCheckBox( bg );
     cb[0]->setText( "Read" );
-    cb[0]->setGeometry( 10,15, 100,25 );
+    vbox->addWidget( cb[0] );
+    cb[0]->setMinimumSize( cb[0]->sizeHint() );
     cb[1] = new QCheckBox( bg );
     cb[1]->setText( "Write" );
-    cb[1]->setGeometry( 10,45, 100,25 );
+    vbox->addWidget( cb[1] );
+    cb[1]->setMinimumSize( cb[1]->sizeHint() );
     cb[2] = new QCheckBox( bg );
     cb[2]->setText( "Execute" );
-    cb[2]->setGeometry( 10,75, 100,30 );
-    bg->setGeometry( 10,80, 120,110 );
+    cb[2]->setMinimumSize( cb[2]->sizeHint() );
+    vbox->addWidget( cb[2] );
+    bg->setMinimumSize( bg->childrenRect().size() );
+    vbox->activate();
+
     connect( bg, SIGNAL(clicked(int)), SLOT(checkBoxClicked(int)) );
 
     QToolTip::add( cb[0], "check box 1" );
@@ -160,18 +209,30 @@ WidgetView::WidgetView( QWidget *parent, const char *name )
     QRadioButton *rb;
     bg = new QButtonGroup( this, "radioGroup" );
     bg->setTitle( "Radio buttons" );
+
+    grid->addWidget( bg, 1, 1 );
+
+  // Create a layout for the radio buttons
+    vbox = new QVBoxLayout(bg, 10);
+
+    vbox->addSpacing( bg->fontMetrics().height() );
     rb = new QRadioButton( bg );
     rb->setText( "AM" );
-    rb->setGeometry( 10, 15, 100,25 );
+    rb->setChecked( TRUE );
+    vbox->addWidget(rb);
+    rb->setMinimumSize( rb->sizeHint() );
     QToolTip::add( rb, "radio button 1" );
     rb = new QRadioButton( bg );
     rb->setText( "FM" );
-    rb->setGeometry( 10, 45, 100,25 );
+    vbox->addWidget(rb);
+    rb->setMinimumSize( rb->sizeHint() );
     QToolTip::add( rb, "radio button 2" );
     rb = new QRadioButton( bg );
     rb->setText( "Short Wave" );
-    rb->setGeometry( 10, 75, 100,25 );
-    bg->setGeometry( 140, 80, 120, 110 );
+    vbox->addWidget(rb);
+    rb->setMinimumSize( rb->sizeHint() );
+    vbox->activate();
+
     connect( bg, SIGNAL(clicked(int)), SLOT(radioButtonClicked(int)) );
     QToolTip::add( rb, "radio button 3" );
 
@@ -186,30 +247,36 @@ WidgetView::WidgetView( QWidget *parent, const char *name )
 	else
 	    lb->insertItem( str );
     }
-    lb->setGeometry( 10, 200, 120, 140 );
-    int lbIH = lb->itemHeight();
-    lb->resize( lb->width(), ((lb->height()+lbIH)/lbIH)*lbIH +
-		lb->frameWidth()*2 );		// round up to next line
+    grid->addMultiCellWidget( lb, 2, 4, 0, 0 );
     connect( lb, SIGNAL(selected(int)), SLOT(listBoxItemSelected(int)) );
     QToolTip::add( lb, "list box" );
 
+    vbox = new QVBoxLayout(8);
+    grid->addLayout( vbox, 2, 1 );
+
   // Create a slider
 
-    QSlider *sb = new QSlider( QSlider::Horizontal,this,"Slider" );
+    QSlider *sb = new QSlider( 0, 300, 1, 100, QSlider::Horizontal,
+			       this, "Slider" );
     sb->setTickmarks( QSlider::Below );
-    sb->setGeometry( 140, 200, 120, sb->sizeHint().height() );
+    sb->setTickInterval( 10 );
+    sb->setFocusPolicy( QWidget::TabFocus );
+    sb->setFixedHeight(sb->sizeHint().height());
+    vbox->addWidget( sb );
+
     connect( sb, SIGNAL(valueChanged(int)), SLOT(sliderValueChanged(int)) );
     QToolTip::add( sb, "slider" );
 
   // Create a combo box
 
     QComboBox *combo = new QComboBox( FALSE, this, "comboBox" );
-    combo->insertItem( "True" );
-    combo->insertItem( "False" );
-    combo->insertItem( "Maybe" );
-    combo->insertItem( "Certainly" );
-    combo->insertItem( "Nope" );    
-    combo->setGeometry( 140, 250, 120, 30 );
+    combo->insertItem( "darkBlue" );
+    combo->insertItem( "darkRed" );
+    combo->insertItem( "darkGreen" );
+    combo->insertItem( "blue" );
+    combo->insertItem( "red" );    
+    combo->setFixedHeight(combo->sizeHint().height());
+    vbox->addWidget( combo );
     connect( combo, SIGNAL(activated(int)), SLOT(comboBoxItemActivated(int)) );
     QToolTip::add( combo, "read-only combo box" );
 
@@ -221,45 +288,80 @@ WidgetView::WidgetView( QWidget *parent, const char *name )
     edCombo->insertItem( "Adaptable" );
     edCombo->insertItem( "Alterable" );
     edCombo->insertItem( "Inconstant" );
-    edCombo->setGeometry( 140, 290, 120, 30 );
+    edCombo->setFixedHeight(edCombo->sizeHint().height());
+    vbox->addWidget( edCombo );
     connect( edCombo, SIGNAL(activated(const char *)),
 	     SLOT(edComboBoxItemActivated(const char *)) );
     QToolTip::add( edCombo, "editable combo box" );
 
-  // Create a line edit
+    vbox->addStretch( 1 );
+
+  // Create a multi line edit
+
+    QMultiLineEdit *mle = new QMultiLineEdit( this, "multiLineEdit" );
+    grid->addMultiCellWidget( mle, 3, 3, 1, 2 );
+    mle->setMinimumHeight(mle->fontMetrics().height()*3);
+    QToolTip::add( mle, "multi line editor" );
+
+  // Create a single line edit
 
     QLineEdit *le = new QLineEdit( this, "lineEdit" );
-    le->setGeometry( 140, 340, 230, 25 );
+    grid->addMultiCellWidget( le, 4, 4, 1, 2 );
+    le->setFixedHeight(le->sizeHint().height());
     connect( le, SIGNAL(textChanged(const char *)),
 	         SLOT(lineEditTextChanged(const char *)) );
     QToolTip::add( le, "single line editor" );
 
-  // Create an message label
-  // The message is updated when buttons are clicked etc.
-
-    QLabel *msgLabel = new QLabel( this, "msgLabel" );
-    msgLabel->setText( "Message:" );
-    msgLabel->setAlignment( AlignRight|AlignVCenter );
-    msgLabel->setGeometry( 10,400, 85, 30 );
-    QToolTip::add( msgLabel, "label 1" );
-
-    msg = new QLabel( this, "message" );
-    msg->setFrameStyle( QFrame::Panel | QFrame::Sunken );
-    msg->setAlignment( AlignCenter );
-    msg->setGeometry( 100,400, 300,30 );
-    msg->setFont( QFont("times",12,QFont::Bold) );
-    QToolTip::add( msg, "label 2" );
-
   // Create a horizontal line (sort of QFrame) above the quit button
 
     QFrame *separator = new QFrame( this, "separatorLine" );
     separator->setFrameStyle( QFrame::HLine | QFrame::Sunken );
-    separator->setGeometry( 5, 390, 440, 4 );
+    separator->setFixedHeight( separator->sizeHint().height() );
+    grid->addMultiCellWidget( separator, 5, 5, 0, 2 );
     QToolTip::add( separator, "tool tips on a separator! wow!" );
 
-    adjustSize();
+    grid->setRowStretch(0,0);
+    grid->setRowStretch(1,0);
+    grid->setRowStretch(2,0);
+    grid->setRowStretch(3,1);
+    grid->setRowStretch(4,1);
+    grid->setRowStretch(5,0);
+
+    grid->setColStretch(0,1);
+    grid->setColStretch(1,1);
+    grid->setColStretch(2,1);
+
+
+  // Create an label and a message in a plain widget
+  // The message is updated when buttons are clicked etc.
+
+    QHBoxLayout *hbox = new QHBoxLayout();
+    topLayout->addLayout( hbox );
+    QLabel *msgLabel = new QLabel( this, "msgLabel" );
+    msgLabel->setText( "Message:" );
+    msgLabel->setAlignment( AlignHCenter|AlignVCenter );
+    msgLabel->setFixedSize( msgLabel->sizeHint() );
+    hbox->addWidget( msgLabel );
+    QToolTip::add( msgLabel, "label 1" );
+
+    msg = new QLabel( this, "message" );
+    msg->setFrameStyle( QFrame::Panel | QFrame::Sunken );
+    msg->setAlignment( AlignCenter );
+    msg->setFont( QFont("times",12,QFont::Bold) );
+    msg->setText( "Message" );
+    msg->setFixedHeight( msg->sizeHint().height() );
+    msg->setText( "" );
+    hbox->addWidget( msg, 5 );
+    QToolTip::add( msg, "label 2" );
+
+
+    topLayout->activate();
 }
 
+void WidgetView::setStatus(const char* text)
+{
+    msg->setText(text);
+}
 
 void WidgetView::button1Clicked()
 {
@@ -267,9 +369,24 @@ void WidgetView::button1Clicked()
 }
 
 
-void WidgetView::button2Clicked()
+void WidgetView::movieUpdate( const QRect& )
 {
-    msg->setText( "The second push button was clicked" );
+    // Uncomment this to test animated icons on your window manager
+    //setIcon( movie.framePixmap() );
+}
+
+void WidgetView::movieStatus( int s )
+{
+    switch ( s ) {
+      case QMovie::SourceEmpty:
+	movielabel->setText("Could not load\n" MOVIEFILENAME );
+	movielabel->setAlignment( AlignCenter );
+	movielabel->setBackgroundColor( backgroundColor() );
+      break;
+      default:
+	if ( movielabel->movie() )	 	// for flicker-free animation:
+	    movielabel->setBackgroundMode( NoBackground );
+    }
 }
 
 
@@ -316,8 +433,9 @@ void WidgetView::listBoxItemSelected( int index )
 void WidgetView::sliderValueChanged( int value )
 {
     QString str;
-    str.sprintf( "New slider value %d", value );
+    str.sprintf( "Movie set to %d%% of normal speed", value );
     msg->setText( str );
+    movie.setSpeed( value );
 }
 
 
@@ -326,6 +444,24 @@ void WidgetView::comboBoxItemActivated( int index )
     QString str;
     str.sprintf( "Combo box item %d activated", index );
     msg->setText( str );
+    switch ( index ) {
+    default:
+    case 0:
+	QApplication::setWinStyleHighlightColor( darkBlue );
+	break;
+    case 1:
+	QApplication::setWinStyleHighlightColor( darkRed );
+	break;
+    case 2:
+	QApplication::setWinStyleHighlightColor( darkGreen );
+	break;
+    case 3:
+	QApplication::setWinStyleHighlightColor( blue );
+	break;
+    case 4:
+	QApplication::setWinStyleHighlightColor( red );
+	break;
+    }
 }
 
 
@@ -381,8 +517,9 @@ int main( int argc, char **argv )
 {
     QApplication::setColorSpec( QApplication::CustomColor );
     QApplication a( argc, argv );
-    WidgetView  *w = new WidgetView;
-    a.setMainWidget( w );
-    w->show();
+    QApplication::setFont( QFont("Helvetica") );
+    WidgetView w;
+    a.setMainWidget( &w );
+    w.show();
     return a.exec();
 }
