@@ -1,23 +1,11 @@
 /****************************************************************************
-** $Id: qnp.cpp,v 1.23 1998/07/03 00:09:26 hanord Exp $
+** $Id: qnp.cpp,v 1.12 1997/09/26 11:22:35 hanord Exp $
 **
 ** Implementation of Qt extension classes for Netscape Plugin support.
 **
 ** Created : 970601
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
-**
-** This file is part of Qt Free Edition, version 1.40.
-**
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
-**
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
-**
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** Copyright (C) 1997 by Troll Tech AS.  All rights reserved.
 **
 *****************************************************************************/
 
@@ -51,17 +39,10 @@
 //   - Provide the "reason" parameter to streamDestroyed
 
 // Qt stuff
-#include <qapplication.h>
+#include <qapp.h>
 #include <qwidget.h>
-#include <qobjectlist.h>
-
-#include <qprinter.h>
-#ifdef _WS_X11_
-#include <qpsprinter.h>
-#endif
-#include <qfile.h>
-#include <qpainter.h>
-#include <qpaintdevicedefs.h>
+#include <qobjcoll.h>
+#include <qwidcoll.h>
 
 #include "qnp.h"
 
@@ -164,7 +145,7 @@ static int qnps_no_call_back = 0;
 static QNPWidget* focussedWidget=0;
 
 #ifdef _WS_WIN_
-// defined in qapplication_win.cpp
+// defined in qapp_win.cpp
 extern bool qt_win_use_simple_timers;
 #endif
 
@@ -177,10 +158,10 @@ typedef void (*ForeignEventProc)(XEvent*);
 
 extern XtEventDispatchProc
  qt_np_cascade_event_handler[LASTEvent];      // defined in qnpsupport.cpp
-void            qt_reset_color_avail();       // defined in qcolor_x11.cpp
-void            qt_activate_timers();         // defined in qapplication_x11.cpp
-timeval        *qt_wait_timer();              // defined in qapplication_x11.cpp
-void		qt_x11SendPostedEvents();     // defined in qapplication_x11.cpp
+void            qt_reset_color_avail();       // defined in qcol_x11.cpp
+void            qt_activate_timers();         // defined in qapp_x11.cpp
+timeval        *qt_wait_timer();              // defined in qapp_x11.cpp
+void		qt_x11SendPostedEvents();     // defined in qapp_x11.cpp
 Boolean  qt_event_handler( XEvent* event );   // defined in qnpsupport.cpp
 extern int      qt_np_count;                  // defined in qnpsupport.cpp
 void qt_np_timeout( void* p, void* id );      // defined in qnpsupport.cpp
@@ -301,7 +282,7 @@ public:
 	POINT curPos;
 	if ( GetCursorPos( &curPos ) ) {
 	    QPoint p(curPos.x, curPos.y);
-	
+	    
 	    QNPWidget *newFocussedWidget = 0;
 	    for ( QNPWidget* npw = npwidgets.first();
 		npw; npw = npwidgets.next() )
@@ -315,12 +296,12 @@ public:
 	    }
 	    if (newFocussedWidget != focussedWidget && focussedWidget)
 		focussedWidget->leaveInstance();
-	
+	    
 	    if (newFocussedWidget) {
 		if (newFocussedWidget != focussedWidget)
 		    newFocussedWidget->enterInstance();
 	    }
-	
+	    
 	    focussedWidget = newFocussedWidget;
 	}
     }
@@ -486,7 +467,7 @@ extern "C" jref
 NPP_GetJavaClass(void)
 {
     if (!qNP) qNP = QNPlugin::create();
-    plugin_java_class = (jref)qNP->getJavaClass();
+    plugin_java_class = qNP->getJavaClass();
     return plugin_java_class;
 }
 
@@ -544,7 +525,7 @@ struct NS_Private {
 ** NPP_New is called when your plugin is instantiated (i.e. when an EMBED
 ** tag appears on a page).
 */
-extern "C" NPError
+extern "C" NPError 
 NPP_New(NPMIMEType /*pluginType*/,
     NPP instance,
     uint16 mode,
@@ -558,7 +539,7 @@ NPP_New(NPMIMEType /*pluginType*/,
 
     if (instance == NULL)
         return NPERR_INVALID_INSTANCE_ERROR;
-
+        
     instance->pdata = new _NPInstance;
 
     This = (_NPInstance*) instance->pdata;
@@ -596,7 +577,7 @@ NPP_New(NPMIMEType /*pluginType*/,
     return result;
 }
 
-extern "C" NPError
+extern "C" NPError 
 NPP_Destroy(NPP instance, NPSavedData** /*save*/)
 {
     _NPInstance* This;
@@ -631,7 +612,7 @@ NPP_Destroy(NPP instance, NPSavedData** /*save*/)
 }
 
 
-extern "C" NPError
+extern "C" NPError 
 NPP_SetWindow(NPP instance, NPWindow* window)
 {
     if (!qNP) qNP = QNPlugin::create();
@@ -711,7 +692,6 @@ NPP_SetWindow(NPP instance, NPWindow* window)
 	    next_pi = This;
 	    /* This->widget = */ // (happens sooner - in QNPWidget constructor)
 		This->instance->newWindow();
-	    This->widget->show();
 	} else {
 	    // New window for existing widget, and all its children.
 	    This->widget->setWindow(FALSE);
@@ -723,11 +703,11 @@ NPP_SetWindow(NPP instance, NPWindow* window)
 	if ( This->widget->width() != (int)window->width
 	  || This->widget->height() != (int)window->height )
 	{
-//#ifdef _WS_WIN_
-	    //This->widget->setGeometry(window->x, window->y, window->width, window->height);
-//#else
+#ifdef _WS_WIN_
+	    This->widget->setGeometry(window->x, window->y, window->width, window->height);
+#else
 	    This->widget->resize(window->width, window->height);
-//#endif
+#endif
 	} else {
 	    This->widget->update();
 	}
@@ -738,10 +718,10 @@ NPP_SetWindow(NPP instance, NPWindow* window)
 }
 
 
-extern "C" NPError
+extern "C" NPError 
 NPP_NewStream(NPP instance,
           NPMIMEType type,
-          NPStream *stream,
+          NPStream *stream, 
           NPBool seekable,
           uint16 *stype)
 {
@@ -770,7 +750,7 @@ int32 STREAMBUFSIZE = 0X0FFFFFFF; /* If we are reading from a file in NPAsFile
                                    * mode so we can take any size stream in our
                                    * write call (since we ignore it) */
 
-extern "C" int32
+extern "C" int32 
 NPP_WriteReady(NPP instance, NPStream *stream)
 {
     _NPInstance* This;
@@ -790,7 +770,7 @@ NPP_WriteReady(NPP instance, NPStream *stream)
 }
 
 
-extern "C" int32
+extern "C" int32 
 NPP_Write(NPP instance, NPStream *stream, int32 offset, int32 len, void *buffer)
 {
     if (instance != NULL)
@@ -807,7 +787,7 @@ NPP_Write(NPP instance, NPStream *stream, int32 offset, int32 len, void *buffer)
 }
 
 
-extern "C" NPError
+extern "C" NPError 
 NPP_DestroyStream(NPP instance, NPStream *stream, NPError reason)
 {
     _NPInstance* This;
@@ -819,16 +799,6 @@ NPP_DestroyStream(NPP instance, NPStream *stream, NPError reason)
 	This = (_NPInstance*) instance->pdata;
 
 	QNPStream* qnps = (QNPStream*)stream->pdata;
-	switch (reason) {
-	  case NPRES_DONE:
-	    qnps->setComplete(TRUE);
-	    break;
-	  case NPRES_USER_BREAK:
-	    break;
-	  case NPRES_NETWORK_ERR:
-	    qnps->setOkay(FALSE);
-	    break;
-	}
 
 	if (This) {
 	    // Give the instance a chance to do something
@@ -844,7 +814,7 @@ NPP_DestroyStream(NPP instance, NPStream *stream, NPError reason)
 }
 
 
-extern "C" void
+extern "C" void 
 NPP_StreamAsFile(NPP instance, NPStream *stream, const char* fname)
 {
     _NPInstance* This;
@@ -859,69 +829,30 @@ NPP_StreamAsFile(NPP instance, NPStream *stream, const char* fname)
     }
 }
 
-typedef struct
-{
-    int32    type;
-    FILE*    fp;
-} NPPrintCallbackStruct;
-
-#ifdef _WS_X11_
-
-class QNPPrinter : public QPrinter {
-    QFile file;
-public:
-    QNPPrinter(FILE* fp)
-    {
-	file.open(IO_WriteOnly, fp);
-	QPDevCmdParam param;
-	param.device = &file;
-	cmd(PDC_SETDEV, 0, &param);
-    }
-    void end()
-    {
-	QPDevCmdParam param;
-	param.device = 0;
-	cmd(PDC_SETDEV, 0, &param);
-    }
-};
-#endif
-
-extern "C" void
+extern "C" void 
 NPP_Print(NPP instance, NPPrint* printInfo)
 {
     if(printInfo == NULL)
         return;
 
     if (instance != NULL) {
-        _NPInstance* This = (_NPInstance*) instance->pdata;
-
+        // _NPInstance* This = (_NPInstance*) instance->pdata;
+    
         if (printInfo->mode == NP_FULL) {
-            printInfo->print.fullPrint.pluginPrinted =
-		This->instance->printFullPage();
-        } else if (printInfo->mode == NP_EMBED) {
-#ifdef _WS_X11_
-            void* platformPrint =
-                printInfo->print.embedPrint.platformPrint;
-	    FILE* outfile = ((NPPrintCallbackStruct*)platformPrint)->fp;
-	    if (ftell(outfile)) {
-		NPWindow* w =
-		    &(printInfo->print.embedPrint.window);
-		QNPPrinter prn(outfile);
-		QPainter painter(&prn);
-		// #### config viewport with w->{x,y,width,height}
-		This->instance->print(&painter);
-		prn.end();
-	    } else {
-		// Why does the browser make spurious NPP_Print calls?
-	    }
-#endif
-#ifdef _WS_WIN_
-            NPWindow* printWindow =
-                &(printInfo->print.embedPrint.window);
-            void* platformPrint =
-                printInfo->print.embedPrint.platformPrint;
-	    // #### Nothing yet.
-#endif
+            // void* platformPrint =
+            //     printInfo->print.fullPrint.platformPrint;
+            // NPBool printOne =
+            //     printInfo->print.fullPrint.printOne;
+            
+            /* Do the default*/
+            printInfo->print.fullPrint.pluginPrinted = FALSE;
+        }
+        else {    /* If not fullscreen, we must be embedded */
+
+            // NPWindow* printWindow =
+            //     &(printInfo->print.embedPrint.window);
+            // void* platformPrint =
+            //     printInfo->print.embedPrint.platformPrint;
         }
     }
 }
@@ -994,13 +925,10 @@ Window qt_XCreateWindow( const QWidget* qw, Display *display, Window parent,
     // ### This isA will not work - we are still in QWidget's constructor.
     if ( qw->isTopLevel() && !qw->isA("QNPWidget") ) {
 	// ### not sure it is good to use name() and className().
-	bool cmap = valuemask & CWColormap;
 	Widget xtw = XtVaAppCreateShell( qw->name(), qw->className(),
 	    applicationShellWidgetClass, display,
 	    XtNx, x, XtNy, y, XtNwidth, w, XtNheight, h,
 	    XtNborderWidth, borderwidth, XtNdepth, depth,
-	    XtNvisual, visual,
-	    cmap ? XtNcolormap : 0, cmap ? attributes->colormap : 0,
 	    0, 0 );
 
 	// Ensure it has a window, and get it.
@@ -1072,7 +1000,7 @@ void qt_XDestroyWindow( const QWidget* qw, Display *display, Window window )
 
 #ifdef _WS_WIN_
 
-BOOL   WINAPI   DllMain (HANDLE hInst,
+BOOL   WINAPI   DllMain (HANDLE hInst, 
                         ULONG ul_reason_for_call,
                         LPVOID lpReserved)
 {
@@ -1085,7 +1013,7 @@ BOOL   WINAPI   DllMain (HANDLE hInst,
 	case DLL_THREAD_DETACH:
 	    break;
     }
-	
+	    
     return TRUE;
 }
 
@@ -1101,10 +1029,8 @@ int main(int argc, char** argv)
 /*!
   \class QNPWidget qnp.h
   \brief A QWidget that is a Web-browser plugin window
-  
-  \extension NSPlugin
 
-  Derive from QNPWidget to create a widget that can be used as a
+  Derive from QNPWidget to create a widget that can be used as a 
   Browser plugin window, or create one and add child widgets.
   Instances of QNPWidget may only be created
   when QNPInstance::newWindow() is called by the browser.
@@ -1127,6 +1053,9 @@ public:
 
         // Fill the plugin widget
         child->setGeometry( 0, 0, width(), height() );
+
+        // Show now, since the QNPWidget is already shown
+        child->show();
     }
 
     void resizeEvent(QResizeEvent*)
@@ -1246,12 +1175,10 @@ void QNPWidget::setWindow(bool delold)
 
     create((WId)pi->window, FALSE, delold);
 
-   if ( delold ) {
-      // Make sure they get a show()
-      clearWFlags( WState_Visible );
-   }
-
 #ifdef _WS_X11_
+    // It's open.  Believe me.
+    setWFlags( WState_Visible );
+
     Widget w = XtWindowToWidget (qt_xdisplay(), pi->window);
     XtAddEventHandler(w, EnterWindowMask, FALSE, enter_event_handler, pi);
     XtAddEventHandler(w, LeaveWindowMask, FALSE, leave_event_handler, pi);
@@ -1273,11 +1200,11 @@ void QNPWidget::setWindow(bool delold)
 
     createNewWindowsForAllChildren(this);
 
-//#ifdef _WS_WIN_
-    //setGeometry( pi->x, pi->y, pi->width, pi->height );
-//#else
+#ifdef _WS_WIN_
+    setGeometry( pi->x, pi->y, pi->width, pi->height );
+#else
     resize( pi->width, pi->height );
-//#endif
+#endif
 }
 
 /*!
@@ -1306,9 +1233,7 @@ void QNPWidget::unsetWindow()
   \class QNPInstance qnp.h
   \brief a QObject that is a Web-browser plugin
 
-  \extension NSPlugin
-
-  Deriving from QNPInstance creates an object that represents a single
+  Deriving from QNPInstance is creates an object that represents a single
   &lt;EMBED&gt; tag in an HTML document.
 
   The QNPInstance is responsible for creating an appropriate window if
@@ -1387,9 +1312,6 @@ QNPWidget* QNPInstance::widget()
   function which must consume at least as much data as was returned
   by the most recent call to writeReady().
 
-  Note that the AsFileOnly method is not supported by Netscape 2.0
-  and MS-Explorer 3.0.  A future version of Qt will hide this problem.
-
   See also:
   <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgst.htm#nppnewstream>
   Netscape: NPP_NewStream method</a>
@@ -1405,9 +1327,6 @@ bool QNPInstance::newStreamCreated(QNPStream*, StreamMode&)
   precludes any incremental behaviour.
   \sa newStreamCreated(), newStream()
 
-  Note that the AsFileOnly method is not supported by Netscape 2.0
-  and MS-Explorer 3.0.
-
   See also:
   <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgst.htm#nppstreamasfile>
   Netscape: NPP_StreamAsFile method</a>
@@ -1417,10 +1336,7 @@ void QNPInstance::streamAsFile(QNPStream*, const char*)
 }
 
 /*!
-  Called when a stream is destroyed.  At this this, the stream may
-  be complete() and okay().  If it is not okay(), then an error has
-  occurred.  If it is okay(), but not complete(), then the user has
-  cancelled the transmission - do not give an error message in this case.
+  Called when a stream is destroyed.
 
   See also:
   <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgst.htm#nppdestroystream>
@@ -1497,39 +1413,6 @@ void QNPInstance::postURL(const char* url, const char* window,
     NPN_PostURL( pi->npp, url, window, len, buf, file );
 }
 
-/*!
-  Print the instance full-page.  By default, this returns FALSE, causing the
-  browser to call the (embedded) print() function instead.
-
-  This function is not tested.
-  It is an encapsulation of the NPP_Print
-  function of the Netscape Plugin API.
-
-  See also:
-  <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgdr.htm#nppprint>
-  Netscape: NPP_Print method</a>
-*/
-bool QNPInstance::printFullPage()
-{
-    return FALSE;
-}
-
-/*!
-  Print the instance embedded in a page.
-
-  This function is not tested.
-  It is an encapsulation of the NPP_Print
-  function of the Netscape Plugin API.
-
-  See also:
-  <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgdr.htm#nppprint>
-  Netscape: NPP_Print method</a>
-*/
-void QNPInstance::print(QPainter*)
-{
-    // ### default could redirected-print the window.
-}
-
 /*!
   Returns the number of arguments to the instance.  Note that you should
   not normally rely on the ordering of arguments, and also note that
@@ -1619,8 +1502,7 @@ QNPStream* QNPInstance::newStream(const char* mimetype, const char* window,
     bool as_file)
 {
     NPStream* s=0;
-    NPError err = NPN_NewStream(pi->npp, (char*)mimetype, window, &s);
-    if (err != NPERR_NO_ERROR) return 0;
+    NPN_NewStream(pi->npp, (char*)mimetype, window, &s);
     return s ? new QNPStream(this, mimetype, s, as_file) : 0;
 }
 
@@ -1639,10 +1521,10 @@ void QNPInstance::status(const char* msg)
 
 /*!
   Returns the Java object associated with the plug-in instance, an
-  object of the
-  \link QNPlugin::getJavaClass() plug-in's Java class\endlink,
+  object of the 
+  \link QNPlugin::getJavaClass() plug-in's Java class\endlink, 
   or 0 if the plug-in does not have a Java class, Java is disabled, or
-  an error occurred .
+  an error occurred .  
 
   The return value is actually a <tt>jref</tt> we use <tt>void*</tt> so
   as to avoid burdening plugins which do not require Java.
@@ -1661,10 +1543,6 @@ void* QNPInstance::getJavaPeer() const
   \class QNPStream qnp.h
   \brief A stream of data provided to a QNPInstance by the browser.
 
-  \extension NSPlugin
-
-  Note that this is neither a QTextStream or a QDataStream.
-
   \sa QNPInstance::write(), QNPInstance::newStreamCreated()
 */
 
@@ -1678,8 +1556,6 @@ QNPStream::QNPStream(QNPInstance* in,const char* mt, _NPStream* st, bool se) :
     mtype(mt),
     seek(se)
 {
-    isokay = TRUE;
-    iscomplete = FALSE;
 }
 
 /*!
@@ -1689,7 +1565,7 @@ QNPStream::~QNPStream()
 {
     if (!qnps_no_call_back) {
 	qnps_no_call_back++;
-	NPN_DestroyStream(inst->pi->npp, stream, NPRES_USER_BREAK);
+	NPN_DestroyStream(inst->pi->npp, stream, 0);
 	qnps_no_call_back--;
     }
 }
@@ -1740,39 +1616,6 @@ bool QNPStream::seekable() const
     return seek;
 }
 
-/*!
-  \internal
-*/
-void QNPStream::setOkay(bool y)
-{
-    isokay = y;
-}
-
-/*!
-  \internal
-*/
-void QNPStream::setComplete(bool y)
-{
-    iscomplete = y;
-}
-
-/*!
-  Returns TRUE if no errors have occurred on the stream.
-*/
-bool QNPStream::okay() const
-{
-    return isokay;
-}
-
-/*!
-  Returns TRUE if the stream has received all the data from
-  the source.
-*/
-bool QNPStream::complete() const
-{
-    return iscomplete;
-}
-
 /*!
   Requests the given section of the stream be sent to the
   QNPInstance::write() function of the instance() of this stream.
@@ -1813,8 +1656,6 @@ int QNPStream::write( int len, void* buffer )
   \class QNPlugin qnp.h
   \brief The plugin central factory.
 
-  \extension NSPlugin
-
   This class is the heart of the plugin.  One instance of this object is
   created when the plugin is \e first needed, by calling
   QNPlugin::create(), which must be implemented in your plugin code to
@@ -1845,16 +1686,12 @@ QNPlugin* QNPlugin::actual()
 }
 
 /*!
-  Creates a QNPlugin.  This may only be used by the constructor
+  Creates a QNPlugin.  This may only be used by the constructor 
   derived class
   returned by plugin's implementation of the QNPlugin::create() function.
 */
 QNPlugin::QNPlugin()
 {
-    // Encourage linker to include stuff.
-    static void* a;
-    a = (void*)NP_Initialize;
-    a = (void*)NP_Shutdown;
 }
 
 /*!
