@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: glbox.h,v 1.3 1997/10/17 05:27:05 aavit Exp $
+** $Id: glbox.h,v 1.2 1997/08/19 12:21:05 aavit Exp $
 **
 ** Definition of GLBox
 ** This is a simple QGLWidget displaying an openGL wireframe box
@@ -14,7 +14,7 @@
 
 class GLBox : public QGLWidget
 {
-    Q_OBJECT
+    Q_OBJECT;
 
 public:
 
