@@ -1,73 +1,42 @@
 /****************************************************************************
-** $Id: qgl.cpp,v 1.24 1998/07/03 00:09:27 hanord Exp $
+** $Id: qgl.cpp,v 1.3 1997/07/08 13:25:16 aavit Exp $
 **
 ** Implementation of OpenGL classes for Qt
 **
 ** Created : 970112
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
-**
-** This file is part of Qt Free Edition, version 1.40.
-**
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
-**
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
-**
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** Copyright (C) 1997 by Troll Tech AS.  All rights reserved.
 **
 *****************************************************************************/
 
 #include "qgl.h"
 
 #if defined(Q_GLX)
-#include <qintdict.h>
 #define INT8  dummy_INT8
 #define INT32 dummy_INT32
 #include <GL/glx.h>
 #undef  INT8
 #undef  INT32
-#define	 GC GC_QQQ
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include <X11/Xos.h>
-#include <X11/Xatom.h>
-#include <X11/Xmu/StdCmap.h>
 #endif
 
+RCSTAG("$Id: qgl.cpp,v 1.3 1997/07/08 13:25:16 aavit Exp $");
+
+
 #if defined(_CC_MSVC_)
 #pragma warning(disable:4355) // 'this' : used in base member initializer list
 #endif
 
 
-/*!
-  \relates QGLFormat
-  Returns the version number string for the Qt OpenGL extension,
-  e.g. "1.0".
-*/
-
-const char *qGLVersion()
-{
-    return QGL_VERSION_STR;
-}
-
-
 /*****************************************************************************
   QGLFormat implementation
  *****************************************************************************/
 
-
-/*!
+ 
+ /*!
   \class QGLFormat qgl.h
-  \brief The QGLFormat class specifies the display format of an OpenGL
-  rendering context.
+  \brief The QGLFormat class specifies the format of an OpenGL rendering context.
 
-  \extension OpenGL
-
-  A display format has several characteristics:
+  A rendering context has several characteristics:
   <ul>
   <li> \link setDoubleBuffer() Double or single buffering.\endlink
   <li> \link setDepth() Depth buffer.\endlink
@@ -80,16 +49,15 @@ const char *qGLVersion()
 
   You create and tell a QGLFormat object what rendering options
   you want from an OpenGL rendering context.
-
   OpenGL drivers or accelerated hardware may or may not support
   advanced features like alpha blending or stereographic viewing.
   If you request alpha blending and the driver/hardware does not
   provide it, you will get an invalid context. You can then try
   again with a less demanding format.
 
-  There are different ways of defining the display characteristics
-  of a rendering context. One is to create a QGLFormat and make
-  it default for the entire application:
+  There are many different ways of defining the characteristics
+  of a rendering context. One is to create a QGLContext and make
+  it default for the entire application.
   \code
     QGLFormat f;
     f.setAlpha( TRUE );
@@ -97,19 +65,17 @@ const char *qGLVersion()
     QGLFormat::setDefaultFormat( f );
   \endcode
 
-  You can also set a format for a specific widget (subclassed from
-  QGLWidget):
+  You can also set a format for your subclassed QGLWidget:
   \code
-    MyGLWidget *w;
+    QGLWidget *w;
       ...
     QGLFormat f;
     f.setAlpha( TRUE );
     f.setStereo( TRUE );
     w->setFormat( f );
-    if ( !w->isValid() ) {
-        f.setStereo( FALSE );	// ok, goggles off
-        w->setFormat( f );
-        if ( !w->isValid() ) {
+    if ( !w->create() ) {
+        f.setStereo( FALSE );	// ok, googles off
+        if ( !w->create() ) {
             fatal( "Cool hardware wanted" );
         }
     }
@@ -118,7 +84,7 @@ const char *qGLVersion()
   <em>Implementation note:</em>
   QGLFormat objects use implicit sharing. This is an internal
   optimization that minimizes copying and memory overhead.
-  When you pass a QGLFormat as an argument to a function, or
+  When you pass a QGLFormat as an argument to a function,or
   return a QGLFormat from a function, a reference count is
   incremented or decremented. Objects stay shared until one
   of them is changed.
@@ -143,7 +109,7 @@ const char *qGLVersion()
 
 QGLFormat::QGLFormat( bool doubleBuffer )
 {
-    data = new FormatFlags;
+    data = new Internal;
     CHECK_PTR( data );
     data->doubleBuffer = doubleBuffer;
     data->depth	       = TRUE;
@@ -453,7 +419,7 @@ const QGLFormat &QGLFormat::defaultFormat()
     QApplication a(argc, argv);
     QGLFormat f;
     f.setDoubleBuffer( FALSE );
-    QGLFormat::setDefaultFormat( f );
+    QGLFormat::setDefault( f );
   \endcode
 */
 
@@ -471,23 +437,19 @@ void QGLFormat::setDefaultFormat( const QGLFormat &f )
 
 /*!
   \class QGLContext qgl.h
-  \brief The QGLContext class encapsulates an OpenGL rendering context.
-
-  \extension OpenGL
-
-  An OpenGL rendering context is a complete set of OpenGL state
-  variables.
+  \brief The QGLContext class provides an OpenGL context.
 
+  A context is...
 */
 
 
 /*!
   Constructs an OpenGL context for the paint device \a device,
   which must be a widget or a pixmap.
-  The \a format specifies several display options for this context.
+  The \a format specify several display options for this context.
 
-  The context will be \link isValid() invalid\endlink if the \a format
-  settings cannot be satified by the underlying OpenGL/Window system.
+  The context will be \link isValid() invalid\endlink if the
+  \a format settings cannot be satified.
 
   \sa isValid()
 */
@@ -520,7 +482,7 @@ QGLContext::~QGLContext()
 
 
 /*!
-  \fn const QGLFormat& QGLContext::format() const
+  \fn const QGLFormat QGLContext::format() const
   Returns the format.
   \sa setFormat()
 */
@@ -554,9 +516,7 @@ void QGLContext::setFormat( const QGLFormat &format )
 
 /*!
   \fn bool QGLContext::isValid() const
-  Returns TRUE if the characteristics specified in the \link setFormat()
-  display format\endlink could be fulfilled by the OpenGL driver/window
-  system.
+  Returns TRUE if the widget was able to satisfy the specified constraints 
 */
 
 /*!
@@ -564,9 +524,7 @@ void QGLContext::setFormat( const QGLFormat &format )
   a context that satisfies the requested \link setFormat() format\endlink,
   otherwise FALSE is returned (the context is invalid).
 
-  If successful and the \a shareContext points to a valid QGLContext,
-  this context will share OpenGL display lists with \a shareContext.
-
+  \internal
   <strong>Implementation note:</strong> Initialization of C++ class members
   usually takes place in the class constructor. QGLContext is an exception
   because it must be simple to customize. The virtual functions
@@ -578,18 +536,22 @@ void QGLContext::setFormat( const QGLFormat &format )
   \sa chooseContext(), isValid()
 */
 
-bool QGLContext::create( const QGLContext* shareContext )
+bool QGLContext::create()
 {
-    reset();
-    valid = chooseContext( shareContext );
+    if ( valid )
+	reset();
+    valid = chooseContext();
     return valid;
 }
 
 
+/*****************************************************************************
+  QGLContext Win32/WGL-specific code
+ *****************************************************************************/
+
+#if defined(Q_WGL)
 
 /*!
-  \fn bool QGLContext::chooseContext( const QGLContext* shareContext = 0 )
-
   This semi-internal function is called by create(). It creates a
   system-dependent OpenGL handle that matches the specified \link
   setFormat() format\endlink.
@@ -604,60 +566,8 @@ bool QGLContext::create( const QGLContext* shareContext )
   subclass if you need to choose a very custom context.
 */
 
-
-/*!
-  \fn void QGLContext::reset()
-
-  Resets the context and makes it invalid.
-  \sa create(), isValid()
-*/
-
-
-/*!
-  \fn void QGLContext::makeCurrent()
-
-  Makes this context the current OpenGL rendering context.  All gl
-  functions you call operate on this context until another context is
-  made current.
-*/
-
-
-/*!
-  \fn void QGLContext::swapBuffers()
-
-  Swaps the screen contents with an off-screen buffer. Works only if
-  the  context is in double buffer mode.
-  \sa QGLFormat::setDoubleBuffer()
-*/
-
-
-/*!
-  \fn void QGLContext::doneCurrent()
-
-  Resets the current OpenGL rendering context.  Normally, you do not need
-  to call this function, because makeCurrent() takes care of that.
-*/
-
-
-/*!
-  \fn QPaintDevice* QGLContext::device() const
-
-  Returns the paint device set for this context.
-
-  \sa QGLContext::QGLContext()
-*/
-
-
-#if defined(Q_WGL)
-
-/*****************************************************************************
-  QGLContext Win32/WGL-specific code
- *****************************************************************************/
-
-
-bool QGLContext::chooseContext( const QGLContext* shareContext )
+bool QGLContext::chooseContext()
 {
-    bool success = FALSE;
     if ( paintDevice->devType() == PDT_WIDGET ) {
 	win = ((QWidget *)paintDevice)->winId();
 	dc  = GetDC( win );
@@ -665,41 +575,33 @@ bool QGLContext::chooseContext( const QGLContext* shareContext )
 	win = 0;
 	dc  = paintDevice->handle();
     }
-    ASSERT( dc != 0 );
     PIXELFORMATDESCRIPTOR pfd;
     int pixelFormatId = choosePixelFormat( &pfd );
-    if ( pixelFormatId != 0 ) {
-	if ( SetPixelFormat(dc, pixelFormatId, &pfd) ) {
-	    rc = wglCreateContext( dc );
-	    success = TRUE;
-	} else {
-#if 0
+    if ( pixelFormatId == 0 ) {
+	rc = 0;
+	dc = 0;
+    } else {
+	if ( !SetPixelFormat( dc, pixelFormatId, &pfd ) ) {
 	    LPVOID lpMsgBuf;
-	    FormatMessage(
+	    FormatMessage( 
 		FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
 		0, GetLastError(),
 		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
 		(LPTSTR) &lpMsgBuf, 0, 0 );
-	    warning( (const char *)lpMsgBuf );
+	    //debug( (const char *)lpMsgBuf );
 	    LocalFree( lpMsgBuf );
-#endif
-	}
+ 	}
+	rc = wglCreateContext( dc );
     }
-    if ( success && shareContext && shareContext->isValid() )
-	success = wglShareLists( shareContext->rc, rc );
     if ( win ) {
 	ReleaseDC( win, dc );
 	dc = 0;
     }
-    if ( !success ) {
-	rc = 0;
-	dc = 0;
-    }
-    return success;
+    return TRUE;
 }
 
 
-/*
+/*!
   <strong>Win32 only</strong>: This virtual function chooses a pixel format
   that matchesthe OpenGL \link setFormat() format\endlink. Reimplement this
   function in a subclass if you need a custom context.
@@ -746,6 +648,10 @@ int QGLContext::choosePixelFormat( void *pfd )
 }
 
 
+/*!
+  Resets the context and makes it invalid.
+  \sa create(), isValid()
+*/
 
 void QGLContext::reset()
 {
@@ -753,35 +659,39 @@ void QGLContext::reset()
 	return;
     doneCurrent();
     wglDeleteContext( rc );
-    rc  = 0;
-    dc  = 0;
+    rc = 0;
+    dc = 0;
     win = 0;
     valid = FALSE;
 }
 
-
-static QGLContext *currentContext = 0;
-
 //
 // NOTE: In a multi-threaded environment, each thread has a current
 // context. If we want to make this code thread-safe, we probably
 // have to use TLS (thread local storage) for keeping current contexts.
 //
 
+static QGLContext *currentContext = 0;
+
+/*!
+  Makes this context the current context
+ */
 void QGLContext::makeCurrent()
 {
+    /*
     if ( currentContext ) {
 	if ( currentContext == this )		// already current
 	    return;
 	currentContext->doneCurrent();
     }
-    if ( !valid )
-	return;
-    if ( win )
+    */
+    dc = paintDevice->handle();
+    if ( dc ) {
+	tmp_dc = FALSE;
+    } else {
+	tmp_dc = TRUE;
 	dc = GetDC( win );
-    else
-	dc = paintDevice->handle();
-    ASSERT( dc != 0 );
+    }
     if ( QColor::hPal() ) {
 	SelectPalette( dc, QColor::hPal(), FALSE );
 	RealizePalette( dc );
@@ -790,19 +700,27 @@ void QGLContext::makeCurrent()
     currentContext = this;
 }
 
-
 void QGLContext::doneCurrent()
 {
+    return;
     if ( currentContext != this )
 	return;
-    currentContext = 0;
-    wglMakeCurrent( dc, 0 );
-    if ( win ) {
+    wglMakeCurrent( 0, 0 );
+    if ( tmp_dc ) {
+#if defined(DEBUG)
+	ASSERT( win != 0 );
+#endif
 	ReleaseDC( win, dc );
 	dc = 0;
     }
+    currentContext = 0;
 }
 
+/*!
+  Swaps the screen contents with an off-screen buffer. Works only if
+  the  context is in double buffer mode.
+  \sa QGLFormat::setDoubleBuffer()
+*/
 
 void QGLContext::swapBuffers()
 {
@@ -813,28 +731,13 @@ void QGLContext::swapBuffers()
 #endif // Q_WGL
 
 
+/*****************************************************************************
+  QGLContext Unix/GLX-specific code
+ *****************************************************************************/
 
 #if defined(Q_GLX)
 
-/*****************************************************************************
-  QGLContext UNIX/GLX-specific code
- *****************************************************************************/
-
-bool QGLContext::chooseContext( const QGLContext* shareContext )
-{
-    vi = chooseVisual();
-    if ( vi == 0 )
-	return FALSE;
-    if ( shareContext && shareContext->cx )
-	cx = glXCreateContext( paintDevice->x11Display(), (XVisualInfo *)vi,
-			       (GLXContext)shareContext->cx, TRUE );
-    else
-	cx = glXCreateContext( paintDevice->x11Display(), (XVisualInfo *)vi,
-			       None, TRUE );
-    return cx != 0;
-}
-
-/*
+/*!
   <strong>X11 only</strong>: This virtual function chooses a visual
   that matches the OpenGL \link setFormat() format\endlink. Reimplement this
   function in a subclass if you need a custom visual.
@@ -897,6 +800,16 @@ void *QGLContext::chooseVisual()
 }
 
 
+bool QGLContext::chooseContext()
+{
+    vi = chooseVisual();
+    if ( vi == 0 )
+	return FALSE;
+    cx = glXCreateContext( paintDevice->x11Display(), (XVisualInfo *)vi,
+			   None, TRUE );
+    return cx != 0;
+}
+
 void QGLContext::reset()
 {
     if ( !valid )
@@ -922,7 +835,6 @@ void QGLContext::makeCurrent()
 
 void QGLContext::doneCurrent()
 {
-    glXMakeCurrent( paintDevice->x11Display(), 0, 0 );
 }
 
 
@@ -939,7 +851,6 @@ void QGLContext::swapBuffers()
 #endif // Q_GLX
 
 
-
 /*****************************************************************************
   QGLWidget implementation
  *****************************************************************************/
@@ -949,23 +860,18 @@ void QGLContext::swapBuffers()
   \class QGLWidget qgl.h
   \brief The QGLWidget class is a widget for rendering OpenGL graphics.
 
-  \extension OpenGL
-
-  It is easy to render OpenGL graphics in Qt applications. You can
-  create a subclass of QGLWidget and reimplement two functions:
-  resizeGL() and paintGL(). The resizeGL() method is called whenever
-  the widget has been resized (and also when it shown for the first
-  time, since all newly created widgets get a resize event
-  automatically). paintGL() is called when the widget needs to be
-  updated.
+  It is easy to render OpenGL graphics in Qt applications. You can create
+  a subclass of QGLWidget and reimplement two functions: resizeGL() and
+  paintGL(). resizeGL() is called whenever the widget has been resized.
+  paintGL() is called when the widget needs to be updated.
 
   \code
-    class MyGLDrawer : public QGLWidget
+    class DrawGL : public QGLWidget
     {
-        Q_OBJECT	// include this if you use Qt signals/slots
+        QOBJECT;   // Include this if you want to use Qt signals & slots etc.
 
     public:
-        MyGLDrawer( QWidget *parent, const char *name )
+        DrawGL( QWidget *parent, const char *name )
 	    : QGLWidget(parent,name) {}
 
     protected:
@@ -976,22 +882,18 @@ void QGLContext::swapBuffers()
 
 	void resizeGL( int w, int h )
 	{
-	  // setup viewport, projection etc.
+	  // setup viewport, display lists etc.
 	}
     };
   \endcode
 
   If you need to repaint from other places than paintGL() (a typical
-  example is when using \link QTimer timers\endlink to animate scenes),
-  you can call the updateGL() function.
+  example is when using \link QTimer timers\endlink to animate scenes)
+  you must call makeCurrent() before you draw and probably swapBuffers()
+  when you are finished.
 
-  When paintGL() or resizeGL() is called, your widget has been made
-  current.  If you need to call the standard OpenGL API functions from
-  other places (e.g. in your widget's constructor), you must call
-  makeCurrent() first.
-
-  Like QGLContext, QGLWidget has advanced functions for requesting a
-  new display format, and you can even set a new rendering context.
+  Like QGLContext, QGLWidget has advanced functions for requesting
+  a new contex format and you can even set a new context.
 */
 
 
@@ -1001,31 +903,15 @@ void QGLContext::swapBuffers()
   The \link QGLFormat::defaultFormat() default format\endlink is
   used.
 
-  The \e parent, \e name and \e f arguments are passed to the QWidget
-  constructor.
-
-  If the \a shareWidget parameter points to a valid QGLWidget, this
-  widget will share OpenGL display lists with \a shareWidget.
-
-  Note: Initialization of OpenGL rendering state etc. should be done
-  by overriding the initializeGL() function, not in the constructor of
-  your QGLWidget subclass.
-
   \sa QGLFormat::defaultFormat()
 */
 
-QGLWidget::QGLWidget( QWidget *parent, const char *name,
-		      const QGLWidget* shareWidget, WFlags f )
-    : QWidget(parent, name, f)
+QGLWidget::QGLWidget( QWidget *parent, const char *name )
+    : QWidget(parent, name)
 {
-    setBackgroundMode( NoBackground );
-    initDone = FALSE;
+    setBackgroundColor( black );
     glcx = 0;
-    if ( shareWidget )
-	setContext( new QGLContext( QGLFormat::defaultFormat(), this ),
-		    shareWidget->context() );
-    else
-	setContext( new QGLContext( QGLFormat::defaultFormat(), this ) );
+    setContext( new QGLContext(QGLFormat::defaultFormat(),this) );
 }
 
 
@@ -1036,44 +922,16 @@ QGLWidget::QGLWidget( QWidget *parent, const char *name,
   The widget becomes invalid if the driver/hardware cannot satisfy
   the requested format.
 
-  The \e parent, \e name and \e f arguments are passed to the QWidget
-  constructor.
-
-  If the \a shareWidget parameter points to a valid QGLWidget, this
-  widget will share OpenGL display lists with \a shareWidget.
-
-  Note: Initialization of OpenGL rendering state etc. should be done
-  by overriding the initializeGL() function, not in the constructor of
-  your QGLWidget subclass.
-
-  \sa QGLFormat::defaultFormat(), isValid()
+  \sa isValid()
 */
 
 QGLWidget::QGLWidget( const QGLFormat &format, QWidget *parent,
-		      const char *name, const QGLWidget* shareWidget,
-		      WFlags f )
-    : QWidget(parent, name, f)
+		      const char *name )
+    : QWidget(parent, name)
 {
-    setBackgroundMode( NoBackground );
-    initDone = FALSE;
+    setBackgroundColor( black );
     glcx = 0;
-    if ( shareWidget )
-	setContext( new QGLContext( format, this ), shareWidget->context() );
-    else
-	setContext( new QGLContext( format, this ) );
-}
-
-
-/*!
-  Destroys the widget.
-*/
-
-QGLWidget::~QGLWidget()
-{
-    delete glcx;
-#if defined(GLX_MESA_release_buffers)
-    glXReleaseBuffersMESA( dpy, winId() );
-#endif
+    setContext( new QGLContext(format,this) );
 }
 
 
@@ -1085,150 +943,42 @@ QGLWidget::~QGLWidget()
 
 /*!
   \fn bool QGLWidget::doubleBuffer() const
-  Returns TRUE if double buffering is set in the format for this widget.
+  Returns TRUE if double buffering is set for this widget.
   \sa QGLFormat::doubleBuffer()
 */
 
 
 /*!
   \fn bool QGLWidget::isValid() const
-  Returns TRUE if the widget was able to satisfy the specified constraints
+  Returns TRUE if the widget was able to satisfy the specified constraints 
 */
 
 /*!
   \fn void QGLWidget::makeCurrent()
-  Makes this widget the current widget for OpenGL
-  operations. I.e. makes this widget's rendering context the current
-  OpenGL rendering context.
+  Makes this widget the current widget for OpenGL operations.
 */
 
 /*!
   \fn void QGLWidget::swapBuffers()
   Swaps the screen contents with an off-screen buffer. Works only if
-  the widget's format specifies double buffer mode.
-  \sa doubleBuffer(), QGLFormat::setDoubleBuffer()
+  the  context is in double buffer mode.
+  \sa QGLFormat::setDoubleBuffer()
 */
 
 
 /*!
   Sets a new format for this widget. The widget becomes invalid if the
   requested format cannot be satisfied.
-
-  Calling setFormat() will not alter any display list sharing this
-  widget currently has.
-
   \sa format(), setContext(), isValid()
 */
 
 void QGLWidget::setFormat( const QGLFormat &format )
 {
-    setContext( new QGLContext(format,this) );
+    glcx->setFormat( format );
+    glcx->create();
 }
 
 
-#if defined(Q_GLX)
-
-/*
-  The create_cmap function is internal and used by QGLWidget::setContext()
-  and GLX (not Windows).  If the application can't find any sharable
-  colormaps, it must at least create as few colormaps as possible.  The
-  dictionary solution below ensures only one colormap is created per visual.
-  Colormaps are also deleted when the application terminates.
-*/
-
-struct CMapEntry {
-    CMapEntry( Colormap m, bool a ) : cmap(m), alloc(a) {}
-   ~CMapEntry();
-    Colormap cmap;
-    bool     alloc;
-};
-
-CMapEntry::~CMapEntry()
-{
-    if ( alloc )
-	XFreeColormap( QPaintDevice::x__Display(), cmap );
-}
-
-static bool		    cmap_init = FALSE;
-static QIntDict<CMapEntry> *cmap_dict = 0;
-static bool		    mesa_gl   = FALSE;
-
-static void cleanup_cmaps()
-{
-    if ( !cmap_dict )
-	return;
-    cmap_dict->setAutoDelete( TRUE );
-    delete cmap_dict;
-    cmap_dict = 0;
-}
-
-static Colormap choose_cmap( Display *dpy, XVisualInfo *vi )
-{
-    if ( !cmap_init ) {
-	cmap_init = TRUE;
-	cmap_dict = new QIntDict<CMapEntry>;
-	const char *v = glXQueryServerString( dpy, vi->screen, GLX_VERSION );
-	mesa_gl = strstr(v,"Mesa") != 0;
-	qAddPostRoutine( cleanup_cmaps );
-    }
-
-    CMapEntry *x = cmap_dict->find( (long)vi->visualid );
-    if ( x )					// found colormap for visual
-	return x->cmap;
-
-    Colormap cmap = 0;
-    bool alloc = FALSE;
-    XStandardColormap *c;
-    int n, i;
-
-    if ( vi->visual==DefaultVisual(dpy,vi->screen) )
-	return DefaultColormap( dpy, vi->screen );
-
-    if ( mesa_gl ) {				// we're using MesaGL
-	Atom hp_cmaps = XInternAtom( dpy, "_HP_RGB_SMOOTH_MAP_LIST", TRUE );
-	if ( hp_cmaps && vi->visual->c_class == TrueColor && vi->depth == 8 ) {
-	    if ( XGetRGBColormaps(dpy,RootWindow(dpy,vi->screen),&c,&n,
-				  hp_cmaps) ) {
-		i = 0;
-		while ( i < n && cmap == 0 ) {
-		    if ( c[i].visualid == vi->visual->visualid )
-			cmap = c[i].colormap;
-		    i++;
-		}
-		XFree( (char *)c );
-	    }
-	}
-    }
-#if !defined(_OS_SOLARIS_)
-    if ( !cmap ) {
-	if ( XmuLookupStandardColormap(dpy,vi->screen,vi->visualid,vi->depth,
-				       XA_RGB_DEFAULT_MAP,FALSE,TRUE) ) {
-	    if ( XGetRGBColormaps(dpy,RootWindow(dpy,vi->screen),&c,&n,
-				  XA_RGB_DEFAULT_MAP) ) {
-		i = 0;
-		while ( i < n && cmap == 0 ) {
-		    if ( c[i].visualid == vi->visualid )
-			cmap = c[i].colormap;
-		    i++;
-		}
-		XFree( (char *)c );
-	    }
-	}
-    }
-#endif
-    if ( !cmap ) {				// no shared cmap found
-	cmap = XCreateColormap( dpy, RootWindow(dpy,vi->screen), vi->visual,
-				AllocNone );
-	alloc = TRUE;
-    }
-    x = new CMapEntry( cmap, alloc );		// associate cmap with visualid
-    cmap_dict->insert( (long)vi->visualid, x );
-    return cmap;
-}
-
-#endif // Q_GLX
-
-
 /*!
   \fn const QGLContext *QGLWidget::context() const
   Returns the current context.
@@ -1236,20 +986,11 @@ static Colormap choose_cmap( Display *dpy, XVisualInfo *vi )
 */
 
 /*!
-  Sets a new context for this widget. The context must be created using
-  \e new.  QGLWidget will delete the context when another context is set
-  or when the widget is destroyed.
-
-  Calling setContext() will not alter any display list sharing this
-  widget currently has, unless you explicitly request sharing with a
-  different context by giving a \a shareContext parameter that points
-  to a valid context.
-
+  Sets a new context for this widget. The old context is deleted.
   \sa context(), setFormat()
 */
 
-void QGLWidget::setContext( QGLContext *context,
-			    const QGLContext* shareContext )
+void QGLWidget::setContext( QGLContext *context )
 {
     if ( context == 0 ) {
 #if defined(CHECK_NULL)
@@ -1261,43 +1002,31 @@ void QGLWidget::setContext( QGLContext *context,
 	warning( "QGLWidget::setContext: Context must refer this widget" );
 #endif
     }
-
-    QGLContext* oldcx = glcx;
-
+    delete glcx;
     glcx = context;
-    initDone = FALSE;
-    bool createFailed = FALSE;
-
-#if defined(Q_WGL)
-
-    if ( oldcx ) {
-	// We already have a context and must therefore create a new
-	// window since Windows does not permit setting a new OpenGL
-	// context for a window that already has one set.
-	destroy( TRUE, TRUE );
-	create( 0, TRUE, TRUE );
-    }
-
-#endif
-
     if ( !glcx->isValid() ) {
-	if ( !glcx->create( shareContext ? shareContext : oldcx ) )
-	    createFailed = TRUE;
+	if ( !glcx->create() )
+	    return;
     }
-    delete oldcx;
-    if ( createFailed )
-	return;
-
 #if defined(Q_GLX)
-
     bool visible = isVisible();
     if ( visible )
-	hide();
-
+        hide();
     XVisualInfo *vi = (XVisualInfo*)glcx->vi;
-    XSetWindowAttributes a;
+    if ( XVisualIDFromVisual(vi->visual) ==
+	 XVisualIDFromVisual((Visual*)QPaintDevice::x11Visual()) )
+	return;
 
-    a.colormap = choose_cmap( dpy, vi );	// find best colormap
+    /*
+      Here we can optimize:
+	1) Create one colormap for each visual.
+	2) Recreate window only if necessary.
+    */
+    Colormap cmap;
+    cmap = XCreateColormap( dpy, RootWindow(dpy,vi->screen),
+			    vi->visual, AllocNone );
+    XSetWindowAttributes a;
+    a.colormap = cmap;
     a.background_pixel = backgroundColor().pixel();
     a.border_pixel = black.pixel();
     Window p = RootWindow( dpy, DefaultScreen(dpy) );
@@ -1306,74 +1035,23 @@ void QGLWidget::setContext( QGLContext *context,
     Window w = XCreateWindow( dpy, p,  x(), y(), width(), height(),
 			      0, vi->depth, InputOutput,  vi->visual,
 			      CWBackPixel|CWBorderPixel|CWColormap, &a );
-
-    Window *cmw;
-    Window *cmwret;
-    int count;
-    if ( XGetWMColormapWindows(dpy,topLevelWidget()->winId(),&cmwret,&count) ){
-	cmw = new Window[count+1];
-        memcpy( (char *)cmw, (char *)cmwret, sizeof(Window)*count );
-	XFree( (char *)cmwret );
-	int i;
-	for ( i=0; i<count; i++ ) {
-	    if ( cmw[i] == winId() ) {		// replace old window
-		cmw[i] = w;
-		break;
-	    }
-	}
-	if ( i >= count )			// append new window
-	    cmw[count++] = w;
-    } else {
-	count = 1;
-	cmw = new Window[count];
-	cmw[0] = w;
-    }
-
-#if defined(GLX_MESA_release_buffers)
-    glXReleaseBuffersMESA( dpy, winId() );
-#endif
     create( w );
-    clearWFlags( WState_Visible );   //## workaround for Qt 1.30 bug
-
-    XSetWMColormapWindows( dpy, topLevelWidget()->winId(), cmw, count );
-    delete [] cmw;
-
     if ( visible )
 	show();
-    XFlush( dpy );
-#endif // Q_GLX
+#endif
 }
 
 
 /*!
   \fn void QGLWidget::updateGL()
+
   Updates the widget by calling paintGL().
 */
 
 
-/*!
-  This virtual function is called one time before the first call to
-  paintGL() or resizeGL(), and then one time whenever the widget has
-  been assigned a new OpenGL context.  Reimplement it in a subclass.
-
-  This function should take care of setting OpenGL rendering flags,
-  defining display lists, etc.
-
-  There is no need to call makeCurrent() because this has already been
-  done when this function is called.
-*/
-
-void QGLWidget::initializeGL()
-{
-}
-
-
 /*!
   This virtual function is called whenever the widget needs to be painted.
   Reimplement it in a subclass.
-
-  There is no need to call makeCurrent() because this has already been
-  done when this function is called.
 */
 
 void QGLWidget::paintGL()
@@ -1385,9 +1063,6 @@ void QGLWidget::paintGL()
   \fn void QGLWidget::resizeGL( int width , int height )
   This virtual function is called whenever the widget has been resized.
   Reimplement it in a subclass.
-
-  There is no need to call makeCurrent() because this has already been
-  done when this function is called.
 */
 
 void QGLWidget::resizeGL( int, int )
@@ -1395,69 +1070,38 @@ void QGLWidget::resizeGL( int, int )
 }
 
 
-
-
 /*!
   Handles paint events. Calls the virtual function paintGL().
 */
 
 void QGLWidget::paintEvent( QPaintEvent * )
 {
-    makeCurrent();
-    if ( !initDone ) {
-	gl_init();
-	resizeGL( width(), height() );	// New context needs this "resize"
-    }
+    glcx->makeCurrent();
     paintGL();
-    if ( doubleBuffer() )
-	swapBuffers();
+    if ( glcx->format().doubleBuffer() )
+	glcx->swapBuffers();
     else
 	glFlush();
 }
 
-
 /*!
   Handles resize events. Calls the virtual function resizeGL().
 */
 
 void QGLWidget::resizeEvent( QResizeEvent * )
 {
-    makeCurrent();
-    if ( !initDone )
-	gl_init();
-#if defined(Q_GLX)
-    glXWaitX();
-#endif
+    glcx->makeCurrent();
     resizeGL( width(), height() );
+    glcx->doneCurrent();
 }
 
 
-/*!
-  Initializes OpenGL for this widget's context. Calls the virtual
-  function initializeGL().
-*/
-
-void QGLWidget::gl_init()
-{
-    /*
-    if ( isValid() ) {
-	glClearColor( 0.0, 0.0, 0.0, 0.0 ); // Let OpenGL clear to black
-	QGLFormat* fmt = format();
-	if ( fmt->depth() )
-	    glEnable( GL_DEPTH_TEST );
-	if ( fmt->stencil() )
-	    glEnable( GL_STENCIL_TEST );
-    }
-    */
-    initializeGL();
-    initDone = TRUE;
-}
-
 /*****************************************************************************
   QGL classes overview documentation.
  *****************************************************************************/
 
-/*! \page qgl.html
+/*!
+\page qgl.html
 
 <title>Qt OpenGL Classes</title>
 </head><body bgcolor="#ffffff">
@@ -1481,11 +1125,12 @@ The OpenGL support classes in Qt are:
 <ul>
 <li> <strong>\link QGLWidget QGLWidget\endlink:</strong> An easy-to-use Qt
   widget for rendering OpenGL scenes.
-<li> <strong>\link QGLContext QGLContext\endlink:</strong> Encapsulates an OpenGL rendering context.
-<li> <strong>\link QGLFormat QGLFormat\endlink:</strong> Specifies the
-display format of a rendering context.
+<li> <strong>\link QGLContext QGLContext\endlink:</strong> An OpenGL context.
+<li> <strong>\link QGLFormat QGLFormat\endlink:</strong> Spesifies the
+device settings of a context.
 </ul>
 
 Many applications need only the high-level QGLWidget class. The other QGL
 classes provide advanced features.
+
 */
