@@ -1,32 +1,29 @@
 /****************************************************************************
-** $Id: qarray.h,v 2.3 1998/07/03 00:09:42 hanord Exp $
+** $Id: qarray.h,v 2.1 1997/08/06 15:05:49 hanord Exp $
 **
 ** Definition of QArray template/macro class
 **
 ** Created : 930906
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #ifndef QARRAY_H
 #define QARRAY_H
 
-#ifndef QT_H
 #include "qgarray.h"
-#endif // QT_H
 
 
 #if defined(USE_MACROCLASS)
