@@ -1,32 +1,29 @@
 /****************************************************************************
-** $Id: qcolor.h,v 2.11 1998/07/03 00:09:31 hanord Exp $
+** $Id: qcolor.h,v 2.6 1997/09/11 17:22:42 warwick Exp $
 **
 ** Definition of QColor class
 **
 ** Created : 940112
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #ifndef QCOLOR_H
 #define QCOLOR_H
 
-#ifndef QT_H
-#include "qwindowdefs.h"
-#endif // QT_H
+#include "qwindefs.h"
 
 
 const QRgb  RGB_DIRTY	= 0x80000000;		// flags unset color
@@ -113,9 +110,8 @@ public:
 
 private:
     void   setSystemNamedColor( const char *name );
-    static void initGlobalColors();
-    static bool color_init;
-    static bool globals_init;
+    static void initglobals();
+    static bool ginit;
     static bool lalloc;
 #if defined(_WS_WIN_)
     static HANDLE hpal;
@@ -152,23 +148,24 @@ inline int QColor::blue() const
 inline uint QColor::pixel() const
 { return (rgbVal & RGB_DIRTY) == 0 ? pix : ((QColor*)this)->alloc(); }
 
-inline bool QColor::lazyAlloc()
-{ return lalloc; }
-
 
 inline bool QColor::operator==( const QColor &c ) const
 {
-    return (((rgbVal | c.rgbVal) & RGB_DIRECT) == 0 &&
-	    (rgbVal & 0x00ffffff) == (c.rgbVal & 0x00ffffff)) ||
-	   ((rgbVal & c.rgbVal & RGB_DIRECT) != 0 &&
-	    (rgbVal & 0x00ffffff) == (c.rgbVal & 0x00ffffff) && pix == c.pix);
+    return ((rgbVal | c.rgbVal) & RGB_DIRECT) == 0 ?
+		rgbVal == c.rgbVal :
+		rgbVal == c.rgbVal && pix == c.pix;
 }
 
 inline bool QColor::operator!=( const QColor &c ) const
 {
-    return !operator==(c);
+    return ((rgbVal | c.rgbVal) & RGB_DIRECT) == 0 ?
+		rgbVal != c.rgbVal :
+		rgbVal != c.rgbVal || pix != c.pix;
 }
 
+inline bool QColor::lazyAlloc()
+{ return lalloc; }
+
 
 /*****************************************************************************
   Global colors
