@@ -1,33 +1,30 @@
 /****************************************************************************
-** $Id: qdir.h,v 2.9 1998/07/03 00:09:43 hanord Exp $
+** $Id: qdir.h,v 2.5 1997/09/22 17:38:24 paul Exp $
 **
 ** Definition of QDir class
 **
 ** Created : 950427
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #ifndef QDIR_H
 #define QDIR_H
 
-#ifndef QT_H
 #include "qstrlist.h"
-#include "qfileinfo.h"
-#endif // QT_H
+#include "qfileinf.h"
 
 typedef Q_DECLARE(QListM,QFileInfo)	    QFileInfoList;
 typedef Q_DECLARE(QListIteratorM,QFileInfo) QFileInfoListIterator;
@@ -112,8 +109,6 @@ public:
 					int filterSpec = DefaultFilter,
 					int sortSpec   = DefaultSort   ) const;
 
-    static const QFileInfoList *drives();
-
     bool	mkdir( const char *dirName,
 			      bool acceptAbsPath = TRUE ) const;
     bool	rmdir( const char *dirName,
