@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qdrawutl.h,v 2.2 1997/02/28 15:46:09 hanord Exp $
+** $Id: qdrawutl.h,v 2.5 1997/08/06 15:05:35 hanord Exp $
 **
 ** Definition of draw utilities
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -81,9 +84,19 @@ void qDrawPlainRect( QPainter *p, const QRect &r, const QColor &,
 // Other useful drawing functions
 //
 
+QRect qItemRect( QPainter *p, GUIStyle gs, int x, int y, int w, int h,
+		int flags, bool enabled,
+		const QPixmap *pixmap, const char *text, int len=-1 );
+
 void qDrawItem( QPainter *p, GUIStyle gs, int x, int y, int w, int h,
 		int flags, const QColorGroup &g, bool enabled,
 		const QPixmap *pixmap, const char *text, int len=-1 );
 
+enum ArrowType
+    { UpArrow, DownArrow, LeftArrow, RightArrow };
+
+void qDrawArrow( QPainter *p, ArrowType type, GUIStyle style, bool down,
+		 int x, int y, int w, int h,
+		 const QColorGroup &g, bool enabled );
 
 #endif // QDRAWUTL_H
