@@ -1,34 +1,31 @@
 /****************************************************************************
-** $Id: qfile.h,v 2.6 1998/07/03 00:09:44 hanord Exp $
+** $Id: qfile.h,v 2.1 1997/08/06 15:05:51 hanord Exp $
 **
 ** Definition of QFile class
 **
 ** Created : 930831
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #ifndef QFILE_H
 #define QFILE_H
 
-#ifndef QT_H
-#include "qiodevice.h"
+#include "qiodev.h"
 #include "qstring.h"
 #include <stdio.h>
-#endif // QT_H
 
 class QDir;
 
@@ -46,9 +43,6 @@ public:
     bool	exists()   const;
     static bool exists( const char *fileName );
 
-    bool	remove();
-    static bool remove( const char *fileName );
-
     bool	open( int );
     bool	open( int, FILE * );
     bool	open( int, int );
@@ -81,8 +75,8 @@ private:
     void	init();
 
 private:	// Disabled copy constructor and operator=
-    QFile( const QFile & );
-    QFile &operator=( const QFile & );
+    QFile( const QFile & ) {}
+    QFile &operator=( const QFile & ) { return *this; }
 };
 
 
