@@ -1,5 +1,5 @@
 /**********************************************************************
-** $Id: qlistbox.h,v 2.3 1997/02/19 10:53:12 hanord Exp $
+** $Id: qlistbox.h,v 2.11 1997/09/09 17:13:41 paul Exp $
 **
 ** Definition of QListBox widget class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -157,9 +160,22 @@ public:
 
     long	maxItemWidth();
 
+    bool	isMultiSelection() const;
+    void	setMultiSelection( bool );
+
+    void	setSelected( int, bool );
+    bool	isSelected( int ) const;
+
+public slots:
+    void	clearSelection();
+
 signals:
     void	highlighted( int index );
     void	selected( int index );
+    void	highlighted( const char * );
+    void	selected( const char * );
+
+    void	selectionChanged();
 
 protected:
     QListBoxItem *item( int index ) const;
@@ -174,6 +190,7 @@ protected:
     void	mouseMoveEvent( QMouseEvent * );
     void	keyPressEvent( QKeyEvent *e );
     void	focusInEvent( QFocusEvent *e );
+    void	focusOutEvent( QFocusEvent *e );
     void	resizeEvent( QResizeEvent * );
     void	timerEvent( QTimerEvent * );
 
@@ -183,12 +200,16 @@ protected:
     void	clearList();
     void	updateCellWidth();
 
+    void	toggleCurrentItem();
+
 private:
     void	updateNumRows( bool );
     void	insert( const QListBoxItem *, int, bool );
     void	change( const QListBoxItem *lbi, int );
     void	setMaxItemWidth( int );
 
+    void	emitChangedSignal( bool );
+
     uint	doDrag		: 1;
     uint	doAutoScroll	: 1;
     uint	isTiming	: 1;
@@ -205,4 +226,10 @@ private:	// Disabled copy constructor and operator=
 };
 
 
+inline bool QListBox::isMultiSelection() const
+{
+    return multiSelect;
+}
+
+
 #endif // QLISTBOX_H
