@@ -1,33 +1,30 @@
 /****************************************************************************
-** $Id: qpicture.h,v 2.6 1998/07/03 00:09:37 hanord Exp $
+** $Id: qpicture.h,v 2.2 1997/08/06 15:05:41 hanord Exp $
 **
 ** Definition of QPicture class
 **
 ** Created : 940729
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #ifndef QPICTURE_H
 #define QPICTURE_H
 
-#ifndef QT_H
-#include "qpaintdevice.h"
+#include "qpaintd.h"
 #include "qbuffer.h"
-#endif // QT_H
 
 
 class QPicture : public QPaintDevice		// picture class
@@ -58,8 +55,8 @@ private:
     bool	formatOk;
 
 private:	// Disabled copy constructor and operator=
-    QPicture( const QPicture & );
-    QPicture &operator=( const QPicture & );
+    QPicture( const QPicture & ) :QPaintDevice(0) {}
+    QPicture &operator=( const QPicture & ) { return *this; }
 };
 
 
