@@ -1,34 +1,31 @@
 /****************************************************************************
-** $Id: qpixmap.h,v 2.22 1998/07/03 00:09:37 hanord Exp $
+** $Id: qpixmap.h,v 2.15 1997/08/06 15:05:41 hanord Exp $
 **
 ** Definition of QPixmap class
 **
 ** Created : 940501
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #ifndef QPIXMAP_H
 #define QPIXMAP_H
 
-#ifndef QT_H
-#include "qpaintdevice.h"
+#include "qpaintd.h"
 #include "qcolor.h"
-#include "qstring.h"
-#endif // QT_H
+#include "qshared.h"
 
 
 class QPixmap : public QPaintDevice		// pixmap class
@@ -90,9 +87,6 @@ public:
     bool	loadFromData( const uchar *buf, uint len,
 			      const char *format,
 			      int conversion_flags );
-    bool	loadFromData( QByteArray data,
-			      const char *format=0,
-			      int conversion_flags=0 );
     bool	save( const char *fileName, const char *format ) const;
 
 #if defined(_WS_WIN_) || defined(_WS_PM_)
@@ -101,19 +95,10 @@ public:
 
     int		serialNumber()	const;
 
-    enum Optimization { NoOptim, NormalOptim, BestOptim };
-
-    Optimization	optimization() const;
-    void		setOptimization( Optimization );
-    static Optimization defaultOptimization();
-    static void		setDefaultOptimization( Optimization );
-    
-#if 1
     bool	isOptimized()	const;
     void	optimize( bool );
     static bool isGloballyOptimized();
     static void optimizeGlobally( bool );
-#endif
 
     virtual void detach();
 
@@ -131,7 +116,7 @@ protected:
     struct QPixmapData : public QShared {	// internal pixmap data
 	QCOORD	w, h;
 	short	d;
-	uint	unused	 : 1;
+	uint	dirty	 : 1;
 	uint	optim	 : 1;
 	uint	uninit	 : 1;
 	uint	bitmap	 : 1;
@@ -141,27 +126,20 @@ protected:
 #if defined(_WS_WIN_)
 	HANDLE	hbm;
 	void   *bits;
-	QPixmap *maskpm;
 #elif defined(_WS_PM_)
 	HANDLE	hdcmem;
 	HANDLE	hbm;
 #elif defined(_WS_X11_)
 	void   *ximage;
-	void   *maskgc;
 #endif
-	Optimization opt;
     } *data;
 
 private:
     void	init( int, int, int );
-    void	deref();
     QPixmap	copy() const;
     static bool optimAll;
-    static Optimization defOpt;
     friend void bitBlt( QPaintDevice *, int, int, const QPaintDevice *,
 			int, int, int, int, RasterOp, bool );
-    friend void bitBlt( QPaintDevice *, int, int, const QImage* src,
-			int, int, int, int, int conversion_flags );
 };
 
 
@@ -206,11 +184,6 @@ inline int QPixmap::serialNumber() const
     return data->ser_no;
 }
 
-inline QPixmap::Optimization QPixmap::optimization() const
-{
-    return data->opt;
-}
-
 inline bool QPixmap::isOptimized() const
 {
     return data->optim;
