@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qpntarry.h,v 2.1 1996/10/06 19:13:49 agulbra Exp $
+** $Id: qpntarry.h,v 2.5 1997/08/28 16:08:33 warwick Exp $
 **
 ** Definition of QPointArray class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -59,7 +62,9 @@ class QPointVal
 public:
     QPointVal( QPointData *ptr ) : p(ptr) {}
     QPointVal &operator=( const QPoint &point );
-	       operator QPoint()	{ return QPoint(p->x,p->y); }
+    QPointVal &operator+=( const QPoint &point );
+    QPointVal &operator-=( const QPoint &point );
+	       operator QPoint() const	{ return QPoint(p->x,p->y); }
     int	       x() const		{ return (int)p->x; }
     int	       y() const		{ return (int)p->y; }
 private:
@@ -106,6 +111,10 @@ public:
 		{ return QPointVal( data()+i ); }
     QPointVal operator[]( uint i )
 		{ return QPointVal( data()+i ); }
+    QPoint operator[]( int i ) const
+		{ return (QPoint)QPointVal( data()+i ); }
+    QPoint operator[]( uint i ) const
+		{ return (QPoint)QPointVal( data()+i ); }
 
     QRect   boundingRect() const;
 
@@ -140,5 +149,19 @@ inline QPointVal &QPointVal::operator=( const QPoint &point )
     return *this;
 }
 
+inline QPointVal &QPointVal::operator+=( const QPoint &point )
+{
+    p->x += (Qpnta_t)point.x();
+    p->y += (Qpnta_t)point.y();
+    return *this;
+}
+
+inline QPointVal &QPointVal::operator-=( const QPoint &point )
+{
+    p->x -= (Qpnta_t)point.x();
+    p->y -= (Qpnta_t)point.y();
+    return *this;
+}
+
 
 #endif // QPNTARRY_H
