@@ -1,34 +1,30 @@
 /****************************************************************************
-** $Id: qstring.h,v 2.10 1998/07/03 00:09:47 hanord Exp $
+** $Id: qstring.h,v 2.8 1997/08/06 15:05:54 hanord Exp $
 **
 ** Definition of extended char array operations, and QByteArray and
 ** QString classes
 **
 ** Created : 920609
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #ifndef QSTRING_H
 #define QSTRING_H
 
-#ifndef QT_H
 #include "qarray.h"
-#endif // QT_H
-
 #include <string.h>
 
 #if defined(_OS_SUN_) && defined(_CC_GNU_)
