@@ -1,31 +1,28 @@
 /****************************************************************************
-** $Id: qtooltip.h,v 2.19 1998/07/03 00:09:54 hanord Exp $
+** $Id: qtooltip.h,v 2.15 1997/08/06 15:06:03 hanord Exp $
 **
-** Definition of Tool Tips (or Balloon Help) for any widget or rectangle
+** Tool Tips (or Balloon Help) for any widget or rectangle
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #ifndef QTOOLTIP_H
 #define QTOOLTIP_H
 
-#ifndef QT_H
 #include "qwidget.h"
 #include "qtimer.h"
-#endif // QT_H
 
 
 class QTipManager;
@@ -53,6 +50,11 @@ class QToolTip
 public:
     QToolTip( QWidget *, QToolTipGroup * = 0 );
 
+    static QFont    font();
+    static void	    setFont( const QFont & );
+    static QPalette palette();
+    static void	    setPalette( const QPalette & );
+
     static void add( QWidget *, const char * );
     static void add( QWidget *, const char *,
 		     QToolTipGroup *, const char * );
@@ -63,11 +65,6 @@ public:
 		     QToolTipGroup *, const char * );
     static void remove( QWidget *, const QRect & );
 
-    static QFont    font();
-    static void	    setFont( const QFont & );
-    static QPalette palette();
-    static void	    setPalette( const QPalette & );
-
 protected:
     virtual void maybeTip( const QPoint & ) = 0;
     void    tip( const QRect &, const char * );
