@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qfiledlg.cpp,v 2.4 1997/01/09 16:21:27 paul Exp $
+** $Id: qfiledlg.cpp,v 2.18 1997/09/09 22:34:24 hanord Exp $
 **
 ** Implementation of QFileDialog class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -35,7 +38,7 @@
 #endif
 #endif
 
-RCSTAG("$Id: qfiledlg.cpp,v 2.4 1997/01/09 16:21:27 paul Exp $");
+RCSTAG("$Id: qfiledlg.cpp,v 2.18 1997/09/09 22:34:24 hanord Exp $");
 
 
 /*!
@@ -61,6 +64,8 @@ RCSTAG("$Id: qfiledlg.cpp,v 2.4 1997/01/09 16:21:27 paul Exp $");
     open( s ); // open() being your function to read the file
   \endcode
 
+  <img src=qfiledlg-m.gif> <img src=qfiledlg-w.gif>
+
   \sa QPrintDialog
 */
 
@@ -78,7 +83,6 @@ QFileDialog::QFileDialog( QWidget *parent, const char *name, bool modal )
     filterEdit->setText( "*" );
     d.convertToAbs();
     rereadDir();
-    resize( 300, 300 );
 }
 
 /*!
@@ -99,7 +103,6 @@ QFileDialog::QFileDialog( const char *dirName, const char *filter,
 	d.setPath( dirName );
     d.convertToAbs();
     rereadDir();
-    resize( 300, 300 );
 }
 
 /*!
@@ -109,6 +112,12 @@ QFileDialog::QFileDialog( const char *dirName, const char *filter,
 
 void QFileDialog::init()
 {
+    QFontMetrics fm = fontMetrics();
+    QWidget *desk = QApplication::desktop();
+    int w = QMIN(fm.width("x")*50, desk->width()-50);
+    int h = QMIN(fm.height()*23, desk->height()-50);
+    resize( w, h );
+
     filterEdit = new QLineEdit(		     this, "filterEdit"	  );
     pathBox    = new QComboBox(		     this, "pathBox"	  );
     dirs       = new QListBox(		     this, "dirList"	  );
@@ -128,9 +137,9 @@ void QFileDialog::init()
     dirL   ->setAutoResize( TRUE );
     fileL  ->setAutoResize( TRUE );
 
-    okB	   ->resize( 50, 25 );
-    filterB->resize( 50, 25 );
-    cancelB->resize( 50, 25 );
+    okB	   ->resize( width()/6, height()/12 );
+    filterB->resize( width()/6, height()/12 );
+    cancelB->resize( width()/6, height()/12 );
 
     connect( files,	SIGNAL(selected(int)),	 SLOT(fileSelected(int)) );
     connect( files,	SIGNAL(highlighted(int)),SLOT(fileHighlighted(int)) );
@@ -143,6 +152,8 @@ void QFileDialog::init()
     connect( cancelB,	SIGNAL(clicked()),	 SLOT(cancelClicked()) );
     d.setMatchAllDirs( TRUE );
     d.setSorting( d.sorting() | QDir::DirsFirst );
+
+    nameEdit->setFocus();
 }
 
 /*!
@@ -295,6 +306,14 @@ void QFileDialog::rereadDir()
   This signal is emitted when the user has selected a new directory.
 */
 
+#if defined(_WS_WIN_)
+static char *win_filter[] = {
+    "All Files", "*.*",
+    "" };
+#endif
+
+static QString filedlg_dir;
+
 
 /*!
   Opens a modal file dialog and returns the name of the file to be opened.
@@ -322,23 +341,20 @@ void QFileDialog::rereadDir()
   \sa getSaveFileName()
 */
 
-#if defined(_WS_WIN_)
-static char *win_filter[] = {
-    "All Files", "*.*",
-    "" };
-#endif
-
 QString QFileDialog::getOpenFileName( const char *dirName, const char *filter,
 				      QWidget *parent, const char *name )
 {
+    if ( dirName && *dirName ) {
+	filedlg_dir = dirName;
+    } else if ( filedlg_dir.isNull() ) {
+	filedlg_dir = QDir::currentDirPath();
+    }
+
 #if defined(_WS_WIN_)
 
+    filedlg_dir = QDir::convertSeparators( filedlg_dir );
+
     const int maxstrlen = 256;
-    char *dir = 0;
-    if ( !dirName || !*dirName ) {
-	dir = new char[maxstrlen];
-	GetCurrentDirectory( maxstrlen, dir );
-    }
     char *file = new char[maxstrlen];
     file[0] = '\0';
 
@@ -349,26 +365,29 @@ QString QFileDialog::getOpenFileName( const char *dirName, const char *filter,
     ofn.lpstrFilter	= win_filter[0];
     ofn.lpstrFile	= file;
     ofn.nMaxFile	= maxstrlen;
-    ofn.lpstrInitialDir = dir ? dir : dirName;
+    ofn.lpstrInitialDir = filedlg_dir;
     ofn.lpstrTitle	= "Open";
-    ofn.Flags		= OFN_CREATEPROMPT;
+    ofn.Flags		= (OFN_CREATEPROMPT|OFN_NOCHANGEDIR);
 
     QString result;
-    if ( GetOpenFileName(&ofn) )
+    if ( GetOpenFileName(&ofn) ) {
 	result = file;
+	filedlg_dir = QFileInfo(file).dirPath();
+    }
 
     delete [] file;
-    delete [] dir;
     return result;
 
 #else
 
-    QFileDialog *dlg = new QFileDialog( dirName, filter, parent, name, TRUE );
+    QFileDialog *dlg = new QFileDialog( filedlg_dir, filter, parent, name, TRUE );
     CHECK_PTR( dlg );
     dlg->setCaption( "Open" );
     QString result;
-    if ( dlg->exec() == QDialog::Accepted )
+    if ( dlg->exec() == QDialog::Accepted ) {
 	result = dlg->selectedFile();
+	filedlg_dir = dlg->dirPath();
+    }
     delete dlg;
     return result;
 
@@ -386,7 +405,7 @@ QString QFileDialog::getOpenFileName( const char *dirName, const char *filter,
   Example:
   \code
     // start at the current working directory and with *.cpp as filter
-    QString f = QFileDialog::getSaveFileDialog( 0, "*.cpp", this );
+    QString f = QFileDialog::getSaveFileName( 0, "*.cpp", this );
     if ( !f.isEmpty() ) {
         // the user gave a file name
     } else {
@@ -404,14 +423,17 @@ QString QFileDialog::getOpenFileName( const char *dirName, const char *filter,
 QString QFileDialog::getSaveFileName( const char *dirName, const char *filter,
 				      QWidget *parent, const char *name )
 {
+    if ( dirName && *dirName ) {
+	filedlg_dir = dirName;
+    } else if ( filedlg_dir.isNull() ) {
+	filedlg_dir = QDir::currentDirPath();
+    }
+
 #if defined(_WS_WIN_)
 
+    filedlg_dir = QDir::convertSeparators( filedlg_dir );
+
     const int maxstrlen = 256;
-    char *dir = 0;
-    if ( !dirName || !*dirName ) {
-	dir = new char[maxstrlen];
-	GetCurrentDirectory( maxstrlen, dir );
-    }
     char *file = new char[maxstrlen];
     file[0] = '\0';
 
@@ -422,26 +444,29 @@ QString QFileDialog::getSaveFileName( const char *dirName, const char *filter,
     ofn.lpstrFilter	= win_filter[0];
     ofn.lpstrFile	= file;
     ofn.nMaxFile	= maxstrlen;
-    ofn.lpstrInitialDir = dir ? dir : dirName;
+    ofn.lpstrInitialDir = filedlg_dir;
     ofn.lpstrTitle	= "Save";
-    ofn.Flags		= OFN_CREATEPROMPT;
+    ofn.Flags		= (OFN_CREATEPROMPT|OFN_NOCHANGEDIR);
 
     QString result;
-    if ( GetSaveFileName(&ofn) )
+    if ( GetSaveFileName(&ofn) ) {
 	result = file;
+	filedlg_dir = QFileInfo(file).dirPath();
+    }
 
     delete [] file;
-    delete [] dir;
     return result;
 
 #else
 
-    QFileDialog *dlg = new QFileDialog( dirName, filter, parent, name, TRUE );
+    QFileDialog *dlg = new QFileDialog( filedlg_dir, filter, parent, name, TRUE );
     CHECK_PTR( dlg );
     dlg->setCaption( "Save As" );
     QString result;
-    if ( dlg->exec() == QDialog::Accepted )
+    if ( dlg->exec() == QDialog::Accepted ) {
 	result = dlg->selectedFile();
+	filedlg_dir = dlg->dirPath();
+    }
     delete dlg;
     return result;
 
@@ -558,11 +583,12 @@ void QFileDialog::resizeEvent( QResizeEvent * )
     filterL->move( 10, 10 );
 
     wTmp = filterL->width();
-    filterEdit->setGeometry( wTmp + 15, 10, w - wTmp - 15 - 10, 20 );
+    filterEdit->setGeometry( wTmp + 15, 10, w - wTmp - 15 - 10,
+	filterEdit->sizeHint().height() );
 
     rTmp = filterL->geometry();
     wTmp = pathBox->width();
-    pathBox->move( (w - wTmp)/2, rTmp.bottom() + 10 );
+    pathBox->move( (w - wTmp)/2, rTmp.bottom() + 12 );
 
     rTmp = pathBox->geometry();
     dirL->move( 10, rTmp.bottom() + 5 );
@@ -570,30 +596,32 @@ void QFileDialog::resizeEvent( QResizeEvent * )
     rTmp = dirL->geometry();
     fileL->move( w / 2 + 5, rTmp.y() );
 
-    rTmp = dirL->geometry();
-    dirs->setGeometry(10, rTmp.bottom() + 5,
-		      w/2 - 15, h - rTmp.bottom() - 10 - 25 - 10 - 20 - 10 );
-
-    rTmp = dirs->geometry();
-    files->setGeometry( rTmp.right() + 10, rTmp.y(),
-			rTmp.width(), rTmp.height() );
-
-    rTmp = dirs->geometry();
-    nameL->move( 10, rTmp.bottom() + 10 );
-
-    wTmp = nameL->width();
-    nameEdit->setGeometry( wTmp + 15, rTmp.bottom() + 10,
-			     w - wTmp - 15 - 10, 20 );
-
-    rTmp = nameEdit->geometry();
-    okB->move( 10, rTmp.bottom() + 10 );
-
     rTmp = okB->geometry();
+    rTmp.moveBottomRight( QPoint(rTmp.right(), h-7 ) ) ;
+    okB->move( 10, rTmp.y() );
+
     wTmp = cancelB->width();
     cancelB->move( w - 10 - wTmp, rTmp.y() );
 
     wTmp = filterB->width();
     filterB->move( (w - wTmp)/2, rTmp.y() );
+
+    int neh = nameEdit->sizeHint().height();
+    rTmp.moveBottomRight( QPoint( rTmp.right(), rTmp.y() - 5 ) );
+    wTmp = nameL->width();
+    nameEdit->setGeometry( wTmp + 15, rTmp.y(),
+			     w - wTmp - 15 - 10, neh );
+    int net = nameEdit->geometry().top();
+
+    nameL->move( 10, rTmp.y() );
+
+    rTmp = dirL->geometry();
+    dirs->setGeometry(10, rTmp.bottom() + 5,
+		      w/2 - 15, net - 9 - rTmp.bottom() - 5 );
+
+    rTmp = dirs->geometry();
+    files->setGeometry( rTmp.right() + 10, rTmp.y(),
+			rTmp.width(), rTmp.height() );
 }
 
 /*!
