@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qtabdlg.cpp,v 2.14 1997/03/13 20:00:54 agulbra Exp $
+** $Id: qtabdlg.cpp,v 2.32 1997/09/29 10:48:37 paul Exp $
 **
 ** Implementation of QTabDialog class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -22,8 +25,9 @@
 #include "qpushbt.h"
 #include "qpainter.h"
 #include "qpixmap.h"
+#include "qapp.h"
 
-RCSTAG("$Id: qtabdlg.cpp,v 2.14 1997/03/13 20:00:54 agulbra Exp $");
+RCSTAG("$Id: qtabdlg.cpp,v 2.32 1997/09/29 10:48:37 paul Exp $");
 
 
 /*!
@@ -32,8 +36,8 @@ RCSTAG("$Id: qtabdlg.cpp,v 2.14 1997/03/13 20:00:54 agulbra Exp $");
   \brief The QTabDialog class provides a tabbed dialog.
 
   A tabbed dialog is one in which several "pages" are available, and
-  the user selects which page to see and use by clicking on its tab.
-  No keyboard shortcuts are available.
+  the user selects which page to see and use by clicking on its tab,
+  or by pressing the indicated Alt-(letter) key combination.
 
   QTabDialog does not provide more than one row of tabs, and does not
   provide tabs along the sides or bottom of the pages.  It also does
@@ -46,16 +50,19 @@ RCSTAG("$Id: qtabdlg.cpp,v 2.14 1997/03/13 20:00:54 agulbra Exp $");
   The normal way to use QTabDialog is to do the following in the
   constructor: <ol> <li> Create a QTabDialog, <li> create a QWidget
   for each of the pages in the tab dialog, insert children into it,
-  set up geometry management for it, and finally use addTab() to set
-  up a tab for it, <li> set up the buttons for the tab dialog (Apply,
-  Cancel and so on), and finally <li> connect to the signals and
-  slots. </ol>
+  set up geometry management for it, and use addTab() to set up a tab
+  and keyboard accelerator for it, <li> set up the buttons for the tab
+  dialog (Apply, Cancel and so on), and finally <li> connect to the
+  signals and slots. </ol>
 
   The pref.cpp example does all this.
 
   If you don't call addTab(), the page you have created will not be
   visible.  Please don't confuse the object name you supply to the
-  QWidget constructor and the tab label you supply to addTab().
+  QWidget constructor and the tab label you supply to addTab():
+  addTab() takes a name which indicates an accelerator and is
+  meaningful and descriptive to the user, while the widget name is
+  used primarily for debugging.
 
   Almost all applications have to connect the applyButtonPressed()
   signal to something.  applyButtonPressed() is emitted when either OK
@@ -70,19 +77,21 @@ RCSTAG("$Id: qtabdlg.cpp,v 2.14 1997/03/13 20:00:54 agulbra Exp $");
   show(); if there is any chance that the state of the application may
   change between the creation of the tab dialog and the time it show()
   is called, you must connect this signal to a slot which resets the
-  state of the dialog. </ul>
+  state of the dialog. <li> selected() is emitted when the user
+  selects some page. </ul>
 
   Each tab is either enabled or disabled at any given time.  If a tab
   is enabled, the tab text is drawn in black and the user can select
-  that tab.  If it is disabled, the tab is drawn in (usually) gray and
-  the user can not select that tab.  Note that even though a tab is
-  disabled, the page can still be visible, for example if it is
-  visible at the moment you call setTabEnabled().
+  that tab.  If it is disabled, the tab is drawn in a different way
+  and the user can not select that tab.  Note that even though a tab
+  is disabled, the page can still be visible, for example if all of
+  the tabs happen to be disabled.
 
   While tab dialogs can be a very good way to split up a complex
-  dialog, it's also very easy to make a royal mess.  Here is some
-  advice (greatly inspired by a USENET posting by Jared M. Spool
-  \<jared@uie.com\>):
+  dialog, it's also very easy to make a royal mess out of a tab
+  dialog.  Here is some advice (greatly inspired by a USENET posting
+  by Jared M. Spool of <a href="http://www.uie.com">User Interface
+  Engineering</a>):
 
   <ol><li> Make sure that each page forms a logical whole which is
   adequately described by the label on the tab.
@@ -129,16 +138,25 @@ RCSTAG("$Id: qtabdlg.cpp,v 2.14 1997/03/13 20:00:54 agulbra Exp $");
   separate dialog.
 
   <li> There should be no implicit ordering of the pages.  If there
-  is, it is probably better to use a \link QWizardDialog wizard dialog.
-  \endlink
+  is, it is probably better to use a wizard dialog.
 
   If some of the pages seem to be ordered and others not, perhaps they
   ought not to be joined in a tab dialog.
 
   </ol>
 
+  <img src=qtabdlg-m.gif> <img src=qtabdlg-w.gif>
 */
 
+/*! \fn void QTabDialog::selected( const char * tabLabel );
+
+  This signal is emitted whenever a tab is selected (raised),
+  including during the first show().
+
+  \sa raise()
+*/
+  
+
 // add comments about delete, ok and apply
 
 struct QTabPrivate
@@ -152,8 +170,9 @@ struct QTabPrivate
     QPushButton * db;
     QPushButton * ab;
     int bh;
-};
 
+    QCOORD l_marg, r_marg, t_marg, b_marg;
+};
 
 /*!
   Constructs a QTabDialog with only an Ok button.
@@ -166,10 +185,13 @@ QTabDialog::QTabDialog( QWidget *parent, const char *name, bool modal,
     d = new QTabPrivate;
     CHECK_PTR( d );
 
-    d->tabs = new QTabBar( this, "tab control" );
-    connect( d->tabs, SIGNAL(selected(int)),
-	     this,    SLOT(showTab(int)) );
-    d->tabs->move( 6, 6 );
+    d->l_marg = 7;
+    d->t_marg = 6;
+    d->r_marg = d->l_marg;
+    d->b_marg = 8;
+
+    d->tabs = 0;
+    setTabBar( new QTabBar( this, "tab control" ) );
 
     d->ab = d->cb = d->db = 0;
 
@@ -181,12 +203,11 @@ QTabDialog::QTabDialog( QWidget *parent, const char *name, bool modal,
 	     this, SIGNAL(applyButtonPressed()) );
     connect( d->ok, SIGNAL(clicked()),
 	     this, SLOT(accept()) );
-    setFont( QFont( "helvetica" ) );
 }
 
 
 /*!
-  Destroys the tab view and all its children.
+  Destroys the tab dialog.
 */
 
 QTabDialog::~QTabDialog()
@@ -295,6 +316,19 @@ bool QTabDialog::hasApplyButton() const
 }
 
 
+/*!
+  Returns TRUE if the tab dialog has an OK button, FALSE if not.
+
+  \sa setDefaultsButton() defaultButtonPressed() hasOkButton()
+  hasCancelButton()
+*/
+
+bool QTabDialog::hasOkButton() const
+{
+    return d->ok != 0;
+}
+
+
 /*!
   \fn void QTabDialog::aboutToShow()
 
@@ -322,6 +356,7 @@ bool QTabDialog::hasApplyButton() const
 
 void QTabDialog::show()
 {
+    d->tabs->setFocus();
     emit aboutToShow();
     setSizes();
 
@@ -346,10 +381,25 @@ void QTabDialog::showTab( int i )
     if ( d && (uint)i < d->children.size() ) {
 	d->children[i]->setGeometry( childRect() );
 	d->children[i]->show();
-	d->children[i]->raise();
+	QWidget * f = qApp ? qApp->focusWidget() : 0;
+	while ( f && f->parent() != this )
+	    f = f->parentWidget();
 	for ( int t = 0; t< (int)d->children.size(); t++ )
-	    if ( i != t )
-		d->children[t]->hide();
+	    if ( i != t ) {
+		if ( f == d->children[t] ) {
+		    // move focus to first object on the incoming page,
+		    // without ugly flicker.
+		    bool b = d->tabs->isUpdatesEnabled();
+		    d->tabs->setUpdatesEnabled( FALSE );
+		    d->tabs->setFocus();
+		    d->children[t]->hide();
+		    focusNextPrevChild( TRUE );
+		    d->tabs->setUpdatesEnabled( b );
+		} else if ( d->children[t]->isVisible() ) {
+		    d->children[t]->hide();
+		}
+	    }
+	emit selected( d->tabs->tab( i )->label );
     }
 }
 
@@ -363,6 +413,17 @@ void QTabDialog::showTab( int i )
   label: The name is internal to the program and invariant, while the
   label is shown on screen and may vary according to e.g. language.
 
+  \a label is written in the QButton style, where &P makes Qt create
+  an accelerator key on Alt-P for this page.  For example:  
+
+  \code
+    td->addTab( graphicsPane, "&Graphics" );
+    td->addTab( soundPane, "&Sound" );
+  \endcode
+
+  If the user presses Alt-S the sound page of the tab dialog is shown,
+  if the user presses Alt-P the graphics page is shown.
+
   If you call addTab() after show(), the screen will flicker and the
   user will be confused.
 */
@@ -372,8 +433,19 @@ void QTabDialog::addTab( QWidget * child, const char * label )
     QTab * t = new QTab();
     CHECK_PTR( t );
     t->label = label;
-    t->enabled = TRUE;
-    int id = d->tabs->addTab( t );
+    addTab( child, t );
+}
+
+/*!
+  This is a lower-level method for adding tabs, similar to
+  the method above.  It is useful if you are using setTabBar()
+  to set a QTabBar subclass with an overridden
+  QTabBar::paint() routine for a subclass of QTab.
+*/
+void QTabDialog::addTab( QWidget * child, QTab* tab )
+{
+    tab->enabled = TRUE;
+    int id = d->tabs->addTab( tab );
     if ( id ==(int)d->children.size() ) {
 	d->children.resize( id+1 );
 	d->children[id] = child;
@@ -385,6 +457,46 @@ void QTabDialog::addTab( QWidget * child, const char * label )
     }
 }
 
+/*!
+  Replaces the QTabBar heading the dialog by the given tab bar.
+  Note that this must be called \e before any tabs have been added,
+  or the behavior is undefined.
+  \sa tabBar()
+*/
+void QTabDialog::setTabBar( QTabBar* tb )
+{
+    delete d->tabs;
+    d->tabs = tb;
+    connect( d->tabs, SIGNAL(selected(int)),
+	     this,    SLOT(showTab(int)) );
+    d->tabs->move( d->l_marg, d->t_marg );
+}
+
+/*!
+  Returns the currently set QTabBar.
+  \sa setTabBar()
+*/
+QTabBar* QTabDialog::tabBar() const
+{
+    return d->tabs;
+}
+
+/*!  Ensures that \a w is shown.  This is useful mainly for accelerators.
+
+  If you use this function, take care not to surprise or confuse the
+  user.
+
+  \sa QTabBar::setCurrentTab()
+*/
+
+void QTabDialog::showPage( QWidget * w )
+{
+    int i;
+    for( i=0; i<(int)d->children.size(); i++ )
+	if ( d->children[i] == w )
+	    d->tabs->setCurrentTab( i );
+}
+
 
 /*!
   Returns TRUE if the page with object name \a name is enabled, and
@@ -428,9 +540,12 @@ void QTabDialog::setTabEnabled( const char * name, bool enable )
     if ( !name || !*name )
 	return;
     int i;
-    for( i=0; i<(int)d->children.size(); i++ )
-	if ( qstrcmp( d->children[i]->name(), name ) == 0 )
+    for( i=0; i<(int)d->children.size(); i++ ) {
+	if ( qstrcmp( d->children[i]->name(), name ) == 0 ) {
 	    d->tabs->setTabEnabled( i, enable );
+	    return;
+	}
+    }
 }
 
 
@@ -448,14 +563,20 @@ void QTabDialog::setTabEnabled( const char * name, bool enable )
 
 void QTabDialog::setApplyButton( const char * text )
 {
-    if ( !d->ab ) {
-	d->ab = new QPushButton( this, "apply settings" );
-	connect( d->ab, SIGNAL(clicked()),
-		 this, SIGNAL(applyButtonPressed()) );
+    if ( !text ) {
+	delete d->ab;
+	d->ab = 0;
+	setSizes();
+    } else {
+	if ( !d->ab ) {
+	    d->ab = new QPushButton( this, "apply settings" );
+	    connect( d->ab, SIGNAL(clicked()),
+		     this, SIGNAL(applyButtonPressed()) );
+	}
+	d->ab->setText( text );
+	setSizes();
+	d->ab->show();
     }
-    d->ab->setText( text );
-    setSizes();
-    d->ab->show();
 }
 
 
@@ -473,14 +594,20 @@ void QTabDialog::setApplyButton( const char * text )
 
 void QTabDialog::setDefaultButton( const char * text )
 {
-    if ( !d->db ) {
-	d->db = new QPushButton( this, "back to default" );
-	connect( d->db, SIGNAL(clicked()),
-		 this, SIGNAL(defaultButtonPressed()) );
+    if ( !text ) {
+	delete d->db;
+	d->db = 0;
+	setSizes();
+    } else {
+	if ( !d->db ) {
+	    d->db = new QPushButton( this, "back to default" );
+	    connect( d->db, SIGNAL(clicked()),
+		     this, SIGNAL(defaultButtonPressed()) );
+	}
+	d->db->setText( text );
+	setSizes();
+	d->db->show();
     }
-    d->db->setText( text );
-    setSizes();
-    d->db->show();
 }
 
 
@@ -500,16 +627,22 @@ void QTabDialog::setDefaultButton( const char * text )
 
 void QTabDialog::setCancelButton( const char * text )
 {
-    if ( !d->cb ) {
-	d->cb = new QPushButton( this, "cancel dialog" );
-	connect( d->cb, SIGNAL(clicked()),
-		 this, SIGNAL(cancelButtonPressed()) );
-	connect( d->cb, SIGNAL(clicked()),
-		 this, SLOT(reject()) );
+    if ( !text ) {
+	delete d->cb;
+	d->cb = 0;
+	setSizes();
+    } else {
+	if ( !d->cb ) {
+	    d->cb = new QPushButton( this, "cancel dialog" );
+	    connect( d->cb, SIGNAL(clicked()),
+		     this, SIGNAL(cancelButtonPressed()) );
+	    connect( d->cb, SIGNAL(clicked()),
+		     this, SLOT(reject()) );
+	}
+	d->cb->setText( text );
+	setSizes();
+	d->cb->show();
     }
-    d->cb->setText( text );
-    setSizes();
-    d->cb->show();
 }
 
 
@@ -518,21 +651,25 @@ void QTabDialog::setCancelButton( const char * text )
 
   Finally set the minimum and maximum sizes for the dialog.
 
-  This function does not resize or move the pages - only resizeEvent()
-  does that.
-
   \sa setApplyButton() setCancelButton() setDefaultButton()
 */
 
 void QTabDialog::setSizes()
 {
+    d->tabs->move( d->l_marg, d->t_marg );
+
     // compute largest button size
     int bw;
-    QSize s( d->ok->sizeHint() );
+    QWidget* first_button = d->ok ?
+		    d->ok : d->db ?
+		    d->db : d->ab ?
+		    d->ab : d->cb ? 
+		    d->cb : 0;
+    QSize s( first_button ? d->ok->sizeHint() : QSize(0,0) );
     bw = s.width();
     d->bh = s.height(); // private member, used for GM
 
-    if ( d->ab ) {
+    if ( d->ab && d->ab != first_button ) {
 	s = d->ab->sizeHint();
 	if ( s.width() > bw )
 	    bw = s.width();
@@ -540,7 +677,7 @@ void QTabDialog::setSizes()
 	    d->bh = s.height();
     }
 
-    if ( d->db ) {
+    if ( d->db && d->ab != first_button ) {
 	s = d->db->sizeHint();
 	if ( s.width() > bw )
 	    bw = s.width();
@@ -548,7 +685,7 @@ void QTabDialog::setSizes()
 	    d->bh = s.height();
     }
 
-    if ( d->cb ) {
+    if ( d->cb && d->ab != first_button ) {
 	s = d->cb->sizeHint();
 	if ( s.width() > bw )
 	    bw = s.width();
@@ -556,8 +693,12 @@ void QTabDialog::setSizes()
 	    d->bh = s.height();
     }
 
+    if ( style() == WindowsStyle && bw < 75 )
+	bw = 75;
+
     // and set all the buttons to that size
-    d->ok->resize( bw, d->bh );
+    if ( d->ok )
+	d->ok->resize( bw, d->bh );
     if ( d->ab )
 	d->ab->resize( bw, d->bh );
     if ( d->db )
@@ -586,6 +727,13 @@ void QTabDialog::setSizes()
     }
     min.setHeight( min.height() + th );
 
+    // factor in the buttons.  six pixels between each button, seven
+    // (6+1) to the right and seven to the left.
+    bw = (bw + 6) * ( (d->ab ? 1 : 0) + (d->db ? 1 : 0) + (d->cb ? 1 : 0)
+	 + (d->ok ? 1 : 0 ) ) + 1 + d->l_marg;
+    if ( min.width() < bw )
+	min.setWidth( bw );
+
     // max must be >= min
     if ( max.width() < min.width() )
 	max.setWidth( min.width() );
@@ -593,16 +741,34 @@ void QTabDialog::setSizes()
 	max.setHeight( min.height() );
 
     // allow for own borders, buttons and tabs, and set own sizes
-    min.setWidth( QMIN( min.width() + 13, 32767 ) );
-    min.setHeight( QMIN( min.height() + d->bh + th + 18, 32767 ) );
-    max.setWidth( QMIN( max.width() + 13, 32767 ) );
-    max.setHeight( QMIN( max.height() + d->bh + th + 18, 32767 ) );
+    min.setWidth( QMIN( min.width() + 17, 32767 ) );
+    min.setHeight( QMIN( d->t_marg + 1 + th + min.height()
+		       + d->t_marg + d->bh + 8, 32767 ) );
+    max.setWidth( QMIN( max.width() + 17, 32767 ) );
+    max.setHeight( QMIN( d->t_marg + 1 + th + max.height()
+		       + d->t_marg + d->bh + 8, 32767 ) );
     setMinimumSize( min );
     setMaximumSize( max );
 
     // fake a resize event to trigger child widget moves
     QResizeEvent r( size(), size() );
     resizeEvent( &r );
+
+    // fiddle the tab chain so the buttons are in their natural order
+    QWidget * w = first_button;
+    if ( d->db ) {
+	setTabOrder( w, d->db );
+	w = d->db;
+    }
+    if ( d->ab ) {
+	setTabOrder( w, d->ab );
+	w = d->ab;
+    }
+    if ( d->cb ) {
+	setTabOrder( w, d->cb );
+	w = d->cb;
+    }
+    setTabOrder( w, d->tabs );
 }
 
 
@@ -617,26 +783,32 @@ void QTabDialog::setSizes()
 void QTabDialog::resizeEvent( QResizeEvent * )
 {
     if ( d->tabs ) {
+	QSize ts = d->tabs->sizeHint();
+	if ( ts.width() > width() ) {
+	    ts.setWidth( width() );
+	}
+	d->tabs->resize( ts );
+
 	int x;
 	x = width();
 
 	if ( d->cb ) {
-	    d->cb->move( x - 5 - d->cb->width(), height() - 5 - d->bh );
+	    d->cb->move( x - d->l_marg - d->cb->width(), height() - d->b_marg - d->bh );
 	    x = d->cb->geometry().x();
 	}
 
 	if ( d->ab ) {
-	    d->ab->move( x - 5 - d->ab->width(), height() - 5 - d->bh );
+	    d->ab->move( x - d->l_marg - d->ab->width(), height() - d->b_marg - d->bh );
 	    x = d->ab->geometry().x();
 	}
 
 	if ( d->db ) {
-	    d->db->move( x - 5 - d->db->width(), height() - 5 - d->bh );
+	    d->db->move( x - d->l_marg - d->db->width(), height() - d->b_marg - d->bh );
 	    x = d->db->geometry().x();
 	}
 
 	if ( d->ok ) {
-	    d->ok->move( x - 5 - d->ok->width(), height() - 5 - d->bh );
+	    d->ok->move( x - d->l_marg - d->ok->width(), height() - d->b_marg - d->bh );
 	}
 
 	int i;
@@ -680,8 +852,10 @@ void QTabDialog::paintEvent( QPaintEvent * )
 
 QRect QTabDialog::childRect() const
 {
-    return QRect( 6, d->tabs->height() + 5, width() - 12,
-		  height() - d->bh - d->tabs->height() - 18 );
+    int y = d->tabs->height() + d->tabs->y();
+    return QRect( d->l_marg + 1, y,
+	    width() - d->l_marg - d->l_marg - 1 - 2,
+	    height() - d->t_marg - 2 - d->bh - d->b_marg - y );
 }
 
 
@@ -696,9 +870,55 @@ QRect QTabDialog::childRect() const
   \sa setCancelButton() setDefaultButton() applyButtonPressed()
 */
 
+void QTabDialog::setOkButton( const char * text )
+{
+    if ( !text ) {
+	delete d->ok;
+	d->ok = 0;
+	setSizes();
+    } else {
+	if ( !d->ok ) {
+	    d->ok = new QPushButton( this, "ok" );
+	    connect( d->ok, SIGNAL(clicked()),
+		     this, SIGNAL(applyButtonPressed()) );
+	}
+	d->ok->setText( text );
+	setSizes();
+	d->ok->show();
+    }
+}
+
+/*!
+  Old version of setOkButton(), provided for backward compatibility.
+ */
 void QTabDialog::setOKButton( const char * text )
 {
-    d->ok->setText( text );
+    setOkButton( text );
+}
+
+
+/*!  Returns the text in the tab for page \a w.
+*/
+
+const char * QTabDialog::tabLabel( QWidget * w )
+{
+    int i;
+    for( i=0; i<(int)d->children.size(); i++ ) {
+	if ( d->children[i] == w ) {
+	    QTab * t = d->tabs->tab(i);
+	    if ( t )
+		return t->label;
+	}
+    }
+    return 0;
+}	    
+
+
+/*!  Reimplemented to hndle a change of GUI style while on-screen.
+*/
+
+void QTabDialog::styleChange( GUIStyle s )
+{
+    QDialog::styleChange( s );
     setSizes();
-    d->ok->show();
 }
