@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qapp_x11.cpp,v 2.96.2.2 1998/03/16 22:02:09 agulbra Exp $
+** $Id: qapp_x11.cpp,v 2.95 1997/10/09 13:10:27 agulbra Exp $
 **
 ** Implementation of X11 startup routines and event handling
 **
@@ -7,7 +7,7 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
@@ -96,7 +96,7 @@ static inline void bzero( void *s, int n )
 #endif
 
 
-RCSTAG("$Id: qapp_x11.cpp,v 2.96.2.2 1998/03/16 22:02:09 agulbra Exp $");
+RCSTAG("$Id: qapp_x11.cpp,v 2.95 1997/10/09 13:10:27 agulbra Exp $");
 
 
 /*****************************************************************************
@@ -164,11 +164,7 @@ static void	initTimers();
 static void	cleanupTimers();
 static timeval	watchtime;			// watch if time is turned back
 
-#if defined(X11R4) || (defined(_OS_OSF_) && (XlibSpecificationRelease < 6)) || defined(_OS_AIX_) || defined(_OS_SUN_)
-#define NO_XIM
-#endif
-
-#if !defined(NO_XIM)
+#if !defined(X11R4)
 static XIM	xim;
 #endif
 
@@ -426,7 +422,7 @@ static void qt_init_internal( int *argcptr, char **argv, Display *display )
 	QPalette pal( cg, dcg, cg );
 	QApplication::setPalette( pal );
     }
-#if !defined(NO_XIM)
+#if !defined(X11R4)
     setlocale( LC_ALL, "" );		// use correct char set mapping
     setlocale( LC_NUMERIC, "C" );	// make sprintf()/scanf() work
     if ( XSupportsLocale() &&
@@ -475,7 +471,7 @@ void qt_cleanup()
     QFont::cleanup();
     QColor::cleanup();
 
-#if !defined(NO_XIM)
+#if !defined(X11R4)
     if ( xim ) {
 	// Calling XCloseIM gives a Purify FMR error
 	// Instead we get a non-critical memory leak
@@ -1340,7 +1336,7 @@ bool qt_set_socket_handler( int sockfd, int type, QObject *obj, bool enable )
 		warning( "QSocketNotifier: Multiple socket notifiers for "
 			 "same socket %d and type %s", sockfd, t[type] );
 	    }
-#endif	
+#endif	    
 	    if ( p )
 		list->insert( list->at(), sn );
 	    else
@@ -2010,7 +2006,7 @@ void qt_close_popup( QWidget *popup )
   widget flag, e.g. the QPopupMenu widget.  When the application opens a
   popup widget, all events are sent to the popup and normal widgets and
   modal widgets cannot be accessed before the popup widget is closed.
-
+  
   Only other popup widgets may be opened when a popup widget is shown.
   The popup widgets are organized in a stack.
   This function returns the active popup widget on top of the stack.
@@ -2640,7 +2636,7 @@ bool QETWidget::translateKeyEvent( const XEvent *event, bool grab )
 
     type = (event->type == KeyPress) ? Event_KeyPress : Event_KeyRelease;
 
-#if defined(NO_XIM)
+#if defined(X11R4) || (defined(_OS_OSF_) && (XlibSpecificationRelease < 6)) || defined(_OS_AIX_)
     // Implementation for X11R4 not using XIM, and for OSF/1 with broken
     // Xlib
 
@@ -2698,7 +2694,7 @@ bool QETWidget::translateKeyEvent( const XEvent *event, bool grab )
 	    delete s;
 	}
     }
-#endif // !NO_XIM
+#endif // !X11R4
 
     state = translateButtonState( event->xkey.state );
 
@@ -2902,7 +2898,7 @@ bool QETWidget::translatePaintEvent( const XEvent *event )
     bool should_clip = translateBySips( this, paintRect );
 
     if ( merging_okay ) {
-	while ( XCheckIfEvent(dpy,&xevent,isPaintOrScrollDoneEvent,(XPointer)&info)
+	while ( XCheckIfEvent(dpy,&xevent,isPaintOrScrollDoneEvent,(XPointer)&info) 
 	    && !qApp->x11EventFilter(&xevent) )	// send event through filter
 	{
 	    if ( !info.config ) {
@@ -2912,7 +2908,7 @@ bool QETWidget::translatePaintEvent( const XEvent *event )
 				   xevent.xexpose.width,
 				   xevent.xexpose.height);
 		    if ( translateBySips( this, exposure ) )
-			should_clip = TRUE;
+			should_clip = TRUE; // WARWICK! if !should_clip && ???
 		    paintRect = paintRect.unite( exposure );
 		} else {
 		    translateScrollDoneEvent( &xevent );
