@@ -1,68 +1,70 @@
 /****************************************************************************
-** $Id: qasyncimageio.cpp,v 1.38 1998/07/03 00:09:30 hanord Exp $
+** $Id: qasyncimageio.cpp,v 1.21 1997/09/26 17:37:56 warwick Exp $
 **
 ** Implementation of asynchronous image/movie loading classes
 **
 ** Created : 970617
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
-#include "qpainter.h"
-#include "qlist.h"
+#include <qpainter.h>
+#include <qlist.h>
 #include "qasyncimageio.h"
+
 #include <stdlib.h>
 
 /*!
   \class QImageConsumer qasyncimageio.h
   \brief An abstraction used by QImageDecoder.
 
-  \ingroup images
+  \internal
 
   A QImageConsumer consumes information about changes to the
   QImage maintained by a QImageDecoder.  It represents the
-  a view of the image which the decoder produces.
+  decoder's view of the obhect which uses the image it produces.
+  Each method of QImageConsumer returns a bool which should be
+  TRUE if the decoder should continue decoding.
 
   \sa QImageDecoder
 */
 
 /*!
-  \fn void QImageConsumer::changed(const QRect&)
+  \fn bool QImageConsumer::changed(const QRect&)
 
   Called when the given area of the image has changed.
 */
 
 /*!
-  \fn void QImageConsumer::end()
+  \fn bool QImageConsumer::end()
 
   Called when all data of all frames has been decoded and revealed
   as changed().
 */
 
 /*!
-  \fn void QImageConsumer::frameDone()
+  \fn bool QImageConsumer::frameDone()
 
   Called when a frame of an animated image has ended and been revealed
-  as changed().  The decoder will not make
+  as changed().  If the consumer returns TRUE, the decoder will not make
   any further changes to the image until the next call to
-  QImageFormat::decode().
+  QImageFormatDecoder::decode().
 */
 
 /*!
-  \fn void QImageConsumer::setLooping(int n)
+  \fn bool QImageConsumer::setLooping(int n)
 
   Called to indicate that the sequence of frames in the image
   should be repeated \a n times, including the sequence during
@@ -83,7 +85,7 @@
 */
 
 /*!
-  \fn void QImageConsumer::setFramePeriod(int milliseconds)
+  \fn bool QImageConsumer::setFramePeriod(int milliseconds)
 
   Notes that the frame about to be decoded should not be displayed until
   the given number of \a milliseconds after the time that this function
@@ -94,7 +96,7 @@
 */
 
 /*!
-  \fn void QImageConsumer::setSize(int, int)
+  \fn bool QImageConsumer::setSize(int, int)
 
   This function is called as soon as the size of the image has
   been determined.
@@ -105,12 +107,10 @@
   \class QImageDecoder qasyncimageio.h
   \brief Incremental image decoder for all supported image formats.
 
-  \ingroup images
+  \internal
 
   New formats are installed by creating objects of class
-  QImageFormatType, and the QMovie class can be used for using
-  all installed incremental image formats; QImageDecoder is only
-  useful for creating new ways of feeding data to an QImageConsumer.
+  QImageFormatDecoderFactory.
 */
 
 static const int max_header = 32;
@@ -121,27 +121,12 @@ struct QImageDecoderPrivate {
 	count = 0;
     }
 
-    static QList<QImageFormatType> * factories;
-
-    // Builtins...
-    static QGIFFormatType * gif_decoder_factory;
+    static QList<QImageFormatDecoderFactory> factories;
 
     uchar header[max_header];
     int count;
 };
-
-QList<QImageFormatType> * QImageDecoderPrivate::factories = 0;
-QGIFFormatType * QImageDecoderPrivate::gif_decoder_factory = 0;
-
-
-static void cleanup()
-{
-    delete QImageDecoderPrivate::factories;
-    QImageDecoderPrivate::factories = 0;
-    delete QImageDecoderPrivate::gif_decoder_factory;
-    QImageDecoderPrivate::gif_decoder_factory = 0;
-}
-
+QList<QImageFormatDecoderFactory> QImageDecoderPrivate::factories;
 
 /*!
   Constructs a QImageDecoder which will send change information to
@@ -190,16 +175,10 @@ int QImageDecoder::decode(const uchar* buffer, int length)
 	while (consumed < length && d->count < max_header) {
 	    d->header[d->count++] = buffer[consumed++];
 	}
-	
-	if ( !QImageDecoderPrivate::factories ) {
-	    QImageDecoderPrivate::factories = new QListT<QImageFormatType>;
-	    QImageDecoderPrivate::gif_decoder_factory = new QGIFFormatType;
-	    qAddPostRoutine( cleanup );
-	}
 
-	for (QImageFormatType* f = QImageDecoderPrivate::factories->first();
+	for (QImageFormatDecoderFactory* f = QImageDecoderPrivate::factories.first();
 	    f && !actual_decoder;
-	    f = QImageDecoderPrivate::factories->next())
+	    f = QImageDecoderPrivate::factories.next())
 	{
 	    actual_decoder = f->decoderFor(d->header, d->count);
 	}
@@ -234,18 +213,12 @@ int QImageDecoder::decode(const uchar* buffer, int length)
 */
 const char* QImageDecoder::formatName(const uchar* buffer, int length)
 {
-    if ( !QImageDecoderPrivate::factories ) {
-	QImageDecoderPrivate::factories = new QListT<QImageFormatType>;
-	QImageDecoderPrivate::gif_decoder_factory = new QGIFFormatType;
-	qAddPostRoutine( cleanup );
-    }
-
     const char* name = 0;
-    for (QImageFormatType* f = QImageDecoderPrivate::factories->first();
+    for (QImageFormatDecoderFactory* f = QImageDecoderPrivate::factories.first();
 	f && !name;
-	f = QImageDecoderPrivate::factories->next())
+	f = QImageDecoderPrivate::factories.next())
     {
-	QImageFormat *decoder = f->decoderFor(buffer, length);
+	QImageFormatDecoder *decoder = f->decoderFor(buffer, length);
 	if (decoder) {
 	    name = f->formatName();
 	    delete decoder;
@@ -259,17 +232,10 @@ const char* QImageDecoder::formatName(const uchar* buffer, int length)
 */
 QStrList QImageDecoder::inputFormats()
 {
-    if ( !QImageDecoderPrivate::factories ) {
-	QImageDecoderPrivate::factories = new QListT<QImageFormatType>;
-	QImageDecoderPrivate::gif_decoder_factory = new QGIFFormatType;
-	qAddPostRoutine( cleanup );
-    }
-
     QStrList result;
 
-    for (QImageFormatType* f = QImageDecoderPrivate::factories->first();
-	 f;
-	 f = QImageDecoderPrivate::factories->next())
+    for (QImageFormatDecoderFactory* f = QImageDecoderPrivate::factories.first();
+	f; f = QImageDecoderPrivate::factories.next())
     {
 	if ( !result.contains(  f->formatName() ) ) {
 	    result.inSort(  f->formatName() );
@@ -280,48 +246,28 @@ QStrList QImageDecoder::inputFormats()
 }
 
 /*!
-  Registers a new QImageFormatType.  This is not needed in
+  Registers a new QImageFormatDecoderFactory.  This is not needed in
   application code as factories call this themselves.
 */
-void QImageDecoder::registerDecoderFactory(QImageFormatType* f)
+void QImageDecoder::registerDecoderFactory(QImageFormatDecoderFactory* f)
 {
-    if ( !QImageDecoderPrivate::factories ) {
-	QImageDecoderPrivate::factories = new QListT<QImageFormatType>;
-	QImageDecoderPrivate::gif_decoder_factory = new QGIFFormatType;
-	qAddPostRoutine( cleanup );
-    }
-
-    QImageDecoderPrivate::factories->insert(0,f);
+    QImageDecoderPrivate::factories.insert(0,f);
 }
 
 /*!
-  Unregisters a new QImageFormatType.  This is not needed in
+  Unregisters a new QImageFormatDecoderFactory.  This is not needed in
   application code as factories call this themselves.
 */
-void QImageDecoder::unregisterDecoderFactory(QImageFormatType* f)
+void QImageDecoder::unregisterDecoderFactory(QImageFormatDecoderFactory* f)
 {
-    if ( !QImageDecoderPrivate::factories )
-	return;
-
-    QImageDecoderPrivate::factories->remove(f);
+    QImageDecoderPrivate::factories.remove(f);
 }
 
 /*!
-  \class QImageFormat qasyncimageio.h
+  \class QImageFormatDecoder qasyncimageio.h
   \brief Incremental image decoder for a specific image format.
 
-  \ingroup images
-
-  By making a derived classes of QImageFormatType which in turn
-  creates objects that are a subclass of QImageFormat, you can add
-  support for more incremental image formats, allowing such formats to
-  be sources for a QMovie, or for the first frame of the image stream
-  to be loaded as a QImage or QPixmap.
-
-  Your new subclass must override the decode() function in order to
-  process your new format.
-
-  New QImageFormat objects are generated by new QImageFormatType factories.
+  \internal
 */
 
 /*!
@@ -330,58 +276,33 @@ void QImageDecoder::unregisterDecoderFactory(QImageFormatType* f)
   \internal
   More importantly, destructs derived classes.
 */
-QImageFormat::~QImageFormat()
+QImageFormatDecoder::~QImageFormatDecoder()
 {
 }
 
 /*!
-  \fn int QImageFormat::decode(QImage& img, QImageConsumer* consumer,
+  \fn int QImageFormatDecoder::decode(QImage& img, QImageConsumer* consumer,
 	    const uchar* buffer, int length)
 
-  New subclasses must override this method.
-
-  It should decode some or all of the bytes from \a buffer into
-  \a img, calling the methods of \a consumer as the decoding proceeds to
-  inform that consumer of changes to the image.
-  The consumer may be 0, in which case the function should just process
-  the data into \a img without telling any consumer about the changes.
-  Note that the decoder must store enough state
-  to be able to continue in subsequent calls to this method - this is
-  the essence of the incremental image loading.
-
-  The function should return without processing all the data if it
-  reaches the end of a frame in the input.
-
-  The function must return the number of bytes it has processed.
+  Image decoders for specific image formats must override this method.
+  It should decode some or all of the bytes in the given buffer into the
+  given image, calling the given consumer as the decoding proceeds.
+  The consumer may be 0.
 */
 
 /*!
-  \class QImageFormatType qasyncimageio.h
-  \brief Factory that makes QImageFormat objects.
+  \class QImageFormatDecoderFactory qasyncimageio.h
+  \brief Factory that makes QImageFormatDecoder objects.
 
-  \ingroup images
-
-  While the QImageIO class allows for \e complete loading of images,
-  QImageFormatType allows for \e incremental loading of images.
+  \internal
 
   New image file formats are installed by creating objects of derived
-  classes of QImageFormatType.  They must implement decoderFor()
+  classes of QImageFormatDecoderFactory.  They must implement decoderFor()
   and formatName().
-
-  QImageFormatType is a very simple class.  Its only task is to
-  recognize image data in some format and make a new object, subclassed
-  from QImageFormat, which can decode that format.
-
-  The factories for formats built into Qt
-  (currently only GIF)
-  are automatically defined before any other factory is initialized.
-  If two factories would recognize an image format, the factory created
-  last will override the earlier one, thus you can override current
-  and future built-in formats.
 */
 
 /*!
-  \fn virtual QImageFormat* QImageFormatType::decoderFor(const
+  \fn virtual QImageFormatDecoder* QImageFormatDecoderFactory::decoderFor(const
 	    uchar* buffer, int length)
 
   Returns a decoder for decoding an image which starts with the give bytes.
@@ -392,7 +313,7 @@ QImageFormat::~QImageFormat()
 */
 
 /*!
-  \fn virtual const char* QImageFormatType::formatName() const
+  \fn virtual const char* QImageFormatDecoderFactory::formatName() const
 
   Returns the name of the format supported by decoders from this factory.
   The string is statically allocated.
@@ -401,7 +322,7 @@ QImageFormat::~QImageFormat()
 /*!
   Creates a factory.  It automatically registers itself with QImageDecoder.
 */
-QImageFormatType::QImageFormatType()
+QImageFormatDecoderFactory::QImageFormatDecoderFactory()
 {
     QImageDecoder::registerDecoderFactory(this);
 }
@@ -409,29 +330,25 @@ QImageFormatType::QImageFormatType()
 /*!
   Destroys a factory.  It automatically unregisters itself from QImageDecoder.
 */
-QImageFormatType::~QImageFormatType()
+QImageFormatDecoderFactory::~QImageFormatDecoderFactory()
 {
     QImageDecoder::unregisterDecoderFactory(this);
 }
 
 /*!
-  \class QGIFFormat qasyncimageio.h
+  \class QGIFDecoder qasyncimageio.h
   \brief Incremental image decoder for GIF image format.
 
-  \ingroup images
-
-  This subclass of QImageFormat decodes GIF format images,
-  including animated GIFs.  Internally in
+  \internal
 */
 
 /*!
-  Constructs a QGIFFormat.
+  Constructs a QGIFDecoder.
 */
-QGIFFormat::QGIFFormat()
+QGIFDecoder::QGIFDecoder()
 {
-    globalcmap = 0;
+    globalcmap_hold = 0;
     disposal = NoDisposal;
-    out_of_bounds = FALSE;
     disposed = TRUE;
     frame = -1;
     state = Header;
@@ -440,27 +357,24 @@ QGIFFormat::QGIFFormat()
 }
 
 /*!
-  Destructs a QGIFFormat.
+  Destructs a QGIFDecoder.
 */
-QGIFFormat::~QGIFFormat()
+QGIFDecoder::~QGIFDecoder()
 {
-    delete globalcmap;
+    delete globalcmap_hold;
 }
 
 
-/*!
-  \class QGIFFormatType qasyncimageio.h
-  \brief Incremental image decoder for GIF image format.
+class QGIFDecoderFactory : public QImageFormatDecoderFactory {
+    QImageFormatDecoder* decoderFor(const uchar* buffer, int length);
+    const char* formatName() const;
+};
 
-  \ingroup images
+// this variable must exist to read gifs, and if it's static sgi cc 7
+// complains... so give it an obscure name and make it non-static
+QGIFDecoderFactory qt_internal_gif_decoder_factory;
 
-  This subclass of QImageFormatType recognizes GIF
-  format images, creating a QGIFFormat when required.  An instance
-  of this class is created automatically before any other factories,
-  so you should have no need for such objects.
-*/
-
-QImageFormat* QGIFFormatType::decoderFor(
+QImageFormatDecoder* QGIFDecoderFactory::decoderFor(
     const uchar* buffer, int length)
 {
     if (length < 6) return 0;
@@ -470,30 +384,22 @@ QImageFormat* QGIFFormatType::decoderFor(
      && buffer[3]=='8'
      && (buffer[4]=='9' || buffer[4]=='7')
      && buffer[5]=='a')
-        return new QGIFFormat;
+        return new QGIFDecoder;
     return 0;
 }
 
-const char* QGIFFormatType::formatName() const
+const char* QGIFDecoderFactory::formatName() const
 {
     return "GIF";
 }
 
 
-void QGIFFormat::disposePrevious( QImage& img, QImageConsumer* consumer )
+void QGIFDecoder::disposePrevious( QImage& img, QImageConsumer* consumer )
 {
-    if ( out_of_bounds ) // flush anything that survived
-	consumer->changed(QRect(0,0,swidth,sheight));
-
     // Handle disposal of previous image before processing next one
 
     if ( disposed ) return;
 
-    int l = QMIN(swidth-1,left);
-    int r = QMIN(swidth-1,right);
-    int t = QMIN(sheight-1,top);
-    int b = QMIN(sheight-1,bottom);
-
     switch (disposal) {
       case NoDisposal:
 	break;
@@ -503,27 +409,34 @@ void QGIFFormat::disposePrevious( QImage& img, QImageConsumer* consumer )
 	preserve_trans = FALSE;
 	if (trans>=0) {
 	    // Easy:  we use the transparent colour
-	    fillRect(img, l, t, r-l+1, b-t+1, trans);
+	    fillRect(img, left, top, right-left+1, bottom-top+1, trans);
 	} else if (bgcol>=0) {
 	    // Easy:  we use the bgcol given
-	    fillRect(img, l, t, r-l+1, b-t+1, bgcol);
+	    fillRect(img, left, top, right-left+1, bottom-top+1, bgcol);
 	} else {
 	    // Impossible:  We don't know of a bgcol - use pixel 0
 	    uchar** line = img.jumpTable();
-	    fillRect(img, l, t, r-l+1, b-t+1, line[0][0]);
+	    fillRect(img, left, top, right-left+1, bottom-top+1, line[0][0]);
+	}
+	if (consumer) {
+	    bool d = !consumer->changed(QRect(left, top,
+					      right-left+1, bottom-top+1));
+	    digress = digress || d;
 	}
-	if (consumer)
-	    consumer->changed(QRect(l, t, r-l+1, b-t+1));
 	break;
       case RestoreImage: {
 	uchar** line = img.jumpTable();
 	preserve_trans = FALSE;
-	for (int ln=t; ln<=b; ln++) {
-	    memcpy(line[ln]+l,
-		backingstore.scanLine(ln-t),
-		r-l+1);
+	for (int ln=top; ln<=bottom; ln++) {
+	    memcpy(line[ln]+left,
+		backingstore.scanLine(ln-top),
+		right-left+1);
+	}
+	if (consumer) {
+	    bool d = !consumer->changed(QRect(left, top,
+					      right-left+1, bottom-top+1));
+	    digress = digress || d;
 	}
-	consumer->changed(QRect(l, t, r-l+1, b-t+1));
       }
     }
     disposal = NoDisposal; // Until an extension says otherwise.
@@ -536,7 +449,7 @@ void QGIFFormat::disposePrevious( QImage& img, QImageConsumer* consumer )
 
   Returns the number of bytes consumed.
 */
-int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
+int QGIFDecoder::decode(QImage& img, QImageConsumer* consumer,
 	const uchar* buffer, int length)
 {
     // We are required to state that
@@ -574,6 +487,10 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 		bgcol=(gcmap && hold[5]) ? hold[5] : -1;
 		//aspect=hold[6] ? double(hold[6]+15)/64.0 : 1.0;
 
+		img.create(swidth, sheight, 8, gcmap ? gncols : 256);
+		line = img.jumpTable();
+		if (consumer) digress = !consumer->setSize(swidth, sheight);
+
 		trans = -1;
 		preserve_trans = FALSE;
 		count=0;
@@ -581,7 +498,6 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 		if (gcmap) {
 		    ccount=0;
 		    state=GlobalColorMap;
-		    globalcmap = new QRgb[gncols];
 		} else {
 		    state=ImageDescriptor;
 		}
@@ -590,13 +506,9 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 	  case GlobalColorMap: case LocalColorMap:
 	    hold[count++]=ch;
 	    if (count==3) {
-		QRgb rgb = (ccount==trans ? 0 : 0xff000000)
-		    | qRgb(hold[0], hold[1], hold[2]);
-		if ( state == LocalColorMap ) {
-		    img.setColor(ccount, rgb);
-		} else {
-		    globalcmap[ccount] = rgb;
-		}
+		img.setColor(ccount,
+		    (ccount==trans ? 0 : 0xff000000)
+		    | qRgb(hold[0], hold[1], hold[2]));
 		if (++ccount >= ncols) {
 		    if ( state == LocalColorMap )
 			state=TableImageLZWSize;
@@ -616,11 +528,7 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 		state=ExtensionLabel;
 		break;
 	      case ';':
-		if (consumer) {
-		    if ( out_of_bounds ) // flush anything that survived
-			consumer->changed(QRect(0,0,swidth,sheight));
-		    consumer->end();
-		}
+		if (consumer) digress = !consumer->end();
 		state=Done;
 		break;
 	      default:
@@ -632,47 +540,15 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 	  case ImageDescriptor:
 	    hold[count++]=ch;
 	    if (count==10) {
-		int newleft=LM(hold[1], hold[2]);
-		int newtop=LM(hold[3], hold[4]);
-		int width=LM(hold[5], hold[6]);
-		int height=LM(hold[7], hold[8]);
-
-		if ( swidth <= 0 )
-		    swidth = newleft + width;
-		if ( sheight <= 0 )
-		    sheight = newtop + height;
-
-		if (img.isNull()) {
-		    img.create(swidth, sheight, 8, gcmap ? gncols : 256);
-		    if (consumer) consumer->setSize(swidth, sheight);
-		}
-		img.setAlphaBuffer(trans >= 0);
-		line = img.jumpTable();
-
 		disposePrevious( img, consumer );
 		disposed = FALSE;
-
-		left = newleft;
-		top = newtop;
-
-		// Sanity check frame size - must fit on "screen".
-		if (left >= swidth) left=swidth-1;
-		if (top >= sheight) top=sheight-1;
-		if (left+width >= swidth) {
-		    if ( width <= swidth )
-			left=swidth-width;
-		    else
-			width=swidth-left;
-		}
-		if (top+height >= sheight) {
-		    if ( height <= sheight )
-			top=sheight-height;
-		    else
-			height=sheight-top;
-		}
-
+		left=LM(hold[1], hold[2]);
+		top=LM(hold[3], hold[4]);
+		int width=LM(hold[5], hold[6]);
+		int height=LM(hold[7], hold[8]);
 		right=left+width-1;
 		bottom=top+height-1;
+		bool hadlcmap = lcmap;
 		lcmap=!!(hold[9]&0x80);
 		interlace=!!(hold[9]&0x40);
 		//bool lcmsortflag=!!(hold[9]&0x20);
@@ -689,34 +565,27 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 			// Not full-size image - erase with bg or transparent
 			if ( bgcol>=0 ) {
 			    fillRect(img, 0, 0, swidth, sheight, bgcol);
-			    if (consumer) consumer->changed(QRect(0,0,swidth,sheight));
+			    if (consumer) digress = !consumer->changed(QRect(0,0,swidth,sheight));
 			} else if ( trans > 0 ) {
 			    fillRect(img, 0, 0, swidth, sheight, trans);
-			    if (consumer) consumer->changed(QRect(0,0,swidth,sheight));
+			    if (consumer) digress = !consumer->changed(QRect(0,0,swidth,sheight));
 			}
 		    }
 		}
 
 		if ( disposal == RestoreImage ) {
-		    int l = QMIN(swidth-1,left);
-		    int r = QMIN(swidth-1,right);
-		    int t = QMIN(sheight-1,top);
-		    int b = QMIN(sheight-1,bottom);
-		    int w = r-l+1;
-		    int h = b-t+1;
-
-		    if (backingstore.width() < w
-			|| backingstore.height() < h) {
+		    if (backingstore.width() < width
+			|| backingstore.height() < height) {
 			// We just use the backing store as a byte array
 			backingstore.create( QMAX(backingstore.width(),
-						  w),
+						  width),
 					     QMAX(backingstore.height(),
-						  h),
+						  height),
 					     8,1);
 		    }
-		    for (int ln=0; ln<h; ln++) {
+		    for (int ln=0; ln<height; ln++) {
 			memcpy(backingstore.scanLine(ln),
-			       line[t+ln]+l, w);
+			       line[top+ln]+left, width);
 		    }
 		}
 
@@ -724,9 +593,17 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 		if (lcmap) {
 		    ccount=0;
 		    state=LocalColorMap;
-		} else {
 		    if (gcmap) {
-			memcpy(img.colorTable(), globalcmap,
+			if (!globalcmap_hold) {
+			    globalcmap_hold = new QRgb[256];
+			    CHECK_PTR(globalcmap_hold);
+			}
+			memcpy(globalcmap_hold, img.colorTable(),
+			    ncols * sizeof(QRgb));
+		    }
+		} else {
+		    if (hadlcmap && gcmap) {
+			memcpy(img.colorTable(), globalcmap_hold,
 			    ncols * sizeof(QRgb));
 		    }
 		    state=TableImageLZWSize;
@@ -737,7 +614,6 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 		bitcount = 0;
 		sp = stack;
 		needfirst = FALSE;
-		out_of_bounds = FALSE;
 	    }
 	    break;
 	  case TableImageLZWSize: {
@@ -767,10 +643,7 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 	    if (expectcount) {
 		state=ImageDataBlock;
 	    } else {
-		if (consumer) {
-		    consumer->frameDone();
-		    digress = TRUE;
-		}
+		if (consumer) digress = !consumer->frameDone();
 
 		state=Introducer;
 	    }
@@ -805,15 +678,12 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 		} else {
 		    if (needfirst) {
 			firstcode=oldcode=code;
-			if (!out_of_bounds && !(preserve_trans && firstcode==trans))
+			if (!(preserve_trans && firstcode==trans))
 			    line[y][x] = firstcode;
 			x++;
-			if (x>=swidth) out_of_bounds = TRUE;
 			needfirst=FALSE;
 			if (x>right) {
 			    x=left;
-			    if (out_of_bounds)
-				out_of_bounds = left>=swidth || y>=sheight;
 			    nextY(img,consumer);
 			}
 		    } else {
@@ -850,14 +720,11 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 			oldcode=incode;
 			while (sp>stack) {
 			    --sp;
-			    if (!out_of_bounds && !(preserve_trans && *sp==trans))
+			    if (!(preserve_trans && *sp==trans))
 				line[y][x] = *sp;
 			    x++;
-			    if (x>=swidth) out_of_bounds = TRUE;
 			    if (x>right) {
 				x=left;
-				if (out_of_bounds)
-				    out_of_bounds = left>=swidth || y>=sheight;
 				nextY(img,consumer);
 			    }
 			}
@@ -938,8 +805,7 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 		    newtrans=-1;
 		    havetrans=FALSE;
 		}
-		if (newtrans >= 0 && frame>=0)
-		    preserve_trans = TRUE;
+		if (trans >= 0) preserve_trans = TRUE;
 		if (newtrans != trans) {
 		    // Unset old transparency
 		    if (trans >= 0) {
@@ -959,10 +825,13 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 		    }
 		    trans = newtrans;
 		    if (trans >= 0) {
-			if (globalcmap)
-			    globalcmap[trans]&=0x00ffffff;
+			if (globalcmap_hold) {
+			    globalcmap_hold[trans]&=0x00ffffff;
+			}
+			img.setColor(trans, 0x00ffffff&img.color(trans));
 		    }
 		}
+		img.setAlphaBuffer(havetrans);
 		if (consumer) consumer->setFramePeriod(delay*10);
 		count=0;
 		state=SkipBlockSize;
@@ -990,7 +859,7 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
     return initial-length;
 }
 
-void QGIFFormat::fillRect(QImage& img, int col, int row, int w, int h, uchar color)
+void QGIFDecoder::fillRect(QImage& img, int col, int row, int w, int h, uchar color)
 {
     if (w>0) {
 	uchar** line = img.jumpTable() + row;
@@ -1000,14 +869,14 @@ void QGIFFormat::fillRect(QImage& img, int col, int row, int w, int h, uchar col
     }
 }
 
-void QGIFFormat::nextY(QImage& img, QImageConsumer* consumer)
+void QGIFDecoder::nextY(QImage& img, QImageConsumer* consumer)
 {
     int my;
     switch (interlace) {
       case 0:
 	// Non-interlaced
-	if (consumer && !out_of_bounds)
-	    consumer->changed(QRect(left, y, right-left+1, 1));
+	if (consumer) digress =
+	    !consumer->changed(QRect(left, y, right-left+1, 1));
 	y++;
 	break;
       case 1:
@@ -1016,8 +885,8 @@ void QGIFFormat::nextY(QImage& img, QImageConsumer* consumer)
 	    my = QMIN(7, bottom-y);
 	    for (i=1; i<=my; i++)
 	        memcpy(img.scanLine(y+i), img.scanLine(y), img.width());
-	    if (consumer && !out_of_bounds)
-	        consumer->changed(QRect(left, y, right-left+1, my+1));
+	    if (consumer) digress =
+	        !consumer->changed(QRect(left, y, right-left+1, my+1));
 	    y+=8;
 	    if (y>bottom) { interlace++; y=4; }
 	} break;
@@ -1027,8 +896,8 @@ void QGIFFormat::nextY(QImage& img, QImageConsumer* consumer)
 	    my = QMIN(3, bottom-y);
 	    for (i=1; i<=my; i++)
 	        memcpy(img.scanLine(y+i), img.scanLine(y), img.width());
-    	    if (consumer && !out_of_bounds)
-	        consumer->changed(QRect(left, y, right-left+1, my+1));
+    	    if (consumer) digress =
+	        !consumer->changed(QRect(left, y, right-left+1, my+1));
 	    y+=8;
 	    if (y>bottom) { interlace++; y=2; }
 	} break;
@@ -1037,17 +906,17 @@ void QGIFFormat::nextY(QImage& img, QImageConsumer* consumer)
 	    my = QMIN(1, bottom-y);
 	    for (int i=1; i<=my; i++)
 	        memcpy(img.scanLine(y+i), img.scanLine(y), img.width());
-	    if (consumer && !out_of_bounds)
-	        consumer->changed(QRect(left, y, right-left+1, my+1));
+	    if (consumer) digress =
+	        !consumer->changed(QRect(left, y, right-left+1, my+1));
 	    y+=4;
 	    if (y>bottom) { interlace++; y=1; }
 	} break;
       case 4:
-	if (consumer && !out_of_bounds)
-	    consumer->changed(QRect(left, y, right-left+1, 1));
+	if (consumer) digress =
+	    !consumer->changed(QRect(left, y, right-left+1, 1));
 	y+=2;
     }
 
     // Consume bogus extra lines
-    if (y >= sheight) out_of_bounds=TRUE; //y=bottom;
+    if (y > bottom) y=bottom;
 }
