@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qclipbrd.cpp,v 2.2 1997/01/14 09:23:58 hanord Exp $
+** $Id: qclipbrd.cpp,v 2.4 1997/09/07 16:01:12 agulbra Exp $
 **
 ** Implementation of QClipboard class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -21,7 +24,7 @@
 #include "qapp.h"
 #include "qpixmap.h"
 
-RCSTAG("$Id: qclipbrd.cpp,v 2.2 1997/01/14 09:23:58 hanord Exp $");
+RCSTAG("$Id: qclipbrd.cpp,v 2.4 1997/09/07 16:01:12 agulbra Exp $");
 
 
 /*!
@@ -59,10 +62,6 @@ RCSTAG("$Id: qclipbrd.cpp,v 2.2 1997/01/14 09:23:58 hanord Exp $");
     // Copy text into the clipboard
     cb->setText( "This text can be pasted by other programs" );
   \endcode
-
-  \warning
-  It is an important GUI principle that all clipboard operations should be
-  initiated by the user.
 */
 
 
