@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qcol_x11.cpp,v 2.14.2.2 1997/11/19 12:20:32 warwick Exp $
+** $Id: qcol_x11.cpp,v 2.14 1997/09/17 14:33:32 warwick Exp $
 **
 ** Implementation of QColor class for X11
 **
@@ -7,7 +7,7 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
@@ -29,7 +29,7 @@
 #include <X11/Xutil.h>
 #include <X11/Xos.h>
 
-RCSTAG("$Id: qcol_x11.cpp,v 2.14.2.2 1997/11/19 12:20:32 warwick Exp $");
+RCSTAG("$Id: qcol_x11.cpp,v 2.14 1997/09/17 14:33:32 warwick Exp $");
 
 
 /*****************************************************************************
@@ -59,8 +59,6 @@ static bool	colors_frozen = FALSE;		// allocating disabled
 static int	current_alloc_context = 0;	// current color alloc context
 static Visual  *g_vis	= 0;			// visual
 static XColor  *g_carr	= 0;			// color array
-static bool	g_carr_fetch = TRUE;		// perform XQueryColors?
-static int	g_cells = 0;			// number of entries in g_carr
 static bool    *g_our_alloc = 0;		// our allocated colors
 static bool	g_truecolor;
 static uint	red_mask , green_mask , blue_mask;
@@ -82,38 +80,12 @@ extern bool	qt_cmap_option;
 void qt_reset_color_avail()
 {
     colorAvail = TRUE;
-    g_carr_fetch = TRUE;		// do XQueryColors if !colorAvail
-}
-
-
-/*
-  Finds the nearest color.  If colors_frozen is TRUE, we only search among
-  the colors we have already allocated and not colors that have been
-  allocated by other applications.
-*/
-
-static int find_nearest_color( int r, int g, int b )
-{
-    int mincol = -1;
-    int mindist = 200000;
-    int rx, gx, bx, dist;
-    XColor *xc = &g_carr[0];
-    for ( int i=0; i<g_cells; i++ ) {
-	if ( colors_frozen && !g_our_alloc[i] ) {
-	    xc++;				// skip color we didn't alloc
-	    continue;
-	}
-	rx = r - (xc->red >> 8);
-	gx = g - (xc->green >> 8);
-	bx = b - (xc->blue>> 8);
-	dist = rx*rx + gx*gx + bx*bx;		// calculate distance
-	if ( dist < mindist ) {			// minimal?
-	    mindist = dist;
-	    mincol = i;
-	}
-	xc++;
+    if ( g_carr ) {				// color array was allocated
+	delete [] g_our_alloc;
+	g_our_alloc = 0;
+	delete [] g_carr;
+	g_carr = 0;
     }
-    return mincol;
 }
 
 
@@ -245,24 +217,6 @@ void QColor::initialize()
     else
 	cmap = XCreateColormap(dpy, RootWindow(dpy,scr), g_vis, AllocNone );
 
-    if ( !g_truecolor ) {
-	// Create the g_our_alloc array, which remembers which color pixels
-	// we allocated.
-	g_cells = QMIN(ncols,256);
-	g_carr  = new XColor[g_cells];
-	CHECK_PTR( g_carr );
-	memset( g_carr, 0, g_cells*sizeof(XColor) );
-	g_carr_fetch = TRUE;		// run XQueryColors on demand
-	g_our_alloc = new bool[g_cells];	
-	CHECK_PTR( g_our_alloc );
-	memset( g_our_alloc, FALSE, g_cells*sizeof(bool) );
-	XColor *xc = &g_carr[0];
-	for ( int i=0; i<g_cells; i++ ) {
-	    xc->pixel = i;		// g_carr[i] = color i
-	    xc++;
-	}
-    }
-
     QPaintDevice::x_display   = dpy;
     QPaintDevice::x_screen    = scr;
     QPaintDevice::x_depth     = depth;
@@ -368,8 +322,8 @@ void QColor::initialize()
 	if (qt_ncols_option>216) qt_ncols_option=216;
 
 	for ( int i = 0; i < qt_ncols_option; i++ ) {
-	    QColor c(cmap[i]);
-	    c.alloc();
+            QColor *c=new QColor(cmap[i]);
+	    c->alloc();
         }
 
 	// No more custom allocations: user has set limit
@@ -408,8 +362,6 @@ void QColor::cleanup()
     if ( !color_init )
 	return;
     color_init = FALSE;
-    delete [] g_carr;
-    delete [] g_our_alloc;
     if ( !QPaintDevice::x_defcmap )
 	XFreeColormap( QPaintDevice::x__Display(), QPaintDevice::x_colormap );
     if ( colorDict ) {
@@ -485,10 +437,8 @@ uint QColor::alloc()
     if ( c ) {					// found color in dictionary
 	rgbVal &= RGB_MASK;			// color ok
 	pix = c->pix;				// use same pixel value
-	if ( c->context != current_alloc_context ) {
-	    c->context = 0;			// convert to default context
-	    g_our_alloc[pix] = TRUE;		// reuse without XAllocColor
-	}
+	if ( c->context != current_alloc_context )
+	    c->context = 0;			// convert to standard context
 	return pix;
     }
     XColor col;
@@ -498,83 +448,60 @@ uint QColor::alloc()
     col.red   = r << 8;
     col.green = g << 8;
     col.blue  = b << 8;
+    if ( !colors_frozen
+    && colorAvail && XAllocColor(dpy, QPaintDevice::x11Colormap(), &col) ) {
+	pix = (uint)col.pixel;			// allocated X11 color
+	rgbVal &= RGB_MASK;
+    } else {					// get closest color
+	int mincol = -1;
+	int mindist = 200000;
+	int rx, gx, bx, dist;
+	int i, maxi = QMIN(QPaintDevice::x11Cells(),256);
+	register XColor *xc;
+	colorAvail = FALSE;			// no more avail colors
+	if ( !g_carr ) {			// get colors in colormap
+	    g_carr = new XColor[maxi];
+	    CHECK_PTR( g_carr );
+	    xc = &g_carr[0];
+	    for ( i=0; i<maxi; i++ ) {
+		xc->pixel = i;			// carr[i] = color i
+		xc++;
+	    }
+	    XQueryColors( dpy, QPaintDevice::x11Colormap(), g_carr, maxi );
+	    g_our_alloc = new bool[maxi];
+	    CHECK_PTR( g_our_alloc );
+	    memset( g_our_alloc, FALSE, maxi*sizeof(*g_our_alloc) );
+	}
 
-    bool try_again = FALSE;
-    int  try_count = 0;
-
-    if ( colors_frozen ) {
-	// Find the nearest color we have already allocated
-	pix = find_nearest_color( r, g, b );
-	if ( (int)pix == -1 ) {			// no nearest color?!
+	xc = &g_carr[0];
+	for ( i=0; i<maxi; i++ ) {		// find closest color
+	    rx = r - (xc->red >> 8);
+	    gx = g - (xc->green >> 8);
+	    bx = b - (xc->blue>> 8);
+	    dist = rx*rx + gx*gx + bx*bx;	// calculate distance
+	    if ( dist < mindist ) {		// minimal?
+		mindist = dist;
+		mincol = i;
+	    }
+	    xc++;
+	}
+	if ( mincol == -1 ) {			// there are no colors, yuck
 	    rgbVal |= RGB_INVALID;
 	    pix = BlackPixel( dpy, scr );
 	    return pix;
-	} else {
-	    rgbVal &= RGB_MASK;
 	}
-    }
-
-    else  do {
-	// This loop is run until we manage to either allocate or
-	// find an approximate color, it stops after 100 iterations.
-
-	if ( colorAvail &&
-	     XAllocColor(dpy,QPaintDevice::x11Colormap(),&col) ) {
-
-	    // We could allocate the color
-	    pix = (uint)col.pixel;
-	    rgbVal &= RGB_MASK;
-	    g_carr[pix] = col;			// update color array
-	    if ( current_alloc_context == 0 )
-		g_our_alloc[pix] = TRUE;	// reuse without XAllocColor
-
-	} else {
-
-	    // No available colors
-	    int i;
-	    colorAvail = FALSE;		// no more available colors
-	    if ( g_carr_fetch ) {		// refetch color array
-		g_carr_fetch = FALSE;
-		XQueryColors(dpy, QPaintDevice::x11Colormap(), g_carr,g_cells);
-	    }
-	    i = find_nearest_color( r, g, b );
-	    if ( i == -1 ) {			// no nearest color?!
-		rgbVal |= RGB_INVALID;
-		pix = BlackPixel( dpy, scr );
-		return pix;
-	    }
-	    if ( g_our_alloc[i] ) {		// we've already allocated it
-		pix = g_carr[i].pixel;
-	    } else {
-		if ( XAllocColor(dpy,QPaintDevice::x11Colormap(),&g_carr[i])){
-		    if ( current_alloc_context == 0 )
-			g_our_alloc[i] = TRUE;	// only in the default context
-		} else {
-		    try_count++;
-		    try_again    = TRUE;
-		    colorAvail = TRUE;
-		    g_carr_fetch = TRUE;
-		}
-	    }
-	    ASSERT(i==(int)g_carr[i].pixel);
-	    if ( !try_again ) {			// got it
-		pix = g_carr[i].pixel;		// allocated X11 color
-		rgbVal &= RGB_MASK;
-	    }
+	if ( !g_our_alloc[mincol] ) {		// we haven't allocated it
+	    XAllocColor( dpy, QPaintDevice::x11Colormap(), &g_carr[mincol] );
+	    g_our_alloc[mincol] = TRUE;
 	}
-
-    } while ( try_again && try_count < 100 );
-
-    if ( try_again ) {				// no hope of allocating color
-	rgbVal |= RGB_INVALID;
-	pix = BlackPixel( dpy, scr );
-	return pix;
+	pix = g_carr[mincol].pixel;		// allocated X11 color
+	rgbVal &= RGB_MASK;
     }
     if ( colorDict->count() < colorDict->size() * 8 ) {
 	c = new QColorData;			// insert into color dict
 	CHECK_PTR( c );
 	c->pix	   = pix;
-	c->context = colors_frozen ? 0 : current_alloc_context;
+	c->context = current_alloc_context;
 	colorDict->insert( (long)rgbVal, c );	// store color in dict
     }
     return pix;
@@ -757,7 +684,7 @@ int QColor::currentAllocContext()
 void QColor::destroyAllocContext( int context )
 {
     init_context_stack();
-    if ( !color_init || g_truecolor || colors_frozen )
+    if ( !color_init || g_truecolor )
 	return; 
     ulong *pixels = new ulong[colorDict->count()];
     QColorData   *d;
@@ -769,7 +696,6 @@ void QColor::destroyAllocContext( int context )
 	++it;
 	if ( d->context == context || context == -1 ) {
 	    pixels[i++] = d->pix;		// delete this color
-	    g_our_alloc[d->pix] = FALSE;	// don't reuse it
 	    colorDict->remove( (long)rgbVal );	// remove from dict
 	}
     }
