@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qconnect.cpp,v 2.1 1996/11/25 15:25:51 hanord Exp $
+** $Id: qconnect.cpp,v 2.3 1997/08/06 15:05:34 hanord Exp $
 **
 ** Implementation of QConnection class
 **
@@ -7,19 +7,22 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qconnect.h"
 
-RCSTAG("$Id: qconnect.cpp,v 2.1 1996/11/25 15:25:51 hanord Exp $");
+RCSTAG("$Id: qconnect.cpp,v 2.3 1997/08/06 15:05:34 hanord Exp $");
 
 
 /*!
@@ -35,7 +38,9 @@ RCSTAG("$Id: qconnect.cpp,v 2.1 1996/11/25 15:25:51 hanord Exp $");
   outside world.
 */
 
-
+/*!
+  \internal
+*/
 QConnection::QConnection( const QObject *object, QMember member,
 			  const char *memberName )
 {
@@ -52,3 +57,33 @@ QConnection::QConnection( const QObject *object, QMember member,
 	}
     }
 }
+
+/*!
+ \fn QConnection::~QConnection()
+ \internal
+*/
+
+/*!
+  \fn bool QConnection::isConnected() const
+  \internal
+*/
+
+/*!
+  \fn QObject *QConnection::object() const
+  \internal
+*/
+
+/*!
+  \fn QMember *QConnection::member() const
+  \internal
+*/
+
+/*!
+  \fn const char *QConnection::memberName() const
+  \internal
+*/
+
+/*!
+  \fn int QConnection::numArgs() const
+  \internal
+*/
