@@ -1,36 +1,38 @@
 /****************************************************************************
-** $Id: qcursor.cpp,v 2.9 1998/07/03 00:09:31 hanord Exp $
+** $Id: qcursor.cpp,v 2.3 1997/08/06 15:05:35 hanord Exp $
 **
 ** Implementation of QCursor class
 **
 ** Created : 940220
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qcursor.h"
 #include "qbitmap.h"
-#include "qdatastream.h"
+#include "qdstream.h"
+
+RCSTAG("$Id: qcursor.cpp,v 2.3 1997/08/06 15:05:35 hanord Exp $");
+
 
 /*!
   \class QCursor qcursor.h
 
   \brief The QCursor class provides a mouse cursor with an arbitrary shape.
 
-  \ingroup kernel
+  \ingroup guitool
   \ingroup shared
 
   This class is used for mainly two things; to create mouse cursors to be
@@ -46,22 +48,19 @@
   use QApplication::setOverrideCursor().
 
   The <a name=cursors>predefined cursor</a> objects are:
-  <ul>
-  <li> arrowCursor : standard arrow cursor
-  <li> upArrowCursor : upwards arrow
-  <li> crossCursor : crosshair
-  <li> waitCursor : hourglass/watch
-  <li> ibeamCursor : ibeam/text entry
-  <li> sizeVerCursor : vertical resize
-  <li> sizeHorCursor : horizontal resize
-  <li> sizeBDiagCursor : diagonal resize (/)
-  <li> sizeFDiagCursor : diagonal resize (\)
-  <li> sizeAllCursor : all directions resize
-  <li> blankCursor : blank/invisible cursor
-  </ul>
-  
-  \sa QWidget
-  <a href="guibooks.html#fowler">GUI Design Handbook: Cursors.</a>
+  <dl compact>
+  <dt> arrowCursor <dd> standard arrow cursor
+  <dt> upArrowCursor <dd> upwards arrow
+  <dt> crossCursor <dd> crosshair
+  <dt> waitCursor <dd> hourglass/watch
+  <dt> ibeamCursor <dd> ibeam/text entry
+  <dt> sizeVerCursor <dd> vertical resize
+  <dt> sizeHorCursor <dd> horizontal resize
+  <dt> sizeBDiagCursor <dd> diagonal resize (/)
+  <dt> sizeFDiagCursor <dd> diagonal resize (\)
+  <dt> sizeAllCursor <dd> all directions resize
+  <dt> blankCursor <dd> blank/invisible cursor
+  </dl>
 */
 
 
