@@ -1,40 +1,38 @@
 /****************************************************************************
-** $Id: qdialog.cpp,v 2.21 1998/07/03 00:09:32 hanord Exp $
+** $Id: qdialog.cpp,v 2.10 1997/09/03 15:27:27 warwick Exp $
 **
 ** Implementation of QDialog class
 **
 ** Created : 950502
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qdialog.h"
-#include "qpushbutton.h"
-#include "qapplication.h"
+#include "qpushbt.h"
+#include "qapp.h"
 #include "qkeycode.h"
-#include "qobjectlist.h"
-#include "qobjectdict.h"
-#include "qwidgetlist.h"
+#include "qobjcoll.h"
+
+RCSTAG("$Id: qdialog.cpp,v 2.10 1997/09/03 15:27:27 warwick Exp $");
+
 
 /*!
   \class QDialog qdialog.h
   \brief The QDialog class is the base class of dialog windows.
 
-  \ingroup dialogs
-
   A dialog window is a widget used to communicate with the user. It offers
   mechanisms such as default buttons.
 
@@ -84,9 +82,6 @@
 
   Modeless dialogs behave just like ordinary widgets. The only difference
   is that they have the default button mechanism.
-
-  \sa QTabDialog QWidget QSemiModal
-  <a href="guibooks.html#fowler">GUI Design Handbook: Dialogs, Standard.</a>
 */
 
 
@@ -94,10 +89,10 @@
   Constructs a dialog named \e name, which has a parent widget \e parent.
 
   The dialog will by default be modeless, unless you set \e modal to
-  TRUE to contruct a modal dialog.
+  TRUE, which constructs a modal dialog.
 
-  The \a f argument is the \link QWidget::QWidget() widget flags,
-  \endlink which can be used to customize the window frame style.
+  The \a f argument is the \link QWidget::QWidget() widget
+  flags \endlink , which can be used to customize the window frame style.
 
   \warning Creating a modeless dialog with a parent makes it an ordinary
   child widget, which is probably not what you want. Expect strange
@@ -180,11 +175,6 @@ int QDialog::exec()
   This function is very useful for modal dialogs. It leaves the local
   event loop and returns from the exec() or show() function.
 
-  \warning Although done() will return to the caller if this dialog is
-  modal, the local event loop is then marked for termination. Hence, a
-  program should not try to do anything that depends on event handling
-  before the corresponding exec() or show() has returned.
-
   \sa accept(), reject()
 */
 
@@ -302,52 +292,15 @@ void QDialog::show()
     if ( !did_move ) {
 	QWidget *w = parentWidget();
 	QPoint p( 0, 0 );
-	int extraw = 0, extrah = 0;
-	QWidget * desk = QApplication::desktop();
 	if ( w )
-	    w = w->topLevelWidget();
-
-	QWidgetList  *list = QApplication::topLevelWidgets();
-	QWidgetListIt it( *list );
-	while ( (extraw == 0 || extrah == 0) &&
-		it.current() != 0 ) {
-	    int w, h;
-	    w = it.current()->frameGeometry().width() - 
-		it.current()->width();
-	    h = it.current()->frameGeometry().height() - 
-		it.current()->height();
-
-	    extraw = QMAX( extraw, w );
-	    extrah = QMAX( extrah, h );
-	    // ### ++it at the end of the loop
-	    ++it;
-	}
-	delete list;
-
-	if ( w )
-	    p = QPoint( w->x() + w->width()/2,
-			w->y() + w->height()/ 2 );
+	    p = w->mapToGlobal( p );
 	else
-	    p = QPoint( desk->width()/2, desk->height()/2 );
-
-	p = QPoint( p.x()-width()/2 - extraw,
-		    p.y()-height()/2 - extraw );
-
-	if ( p.x() + extraw + width() > desk->width() )
-	    p.setX( desk->width() - width() - extraw );
-	if ( p.x() < 0 )
-	    p.setX( 0 );
-
-	if ( p.y() + extrah + height() > desk->height() )
-	    p.setY( desk->height() - height() - extrah );
-	if ( p.y() < 0 )
-	    p.setY( 0 );
-	
-	move( p );
+	    w = QApplication::desktop();
+	move( p.x() + w->width()/2  - width()/2,
+	      p.y() + w->height()/2 - height()/2 );
     }
     QWidget::show();
-    if ( testWFlags(WType_Modal) )
-	qApp->enter_loop();
+    if ( testWFlags(WType_Modal) ) qApp->enter_loop();
 }
 
 
