@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qfnt_x11.cpp,v 2.18.2.1 1997/11/18 14:16:05 paul Exp $
+** $Id: qfnt_x11.cpp,v 2.18 1997/09/10 14:36:29 agulbra Exp $
 **
 ** Implementation of QFont, QFontMetrics and QFontInfo classes for X11
 **
@@ -7,7 +7,7 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
@@ -34,7 +34,7 @@
 #include <X11/Xos.h>
 #include <X11/Xatom.h>
 
-RCSTAG("$Id: qfnt_x11.cpp,v 2.18.2.1 1997/11/18 14:16:05 paul Exp $");
+RCSTAG("$Id: qfnt_x11.cpp,v 2.18 1997/09/10 14:36:29 agulbra Exp $");
 
 
 static const int fontFields = 14;
@@ -521,7 +521,7 @@ void QFont::load( HANDLE ) const
 	if ( !d->fin ) {			// font was never loaded
 	    d->fin = new QFontInternal( n );
 	    CHECK_PTR( d->fin );
-	    fontDict->insert( d->fin->name(), d->fin );
+	    fontDict->insert( n, d->fin );
 	}
     }
     XFontStruct *f = d->fin->f;
@@ -541,7 +541,7 @@ void QFont::load( HANDLE ) const
 	// If we get a cache overflow, we make room for this font only
 	if ( size > fontCache->maxCost() + reserveCost )
 	    fontCache->setMaxCost( size + reserveCost );
-	if ( !fontCache->insert(d->fin->name(), d->fin, size) ) {
+	if ( !fontCache->insert(n, d->fin, size) ) {
 #if defined(DEBUG)
 	    fatal( "QFont::load: Cache overflow error" );
 #endif
