@@ -1,23 +1,22 @@
 /****************************************************************************
-** $Id: qmovie.cpp,v 1.39 1998/07/03 00:09:35 hanord Exp $
+** $Id: qmovie.cpp,v 1.26 1997/09/30 11:18:37 hanord Exp $
 **
 ** Implementation of movie classes
 **
 ** Created : 970617
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -39,8 +38,6 @@
   \class QMovie qmovie.h
   \brief Incrementally loads an animation or image, signalling as it progresses.
 
-  \ingroup images
-
   A QMovie provides a QPixmap as the framePixmap(), and connections can
   be made via connectResize() and connectUpdate() to receive notification
   of size and pixmap changes.  All decoding is driven by
@@ -67,25 +64,11 @@
   disposal, local color maps, and background colors.  The Netscape looping
   extension is obeyed.
 
-  Archives of animated GIFs and tools for building them can be found
-  at <a href=http://www.yahoo.com/Arts/Visual_Arts/Animation/Computer_Animation/Animated_GIFs/>
-  Yahoo!</a>.
-
   We are required to state: The Graphics Interchange Format(c) is the
   Copyright property of CompuServe Incorporated. GIF(sm) is a Service
   Mark property of CompuServe Incorporated.
 
-  \warning Unisys has changed its position regarding GIF.  If you are
-  in a country where Unisys holds a patent on LZW compression and/or
-  decompression, Unisys may require a license from you.  These
-  countries include Canada, Japan, the USA, France, Germany, Italy and
-  the UK.  There is more information on Unisys web site: <a
-  href="http://corp2.unisys.com/LeadStory/lzwfaq.html">Overview of
-  Unisys' position.</a> GIF support may be removed in a future version
-  of Qt.  We recommend using the PNG format, which is available in the
-  <a href="imageio.html">Qt Image IO Extension</a> package.
-
-  <img src="qmovie.gif">
+  <img src=qmovie.gif>
 
   \sa QLabel::setMovie()
 */
@@ -114,13 +97,13 @@ public: // for QMovie
     void showChanges();
 
     // This as QImageConsumer
-    void changed(const QRect& rect);
-    void end();
-    void frameDone();
+    bool changed(const QRect& rect);
+    bool end();
+    bool frameDone();
     void restartTimer();
-    void setLooping(int l);
-    void setFramePeriod(int milliseconds);
-    void setSize(int w, int h);
+    bool setLooping(int l);
+    bool setFramePeriod(int milliseconds);
+    bool setSize(int w, int h);
 
     // This as QDataSink
     int readyToReceive();
@@ -193,8 +176,7 @@ QMoviePrivate::QMoviePrivate(QDataSource* src, QMovie* movie, int bufsize) :
 
 QMoviePrivate::~QMoviePrivate()
 {
-    if ( buffer )				// Avoid purify complaint
-	delete [] buffer;
+    delete buffer;
     delete pump;
     delete decoder;
     delete source;
@@ -210,18 +192,12 @@ bool QMoviePrivate::isNull() const
 void QMoviePrivate::init(bool fully)
 {
     buf_usage = buf_r = buf_w = 0;
-    if ( buffer )				// Avoid purify complaint
-	delete [] buffer;
+    delete buffer;
     buffer = fully ? new uchar[buf_size] : 0;
 
     delete decoder;
     decoder = fully ? new QImageDecoder(this) : 0;
 
-#ifdef AVOID_OPEN_FDS
-    if ( source && !source->isOpen() )
-	source->open(IO_ReadOnly);
-#endif
-
     waitingForFrameTick = FALSE;
     stepping = -1;
     framenumber = 0;
@@ -318,17 +294,19 @@ void QMoviePrivate::showChanges()
 }
 
 // QMoviePrivate as QImageConsumer
-void QMoviePrivate::changed(const QRect& rect)
+bool QMoviePrivate::changed(const QRect& rect)
 {
     if (!frametimer->isActive()) frametimer->start(0);
     changed_area = changed_area.unite(rect);
+    return TRUE;
 }
 
-void QMoviePrivate::end()
+bool QMoviePrivate::end()
 {
+    return TRUE; // Not that it will do anything.
 }
 
-void QMoviePrivate::frameDone()
+bool QMoviePrivate::frameDone()
 {
     if (stepping > 0) {
 	stepping--;
@@ -343,6 +321,7 @@ void QMoviePrivate::frameDone()
     showChanges();
     emit dataStatus(QMovie::EndOfFrame);
     framenumber++;
+    return FALSE;
 }
 
 void QMoviePrivate::restartTimer()
@@ -358,7 +337,7 @@ void QMoviePrivate::restartTimer()
     }
 }
 
-void QMoviePrivate::setLooping(int nloops)
+bool QMoviePrivate::setLooping(int nloops)
 {
     if (loop == -1) { // Only if we don't already know how many loops!
 	if (source->rewindable()) {
@@ -369,21 +348,25 @@ void QMoviePrivate::setLooping(int nloops)
 	    loop = -2;
 	}
     }
+
+    return TRUE;
 }
 
-void QMoviePrivate::setFramePeriod(int milliseconds)
+bool QMoviePrivate::setFramePeriod(int milliseconds)
 {
     // Animation:  only show complete frame
     frameperiod = milliseconds;
     if (stepping<0 && frameperiod >= 0) restartTimer();
+    return TRUE;
 }
 
-void QMoviePrivate::setSize(int w, int h)
+bool QMoviePrivate::setSize(int w, int h)
 {
     if (mypixmap.width() != w || mypixmap.height() != h) {
 	mypixmap.resize(w, h);
 	emit sizeChanged(QSize(w, h));
     }
+    return TRUE;
 }
 
 
@@ -391,8 +374,6 @@ void QMoviePrivate::setSize(int w, int h)
 
 int QMoviePrivate::readyToReceive()
 {
-    // Could pre-fill buffer, but more efficient to just leave the
-    // data back at the source.
     return (waitingForFrameTick || !stepping || buf_usage || error)
 	? 0 : buf_size;
 }
@@ -439,14 +420,9 @@ void QMoviePrivate::eof()
     } else {
 	delete decoder;
 	decoder = 0;
-	if ( buffer )				// Avoid purify complaint
-	    delete [] buffer;
+	delete buffer;
 	buffer = 0;
 	emit dataStatus(QMovie::EndOfMovie);
-#ifdef AVOID_OPEN_FDS
-	if ( source )
-	    source->close();
-#endif
     }
 }
 
@@ -461,11 +437,6 @@ void QMoviePrivate::pause()
 
 void QMoviePrivate::refresh()
 {
-    if (!decoder) {
-	frametimer->stop();
-	return;
-    }
-
     if (frameperiod < 0 && loop == -1) {
 	// Only show changes if probably not an animation
 	showChanges();
@@ -496,9 +467,12 @@ QMovie::QMovie()
     d = new QMoviePrivate();
 }
 
-/*!
+#if 0
+// Removed - QDataSource should be the interface.  QIODevice is not useful,
+// as it doesn't emit signals upon data availability.
+/*
   Creates a QMovie which reads an image sequence from the given
-  QDataSource.  The source must be allocated dynamically,
+  QIODevice.  The device must be allocated dynamically,
   as it becomes owned by the QMovie, and will be destroyed
   when the movie is destroyed.
   The movie starts playing as soon as event processing continues.
@@ -508,10 +482,11 @@ QMovie::QMovie()
   value, the better interleaved the movie playback will be with other
   event processing, but the slower the overall processing.
 */
-QMovie::QMovie(QDataSource* src, int bufsize)
+QMovie::QMovie(QIODevice* src, int bufsize)
 {
-    d = new QMoviePrivate(src, this, bufsize);
+    d = new QMoviePrivate(new QIODeviceSource(src), this, bufsize);
 }
+#endif
 
 /*!
   \overload
@@ -852,7 +827,7 @@ void QMovie::disconnectStatus(QObject* receiver, const char* member)
 ** QMoviePrivate meta object code from reading C++ file 'qmovie.cpp'
 **
 ** Created: Thu Sep 4 15:31:20 1997
-**      by: The Qt Meta Object Compiler ($Revision: 1.39 $)
+**      by: The Qt Meta Object Compiler ($Revision: 1.26 $)
 **
 ** WARNING! All changes made in this file will be lost!
 *****************************************************************************/
@@ -863,7 +838,7 @@ void QMovie::disconnectStatus(QObject* receiver, const char* member)
 #error Moc format conflict - please regenerate all moc files
 #endif
 
-#include <qmetaobject.h>
+#include <qmetaobj.h>
 
 
 const char *QMoviePrivate::className() const
