@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qpic_x11.cpp,v 2.0 1996/09/24 10:15:56 agulbra Exp $
+** $Id: qpic_x11.cpp,v 2.1 1997/08/06 15:05:41 hanord Exp $
 **
 ** Implementation of QPicture class for X11
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -23,7 +26,7 @@
 #include <X11/Xutil.h>
 #include <X11/Xos.h>
 
-RCSTAG("$Id: qpic_x11.cpp,v 2.0 1996/09/24 10:15:56 agulbra Exp $");
+RCSTAG("$Id: qpic_x11.cpp,v 2.1 1997/08/06 15:05:41 hanord Exp $");
 
 
 QPicture::QPicture()
