@@ -1,40 +1,41 @@
 /****************************************************************************
-** $Id: qpicture.cpp,v 2.7 1998/07/03 00:09:37 hanord Exp $
+** $Id: qpicture.cpp,v 2.2 1997/08/06 15:05:41 hanord Exp $
 **
 ** Implementation of QPicture class
 **
 ** Created : 940802
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qpicture.h"
-#include "qpaintdevicedefs.h"
+#include "qpaintdc.h"
 #include "qpainter.h"
 #include "qpixmap.h"
-#include "qimage.h"
 #include "qfile.h"
-#include "qdatastream.h"
+#include "qdstream.h"
+
+RCSTAG("$Id: qpicture.cpp,v 2.2 1997/08/06 15:05:41 hanord Exp $");
+
 
 /*!
   \class QPicture qpicture.h
   \brief The QPicture class is a paint device that records and replays QPainter
   commands.
 
-  \ingroup drawing
+  \ingroup paintdevice
 
   A picture serializes painter commands to an IO device in a
   platform-independent format.	A picture created under Windows
@@ -346,12 +347,6 @@ bool QPicture::exec( QPainter *painter, QDataStream &s, int nrecords )
 		painter->drawPixmap( p, pixmap );
 		}
 		break;
-	    case PDC_DRAWIMAGE: {
-		QImage image;
-		s >> p >> image;
-		painter->drawImage( p, image );
-		}
-		break;
 	    case PDC_BEGIN:
 		s >> ul;			// number of records
 		if ( !exec( painter, s, ul ) )
@@ -535,10 +530,6 @@ bool QPicture::cmd( int c, QPainter *, QPDevCmdParam *p )
 	    s << *p[0].point;
 	    s << *p[1].pixmap;
 	    break;
-	case PDC_DRAWIMAGE:
-	    s << *p[0].point;
-	    s << *p[1].image;
-	    break;
 	case PDC_SAVE:
 	case PDC_RESTORE:
 	    break;
