@@ -1,23 +1,22 @@
 /****************************************************************************
-** $Id: qpixmap.cpp,v 2.31 1998/07/06 01:57:39 agulbra Exp $
+** $Id: qpixmap.cpp,v 2.21 1997/09/29 16:41:55 eiriken Exp $
 **
 ** Implementation of QPixmap class
 **
 ** Created : 950301
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -25,25 +24,22 @@
 #include "qimage.h"
 #include "qwidget.h"
 #include "qpainter.h"
-#include "qdatastream.h"
+#include "qdstream.h"
 #include "qbuffer.h"
 
+RCSTAG("$Id: qpixmap.cpp,v 2.21 1997/09/29 16:41:55 eiriken Exp $");
+
+
 /*!
   \class QPixmap qpixmap.h
-  \brief The QPixmap class is an off-screen pixel-based paint device.
+  \brief The QPixmap class is an off-screen buffer paint device.
 
   \ingroup drawing
   \ingroup shared
-  
-  It is one of the two classes Qt provides for dealing with images,
-  the other being QImage.  QPixmap is designed and optimized for
-  drawing; QImage is designed and optimized for I/O and for direct
-  pixel access/manipulation.  There are (slow) functions to convert
-  between QImage and QPixmp; convertToImage() and convertFromImage().
 
-  One common use of the QPixmap class is to enable smooth updating of
+  A common use of the QPixmap class is to enable smooth updating of
   widgets.  Whenever something complex needs to be drawn, you can use
-  a pixmap to obtain flicker-free drawing, like this:
+  a pixmap to obtain flicker-free drawing.
 
   <ol plain>
   <li> Create a pixmap with the same size as the widget.
@@ -51,19 +47,35 @@
   <li> Paint the pixmap.
   <li> bitBlt() the pixmap contents onto the widget.
   </ol>
-  
-  Pixel data in a pixmap is internal and managed by the underlying
-  window system.  Pixels can only be accessed through QPainter
-  functions, through bitBlt(), and by converting the QPixmap to a
-  QImage.
 
-  You can display a QPixmap on the screen easily using
-  e.g. QLabel::setPixmap(), and all the \link QButton button classes
-  \endlink support pixmap use.
-  
-  There are also convenience functions to get and set single pixels
-  and to load and save the entire pixmap; these work by converting the
-  pixmap to a QImage internally.
+  Example of flicker-free update:
+  \code
+    void MyWidget::paintEvent( QPaintEvent * )
+    {
+	QPixmap	 pm( size() );			// create pixmap
+	QPainter p;				// our painter
+	pm.fill( backgroundColor() );		// initialize pixmap
+	p.begin( &pm );				// start painting pixmap
+	...					// draw something
+	p.end();				// painting done
+	bitBlt( this, 0,0, &pm );		// copy pixmap to widget
+    }
+  \endcode
+
+  The bitBlt() function has quite a few arguments that are not used in
+  this example.
+
+  Pixel data in a pixmap is internal and managed by the underlying window
+  system.  Pixels can only be accessed through QImage, QPainter functions
+  and the bitBlt().
+
+  You can \link load() load\endlink and \link save() save\endlink
+  pixmaps using several image formats.	You can display a QPixmap on
+  the screen easily using QLabel::setPixmap(), and all the \link
+  QButton button classes \endlink support pixmap use.
+
+  A pixmap can be converted to a QImage to get direct access to the pixels.
+  A QImage can also be converted back to a pixmap.
 
   The QPixmap class is optimized by the use of \link shclass.html implicit
   sharing\endlink, so it is very efficient to pass QPixmap objects as
@@ -161,21 +173,38 @@ QPixmap::QPixmap( const char *xpm[] )
 	convertFromImage( image );
 }
 
-/*
-  Constructs a pixmaps by loading from \a data.
-
-  \sa loadFromData()
-*/
-/* Not until Qt 2.0 - adding conversions break sourcecode.
-QPixmap::QPixmap( const QByteArray & data )
-{
-    loadFromData(data);
-}
-*/
-
 
 /*!
-  Returns a
+  Special-purpose function that detaches the pixmap from shared pixmap data.
+
+  A pixmap is automatically detached by Qt whenever its contents is about
+  to change.  This is done in all QPixmap member functions that modify the
+  pixmap (fill(), resize(), convertFromImage(), load() etc.), in bitBlt()
+  for the destination pixmap and in QPainter::begin() on a pixmap.
+
+  It is possible to modify a pixmap without letting Qt know.
+  You can first obtain the \link handle() system-dependent handle\endlink
+  and then call system-specific functions (for instance BitBlt under Windows)
+  that modifies the pixmap contents.  In this case, you can call detach()
+  to cut the pixmap loose from other pixmaps that share data with this one.
+
+  detach() returns immediately if there is just a single reference or if
+  the pixmap has not been initialized yet.
+*/
+
+void QPixmap::detach()
+{
+    if ( data->optim )
+	data->dirty = TRUE;
+    if ( data->uninit || data->count == 1 ) {
+	data->uninit = FALSE;
+	return;
+    }
+    *this = copy();
+}
+
+/*!
+  Returns a 
   \link shclass.html deep copy\endlink of the pixmap using the bitBlt()
   function to copy the pixels.
   \sa operator=()
@@ -184,9 +213,8 @@ QPixmap::QPixmap( const QByteArray & data )
 QPixmap QPixmap::copy() const
 {
     QPixmap tmp( data->w, data->h, data->d );
-    tmp.data->optim  = data->optim;		// copy optimization flag
+    tmp.data->optim  = data->optim;		// copy optim flag
     tmp.data->bitmap = data->bitmap;		// copy bitmap flag
-    tmp.data->opt    = data->opt;		// copy optimization setting
     if ( !tmp.isNull() ) {			// copy the bitmap
 	bitBlt( &tmp, 0,0, this, 0,0, data->w, data->h, CopyROP, TRUE );
 	if ( data->mask )			// copy the mask
@@ -338,7 +366,6 @@ void QPixmap::resize( int w, int h )
 	QPixmap pm;
 	pm.data->optim	= data->optim;		// keep optimization flag
 	pm.data->bitmap = data->bitmap;		// keep is-a flag
-	pm.data->opt	= data->opt;		// keep optimization setting
 	*this = pm;
 	return;
     }
@@ -353,9 +380,8 @@ void QPixmap::resize( int w, int h )
 	bitBlt( &pm, 0, 0, this, 0, 0,		// copy old pixmap
 		QMIN(width(), w),
 		QMIN(height(),h), CopyROP, TRUE );
-    pm.data->optim  = data->optim;		// keep optimization flag
+    pm.data->optim  = data->optim;		// keep optim flag
     pm.data->bitmap = data->bitmap;		// keep bitmap flag
-    pm.data->opt    = data->opt;		// keep optimization setting
     if ( data->mask ) {				// resize mask as well
 	QBitmap m = *data->mask;
 	m.resize( w, h );
@@ -430,7 +456,7 @@ void QPixmap::setMask( const QBitmap &mask )
   things like:
   \code
     pm->setMask( pm->createHeuristicMask() );
-  \endcode
+  \endcode;
 
   This function is slow because it involves transformation to a QImage,
   non-trivial computations and a transformation back to QBitmap.
@@ -614,17 +640,6 @@ bool QPixmap::loadFromData( const uchar *buf, uint len, const char *format,
     return loadFromData( buf, len, format, conversion_flags );
 }
 
-/*!
-  \overload
-*/
-bool QPixmap::loadFromData( QByteArray buf,
-			    const char *format,
-			    int conversion_flags )
-{
-    return loadFromData( (const uchar *)(buf.data()), buf.size(),
-			 format, conversion_flags );
-}
-
 /*!
   Saves the pixmap to the file \e fileName, using the image file format
   \e format.  Returns TRUE if successful, or FALSE if the pixmap could not
