@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qpm_x11.cpp,v 2.9 1997/03/22 16:18:37 hanord Exp $
+** $Id: qpm_x11.cpp,v 2.26 1997/09/25 13:24:28 warwick Exp $
 **
 ** Implementation of QPixmap class for X11
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -35,11 +38,11 @@
 #include <X11/extensions/XShm.h>
 #endif
 
-RCSTAG("$Id: qpm_x11.cpp,v 2.9 1997/03/22 16:18:37 hanord Exp $");
+RCSTAG("$Id: qpm_x11.cpp,v 2.26 1997/09/25 13:24:28 warwick Exp $");
 
 
 /*****************************************************************************
-  MIT Shared Memory Extension support: makes xForm noticeably faster.
+  MIT Shared Memory Extension support: makes xForm noticeably (~20%) faster.
  *****************************************************************************/
 
 #if defined(MITSHM)
@@ -65,14 +68,14 @@ static void qt_cleanup_mitshm()
     shmctl( xshminfo.shmid, IPC_RMID, 0 );
 }
 
-bool qt_create_mitshm_buffer( int w, int h )
+bool qt_create_mitshm_buffer( QPaintDevice* dev, int w, int h )
 {
     static int major, minor;
     static Bool pixmaps_ok;
-    Display *dpy = QPaintDevice::x11Display();
-    int scr	 = QPaintDevice::x11Screen();
-    int dd	 = QPaintDevice::x11Depth();
-    Visual *vis	 = QPaintDevice::x11Visual();
+    Display *dpy = dev->x11Display();
+    int scr	 = dev->x11Screen();
+    int dd	 = dev->x11Depth();
+    Visual *vis	 = (Visual*)dev->x11Visual();
 
     if ( xshminit ) {
 	qt_cleanup_mitshm();
@@ -116,7 +119,7 @@ bool qt_create_mitshm_buffer( int w, int h )
 
 #else
 
-bool qt_create_mitshm_buffer( int, int )
+bool qt_create_mitshm_buffer( QPaintDevice*, int, int )
 {
     return FALSE;
 }
@@ -190,6 +193,7 @@ void QPixmap::init( int w, int h, int d )
     if ( make_null || w < 0 || h < 0 || data->d == 0 ) {
 	data->w = data->h = 0;
 	data->d = 0;
+	hd = None;
 #if defined(CHECK_RANGE)
 	if ( !make_null )
 	    warning( "QPixmap: Invalid pixmap parameters" );
@@ -451,8 +455,9 @@ int QPixmap::metric( int m ) const
   has 8 bit depth.  If the pixmap has greater than 8 bit depth, the
   returned image has 32 bit depth.
 
-  \bug Does not support 2 or 4 bit display hardware. This function
-  needs to be tested on different types of X servers.
+  \bug Does not support 2 or 4 bit display hardware.
+
+  \bug Alpha masks on monochrome images are ignored.
 
   \sa convertFromImage()
 */
@@ -502,6 +507,15 @@ QImage QPixmap::convertToImage() const
     if ( image.isNull() )			// could not create image
 	return image;
 
+    const QBitmap* msk = mask();
+
+    QImage alpha;
+    if (msk) {
+	image.setAlphaBuffer( TRUE );
+	alpha = msk->convertToImage();
+    }
+    bool ale = alpha.bitOrder() == QImage::LittleEndian;
+
     if ( trucol ) {				// truecolor
 	uint red_mask	 = visual->red_mask;
 	uint green_mask	 = visual->green_mask;
@@ -520,6 +534,7 @@ QImage QPixmap::convertToImage() const
 	    bppc++;
 
 	for ( int y=0; y<h; y++ ) {
+	    uchar* asrc = msk ? alpha.scanLine( y ) : 0;
 	    dst = (QRgb *)image.scanLine( y );
 	    src = (uchar *)xi->data + xi->bytes_per_line*y;
 	    for ( int x=0; x<w; x++ ) {
@@ -576,7 +591,18 @@ QImage QPixmap::convertToImage() const
 		    b = (pixel & blue_mask) >> blue_shift;
 		else
 		    b = (pixel & blue_mask) << -blue_shift;
-		*dst++ = qRgb(r, g, b);
+
+		if (msk) {
+		    if ( ale ) {
+			*dst++ = (asrc[x >> 3] & (1 << (x & 7)))
+			    ? (0xff000000 | qRgb(r, g, b)) : qRgb(r, g, b);
+		    } else {
+			*dst++ = (asrc[x >> 3] & (1 << (7 -(x & 7))))
+			    ? (0xff000000 | qRgb(r, g, b)) : qRgb(r, g, b);
+		    }
+		} else {
+		    *dst++ = qRgb(r, g, b);
+		}
 	    }
 	}
     } else if ( xi->bits_per_pixel == d ) {	// compatible depth
@@ -610,11 +636,28 @@ QImage QPixmap::convertToImage() const
 	memset( pix, 0, 256 );
 	bpl = image.bytesPerLine();
 
-	for ( i=0; i<h; i++ ) {			// which pixels are used?
-	    p = image.scanLine( i );
-	    end = p + bpl;
-	    while ( p < end )
-		use[*p++] = 1;
+	if (msk) {				// which pixels are used?
+	    for ( i=0; i<h; i++ ) {
+		uchar* asrc = alpha.scanLine( i );
+		p = image.scanLine( i );
+		for ( int x = 0; x < w; x++ ) {
+		    if ( ale ) {
+			if (asrc[x >> 3] & (1 << (x & 7)))
+			    use[*p] = 1;
+		    } else {
+			if (asrc[x >> 3] & (1 << (7 -(x & 7))))
+			    use[*p] = 1;
+		    }
+		    ++p;
+		}
+	    }
+	} else {
+	    for ( i=0; i<h; i++ ) {
+		p = image.scanLine( i );
+		end = p + bpl;
+		while ( p < end )
+		    use[*p++] = 1;
+	    }
 	}
 	ncols = 0;
 	for ( i=0; i<256; i++ ) {		// build translation table
@@ -637,14 +680,43 @@ QImage QPixmap::convertToImage() const
 	    carr[i].pixel = i;
 	XQueryColors( dpy, cmap, carr, ncells );// get default colormap
 
-	image.setNumColors( ncols );		// create color table
+	if (msk) {
+	    int trans;
+	    if (ncols < 256) {
+		trans = ncols++;
+		image.setNumColors( ncols );	// create color table
+		image.setColor( trans, 0x00000000 );
+	    } else {
+		image.setNumColors( ncols );	// create color table
+		// oh dear... no spare "transparent" pixel.
+		// use first pixel in image (as good as any).
+		trans = image.scanLine( i )[0];
+	    }
+	    for ( i=0; i<h; i++ ) {
+		uchar* asrc = alpha.scanLine( i );
+		p = image.scanLine( i );
+		for ( int x = 0; x < w; x++ ) {
+		    if ( ale ) {
+			if (!(asrc[x >> 3] & (1 << (x & 7))))
+			    *p = trans;
+		    } else {
+			if (!(asrc[x >> 3] & (1 << (7 -(x & 7)))))
+			    *p = trans;
+		    }
+		    ++p;
+		}
+	    }
+	} else {
+	    image.setNumColors( ncols );	// create color table
+	}
 	int j = 0;
 	for ( i=0; i<256; i++ ) {		// translate pixels
 	    if ( use[i] ) {
 		image.setColor( j++,
-				qRgb( (carr[i].red   >> 8) & 255,
-				      (carr[i].green >> 8) & 255,
-				      (carr[i].blue  >> 8) & 255 ) );
+				 ( msk ? 0xff000000 : 0 )
+				 | qRgb( (carr[i].red   >> 8) & 255,
+			                 (carr[i].green >> 8) & 255,
+			                 (carr[i].blue  >> 8) & 255 ) );
 	    }
 	}
 	delete [] carr;
@@ -660,38 +732,66 @@ QImage QPixmap::convertToImage() const
 
 
 /*!
-  \fn bool QPixmap::convertFromImage( const QImage &image, ColorMode mode )
   Converts an image and sets this pixmap. Returns TRUE if successful.
 
-  The \e mode argument specifies whether the resulting pixmap should be
-  a monochrome (\link depth() depth\endlink == 1) or a normal
-  (\link defaultDepth() native depth\endlink) pixmap.  This argument
-  is ignored if this pixmap is a QBitmap.
+  The \a conversion_flags argument is a bitwise-OR from the following choices.
+  The options marked \e (default) are the choice if no other choice from the
+  list is included (they are zero):
 
-  If this pixmap is a QBitmap or \e mode == \c QPixmap::Mono, the pixmap
-  becomes monochrome.  If necessary, it is dithered using the
-  Floyd-Steinberg dithering algorithm.
+  <dl>
+   <dt>Color/Mono preference (ignored for QBitmap)
+   <dd>
+    <ul>
+     <li> \c AutoColor (default) - If the \e image has \link
+               QImage::depth() depth\endlink 1 and contains only
+               black and white pixels, then the pixmap becomes monochrome.
+     <li> \c ColorOnly - The pixmap is dithered/converted to the
+               \link defaultDepth() native display depth\endlink.
+     <li> \c MonoOnly - The pixmap becomes monochrome.  If necessary,
+               it is dithered using the chosen dithering algorithm.
+    </ul>
+   <dt>Dithering mode preference, for RGB channels
+   <dd>
+    <ul>
+     <li> \c DiffuseDither (default) - a high quality dither
+     <li> \c OrderedDither - a faster more ordered dither
+     <li> \c ThresholdDither - no dithering, closest color is used
+    </ul>
+   <dt>Dithering mode preference, for alpha channel
+   <dd>
+    <ul>
+     <li> \c DiffuseAlphaDither - a high quality dither
+     <li> \c OrderedAlphaDither - a faster more ordered dither
+     <li> \c ThresholdAlphaDither (default) - no dithering
+    </ul>
+   <dt>Color matching versus dithering preference
+   <dd>
+    <ul>
+     <li> \c PreferDither - always dither 32-bit images when
+		the image
+		is being converted to 8-bits.
+		This is the default when converting to a pixmap.
+     <li> \c AvoidDither - only dither 32-bit images if
+		the image
+		has more than 256 colours and it
+		is being converted to 8-bits.
+		This is the default when an image is converted
+		for the purpose of saving to a file.
+    </ul>
+  </dl>
 
-  If \e mode == \c QPixmap::Auto (default) and the \e image has \link
-  QImage::depth() depth\endlink 1 and contains only black and white
-  pixels, then the pixmap becomes monochrome, as above.
-
-  If \e mode == \c QPixmap::Color, the pixmap is dithered/converted to the
-  \link defaultDepth() native display depth\endlink.
+  Passing 0 for \a conversion_flags gives all the default options.
 
   Note that even though a QPixmap with depth 1 behaves much like a
   QBitmap, isQBitmap() returns FALSE.
 
-  If \e image has more colors than the number of available colors, we
-  try to pick the most important colors.
+  \bug Does not support 2 or 4 bit display hardware.
 
-  \bug Does not support 2 or 4 bit display hardware. This function
-  needs to be tested on different types of X servers.
-
-  \sa convertToImage(), isQBitmap(), QImage::convertDepth(), defaultDepth()
+  \sa convertToImage(), isQBitmap(), QImage::convertDepth(), defaultDepth(),
+    hasAlphaBuffer()
 */
 
-bool QPixmap::convertFromImage( const QImage &img, ColorMode mode )
+bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 {
     if ( img.isNull() ) {
 #if defined(CHECK_NULL)
@@ -704,8 +804,9 @@ bool QPixmap::convertFromImage( const QImage &img, ColorMode mode )
     int	 w   = image.width();
     int	 h   = image.height();
     int	 d   = image.depth();
-    int	 dd  = x11Depth();
-    bool force_mono = (dd == 1 || isQBitmap() || mode == Mono);
+    int	 dd  = defaultDepth();
+    bool force_mono = (dd == 1 || isQBitmap() ||
+		       (conversion_flags & ColorMode_Mask)==MonoOnly );
 
     if ( data->mask ) {				// get rid of the mask
 	delete data->mask;
@@ -718,20 +819,30 @@ bool QPixmap::convertFromImage( const QImage &img, ColorMode mode )
 
     if ( force_mono ) {				// must be monochrome
 	if ( d != 1 ) {
-	    image = image.convertDepth( 1 );	// dither
+	    image = image.convertDepth( 1, conversion_flags );	// dither
 	    d = 1;
 	}
     } else {					// can be both
 	bool conv8 = FALSE;
-	if ( mode == Color ) {			// native depth wanted
-	    conv8 = d == 1;
-	} else if ( d == 1 && image.numColors() == 2 ) {
-	    QRgb c0 = image.color(0);		// mode==Auto: convert to best
-	    QRgb c1 = image.color(1);
-	    conv8 = QMIN(c0,c1) != 0 || QMAX(c0,c1) != qRgb(255,255,255);
+	if ( d > 8 && dd <= 8 ) {		// convert to 8 bit
+	    if ( (conversion_flags & DitherMode_Mask) == AutoDither )
+		conversion_flags = (conversion_flags & ~DitherMode_Mask)
+					| PreferDither;
+	    conv8 = TRUE;
+	} else if ( (conversion_flags & ColorMode_Mask) == ColorOnly ) {
+	    conv8 = d == 1;			// native depth wanted
+	} else if ( d == 1 ) {
+	    if ( image.numColors() == 2 ) {
+		QRgb c0 = image.color(0);	// Auto: convert to best
+		QRgb c1 = image.color(1);
+		conv8 = QMIN(c0,c1) != 0 || QMAX(c0,c1) != qRgb(255,255,255);
+	    } else {
+		// eg. 1-colour monochrome images (they do exist).
+		conv8 = TRUE;
+	    }
 	}
 	if ( conv8 ) {
-	    image = image.convertDepth( 8 );
+	    image = image.convertDepth( 8, conversion_flags );
 	    d = 8;
 	}
     }
@@ -778,7 +889,7 @@ bool QPixmap::convertFromImage( const QImage &img, ColorMode mode )
 
 	if ( image.hasAlphaBuffer() ) {
 	    QBitmap m;
-	    m = image.createAlphaMask();
+	    m = image.createAlphaMask( conversion_flags );
 	    setMask( m );
 	}
 	return TRUE;
@@ -803,10 +914,11 @@ bool QPixmap::convertFromImage( const QImage &img, ColorMode mode )
 	int   r, g, b;
 
 	if ( d8 ) {				// setup pixel translation
+	    QRgb *ctable = image.colorTable();
 	    for ( int i=0; i<image.numColors(); i++ ) {
-		r = qRed  (image.color(i));
-		g = qGreen(image.color(i));
-		b = qBlue (image.color(i));
+		r = qRed  (ctable[i]);
+		g = qGreen(ctable[i]);
+		b = qBlue (ctable[i]);
 		r = red_shift	> 0 ? r << red_shift   : r >> -red_shift;
 		g = green_shift > 0 ? g << green_shift : g >> -green_shift;
 		b = blue_shift	> 0 ? b << blue_shift  : b >> -blue_shift;
@@ -882,12 +994,6 @@ bool QPixmap::convertFromImage( const QImage &img, ColorMode mode )
 	xi->data = (char *)newbits;
     }
 
-    if ( d == 32 && !trucol ) {			// convert to 8 bit
-	image = image.convertDepth( 8 );
-	d = 8;
-	nbytes = image.numBytes();		// recalc image size
-    }
-
     if ( d == 8 && !trucol ) {			// 8 bit pixmap
 	int  i, j;
 	int  pop[256];				// pixel popularity
@@ -931,11 +1037,12 @@ bool QPixmap::convertFromImage( const QImage &img, ColorMode mode )
 	int  maxpix = 0;
 	CHECK_PTR( pixarr );
 	j = 0;
+	QRgb* ctable = image.colorTable();
 	for ( i=0; i<256; i++ ) {		// init pixel array
 	    if ( pop[i] > 0 ) {
-		px->r = qRed  ( image.color(i) );
-		px->g = qGreen( image.color(i) );
-		px->b = qBlue ( image.color(i) );
+		px->r = qRed  ( ctable[i] );
+		px->g = qGreen( ctable[i] );
+		px->b = qBlue ( ctable[i] );
 		px->use = pop[i];
 		if ( pop[i] > maxpop ) {	// select most popular entry
 		    maxpop = pop[i];
@@ -1049,9 +1156,9 @@ bool QPixmap::convertFromImage( const QImage &img, ColorMode mode )
     }
     data->w = w;  data->h = h;	data->d = dd;
 
-    if ( image.hasAlphaBuffer() ) {
+    if ( img.hasAlphaBuffer() ) {
 	QBitmap m;
-	m = image.createAlphaMask();
+	m = img.createAlphaMask( conversion_flags );
 	setMask( m );
     }
 
@@ -1102,8 +1209,7 @@ QPixmap QPixmap::grabWindow( WId window, int x, int y, int w, int h )
     pm.data->uninit = FALSE;
     GC gc = qt_xget_temp_gc( FALSE );
     XSetSubwindowMode( dpy, gc, IncludeInferiors );
-    XCopyArea( dpy, QApplication::desktop()->handle(), pm.handle(), gc,
-	       x, y, w, h, 0, 0 );
+    XCopyArea( dpy, window, pm.handle(), gc, x, y, w, h, 0, 0 );
     XSetSubwindowMode( dpy, gc, ClipByChildren );
     return pm;
 }
@@ -1241,6 +1347,13 @@ QPixmap QPixmap::xForm( const QWMatrix &matrix ) const
     }
 
 #if defined(MITSHM)
+    static bool try_once = TRUE;
+    if (try_once) {
+	try_once = FALSE;
+	if ( !xshminit )
+	    qt_create_mitshm_buffer( this, 800, 600 );
+    }
+
     bool use_mitshm = xshmimg && !depth1 &&
 		      xshmimg->width >= w && xshmimg->height >= h;
 #endif
@@ -1402,7 +1515,7 @@ QPixmap QPixmap::xForm( const QWMatrix &matrix ) const
 #undef IWX
 #define IWX(b)	if ( trigx < maxws && trigy < maxhs ) {			      \
 		    if ( *(sptr+sbpl*(trigy>>16)+(trigx>>19)) &		      \
-			 (1 << 7-((trigx>>16)&7)) )			      \
+			 (1 << (7-((trigx>>16)&7))) )			      \
 			*p |= b;					      \
 		}							      \
 		trigx += m11;						      \
