@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qpntarry.cpp,v 2.0 1996/09/24 10:15:58 agulbra Exp $
+** $Id: qpntarry.cpp,v 2.9 1997/08/29 10:50:42 warwick Exp $
 **
 ** Implementation of QPointArray class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -23,7 +26,45 @@
 #include "qdstream.h"
 #include <stdarg.h>
 
-RCSTAG("$Id: qpntarry.cpp,v 2.0 1996/09/24 10:15:58 agulbra Exp $");
+RCSTAG("$Id: qpntarry.cpp,v 2.9 1997/08/29 10:50:42 warwick Exp $");
+
+/*!
+  \class QPointVal qpntarry.h
+  \brief The QPointVal class is an internal class, used with QPointArray.
+
+  The QPointVal is required by the indexing [] operator on point arrays.
+  It is probably a bad idea to use it in any other context.
+*/
+
+/*! \fn QPointVal::QPointVal (QPointData* ptr) 
+
+  Constructs a reference to an element in a QPointArray.  This is
+  what QPointArray::operator[] contructs its return value with.
+*/
+
+/*! \fn QPointVal& QPointVal::operator= (const QPoint& point) 
+  Assigns a given point to the value referenced by this QPointVal.
+*/
+
+/*! \fn QPointVal& QPointVal::operator+= (const QPoint& point) 
+  Vector-adds a given point to the value referenced by this QPointVal.
+*/
+
+/*! \fn QPointVal& QPointVal::operator-= (const QPoint& point) 
+  Vector-subtracts a given point to the value referenced by this QPointVal.
+*/
+
+/*! \fn QPointVal::operator QPoint() const
+  Returns the value of the point referenced by this QPointVal.
+*/
+
+/*! \fn int QPointVal::x () const 
+  Returns the X coordinate of the point referenced by this QPointVal.
+*/
+
+/*! \fn int QPointVal::y () const 
+  Returns the Y coordinate of the point referenced by this QPointVal.
+*/
 
 
 /*!
@@ -44,7 +85,8 @@ RCSTAG("$Id: qpntarry.cpp,v 2.0 1996/09/24 10:15:58 agulbra Exp $");
   has member functions that operate on QPoint to make programming
   easier.
 
-  Note that since this class is a QArray, it is explicitly shared
+  Note that since this class is a QArray, it is
+  \link shclass.html explicitly shared\endlink
   and works with shallow copies by default.
 */
 
@@ -69,7 +111,10 @@ RCSTAG("$Id: qpntarry.cpp,v 2.0 1996/09/24 10:15:58 agulbra Exp $");
 
 /*!
   \fn QPointArray::QPointArray( const QPointArray &a )
-  Constructs a shallow copy of the point array \e a.
+  Constructs a 
+  \link shclass.html shallow copy\endlink of the point array \e a.
+
+  \sa copy()
 */
 
 /*!
@@ -115,12 +160,21 @@ QPointArray::QPointArray( int nPoints, const QCOORD *points )
 
 /*!
   \fn QPointArray &QPointArray::operator=( const QPointArray &a )
-  Assigns a shallow copy of \e a to this point array and returns a reference
-  to this point array.
+  Assigns a 
+  \link shclass.html shallow copy\endlink of \e a to this point array
+  and returns a reference to this point array.
 
   Equivalent to assign( a ).
+
+  \sa copy()
 */
 
+/*!
+  \fn QPointArray QPointArray::copy() const
+
+  Creates a 
+  \link shclass.html deep copy\endlink of the array.
+*/
 
 /*!
   Fills the point array with the point \e p.
@@ -220,6 +274,12 @@ bool QPointArray::setPoints( int nPoints, const QCOORD *points )
     return TRUE;
 }
 
+/*!
+  \fn void QPointArray::setPoint( uint i, const QPoint &p )
+
+  Equivalent to setPoint( i, p.x(), p.y() ).
+*/
+
 /*!
   Resizes the array to \e nPoints and sets the points in the array to
   the values taken from the variable argument list.
@@ -351,6 +411,30 @@ QPoint QPointArray::at( uint index ) const
     return QPoint( (QCOORD)p.x, (QCOORD)p.y );
 }
 
+/*!
+  \fn QPointVal QPointArray::operator[]( int index )
+
+  Returns a reference to the point at position \e index in the array.
+*/
+
+/*!
+  \fn QPointVal QPointArray::operator[]( uint index )
+
+  Returns a reference to the point at position \e index in the array.
+*/
+
+/*!
+  \fn QPoint QPointArray::operator[] (int i) const
+
+  Returns the point at position \e index in the array.
+*/
+
+/*!
+  \fn QPoint QPointArray::operator[] (uint i) const
+
+  Returns the point at position \e index in the array.
+*/
+
 
 /*!
   Returns the bounding rectangle of the points in the array, or
@@ -390,6 +474,16 @@ static inline int fix_angle( int a )
     return a;
 }
 
+/*!
+  Sets the points of the array to those describing an arc of an
+  ellipse with size \a w by \a h and position (\a x, \a y ), starting
+  from angle \a1, spanning \a a2.
+  Angles are specified in 16ths of a degree,
+  i.e. a full circle equals 5760 (16*360). Positive values mean
+  counter-clockwise while negative values mean clockwise direction.
+  Zero degrees is at the 3'o clock position.
+*/
+
 void QPointArray::makeArc( int x, int y, int w, int h, int a1, int a2 )
 {
     a1 = fix_angle( a1 );
@@ -405,17 +499,10 @@ void QPointArray::makeArc( int x, int y, int w, int h, int a1, int a2 )
     if ( a2 > 0 ) {
 	j = 0;
 	inc = 1;
-    }
-    else {
+    } else {
 	j = npts - 1;
 	inc = -1;
     }
-#if 0
-    if ( a1 == 90*16 ) {
-	debug( "a1=%d, a2=%d, a3=%d, size=%d, npts=%d, i=%d",
-	       a1,a2,a3,size(),npts,i);
-    }
-#endif
     while ( npts-- ) {
 	if ( i >= (int)size() )			// wrap index
 	    i = 0;
@@ -427,7 +514,10 @@ void QPointArray::makeArc( int x, int y, int w, int h, int a1, int a2 )
     return;
 }
 
-
+/*!
+  Sets the points of the array to those describing an ellipse with
+  size \a w by \a h and position (\a x, \a y ).
+*/
 void QPointArray::makeEllipse( int xx, int yy, int w, int h )
 {						// midpoint, 1/4 ellipse
     if ( w <= 0 || h <= 0 ) {
@@ -449,11 +539,10 @@ void QPointArray::makeEllipse( int xx, int yy, int w, int h )
     int *py = new int[s];
     int x, y, i=0;
     double d1, d2;
-    double a=0.5*w, b=0.5*h;
-    double a2=a*a,  b2=b*b;
+    double a2=(w/2)*(w/2),  b2=(h/2)*(h/2);
     x = 0;
-    y = int(b);
-    d1 = b2 - a2*b + 0.25*a2;
+    y = int(h/2);
+    d1 = b2 - a2*(h/2) + 0.25*a2;
     px[i] = x;
     py[i] = y;
     i++;
@@ -461,8 +550,7 @@ void QPointArray::makeEllipse( int xx, int yy, int w, int h )
 	if ( d1 < 0 ) {
 	    d1 = d1 + b2*(3.0+2*x);
 	    x++;
-	}
-	else {
+	} else {
 	    d1 = d1 + b2*(3.0+2*x) + 2.0*a2*(1-y);
 	    x++;
 	    y--;
@@ -471,16 +559,13 @@ void QPointArray::makeEllipse( int xx, int yy, int w, int h )
 	py[i] = y;
 	i++;
     }
-    double t1 = x+0.5;
-    double t2 = y-1;
-    d2 = b2*t1*t1 + a2*t2*t2 - a2*b2;
+    d2 = b2*(x+0.5)*(x+0.5) + a2*(y-1)*(y-1) - a2*b2;
     while ( y > 0 ) {				// region 2
 	if ( d2 < 0 ) {
 	    d2 = d2 + 2.0*b2*(x+1) + a2*(3-2*y);
 	    x++;
 	    y--;
-	}
-	else {
+	} else {
 	    d2 = d2 + a2*(3-2*y);
 	    y--;
 	}
@@ -488,9 +573,6 @@ void QPointArray::makeEllipse( int xx, int yy, int w, int h )
 	py[i] = y;
 	i++;
     }
-    if ( i > s ) {
-	debug( "i > s!!!    i=%d,  w=%d, h=%d", i, w, h );
-    }
     s = i;
     resize( 4*s );				// make full point array
     xx += w/2;
@@ -508,12 +590,137 @@ void QPointArray::makeEllipse( int xx, int yy, int w, int h )
 }
 
 
+// Work functions for QPointArray::quadBezier()
+static
+void split(const double *p, double *l, double *r)
+{
+    double tmpx;
+    double tmpy;
+
+    l[0] =  p[0];
+    l[1] =  p[1];
+    r[6] =  p[6];
+    r[7] =  p[7];
+
+    l[2] = (p[0]+ p[2])/2;
+    l[3] = (p[1]+ p[3])/2;
+    tmpx = (p[2]+ p[4])/2;
+    tmpy = (p[3]+ p[5])/2;
+    r[4] = (p[4]+ p[6])/2;
+    r[5] = (p[5]+ p[7])/2;
+
+    l[4] = (l[2]+ tmpx)/2;
+    l[5] = (l[3]+ tmpy)/2;
+    r[2] = (tmpx + r[4])/2;
+    r[3] = (tmpy + r[5])/2;
+
+    l[6] = (l[4]+ r[2])/2;
+    l[7] = (l[5]+ r[3])/2;
+    r[0] = l[6];
+    r[1] = l[7];
+}
+// Based on:
+//
+//   A Fast 2D Point-On-Line Test
+//   by Alan Paeth
+//   from "Graphics Gems", Academic Press, 1990
+static
+int pnt_on_line( const double* p, const double* q, const double* t )
+{
+/*
+ * given a line through P:(px,py) Q:(qx,qy) and T:(tx,ty)
+ * return 0 if T is not on the line through      <--P--Q-->
+ *        1 if T is on the open ray ending at P: <--P
+ *        2 if T is on the closed interior along:   P--Q
+ *        3 if T is on the open ray beginning at Q:    Q-->
+ *
+ * Example: consider the line P = (3,2), Q = (17,7). A plot
+ * of the test points T(x,y) (with 0 mapped onto '.') yields:
+ *
+ *     8| . . . . . . . . . . . . . . . . . 3 3
+ *  Y  7| . . . . . . . . . . . . . . 2 2 Q 3 3    Q = 2
+ *     6| . . . . . . . . . . . 2 2 2 2 2 . . .
+ *  a  5| . . . . . . . . 2 2 2 2 2 2 . . . . .
+ *  x  4| . . . . . 2 2 2 2 2 2 . . . . . . . .
+ *  i  3| . . . 2 2 2 2 2 . . . . . . . . . . .
+ *  s  2| 1 1 P 2 2 . . . . . . . . . . . . . .    P = 2
+ *     1| 1 1 . . . . . . . . . . . . . . . . .
+ *      +--------------------------------------
+ *        1 2 3 4 5 X-axis 10        15      19
+ *
+ * Point-Line distance is normalized with the Infinity Norm
+ * avoiding square-root code and tightening the test vs the
+ * Manhattan Norm. All math is done on the field of integers.
+ * The latter replaces the initial ">= MAX(...)" test with
+ * "> (ABS(qx-px) + ABS(qy-py))" loosening both inequality
+ * and norm, yielding a broader target line for selection.
+ * The tightest test is employed here for best discrimination
+ * in merging collinear (to grid coordinates) vertex chains
+ * into a larger, spanning vectors within the Lemming editor.
+ */
+
+    if ( QABS((q[1]-p[1])*(t[0]-p[0])-(t[1]-p[1])*(q[0]-p[0])) >=
+        (QMAX(QABS(q[0]-p[0]), QABS(q[1]-p[1])))) return 0;
+
+    if (((q[0]<p[0])&&(p[0]<t[0])) || ((q[1]<p[1])&&(p[1]<t[1])))
+	return 1 ;
+    if (((t[0]<p[0])&&(p[0]<q[0])) || ((t[1]<p[1])&&(p[1]<q[1])))
+	return 1 ;
+    if (((p[0]<q[0])&&(q[0]<t[0])) || ((p[1]<q[1])&&(q[1]<t[1])))
+	return 3 ;
+    if (((t[0]<q[0])&&(q[0]<p[0])) || ((t[1]<q[1])&&(q[1]<p[1])))
+	return 3 ;
+
+    return 2 ;
+}
+static
+void polygonizeQBezier( double* acc, int& accsize, const double ctrl[],
+			int maxsize )
+{
+    if ( accsize > maxsize / 2 )
+    {
+	// This never happens in practice.
+
+	if ( accsize >= maxsize )
+	    return;
+	// Running out of space - approximate by a line.
+        acc[accsize++] = ctrl[0];
+	acc[accsize++] = ctrl[1];
+	acc[accsize++] = ctrl[6];
+	acc[accsize++] = ctrl[7];
+	return;
+    }
+
+    //intersects:
+    double l[8];
+    double r[8];
+    split( ctrl, l, r);
+
+    if ( pnt_on_line( &ctrl[0], &ctrl[6], &ctrl[2] ) == 2
+      && pnt_on_line( &ctrl[0], &ctrl[6], &ctrl[4] ) == 2 )
+    {
+	// Approximate by 2 lines.
+	acc[accsize++] = l[0];
+	acc[accsize++] = l[1];
+	acc[accsize++] = l[6];
+	acc[accsize++] = l[7];
+	acc[accsize++] = r[6];
+	acc[accsize++] = r[7];
+	return;
+    }
+
+    // Too big and too curved - recusively subdivide.
+    polygonizeQBezier( acc, accsize, l, maxsize );
+    polygonizeQBezier( acc, accsize, r, maxsize );
+}
+
 /*!
   Returns the Bezier points for the four control points in this array.
 */
 
 QPointArray QPointArray::quadBezier() const
 {
+#ifdef USE_SIMPLE_QBEZIER_CODE
     if ( size() != 4 ) {
 #if defined(CHECK_RANGE)
 	warning( "QPointArray::bezier: The array must have 4 control points" );
@@ -521,6 +728,7 @@ QPointArray QPointArray::quadBezier() const
 	QPointArray p;
 	return p;
     }
+
     int v;
     const int n = 3;				// n + 1 control points
     float xvec[4];
@@ -566,6 +774,41 @@ QPointArray QPointArray::quadBezier() const
     pd->y = (Qpnta_t)yvec[3];
 
     return p;
+#else
+
+    if ( size() != 4 ) {
+#if defined(CHECK_RANGE)
+	warning( "QPointArray::bezier: The array must have 4 control points" );
+#endif
+	QPointArray pa;
+	return pa;
+    } else {
+	QRect r = boundingRect();
+	int m = QMAX(r.width(),r.height());
+	double *p = new double[m];
+	double *ctrl = new double[8];
+	int i;
+	for (i=0; i<4; i++) {
+	    ctrl[i*2] = at(i).x();
+	    ctrl[i*2+1] = at(i).y();
+	}
+	int len=0;
+	polygonizeQBezier( p, len, ctrl, m );
+	QPointArray pa(len/2);
+	int j=0;
+	for (i=0; j<len; i++) {
+	    // Don't round - it looks terrible
+	    int x = int(p[j++]);
+	    int y = int(p[j++]);
+	    pa[i] = QPoint(x,y);
+	}
+	delete p;
+	delete ctrl;
+
+	return pa;
+    }
+
+#endif
 }
 
 
