@@ -1,37 +1,37 @@
 /****************************************************************************
-** $Id: qregion.cpp,v 2.15 1998/07/03 00:09:40 hanord Exp $
+** $Id: qregion.cpp,v 2.4 1997/10/09 13:10:56 agulbra Exp $
 **
 ** Implementation of QRegion class
 **
 ** Created : 950726
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qregion.h"
-#include "qpointarray.h"
+#include "qpntarry.h"
 #include "qbuffer.h"
-#include "qdatastream.h"
+#include "qdstream.h"
+
+RCSTAG("$Id: qregion.cpp,v 2.4 1997/10/09 13:10:56 agulbra Exp $");
+
 
 /*!
   \class QRegion qregion.h
   \brief The QRegion class specifies a clip region for a painter.
 
-  \ingroup drawing
-
   A region can be a rectangle, an ellipse, a polygon or a combination
   of these.
 
@@ -67,20 +67,6 @@
   QRegion member functions
  *****************************************************************************/
 
-/*!
-  Constructs a rectangular or elliptic region.
-
-  \a x, \a y, \a w, and \a h specify the region rectangle.
-  \a t is the region type: QRegion::Rectangle (default) or
-  QRegion::Ellipse.
-*/
-QRegion::QRegion( int x, int y, int w, int h, RegionType t )
-{
-    QRegion tmp(QRect(x,y,w,h),t);
-    tmp.data->ref();
-    data = tmp.data;
-}
-
 /*!
   Detaches from shared region data to makes sure that this region is the
   only one referring the data.
@@ -98,14 +84,46 @@ void QRegion::detach()
 
 
 /*!
-  Obsolete function.
+  Stores a region command in an internal buffer so it can be played back
+  later using exec().
+
+  All region commands (unite, intersect etc.) are stored in the buffer.
+  We need to do this because it is not possible to access the native
+  region data on most platforms. This seems to be the only way to support
+  regions for QPictures.  We have sacrified speed for compatibility.
 */
 
-void QRegion::cmd( int, void *, const QRegion *, const QRegion * )
+void QRegion::cmd( int id, void *param, const QRegion *r1, const QRegion *r2 )
 {
-#if defined(DEBUG)
-    debug( "QRegion::cmd: This is an obsolete function and should not be called" );
+    QBuffer buf( data->bop );
+    QDataStream s( &buf );
+    buf.open( IO_WriteOnly );
+    buf.at( buf.size() );
+    s << id;
+    switch ( id ) {
+	case QRGN_SETRECT:
+	case QRGN_SETELLIPSE:
+	    s << *((QRect*)param);
+	    break;
+	case QRGN_SETPTARRAY_ALT:
+	case QRGN_SETPTARRAY_WIND:
+	    s << *((QPointArray*)param);
+	    break;
+	case QRGN_TRANSLATE:
+	    s << *((QPoint*)param);
+	    break;
+	case QRGN_OR:
+	case QRGN_AND:
+	case QRGN_SUB:
+	case QRGN_XOR:
+	    s << r1->data->bop << r2->data->bop;
+	    break;
+#if defined(CHECK_RANGE)
+	default:
+	    warning( "QRegion: Internal cmd error" );
 #endif
+    }
+    buf.close();
 }
 
 
@@ -137,15 +155,21 @@ void QRegion::exec( const QByteArray &buffer )
 	    QRect r;
 	    s >> r;
 	    rgn = QRegion( r, id == QRGN_SETRECT ? Rectangle : Ellipse );
-	} else if ( id == QRGN_SETPTARRAY_ALT || id == QRGN_SETPTARRAY_WIND ) {
+	}
+	else if ( id == QRGN_SETPTARRAY_ALT || id == QRGN_SETPTARRAY_WIND ) {
 	    QPointArray a;
 	    s >> a;
 	    rgn = QRegion( a, id == QRGN_SETPTARRAY_WIND );
-	} else if ( id == QRGN_TRANSLATE ) {
+	}
+	else if ( id == QRGN_TRANSLATE ) {
 	    QPoint p;
 	    s >> p;
+#if defined(DEBUG)
+	    ASSERT( !rgn.data->bop.isNull() );
+#endif
 	    rgn.translate( p.x(), p.y() );
-	} else if ( id >= QRGN_OR && id <= QRGN_XOR ) {
+	}
+	else if ( id >= QRGN_OR && id <= QRGN_XOR ) {
 	    QByteArray bop1, bop2;
 	    QRegion r1, r2;
 	    s >> bop1;	r1.exec( bop1 );
@@ -164,14 +188,6 @@ void QRegion::exec( const QByteArray &buffer )
 		    rgn = r1.eor( r2 );
 		    break;
 	    }
-	} else if ( id == QRGN_RECTS ) {	// ### will appear in Qt 2.0
-	    Q_UINT32 n;
-	    s >> n;
-	    QRect r;
-	    for ( int i=0; i<(int)n; i++ ) {
-		s >> r;
-		rgn = rgn.unite( QRegion(r) );
-	    }
 	}
     }
     buf.close();
@@ -190,28 +206,7 @@ void QRegion::exec( const QByteArray &buffer )
 
 QDataStream &operator<<( QDataStream &s, const QRegion &r )
 {
-    QArray<QRect> a = r.rects();
-    if ( a.isEmpty() ) {
-	s << (Q_UINT32)0;
-    } else {
-#if QT_VERSION >= 200
-	s << (Q_UINT32)(4+4+8*a.size());
-	s << (int)QRGN_RECTS;
-	s << (Q_UINT32)a.size();
-	for ( int i=0; i<(int)a.size(); i++ )
-	    s << a[i];
-#else
-	int i;
-	for ( i=(int)a.size()-1; i>0; i-- ) {
-	    s << (Q_UINT32)(12+i*24);
-	    s << (int)QRGN_OR;
-	}
-	for ( i=0; i<(int)a.size(); i++ ) {
-	    s << (Q_UINT32)(4+8) << (int)QRGN_SETRECT << a[i];
-	}
-#endif
-    }
-    return s;
+    return s << r.data->bop;
 }
 
 /*!
@@ -229,7 +224,7 @@ QDataStream &operator>>( QDataStream &s, QRegion &r )
 
 
 // OBSOLETE
-#if !(defined(__STRICT_ANSI__) && defined(_CC_GNU_)) && !defined(_CC_EDG_) && !defined(_CC_HP_) && !defined(_CC_HP_ACC_) && !defined(_CC_USLC_) && !defined(xor)
+#if !(defined(__STRICT_ANSI__) && defined(_CC_GNU_)) && !defined(_CC_EDG_) && !defined(_CC_HP_) && !defined(_CC_HP_ACC_) && !defined(xor)
 /*!
   OBSOLETE - Use eor() instead.
 */
