@@ -1,36 +1,35 @@
 /****************************************************************************
-** $Id: qsignal.cpp,v 2.6 1998/07/03 00:09:40 hanord Exp $
+** $Id: qsignal.cpp,v 2.1 1997/08/06 15:05:46 hanord Exp $
 **
 ** Implementation of QSignal class
 **
 ** Created : 941201
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qsignal.h"
-#include "qmetaobject.h"
+#include "qmetaobj.h"
 #include <ctype.h>
 
+RCSTAG("$Id: qsignal.cpp,v 2.1 1997/08/06 15:05:46 hanord Exp $");
+
 /*!
   \class QSignal qsignal.h
   \brief The QSignal class can be used to send signals without parameters.
 
-  \ingroup kernel
-
   QSignal is a simple extension of QObject that can send plain signals
   without parameters.  If you want to send signals from a class that does
   not inherit QObject, you can create an internal QSignal object to emit
@@ -138,10 +137,6 @@ bool QSignal::connect( const QObject *receiver, const char *member )
 			     receiver, member );
 }
 
-#if (QT_VERSION >= 200)
-#error "disconnect const foo"
-#endif
-
 /*!
   Disonnects the signal from \e member in object \e receiver.
   \sa connect(), QObject::disconnect()
