@@ -1,39 +1,39 @@
 /****************************************************************************
-** $Id: qsize.cpp,v 2.10 1998/07/03 00:09:41 hanord Exp $
+** $Id: qsize.cpp,v 2.4 1997/08/27 13:39:55 agulbra Exp $
 **
 ** Implementation of QSize class
 **
 ** Created : 931028
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #define QSIZE_C
 #include "qsize.h"
-#include "qdatastream.h"
+#include "qdstream.h"
+
+RCSTAG("$Id: qsize.cpp,v 2.4 1997/08/27 13:39:55 agulbra Exp $");
+
 
 /*!
   \class QSize qsize.h
   \brief The QSize class defines the size of a two-dimensional object.
 
-  \ingroup drawing
-
   A size is specified by a width and a height.
 
-  The coordinate type is QCOORD (defined in qwindowdefs.h as \c short).
+  The coordinate type is QCOORD (defined in qwindefs.h as \c short).
   The minimum value of QCOORD is QCOORD_MIN (-32768) and the maximum
   value is  QCOORD_MAX (32767).
 
@@ -317,16 +317,3 @@ QDataStream &operator>>( QDataStream &s, QSize &sz )
     s >> h;  sz.rheight() = h;
     return s;
 }
-
-
-/*! \fn QSize QSize::expandedTo( const QSize & otherSize ) const
-  
-  Returns a size with the maximum width and height of this size and
-  \a otherSize.
-*/
-
-/*! \fn QSize QSize::boundedTo( const QSize & otherSize ) const
-  
-  Returns a size with the minimum width and height of this size and
-  \a otherSize.
-*/
