@@ -1,30 +1,31 @@
 /****************************************************************************
-** $Id: qtimer.cpp,v 2.10 1998/07/03 00:09:41 hanord Exp $
+** $Id: qtimer.cpp,v 2.4 1997/08/06 15:05:47 hanord Exp $
 **
 ** Implementation of QTimer class
 **
 ** Created : 931111
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qtimer.h"
 #include "qsignal.h"
-#include "qobjectlist.h"
-#include "qobjectdict.h"
+#include "qobjcoll.h"
+
+RCSTAG("$Id: qtimer.cpp,v 2.4 1997/08/06 15:05:47 hanord Exp $");
+
 
 /*!
   \class QTimer qtimer.h
@@ -51,12 +52,12 @@
 
   As a special case, a QTimer with timeout 0 times out as soon as all
   the events in the window system's event queue have been processed.
-  
+
   This can be used to do heavy work while providing a snappy
   user interface: \code
     QTimer *t = new QTimer( myObject );
     connect( t, SIGNAL(timeout()), SLOT(processOneThing()) );
-    t->start( 0, FALSE );
+    t->start( 0, TRUE );
   \endcode
 
   myObject->processOneThing() will be called repeatedly and should
@@ -67,16 +68,12 @@
   on more and more platforms and we expect that null events will
   eventually be replaced by threading.
 
-  Note that QTimer's accuracy depends on the underlying operating
-  system and hardware.  Most platforms support an accuracy of 20ms;
-  some provide more.  If Qt is unable to deliver the requested number
-  of timer clicks, it will silently discard some.
-
-  An alternative to using QTimer is to call QObject::startTimer() for
-  your object and reimplement the QObject::timerEvent() event handler
-  in your class (which must of course inherit QObject).  The
-  disadvantage is that timerEvent() does not support such high-level
-  features as single-shot timers or signals.
+  An alternative is to call QObject::startTimer() for your object and
+  reimplement the QObject::timerEvent() event handler in your class
+  (it must inherit QObject).  The advantage is that you can have
+  multiple running timers, each having a unique identifier.  The
+  disadvantage is that it does not support such high-level features as
+  single-shot timers or signals.
 */
 
 
@@ -93,6 +90,7 @@ const int INV_TIMER = -1;			// invalid timer id
 QTimer::QTimer( QObject *parent, const char *name )
     : QObject( parent, name )
 {
+    initMetaObject();
     id = INV_TIMER;
 }
 
@@ -255,7 +253,7 @@ bool QSingleShotTimer::event( QEvent * )
 
   Example:
   \code
-    #include <qapplication.h>
+    #include <qapp.h>
     #include <qtimer.h>
 
     int main( int argc, char **argv )
