@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qwid_x11.cpp,v 2.23 1997/04/01 17:07:45 hanord Exp $
+** $Id: qwid_x11.cpp,v 2.69 1997/10/08 16:56:23 warwick Exp $
 **
 ** Implementation of QWidget and QWindow classes for X11
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -21,15 +24,22 @@
 #include "qapp.h"
 #include "qpaintdc.h"
 #include "qpainter.h"
-#include "qpixmap.h"
+#include "qbitmap.h"
 #include "qwidcoll.h"
 #include "qobjcoll.h"
+#include "qaccel.h"
 #define	 GC GC_QQQ
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xos.h>
+#if !defined(XlibSpecificationRelease)
+#define X11R4
+typedef char *XPointer;
+#else
+#undef  X11R4
+#endif
 
-RCSTAG("$Id: qwid_x11.cpp,v 2.23 1997/04/01 17:07:45 hanord Exp $");
+RCSTAG("$Id: qwid_x11.cpp,v 2.69 1997/10/08 16:56:23 warwick Exp $");
 
 
 void qt_enter_modal( QWidget * );		// defined in qapp_x11.cpp
@@ -37,6 +47,8 @@ void qt_leave_modal( QWidget * );		// --- "" ---
 bool qt_modal_state();				// --- "" ---
 void qt_open_popup( QWidget * );		// --- "" ---
 void qt_close_popup( QWidget * );		// --- "" ---
+void qt_insert_sip( QWidget*, int, int );	// --- "" ---
+int  qt_sip_count( QWidget* );			// --- "" ---
 void qt_updated_rootinfo();
 
 
@@ -50,8 +62,11 @@ static QWidget *keyboardGrb = 0;
   QWidget member functions
  *****************************************************************************/
 
+#if QT_VERSION == 200
+#error "Make create and destroy virtual, remove the old functions."
+#endif
 
-extern Atom q_wm_delete_window;			// defined in qapp_x11.cpp
+extern Atom qt_wm_delete_window;		// defined in qapp_x11.cpp
 
 const uint stdWidgetEventMask =			// X event mask
 	KeyPressMask | KeyReleaseMask |
@@ -64,13 +79,43 @@ const uint stdWidgetEventMask =			// X event mask
 	StructureNotifyMask | SubstructureRedirectMask;
 
 
-/*!
-  \internal
-  Creates a new widget window if \e window is null, otherwise sets the
-  widget's window.
+/*
+  The qt_ functions below are implemented in qt_x11.cpp.
 */
 
-void QWidget::create( WId window )
+Window qt_XCreateWindow( const QWidget *creator,
+			 Display *display, Window parent,
+			 int x, int y, uint w, uint h,
+			 int borderwidth, int depth,
+			 uint windowclass, Visual *visual,
+			 ulong valuemask, XSetWindowAttributes *attributes );
+Window qt_XCreateSimpleWindow( const QWidget *creator,
+			       Display *display, Window parent,
+			       int x, int y, uint w, uint h, int borderwidth,
+			       ulong border, ulong background );
+void qt_XDestroyWindow( const QWidget *destroyer,
+			Display *display, Window window );
+
+
+
+/*!
+  Creates a new widget window if \a window is null, otherwise sets the
+  widget's window to \a window.
+
+  Initializes the window (sets the geometry etc.) if \a initializeWindow
+  is TRUE.  If \a initializeWindow is FALSE, no initialization is
+  performed.  This parameter makes only sense if \a window is a valid
+  window.
+
+  Destroys the old window if \a destroyOldWindow is TRUE.  If \a
+  destroyOldWindow is FALSE, you are responsible for destroying
+  the window yourself (using platform native code).
+
+  The QWidget constructor calls create(0,TRUE,TRUE) to create a window for
+  this widget.
+*/
+
+void QWidget::create( WId window, bool initializeWindow, bool destroyOldWindow)
 {
     if ( testWFlags(WState_Created) && window == 0 )
 	return;
@@ -90,6 +135,9 @@ void QWidget::create( WId window )
     Window parentw, destroyw = 0;
     WId	   id;
 
+    if ( !window )				// always initialize
+	initializeWindow = TRUE;
+
     if ( popup )				// a popup is a tool window
 	setWFlags(WStyle_Tool);
 
@@ -106,8 +154,8 @@ void QWidget::create( WId window )
     }
 
     if ( desktop ) {				// desktop widget
-	frect.setRect( 0, 0, sw, sh );
 	modal = popup = FALSE;			// force these flags off
+	frect.setRect( 0, 0, sw, sh );
     } else if ( topLevel ) {			// calc pos/size from screen
 	frect.setRect( sw/4, 3*sh/10, sw/2, 4*sh/10 );
     } else {					// child widget
@@ -115,15 +163,13 @@ void QWidget::create( WId window )
     }
     crect = frect;				// default client rect
 
-    if ( topLevel )
-	parentw = root_win;
-    else
-	parentw = parentWidget()->winId();
+    parentw = topLevel ? root_win : parentWidget()->winId();
 
-    XSetWindowAttributes wattr;
+    XSetWindowAttributes wsa;
 
     if ( window ) {				// override the old window
-	destroyw = winid;
+	if ( destroyOldWindow )
+	    destroyw = winid;
 	id = window;
 	setWinId( window );
     } else if ( desktop ) {			// desktop widget
@@ -138,57 +184,60 @@ void QWidget::create( WId window )
 	}
     } else {
 	if ( x11DefaultVisual() && x11DefaultColormap() ) {
-	    id = XCreateSimpleWindow( dpy, parentw,
-				      frect.left(), frect.top(),
-				      frect.width(), frect.height(),
-				      0,
-				      black.pixel(),
-				      bg_col.pixel() );
+	    id = qt_XCreateSimpleWindow( this, dpy, parentw,
+					 frect.left(), frect.top(),
+					 frect.width(), frect.height(),
+					 0,
+					 black.pixel(),
+					 bg_col.pixel() );
 	} else {
-	    wattr.background_pixel = bg_col.pixel();
-	    wattr.border_pixel = black.pixel();		
-	    wattr.colormap = (Colormap)x11Colormap();
-	    id = XCreateWindow( dpy, parentw,
-				frect.left(), frect.top(),
-				frect.width(), frect.height(),
-				0, x11Depth(), InputOutput,
-				(Visual*)x11Visual(),
-				CWBackPixel|CWBorderPixel|CWColormap,
-				&wattr );
+	    wsa.background_pixel = bg_col.pixel();
+	    wsa.border_pixel = black.pixel();		
+	    wsa.colormap = (Colormap)x11Colormap();
+	    id = qt_XCreateWindow( this, dpy, parentw,
+				   frect.left(), frect.top(),
+				   frect.width(), frect.height(),
+				   0, x11Depth(), InputOutput,
+				   (Visual*)x11Visual(),
+				   CWBackPixel|CWBorderPixel|CWColormap,
+				   &wsa );
 	}
 	setWinId( id );				// set widget id/handle + hd
     }
 
     if ( topLevel && !(desktop || popup || modal) ) {
 	if ( testWFlags(WStyle_Customize) ) {	// customize top-level widget
-	    ulong wattr_mask = 0;
+	    ulong wsa_mask = 0;
 	    if ( testWFlags(WStyle_NormalBorder) ) {
 		;				// ok, we already have it
 	    } else {
-		if ( testWFlags(WStyle_DialogBorder) ) {
+		if ( testWFlags(WStyle_DialogBorder) && initializeWindow ) {
 		    XSetTransientForHint( dpy, id, root_win );
 		} else {			// no border
-		    wattr.override_redirect = TRUE;
-		    wattr_mask |= CWOverrideRedirect;
+		    wsa.override_redirect = TRUE;
+		    wsa_mask |= CWOverrideRedirect;
 		}
 	    }
 	    if ( testWFlags(WStyle_Tool) ) {
-		wattr.save_under = TRUE;
-		wattr_mask |= CWSaveUnder;
+		wsa.save_under = TRUE;
+		wsa_mask |= CWSaveUnder;
 	    }
-	    if ( wattr_mask )
-		XChangeWindowAttributes( dpy, id, wattr_mask, &wattr );
+	    if ( wsa_mask && initializeWindow )
+		XChangeWindowAttributes( dpy, id, wsa_mask, &wsa );
 	} else {				// normal top-level widget
 	    setWFlags( WStyle_NormalBorder | WStyle_Title | WStyle_SysMenu |
 		       WStyle_MinMax );
 	}
     }
-    if ( popup ) {				// popup widget
+
+    if ( !initializeWindow ) {
+	// do no initialization
+    } else if ( popup ) {			// popup widget
 	XSetTransientForHint( dpy, id, parentw );
-	wattr.override_redirect = TRUE;
-	wattr.save_under = TRUE;
+	wsa.override_redirect = TRUE;
+	wsa.save_under = TRUE;
 	XChangeWindowAttributes( dpy, id, CWOverrideRedirect | CWSaveUnder,
-				 &wattr );
+				 &wsa );
     } else if ( topLevel && !desktop ) {	// top-level widget
 	if ( modal ) {
 	    QWidget *p = parentWidget();	// real parent
@@ -222,90 +271,157 @@ void QWidget::create( WId window )
 	XResizeWindow( dpy, id, crect.width(), crect.height() );
 	XStoreName( dpy, id, title );
 	Atom protocols[1];
-	protocols[0] = q_wm_delete_window;	// support del window protocol
+	protocols[0] = qt_wm_delete_window;	// support del window protocol
 	XSetWMProtocols( dpy, id, protocols, 1 );
     }
-    if ( testWFlags(WResizeNoErase) ) {
-	wattr.bit_gravity = NorthWestGravity;	// don't erase when resizing
-	XChangeWindowAttributes( dpy, id, CWBitGravity, &wattr );
+
+    if ( testWFlags(WResizeNoErase) && initializeWindow ) {
+	wsa.bit_gravity = NorthWestGravity;	// don't erase when resizing
+	XChangeWindowAttributes( dpy, id, CWBitGravity, &wsa );
     }
+
     setWFlags( WState_TrackMouse );
     setMouseTracking( FALSE );			// also sets event mask
     if ( desktop ) {
 	setWFlags( WState_Visible );
     } else if ( topLevel ) {			// set X cursor
 	QCursor *oc = QApplication::overrideCursor();
-	XDefineCursor( dpy, winid, oc ? oc->handle() : curs.handle() );
+	if ( initializeWindow )
+	    XDefineCursor( dpy, winid, oc ? oc->handle() : curs.handle() );
 	setWFlags( WCursorSet );
     }
+
+    if ( window ) {				// got window from outside
+	XWindowAttributes a;
+	XGetWindowAttributes( dpy, window, &a );
+	frect.setRect( a.x, a.y, a.width, a.height );
+	crect = frect;
+	if ( a.map_state == IsUnmapped )
+	    setWFlags( WState_Visible );
+	else
+	    clearWFlags( WState_Visible );
+    }
+
     if ( destroyw )
-	XDestroyWindow( dpy, destroyw );
+	qt_XDestroyWindow( this, dpy, destroyw );
+}
+
+
+#if QT_VERSION == 200
+#error "Cleanup here"
+//
+// Remove create(window) and create().  Return void.
+// Default arguments: (window=0, destroyW=TRUE)
+#endif
+
+
+/*!
+  \internal
+  Creates the widget's window. Equivalent with create(window,TRUE).
+  This function is usually called from the QWidget constructor.
+*/
+
+void QWidget::create( WId window )
+{
+    create( window, TRUE, TRUE );
 }
 
 
 /*!
   \internal
-  Creates the widget's window. Equivalent with create(0).
+  Creates the widget's window. Equivalent with create(0,TRUE).
   This function is usually called from the QWidget constructor.
 */
 
 bool QWidget::create()
 {
-    create( 0 );
+    create( 0, TRUE, TRUE );
     return TRUE;
 }
 
 
 /*!
-  \internal
-  Destroys the widget window and frees up window system resources.
-  Usually called from the QWidget destructor.
+  Frees up window system resources.
+  Destroys the widget window if \a destroyWindow is TRUE.  
+
+  destroy() calls itself recursively for all the child widgets,
+  passing \a destroySubWindows for the \a destroyWindow parameter.
+  To have more control over destruction of subwidgets,
+  destroy subwidgets selectively first.
+
+  This function is usually called from the QWidget destructor.
 */
 
-bool QWidget::destroy()
+void QWidget::destroy( bool destroyWindow, bool destroySubWindows )
 {
-    if ( qApp->focus_widget == this )
-	qApp->focus_widget = 0;			// reset focus widget
-    if ( parentWidget() && parentWidget()->focusChild == this )
-	parentWidget()->focusChild = 0;
     if ( testWFlags(WState_Created) ) {
 	clearWFlags( WState_Created );
-	focusChild = 0;
 	if ( children() ) {
 	    QObjectListIt it(*children());
 	    register QObject *obj;
 	    while ( (obj=it.current()) ) {	// destroy all widget children
 		++it;
 		if ( obj->isWidgetType() )
-		    ((QWidget*)obj)->destroy();
+		    ((QWidget*)obj)->destroy(destroySubWindows,
+					     destroySubWindows);
 	    }
 	}
 	if ( mouseGrb == this )
 	    releaseMouse();
 	if ( keyboardGrb == this )
 	    releaseKeyboard();
+	if ( extra && extra->xic ) {
+#if !defined(X11R4)
+	    XDestroyIC( (XIC)extra->xic );
+	    extra->xic = 0;
+#endif
+	}
 	if ( testWFlags(WType_Modal) )		// just be sure we leave modal
 	    qt_leave_modal( this );
 	else if ( testWFlags(WType_Popup) )
 	    qt_close_popup( this );
-	if ( !testWFlags(WType_Desktop) )
-	    XDestroyWindow( dpy, winid );
+	if ( destroyWindow && !testWFlags(WType_Desktop) )
+	    qt_XDestroyWindow( this, dpy, winid );
 	setWinId( 0 );
     }
-    return TRUE;
 }
 
 
 /*!
-  This function is provided in case a widget should feel \e really
-  bad, regret that it was even born.
+  \internal
+  Destroys the widget's window and frees up window system resources.
+  Equivalent with destroy(TRUE).
+  This function is usually called from the QWidget destructor.
+*/
 
-  It gives the widget a fresh start, new \e parent, new widget flags
-  (\e f, but as usual, use 0) at a new position in its new parent (\e p).
+bool QWidget::destroy()
+{
+    destroy( TRUE, TRUE );
+    return TRUE;
+}
 
-  If \e showIt is TRUE, show() is called once the widget has been
+
+#if QT_VERSION == 200
+#error "Rename recreate as reparent"
+#endif
+
+/*!
+  Reparents the widget.  The widget gets a new \a parent, new widget
+  flags (\a f, but as usual, use 0) at a new position in its new
+  parent (\a p).
+
+  If \a showIt is TRUE, show() is called once the widget has been
   recreated.
 
+  If the new parent widget is in a different top-level widget, the
+  reparented widget and its children are appended to the end of the
+  \link setFocusPolicy() TAB chain \endlink of the new parent widget,
+  in the same internal order as before.  If one of the moved widgets
+  had keyboard focus, recreate() calls clearFocus() for that widget.
+
+  If the new parent widget is in the same top-level widget as the old
+  parent, recreate doesn't change the TAB order or keyboard focus.
+
   \sa getWFlags()
 */
 
@@ -317,6 +433,9 @@ void QWidget::recreate( QWidget *parent, WFlags f, const QPoint &p,
     if ( testWFlags(WType_Desktop) )
 	old_winid = 0;
     setWinId( 0 );
+
+    reparentFocusWidgets( parent );		// fix focus chains
+
     if ( parentObj )				// remove from parent
 	parentObj->removeChild( this );
     if ( parent ) {				// insert into new parent
@@ -327,6 +446,7 @@ void QWidget::recreate( QWidget *parent, WFlags f, const QPoint &p,
     QSize    s	    = size();
     QPixmap *bgp    = (QPixmap *)backgroundPixmap();
     QColor   bgc    = bg_col;			// save colors
+    const char* capt= caption();
     flags = f;
     clearWFlags( WState_Created | WState_Visible );
     create();
@@ -350,10 +470,23 @@ void QWidget::recreate( QWidget *parent, WFlags f, const QPoint &p,
 	XSetWindowBackground( dpy, winid, bgc.pixel() );
     setGeometry( p.x(), p.y(), s.width(), s.height() );
     setEnabled( enable );
+    if ( capt ) {
+	extra->caption = 0;
+	setCaption( capt );
+    }
     if ( showIt )
 	show();
     if ( old_winid )
-	XDestroyWindow( dpy, old_winid );
+	qt_XDestroyWindow( this, dpy, old_winid );
+
+    QObjectList	*accelerators = queryList( "QAccel" );
+    QObjectListIt it( *accelerators );
+    QObject *obj;
+    while ( (obj=it.current()) != 0 ) {
+	++it;
+	((QAccel*)obj)->repairEventFilter();
+    }
+    delete accelerators;
 }
 
 
@@ -387,38 +520,21 @@ QPoint QWidget::mapFromGlobal( const QPoint &pos ) const
 
 
 /*!
-  Sets the background color of this widget.
+  This function is deprecated.  Use setBackgroundMode() or setPalette(),
+  as they ensure the appropriate clearing color is used when the widget
+  is in the Active, Normal, or Disabled state.
 
-  The background color is semi-independent of the widget color group.
-  Setting a new palette overwrites the background color, but setting
-  the background color does not change the palette in any way.
-
-  The window system clears the widget to the background color just before
-  sending a paint event. This generally causes flicker if the widget does
-  not use colorGroup().background() as background color.  A widget which
-  uses colorGroup().base() as background color can avoid flicker by doing:
-
-  \code
-    setBackgroundColor( colorGroup().base() );
-  \endcode
-
-  If you want to change the color scheme of a widget, the setPalette()
-  function is better suited.  Here is how to set \e thatWidget to use a
-  light green (RGB value 80, 255, 80) as background color, with shades
-  of green used for all the 3D effects:
-
-  \code
-    thatWidget->setPalette( QPalette( QColor(80, 255, 80) ) );
-  \endcode
-
-  You can also use QApplication::setPalette() if you want to change
-  the color scheme of your entire application, or of all new widgets.
-  
-  \sa backgroundColor(), backgroundColorChange(), setPalette(),
-  setBackgroundPixmap(), QApplication::setPalette()
+  \sa setPalette(), QApplication::setPalette(), backgroundColor(),
+      setBackgroundPixmap(), setBackgroundMode()
 */
 
 void QWidget::setBackgroundColor( const QColor &color )
+{
+    setBackgroundModeDirect( FixedColor );
+    setBackgroundColorDirect( color );
+}
+
+void QWidget::setBackgroundColorDirect( const QColor &color )
 {
     QColor old = bg_col;
     bg_col = color;
@@ -430,6 +546,8 @@ void QWidget::setBackgroundColor( const QColor &color )
     backgroundColorChange( old );
 }
 
+static int allow_null_pixmaps = 0;
+
 /*!
   Sets the background pixmap of the widget to \e pixmap.
 
@@ -437,6 +555,9 @@ void QWidget::setBackgroundColor( const QColor &color )
   not work well with a background pixmap.
 
   \sa backgroundPixmap(), backgroundPixmapChange(), setBackgroundColor()
+
+  \internal
+  This function is call with a null pixmap by setBackgroundEmpty().
 */
 
 void QWidget::setBackgroundPixmap( const QPixmap &pixmap )
@@ -444,7 +565,7 @@ void QWidget::setBackgroundPixmap( const QPixmap &pixmap )
     QPixmap old;
     if ( extra && extra->bg_pix )
 	old = *extra->bg_pix;
-    if ( pixmap.isNull() ) {
+    if ( !allow_null_pixmaps && pixmap.isNull() ) {
 	XSetWindowBackground( dpy, winid, bg_col.pixel() );
 	if ( extra && extra->bg_pix ) {
 	    delete extra->bg_pix;
@@ -452,9 +573,11 @@ void QWidget::setBackgroundPixmap( const QPixmap &pixmap )
 	}
     } else {
 	QPixmap pm = pixmap;
-	if ( pm.depth() == 1 && QPixmap::defaultDepth() > 1 ) {
-	    pm = QPixmap( pixmap.size() );
-	    bitBlt( &pm, 0, 0, &pixmap, 0, 0, pm.width(), pm.height() );
+	if (!pm.isNull()) {
+	    if ( pm.depth() == 1 && QPixmap::defaultDepth() > 1 ) {
+		pm = QPixmap( pixmap.size() );
+		bitBlt( &pm, 0, 0, &pixmap, 0, 0, pm.width(), pm.height() );
+	    }
 	}
 	if ( extra && extra->bg_pix )
 	    delete extra->bg_pix;
@@ -465,7 +588,29 @@ void QWidget::setBackgroundPixmap( const QPixmap &pixmap )
 	if ( testWFlags(WType_Desktop) )	// save rootinfo later
 	    qt_updated_rootinfo();
     }
-    backgroundPixmapChange( old );
+    if ( !allow_null_pixmaps ) {
+	setBackgroundModeDirect( FixedPixmap );
+	backgroundPixmapChange( old );
+    }
+}
+
+
+/*!
+  Sets the window-system background of the widget to nothing.
+
+  Note that `nothing' is actually a pixmap that isNull(), thus you
+  can check for an empty background by checking backgroundPixmap().
+
+  \sa setBackgroundPixmap(), setBackgroundColor()
+
+  This class should \e NOT be made virtual - it is an alternate usage
+  of setBackgroundPixmap().
+*/
+void QWidget::setBackgroundEmpty()
+{
+    allow_null_pixmaps++;
+    setBackgroundPixmap(QPixmap());
+    allow_null_pixmaps--;
 }
 
 
@@ -500,6 +645,9 @@ void QWidget::setCursor( const QCursor &cursor )
 
 void QWidget::setCaption( const char *caption )
 {
+    if ( caption && extra && extra->caption &&
+	 !strcmp( extra->caption, caption ) )
+	return; // for less flicker
     if ( extra )
 	delete [] extra->caption;
     else
@@ -515,18 +663,40 @@ void QWidget::setCaption( const char *caption )
 
 void QWidget::setIcon( const QPixmap &pixmap )
 {
-    if ( extra )
+    if ( extra ) {
 	delete extra->icon;
-    else
+	extra->icon = 0;
+    } else {
 	createExtra();
-    extra->icon = new QPixmap( pixmap );
-    XWMHints wm_hints;				// window manager hints
-    wm_hints.input = True;
-    wm_hints.icon_pixmap = extra->icon->handle();
-    wm_hints.flags = IconPixmapHint;
-    XSetWMHints( dpy, winId(), &wm_hints );
+    }
+    Pixmap icon_pixmap;
+    Pixmap mask_pixmap;
+    QBitmap mask;
+    if ( pixmap.isNull() ) {
+	icon_pixmap = 0;
+	mask_pixmap = 0;
+    } else {
+	extra->icon = new QPixmap( pixmap );
+	icon_pixmap = pixmap.handle();
+	mask = pixmap.mask() ? *pixmap.mask() : pixmap.createHeuristicMask();
+	mask_pixmap = mask.handle();
+    }
+    XWMHints *h = XGetWMHints( dpy, winId() );    
+    XWMHints  wm_hints;
+    bool got_hints = h != 0;     
+    if ( !got_hints ) {
+	h = &wm_hints;
+	h->flags = 0;
+    }
+    h->icon_pixmap = icon_pixmap;
+    h->icon_mask   = mask_pixmap;
+    h->flags |= IconPixmapHint | IconMaskHint;
+    XSetWMHints( dpy, winId(), h );
+    if ( got_hints )
+	XFree( h );
 }
 
+
 /*!
   Sets the text of the window's icon to \e iconText.
   \sa iconText(), setIcon(), setCaption()
@@ -545,16 +715,14 @@ void QWidget::setIconText( const char *iconText )
 
 void QWidget::setMouseTracking( bool enable )
 {
-    if ( enable == testWFlags(WState_TrackMouse) )
+    bool gmt = QApplication::hasGlobalMouseTracking();
+    if ( enable == testWFlags(WState_TrackMouse) && !gmt )
 	return;
-    uint m;
-    if ( enable ) {
-	m = PointerMotionMask;
+    uint m = (enable || gmt) ? PointerMotionMask : 0;
+    if ( enable )
 	setWFlags( WState_TrackMouse );
-    } else {
-	m = 0;
+    else
 	clearWFlags( WState_TrackMouse );
-    }
     if ( testWFlags(WType_Desktop) ) {		// desktop widget?
 	if ( testWFlags(WPaintDesktop) )	// get desktop paint events
 	    XSelectInput( dpy, winid, ExposureMask );
@@ -588,7 +756,7 @@ void QWidget::grabMouse()
 	    mouseGrb->releaseMouse();
 	XGrabPointer( dpy, winid, TRUE,
 		      ButtonPressMask | ButtonReleaseMask |
-		      ButtonMotionMask | EnterWindowMask | LeaveWindowMask,
+		      PointerMotionMask | EnterWindowMask | LeaveWindowMask,
 		      GrabModeAsync, GrabModeAsync,
 		      None, None, CurrentTime );
 	mouseGrb = this;
@@ -614,8 +782,7 @@ void QWidget::grabMouse( const QCursor &cursor )
 	    mouseGrb->releaseMouse();
 	XGrabPointer( dpy, winid, TRUE,
 		      ButtonPressMask | ButtonReleaseMask |
-		      ButtonMotionMask |
-		      EnterWindowMask | LeaveWindowMask,
+		      PointerMotionMask | EnterWindowMask | LeaveWindowMask,
 		      GrabModeAsync, GrabModeAsync,
 		      None, cursor.handle(), CurrentTime );
 	mouseGrb = this;
@@ -785,8 +952,12 @@ void QWidget::update( int x, int y, int w, int h )
     }
 }
 
+/*!
+  \overload void QWidget::update( const QRect &r )
+*/
 
-/*! \overload void QWidget::repaint( bool erase )
+/*!
+  \overload void QWidget::repaint( bool erase )
 
   This version repaints the entire widget.
 */
@@ -831,79 +1002,25 @@ void QWidget::repaint( int x, int y, int w, int h, bool erase )
 
 
 /*!
-  Shows the widget and its child widgets.
-
-  If its size or position has changed, Qt guarantees that a widget gets
-  move and resize events just before the widget is shown.
-
-  \sa hide(), iconify(), isVisible()
+  \internal
+  Platform-specific part of QWidget::show().
 */
 
-void QWidget::show()
+void QWidget::showWindow()
 {
-    if ( testWFlags(WState_Visible) )
-	return;
-    if ( extra ) {
-	int w = crect.width();
-	int h = crect.height();
-	if ( w < extra->minw || h < extra->minh ||
-	     w > extra->maxw || h > extra->maxh ) {
-	    w = QMAX( extra->minw, QMIN( w, extra->maxw ));
-	    h = QMAX( extra->minh, QMIN( h, extra->maxh ));
-	    resize( w, h );			// deferred resize
-	}
-    }
-    sendDeferredEvents();
-    if ( children() ) {
-	QObjectListIt it(*children());
-	register QObject *object;
-	QWidget *widget;
-	while ( it ) {				// show all widget children
-	    object = it.current();		//   (except popups)
-	    ++it;
-	    if ( object->isWidgetType() ) {
-		widget = (QWidget*)object;
-		if ( !widget->testWFlags(WState_DoHide) )
-		    widget->show();
-	    }
-	}
-    }
-    if ( testWFlags(WType_Popup) )
-	raise();
-    XMapWindow( dpy, winid );
-    setWFlags( WState_Visible );
-    clearWFlags( WState_DoHide );
-    if ( testWFlags(WType_Modal) )
-	qt_enter_modal( this );
-    else if ( testWFlags(WType_Popup) )
-	qt_open_popup( this );
+    XMapWindow( dpy, winId() );
 }
 
 
 /*!
-  Hides the widget.
-  \sa show(), iconify(), isVisible()
+  \internal
+  Platform-specific part of QWidget::hide().
 */
 
-void QWidget::hide()
+void QWidget::hideWindow()
 {
-    if ( testWFlags(WFocusSet) || focusChild ) {
-	QWidget *w = this;
-	while ( w->focusChild )			// descend focus chain
-	    w = w->focusChild;
-	w->clearFocus();
-    }
-    setWFlags( WState_DoHide );
-    if ( !testWFlags(WState_Visible) )
-	return;
-    if ( testWFlags(WType_Modal) )
-	qt_leave_modal( this );
-    else if ( testWFlags(WType_Popup) )
-	qt_close_popup( this );
-    XUnmapWindow( dpy, winid );
-    clearWFlags( WState_Visible );
-    cancelMove();
-    cancelResize();
+    XUnmapWindow( dpy, winId() );
+    if ( isPopup() ) XFlush( dpy );
 }
 
 
@@ -927,7 +1044,7 @@ void QWidget::iconify()
   Raises this widget to the top of the parent widget's stack.
 
   If there are any siblings of this widget that overlap it on the screen,
-  this widget will be in front of its siblings afterwards.
+  this widget will be visually in front of its siblings afterwards.
 
   \sa lower()
 */
@@ -936,7 +1053,7 @@ void QWidget::raise()
 {
     QWidget *p = parentWidget();
     if ( p && p->childObjects && p->childObjects->findRef(this) >= 0 )
-	p->childObjects->insert( 0, p->childObjects->take() );
+	p->childObjects->append( p->childObjects->take() );
     XRaiseWindow( dpy, winid );
 }
 
@@ -953,7 +1070,7 @@ void QWidget::lower()
 {
     QWidget *p = parentWidget();
     if ( p && p->childObjects && p->childObjects->findRef(this) >= 0 )
-	p->childObjects->append( p->childObjects->take() );
+	p->childObjects->insert( 0, p->childObjects->take() );
     XLowerWindow( dpy, winid );
 }
 
@@ -1216,7 +1333,7 @@ void QWidget::setMinimumSize( int w, int h )
     extra->minh = h;
     int minw = QMAX(w,crect.width());
     int minh = QMAX(h,crect.height());
-    if ( isVisible() && (minw > w || minh > h) )
+    if ( minw > w || minh > h )
 	resize( minw, minh );
     if ( testWFlags(WType_TopLevel) ) {
 	XSizeHints size_hints;
@@ -1251,7 +1368,7 @@ void QWidget::setMaximumSize( int w, int h )
     extra->maxh = h;
     int maxw = QMIN(w,crect.width());
     int maxh = QMIN(h,crect.height());
-    if ( isVisible() && (maxw < w || maxh < h) )
+    if ( maxw < w || maxh < h )
 	resize( maxw, maxh );
     if ( testWFlags(WType_TopLevel) ) {
 	XSizeHints size_hints;
@@ -1356,8 +1473,13 @@ void QWidget::scroll( int dx, int dy )
 	y2 = 0;
 	h += dy;
     }
+
+    if ( dx == 0 && dy == 0 )
+	return;
+
     XCopyArea( dpy, winid, winid, qt_xget_readonly_gc(), x1, y1, w, h, x2, y2);
-    if ( children() ) {				// scroll children
+
+    if ( children() ) {	// scroll children
 	QPoint pd( dx, dy );
 	QObjectListIt it(*children());
 	register QObject *object;
@@ -1370,14 +1492,26 @@ void QWidget::scroll( int dx, int dy )
 	    ++it;
 	}
     }
+
+    // Don't let the server be bogged-down with repaint events
+    bool repaint_immediately = qt_sip_count( this ) < 3;
+
     if ( dx ) {
 	x1 = x2 == 0 ? w : 0;
-	XClearArea( dpy, winid, x1, 0, crect.width()-w, crect.height(), TRUE);
+	if ( repaint_immediately )
+	    repaint( x1, 0, crect.width()-w, crect.height(), TRUE );
+	else
+	    XClearArea( dpy, winid, x1, 0, crect.width()-w, crect.height(), TRUE);
     }
     if ( dy ) {
 	y1 = y2 == 0 ? h : 0;
-	XClearArea( dpy, winid, 0, y1, crect.width(), crect.height()-h, TRUE);
+	if ( repaint_immediately )
+	    repaint( 0, y1, crect.width(), crect.height()-h, TRUE );
+	else
+	    XClearArea( dpy, winid, 0, y1, crect.width(), crect.height()-h, TRUE);
     }
+
+    qt_insert_sip( this, dx, dy );
 }
 
 
@@ -1389,7 +1523,7 @@ void QWidget::scroll( int dx, int dy )
   Writes \e str at position \e x,y.
 
   The \e y position is the base line position of the text.  The text is
-  drawn using the current font and the current foreground color.
+  drawn using the default font and the default foreground color.
 
   This function is provided for convenience.  You will generally get
   more flexible results and often higher speed by using a a \link
