@@ -1,36 +1,31 @@
 /****************************************************************************
-** $Id: qwidget.cpp,v 2.117 1998/07/03 00:09:41 hanord Exp $
+** $Id: qwidget.cpp,v 2.80 1997/10/06 12:57:22 agulbra Exp $
 **
 ** Implementation of QWidget class
 **
 ** Created : 931031
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
-#include "qobjectlist.h"
-#include "qobjectdict.h"
+#include "qobjcoll.h"
 #include "qwidget.h"
-#include "qwidgetlist.h"
-#include "qwidgetintdict.h"
-#include "qptrdict.h"
-#include "qfocusdata.h"
+#include "qwidcoll.h"
 #include "qpixmap.h"
 #include "qkeycode.h"
-#include "qapplication.h"
+#include "qapp.h"
 #include "qbrush.h"
 #if defined(_WS_WIN_)
 #if defined(_CC_BOOL_DEF_)
@@ -42,6 +37,9 @@
 #endif
 #endif
 
+RCSTAG("$Id: qwidget.cpp,v 2.80 1997/10/06 12:57:22 agulbra Exp $");
+
+
 /*!
   \class QWidget qwidget.h
   \brief The QWidget class is the base class of all user interface objects.
@@ -119,16 +117,12 @@
 	setMaximumSize(),
 	setMinimumSize(),
 	setSizeIncrement(),
-	setFixedSize()
+	setFixedSize().
 
   <li> Mode:
-	isVisible(),
-	isVisibleTo(),
-	isVisibleToTLW(),
+	isVisible()
 	isDesktop(),
 	isEnabled(),
-	isEnabledTo(),
-	isEnabledToTLW(),
 	isModal(),
 	isPopup(),
 	isTopLevel(),
@@ -136,11 +130,7 @@
 	hasMouseTracking(),
 	setMouseTracking(),
 	isUpdatesEnabled(),
-	setUpdatesEnabled(),
-	setFontPropagation(),
-	fontPropagation(),
-	setPalettePropagation(),
-	palettePropagation().
+	setUpdatesEnabled().
 
   <li> Look and feel:
 	style(),
@@ -347,14 +337,12 @@
 
   When writing a widget, there are a few more things to look out for.
   In the constructor, be sure to set up your member variables early
-  on, before there's any chance that you might receive an event.
-
-  It is often a good idea to reimplement sizeHint(), so users of your
-  class can set up layout management more easily.  If you do, consider
-  offering size management using autoMinimumSize() too.
-
-  If your widget is a top-level window, setCaption() and setIcon() set
-  the title bar and icon respectively.
+  on, before there's any chance that you might receive an event.  You
+  may want to call setMinimumSize() and perhaps setMaximumSize() or
+  setSizeIncrement() to ensure that your widget will not be resized
+  ridiculously (but neither give you any guarantee, so write
+  carefully!)  If your widget is a top-level window, setCaption() and
+  setIcon() set the title bar and icon respectively.
 
   \sa QEvent, QPainter, QGridLayout, QBoxLayout
 */
@@ -429,11 +417,22 @@ inline bool QWidgetMapper::remove( WId id )
 }
 
 
+// Focus data class - documented in QWidget::focusData()
+
+class QFocusData {
+public:
+    QFocusData()
+	: it(focusWidgets) {}
+    QList<QWidget> focusWidgets;
+    QListIterator<QWidget> it;
+};
+
+
 /*****************************************************************************
   QWidget member functions
  *****************************************************************************/
 
-typedef Q_DECLARE(QPtrDictM,void) QDeferDict;
+typedef Q_DECLARE(QIntDictM,int) QDeferDict;
 
 static QDeferDict *deferredMoves   = 0;
 static QDeferDict *deferredResizes = 0;
@@ -479,10 +478,10 @@ static inline short decompress_b( uint n )
 
 void QWidget::deferMove( const QPoint &oldPos )
 {
-    uint n = (uint)(long)deferredMoves->find( this );
+    uint n = (uint)deferredMoves->find( (long)this );
     if ( !n ) {
 	n = compress(oldPos.x(),oldPos.y());
-	deferredMoves->insert( this, (void*)n );
+	deferredMoves->insert( (long)this, (int *)n );
     }
 }
 
@@ -490,43 +489,42 @@ void QWidget::deferResize( const QSize &oldSize )
 {
     int w = oldSize.width();
     int h = oldSize.height();
-    uint n = (uint)(long)deferredResizes->find( this );
+    uint n = (uint)deferredResizes->find( (long)this );
     if ( n ) {
 	if ( w < 0 && decompress_a(n) > 0 ) {	// did setGeometry
-	    deferredResizes->take( this );
+	    deferredResizes->take( (long)this );
 	    n = compress(-decompress_a(n),-decompress_b(n));
-	    deferredResizes->insert( this, (void*)n );
+	    deferredResizes->insert( (long)this, (int *)n );
 	}
     } else {
 	n = compress(w,h);
-	deferredResizes->insert( this, (void*)n );
+	deferredResizes->insert( (long)this, (int *)n );
     }
 }
 
 
 void QWidget::cancelMove()
 {
-    deferredMoves->take( this );
+    deferredMoves->take( (long)this );
 }
 
 void QWidget::cancelResize()
 {
-    deferredResizes->take( this );
+    deferredResizes->take( (long)this );
 }
 
 
 /*!
-  Send deferred or enforced move, resize and child events for this widget.
+  Send deferred or enforced move and resize events for this widget.
 */
 
 void QWidget::sendDeferredEvents()
 {
-    QApplication::sendPostedEvents( this, Event_ChildInserted );
-    uint m = (uint)(long)deferredMoves->find(this);
-    uint r = (uint)(long)deferredResizes->find(this);
+    uint m = (uint)deferredMoves->find((long)this);
+    uint r = (uint)deferredResizes->find((long)this);
     if ( m && r && decompress_a(r) < 0 ) {
-	// Hack it to work: the old width is negative to indicate that
-	// we wanted to setGeometry and not move + resize.
+	// Hack: the old width is negative to indicate that we wanted
+	// to setGeometry and not move + resize.
 	internalSetGeometry( x(), y(), width(), height() );
     } else {
 	if ( m )
@@ -535,12 +533,12 @@ void QWidget::sendDeferredEvents()
 	    internalResize( width(), height() );
     }
     if ( m ) {
-	deferredMoves->take( this );
+	deferredMoves->take( (long)this );
 	QMoveEvent e( pos(), QPoint(decompress_a(m), decompress_b(m)) );
 	QApplication::sendEvent( this, &e );
     }
     if ( r ) {
-	deferredResizes->take( this );
+	deferredResizes->take( (long)this );
 	int w = decompress_a(r);
 	int h = decompress_b(r);
 	QResizeEvent e( size(), QSize(QABS(w),QABS(h)) );
@@ -548,6 +546,7 @@ void QWidget::sendDeferredEvents()
     }
 }
 
+
 /*!
   Constructs a widget which is a child of \e parent, with the name \e name and
   widget flags set to \e f.
@@ -596,15 +595,15 @@ void QWidget::sendDeferredEvents()
 				  WStyle_Tool );
   \endcode
 
-  The widget flags are defined in qwindowdefs.h (which is included by
+  The widget flags are defined in qwindefs.h (which is included by
   qwidget.h).
 */
 
 QWidget::QWidget( QWidget *parent, const char *name, WFlags f )
     : QObject( parent, name ), QPaintDevice( PDT_WIDGET ),
-      pal( parent ? parent->palette()		// use parent's palette
-           : *qApp->palette() )			// use application palette
+      pal( *qApp->palette() )			// use application palette
 {
+    initMetaObject();				// initialize meta object
     isWidget = TRUE;				// is a widget
     winid = 0;					// default attributes
     flags = f;
@@ -632,10 +631,6 @@ QWidget::QWidget( QWidget *parent, const char *name, WFlags f )
 	    }
 	}
     }
-    if ( parent ) {
-	QChildEvent *e = new QChildEvent( Event_ChildInserted, this );
-	QApplication::postEvent( parent, e );
-    }
 }
 
 
@@ -648,24 +643,12 @@ QWidget::QWidget( QWidget *parent, const char *name, WFlags f )
 
 QWidget::~QWidget()
 {
-    // remove myself from the can-take-focus list
-    QFocusData *f = focusData();
-    if ( f )
-	f->focusWidgets.removeRef( this );
-
-    if ( parentObj ) {
-	QChildEvent e( Event_ChildRemoved, this );
-	QApplication::sendEvent( parentObj, &e );
-    }
     if ( deferredMoves ) {
-	deferredMoves->take( this );	// clean deferred move/resize
-	deferredResizes->take( this );
+	deferredMoves->take( (long)this );	// clean deferred move/resize
+	deferredResizes->take( (long)this );
     }
-    if ( QApplication::main_widget == this ) {	// reset main widget
+    if ( QApplication::main_widget == this )	// reset main widget
 	QApplication::main_widget = 0;
-	if (qApp)
-	    qApp->quit();
-    }
     if ( focusWidget() == this )
 	clearFocus();
     if ( testWFlags(WExportFontMetrics) )	// remove references to this
@@ -682,6 +665,7 @@ QWidget::~QWidget()
 	QObject *obj;
 	while ( (obj=it.current()) ) {
 	    ++it;
+	    obj->parentObj = 0;
 	    delete obj;
 	    if ( !childObjects )		// removeChild resets it
 		break;
@@ -694,10 +678,14 @@ QWidget::~QWidget()
     // better to give focus to someone else
     if ( qApp->focus_widget == this )
 	qApp->focus_widget = 0;
+    // remove myself from the can-take-focus list
+    QFocusData *f = focusData();
+    if ( f )
+	f->focusWidgets.removeRef( this );
 
+    destroy();					// platform-dependent cleanup
     if ( extra )
 	deleteExtra();
-    destroy();					// platform-dependent cleanup
 }
 
 
@@ -735,10 +723,10 @@ void QWidget::destroyMapper()
     ASSERT( it.count() == 0 );
 #endif
 #if 0
-    it.toFirst();				// shouln't be any more widgets
-    while ( (w=it.current()) ) {		// delete the rest
-	++it;
+    w = it.toFirst();				// shouln't be any more widgets
+    while ( w ) {				// delete the rest
 	delete w;
+	w = ++it;
     }
 #endif
     delete mapper;
@@ -826,11 +814,14 @@ void QWidget::createExtra()
 	extra->incw = extra->inch = 0;
 	extra->caption = extra->iconText = 0;
 	extra->icon = extra->bg_pix = 0;
+#if defined(_WS_WIN_)
+	extra->winIcon = 0;
+#endif
 	extra->bg_mode = PaletteBackground;
 	extra->focusData = 0;
-	extra->propagateFont = 0;
-	extra->propagatePalette = 0;
-	createSysExtra();
+#if defined(_WS_X11_)
+	extra->xic = 0;
+#endif
     }
 }
 
@@ -842,14 +833,15 @@ void QWidget::createExtra()
 void QWidget::deleteExtra()
 {
     if ( extra ) {				// if exists
-	if ( extra->caption )			// Avoid purify complaint.
-	    delete [] extra->caption;
-	if ( extra->iconText )			// Avoid purify complaint.
-	    delete [] extra->iconText;
+	delete [] extra->caption;
+	delete [] extra->iconText;
 	delete extra->icon;
+#if defined(_WS_WIN_)
+	if ( extra->winIcon )
+	    DestroyIcon( extra->winIcon );
+#endif
 	delete extra->bg_pix;
 	delete extra->focusData;
-	deleteSysExtra();
 	delete extra;
 	// extra->xic destroyed in QWidget::destroy()
 	extra = 0;
@@ -861,7 +853,7 @@ void QWidget::deleteExtra()
   Returns a pointer to the widget with window identifer/handle \e id.
 
   The window identifier type depends by the underlying window system,
-  see qwindowdefs.h for the actual definition.
+  see qwindefs.h for the actual definition.
   If there is no widget with this identifier, a null pointer is returned.
 
   \sa wmapper(), id()
@@ -935,7 +927,7 @@ GUIStyle QWidget::style() const
 
 /*!
   Sets the GUI style for this widget.  The valid values are listed
-  in qwindowdefs.h.
+  in qwindefs.h.
 
   \sa style(), styleChange(), QApplication::setStyle()
 */
@@ -1022,43 +1014,6 @@ void QWidget::styleChange( GUIStyle )
   \sa setEnabled()
 */
 
-
-/*!
-  Returns TRUE if this widget and every parent up to but excluding
-  \a ancestor is enabled, otherwise returns FALSE.
-
-  \sa setEnabled() isEnabled()
-*/
-
-bool QWidget::isEnabledTo(QWidget* ancestor) const
-{
-    const QWidget * w = this;
-    while ( w && w->isEnabled()
-	    && !w->isTopLevel()
-	    && w->parentWidget()
-	    && w->parentWidget()!=ancestor
-	)
-	w = w->parentWidget();
-    return w->isEnabled();
-}
-
-
-/*!
-  Returns TRUE if this widget and every parent up to the \link
-  topLevelWidget() top level widget \endlink is enabled, otherwise
-  returns FALSE.
-
-  This is equivalent to isEnabledTo(0).
-
-  \sa setEnabled() isEnabled()
-*/
-
-bool QWidget::isEnabledToTLW() const
-{
-    return isEnabledTo(0);
-}
-
-
 /*!
   Enables widget input events if \e enable is TRUE, otherwise disables
   input events.
@@ -1104,7 +1059,7 @@ void QWidget::setEnabled( bool enable )
 
   The default implementation calls repaint(TRUE).
 
-  \sa setEnabled(), isEnabled(), repaint(), update()
+  \sa setEnabled(), enabled(), repaint(), update()
 */
 
 void QWidget::enabledChange( bool )
@@ -1200,7 +1155,7 @@ QRect QWidget::childrenRect() const
   The widget cannot be resized to a smaller size than the minimum widget
   size.
 
-  \sa setMinimumSize() maximumSize(), sizeIncrement()
+  \sa setMinimumSize(), maximumSize(), sizeIncrement()
 */
 
 QSize QWidget::minimumSize() const
@@ -1266,8 +1221,7 @@ void QWidget::setFixedSize( int w, int h )
   Sets the minimum width of the widget to \a w without changing the
   height.  Provided for convenience.
 
-  \sa sizeHint() minimumSize() maximumSize()
-  setFixedSize() and more
+  \sa sizeHint() minimumSize() maximumSize() setFixedSize() and more
 */
 
 void QWidget::setMinimumWidth( int w )
@@ -1436,6 +1390,9 @@ QWidget::BackgroundMode QWidget::backgroundMode() const
 /*!
   Tells the window system which color to clear this widget to when
   sending a paint event.
+  Tells the window system which color
+  from the \link QWidget::palette() palette\endlink
+  to use for clearing this widget when sending a paint event.
 
   In other words, this color is the color of the widget when
   paintEvent() is called.  To minimize flicker, this should be the
@@ -1635,21 +1592,14 @@ const QColorGroup &QWidget::colorGroup() const
 /*!
   \fn const QPalette &QWidget::palette() const
   Returns the widget palette.
-  \sa setPalette(), colorGroup(), QApplication::palette()
+  \sa setPalette(), colorGroup()
 */
 
 /*!
   Sets the widget palette to \e p. The widget background color is set to
   <code>colorGroup().background()</code>.
 
-  If \a palettePropagation() is \c AllChildren or \c SamePalette,
-  setPalette() calls setPalette() for children of the object, or those
-  with whom the object shares the palette, respectively.  The default
-  for QWidget is \a NoChildren, so setPalette() will not change the children's
-  palettes.
-
-  \sa QApplication::setPalette(), palette(), paletteChange(),
-  colorGroup(), setBackgroundColor(), setPalettePropagation()
+  \sa palette(), paletteChange(), colorGroup(), setBackgroundColor()
 */
 
 void QWidget::setPalette( const QPalette &p )
@@ -1658,18 +1608,6 @@ void QWidget::setPalette( const QPalette &p )
     pal = p;
     setBackgroundColorFromMode();
     paletteChange( old );
-    PropagationMode m = palettePropagation();
-    if ( m != NoChildren && children() ) {
-	QObjectListIt it( *children() );
-	QWidget *w;
-	while( (w=(QWidget *)it.current()) != 0 ) {
-	    ++it;
-	    if ( w->isWidgetType() &&
-		 ( m == AllChildren ||
-		   old.isCopyOf( w->pal ) ) )
-		w->setPalette( pal );
-	}
-    }
 }
 
 /*!
@@ -1715,12 +1653,7 @@ void QWidget::paletteChange( const QPalette & )
     setFont( f );
   \endcode
 
-  If \a fontPropagation() is \c AllChildren or \c SameFont, setFont()
-  calls setFont() for children of the object, or those with whom the
-  object shares the font, respectively.  The default for QWidget
-  is \a NoChildren, so setFont() will not change the children's fonts.
-
-  \sa font(), fontChange(), fontInfo(), fontMetrics(), setFontPropagation()
+  \sa font(), fontChange(), fontInfo(), fontMetrics()
 */
 
 void QWidget::setFont( const QFont &font )
@@ -1729,18 +1662,6 @@ void QWidget::setFont( const QFont &font )
     fnt = font;
     fnt.handle();				// force load font
     fontChange( old );
-    PropagationMode m = fontPropagation();
-    if ( m != NoChildren && children() ) {
-	QObjectListIt it( *children() );
-	QWidget *w;
-	while( (w=(QWidget *)it.current()) != 0 ) {
-	    ++it;
-	    if ( w->isWidgetType() &&
-		 ( m == AllChildren ||
-		   old.isCopyOf( w->fnt ) ) )
-		 w->setFont( fnt );
-	}
-    }
 }
 
 
@@ -1775,7 +1696,7 @@ void QWidget::fontChange( const QFont & )
 
   The font metrics object is automatically updated if somebody sets a new
   widget font. We have decided to change this policy in Qt 2.0: setting
-  a new font for a widget should not affect existing QFontMetrics objects.
+  a new font for a widget should not affect existing QFontMetric objects.
 
   \sa font(), fontInfo(), setFont()
 */
@@ -2051,21 +1972,10 @@ void QWidget::clearFocus()
   a web browser might reimplement it to move its "current active link"
   forwards or backwards, and call QWidget::focusNextPrevChild() only
   when it reaches the last/first.
-
-  Child widgets call focusNextPrevChild() on their parent widgets, and
-  only the top-level widget will thus make the choice of where to redirect
-  focus.  By overriding this method for an object, you thus gain control
-  of focus traversal for all child widgets.
-
-  \sa focusData()
 */
 
 bool QWidget::focusNextPrevChild( bool next )
 {
-    QWidget* p = parentWidget();
-    if ( !testWFlags(WType_TopLevel) && p )
-	return p->focusNextPrevChild(next);
-
     QFocusData *f = focusData( TRUE );
 
     QWidget *startingPoint = f->it.current();
@@ -2075,7 +1985,7 @@ bool QWidget::focusNextPrevChild( bool next )
     do {
 	if ( w && w != startingPoint &&
 	     w->testWFlags( WState_TabToFocus ) && !w->focusProxy() &&
-	     w->isVisibleToTLW() && w->isEnabledToTLW() )
+	     w->isVisibleToTLW() && w->isEnabled() )
 	    candidate = w;
 	w = next ? f->focusWidgets.prev() : f->focusWidgets.next();
     } while( w && !(candidate && w==startingPoint) );
@@ -2098,8 +2008,6 @@ QWidget *QWidget::focusWidget() const
 {
     QWidget *that = (QWidget *)this;		// mutable
     QFocusData *f = that->focusData( FALSE );
-    if ( f && f->focusWidgets.count() && f->it.current() == 0 )
-	f->it.toFirst();
     return f ? f->it.current() : 0;
 }
 
@@ -2112,18 +2020,8 @@ QWidget *QWidget::focusWidget() const
   list contains all the widgets in this top-level widget that can
   accept focus, in tab order.  An iterator points to the current focus
   widget (focusWidget() returns a pointer to this widget).
-
-  This information is useful for implementing advanced versions
-  of focusNextPrevChild().
 */
-QFocusData * QWidget::focusData()
-{
-    return focusData(TRUE);
-}
 
-/*!
-  Internal function which lets us not create it too.
-*/
 QFocusData * QWidget::focusData( bool create )
 {
     QWidget * tlw = topLevelWidget();
@@ -2201,9 +2099,8 @@ void QWidget::setTabOrder( QWidget* first, QWidget *second )
 void QWidget::reparentFocusWidgets( QWidget * parent )
 {
     if ( focusData() &&
-	 ( parent == 0 ? parentObj != 0
-		       : parent->topLevelWidget() != topLevelWidget() ) )
-    {
+	 ( parentObj == 0 || parent == 0 ||
+	   parent->topLevelWidget() != topLevelWidget() ) ) {
 	QFocusData * from = focusData();
 	from->focusWidgets.first();
 	QFocusData * to;
@@ -2307,7 +2204,7 @@ void QWidget::setCRect( const QRect &r )
   QWidget::StrongFocus if it accepts both and \c QWidget::NoFocus if it
   does not accept focus at all.
 
-  \sa isFocusEnabled(), setFocusPolicy(), focusInEvent(), focusOutEvent(),
+  \sa isFocusEnabled(), setFocusPolicy(), focusInEvent(), focusOutEvent(), 
   keyPressEvent(), keyReleaseEvent(), isEnabled()
 */
 
@@ -2483,9 +2380,12 @@ void QWidget::show()
 	while ( QApplication::activePopupWidget() )
 	    QApplication::activePopupWidget()->hide();
     }
-
     showWindow();
-
+#if !defined(_WS_WIN_)
+    // The Windows implementation does this in showWindow()
+    setWFlags( WState_Visible );
+    clearWFlags( WState_DoHide );
+#endif
     if ( testWFlags(WType_Modal) )
 	qt_enter_modal( this );
     else if ( testWFlags(WType_Popup) )
@@ -2515,32 +2415,27 @@ void QWidget::hide()
 
     hideWindow();
 
-    clearWFlags( WState_Visible );
-
     // next bit tries to move the focus if the focus widget is now
     // hidden.
     if ( qApp && qApp->focusWidget() == this )
 	focusNextPrevChild( TRUE );
 
-    // ### Can these ever be useful?
+    clearWFlags( WState_Visible );
+
     cancelMove();
     cancelResize();
-
     if ( isTopLevel() ) {			// last TLW hidden?
 	if ( qApp->receivers(SIGNAL(lastWindowClosed())) && noVisibleTLW() )
 	    emit qApp->lastWindowClosed();
     }
 
 #if QT_VERSION == 200
-#error "Do this inherits-stuff by overriding hide() in QDialog"
+#error "Do this inherits-hack by overriding hide() in QDialog"
 #error "Be sure to test for testWFlags(WState_Visible) or anything"
 #error " else that makes QWidget::hide() return early."
 #endif
     if ( testWFlags(WType_Modal)
       && inherits("QDialog") ) qApp->exit_loop();
-
-    QHideEvent e(FALSE);
-    QApplication::sendEvent( this, &e );
 }
 
 
@@ -2553,8 +2448,8 @@ void QWidget::hide()
   close event. The default implementation of QWidget::closeEvent()
   accepts the close event.
 
-  If \e forceKill is TRUE, the widget is deleted whether it accepts
-  the close event or not.
+  If \e forceKill is TRUE, the widget is deleted whether it accepts the
+  close event or not.
 
   The application is \link QApplication::quit() terminated\endlink when
   the \link QApplication::setMainWidget() main widget\endlink is closed.
@@ -2595,43 +2490,19 @@ bool QWidget::close( bool forceKill )
 
   Returns TRUE if the widget itself is set to visible status, or else
   FALSE.  Calling show() sets the widget to visible status; calling
-  hide() sets it to hidden status. Iconified top-level widgets also
-  have hidden status.
+  hide() sets it to hidden status.
 
   If a widget is set to visible status, but its parent widget is set
   to hidden status, this function returns TRUE.  isVisibleToTLW()
   looks at the visibility status of the parent widgets up to the
   top level widget.
 
-  This function returns TRUE if the widget currently is obscured by
-  other windows on the screen, but would be visible if moved.
-
-  \sa show(), hide(), isVisibleToTLW()
-*/
-
-
-/*!
-  Returns TRUE if this widget and every parent up to but excluding
-  \a ancestor is visible, otherwise returns FALSE.
-
   This function returns TRUE if the widget it is obscured by other
   windows on the screen, but would be visible if moved.
 
-  \sa show() hide() isVisible()
+  \sa show(), hide(), isVisibleToTLW(), iconified()
 */
 
-bool QWidget::isVisibleTo(QWidget* ancestor) const
-{
-    const QWidget * w = this;
-    while ( w && w->isVisible()
-	    && !w->isTopLevel()
-	    && w->parentWidget()
-	    && w->parentWidget()!=ancestor
-	)
-	w = w->parentWidget();
-    return w->isVisible();
-}
-
 
 /*!
   Returns TRUE if this widget and every parent up to the \link
@@ -2641,14 +2512,15 @@ bool QWidget::isVisibleTo(QWidget* ancestor) const
   This function returns TRUE if the widget it is obscured by other
   windows on the screen, but would be visible if moved.
 
-  This is equivalent to isVisibleTo(0).
-
-  \sa show(), hide(), isVisible()
+  \sa show(), hide(), isVisible(), iconified()
 */
 
 bool QWidget::isVisibleToTLW() const
 {
-    return isVisibleTo(0);
+    const QWidget * w = this;
+    while ( w && w->isVisible() && !w->isTopLevel() && w->parentWidget() )
+	w = w->parentWidget();
+    return w->isVisible();
 }
 
 
@@ -2701,7 +2573,7 @@ QSize QWidget::sizeHint() const
   \fn bool QWidget::testWFlags( WFlags n ) const
 
   Returns non-zero if any of the widget flags in \e n are set. The
-  widget flags are listed in qwindowdefs.h, and are strictly for
+  widget flags are listed in qwindefs.h, and are strictly for
   internal use.
 
   \internal
@@ -2819,7 +2691,7 @@ bool QWidget::event( QEvent *e )
 	case Event_KeyPress: {
 	    QKeyEvent *k = (QKeyEvent *)e;
 	    bool res = FALSE;
-	    if ( k->key() == Key_Backtab ||
+	    if ( k->key() == Key_Backtab || 
 		 (k->key() == Key_Tab && (k->state() & ShiftButton)) )
 		res = focusNextPrevChild( FALSE );
 	    else if ( k->key() == Key_Tab )
@@ -3253,65 +3125,3 @@ bool QWidget::x11Event( XEvent * )
 }
 
 #endif
-
-
-/*!  Returns the font propagation mode of this widget.  The default
-  font propagation mode is \c NoChildren, but you can set it to \a
-  SameFont or \a AllChildren.
-
-  \sa setFontPropagation()
-*/
-
-QWidget::PropagationMode QWidget::fontPropagation() const
-{
-    return extra ? (PropagationMode)extra->propagateFont : NoChildren;
-}
-
-
-/*!  Sets the font propagation mode to \a m.
-
-  if \a m is \c NoChildren (the default), setFont() does not change
-  any children's fonts.  If it is \c SameFont, setFont() changes the
-  font of the children that have the exact same font as this widget
-  (see \l QFont::isCopyOf() for details).  If it is \c AllChildren,
-  setFont() changes the font of all children.
-
-  \sa fontPropagation() setFont() setPalettePropagation()
-*/
-
-void QWidget::setFontPropagation( PropagationMode m )
-{
-    createExtra();
-    extra->propagateFont = (int)m;
-}
-
-
-/*!  Returns the palette propagation mode of this widget.  The default
-  palette propagation mode is \c NoChildren, but you can set it to \a
-  SamePalette or \a AllChildren.
-
-  \sa setPalettePropagation()
-*/
-
-QWidget::PropagationMode QWidget::palettePropagation() const
-{
-    return extra ? (PropagationMode)extra->propagatePalette : NoChildren;
-}
-
-
-/*!  Sets the palette propagation mode to \a m.
-
-  if \a m is \c NoChildren (the default), setPalette() does not change
-  any children's palettes.  If it is \c SamePalette, setPalette()
-  changes the palette of the children that have the exact same palette
-  as this widget (see \l QPalette::isCopyOf() for details).  If it is
-  \c AllChildren, setPalette() changes the palette of all children.
-
-  \sa palettePropagation() setPalette() setFontPropagation()
-*/
-
-void QWidget::setPalettePropagation( PropagationMode m )
-{
-    createExtra();
-    extra->propagatePalette = (int)m;
-}
