@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qwidget.h,v 2.10 1997/03/18 09:54:43 hanord Exp $
+** $Id: qwidget.h,v 2.36 1997/09/29 15:34:02 eiriken Exp $
 **
 ** Definition of QWidget class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -52,10 +55,12 @@ public:
     bool	 isDesktop()	const;
 
     bool	 isEnabled()	const;
+public slots:
     virtual void setEnabled( bool );
 
   // Widget coordinates
 
+public:
     const QRect &frameGeometry() const;
     const QRect &geometry()	const;
     int		 x()		const;
@@ -73,6 +78,10 @@ public:
     void	 setMinimumSize( int w, int h );
     void	 setMaximumSize( const QSize & );
     void	 setMaximumSize( int w, int h );
+    void	 setMinimumWidth( int w );
+    void	 setMinimumHeight( int h );
+    void	 setMaximumWidth( int w );
+    void	 setMaximumHeight( int h );
 
     QSize	 sizeIncrement() const;
     void	 setSizeIncrement( const QSize & );
@@ -80,6 +89,8 @@ public:
 
     void	 setFixedSize( const QSize & );
     void	 setFixedSize( int w, int h );
+    void	 setFixedWidth( int w );
+    void	 setFixedHeight( int h );
 
   // Widget coordinate mapping
 
@@ -92,12 +103,20 @@ public:
 
   // Widget attribute functions
 
-    const QColor &backgroundColor() const;
-    const QColor &foregroundColor() const;
-    virtual void setBackgroundColor( const QColor & );
+    enum BackgroundMode { FixedColor, FixedPixmap, NoBackground,
+			  PaletteForeground, PaletteBackground, PaletteLight,
+			  PaletteMidlight, PaletteDark, PaletteMid,
+			  PaletteText, PaletteBase };
+
+    BackgroundMode backgroundMode() const;
+    void	   setBackgroundMode( BackgroundMode );
+
+    const QColor  &backgroundColor() const;
+    const QColor  &foregroundColor() const;
+    virtual void  setBackgroundColor( const QColor & );
 
     const QPixmap *backgroundPixmap() const;
-    virtual void setBackgroundPixmap( const QPixmap & );
+    virtual void   setBackgroundPixmap( const QPixmap & );
 
     const QColorGroup &colorGroup() const;
     const QPalette    &palette()    const;
@@ -119,10 +138,12 @@ public:
     void	 setIconText( const char * );
 
     bool	 hasMouseTracking() const;
+public slots:
     void	 setMouseTracking( bool enable );
 
   // Keyboard input focus functions
 
+public:
     enum FocusPolicy 
     { NoFocus = 0, TabFocus = 0x1, ClickFocus = 0x2, StrongFocus = 0x3 };
 
@@ -134,6 +155,9 @@ public:
     bool	 hasFocus() const;
     void	 setFocus();
     void	 clearFocus();
+    static void  setTabOrder( QWidget *, QWidget * );
+    void	 setFocusProxy( QWidget * );
+    QWidget *	 focusProxy() const;
 
   // Grab functions
 
@@ -148,9 +172,11 @@ public:
   // Update/refresh functions
 
     bool	 isUpdatesEnabled() const;
+public slots:
     void	 setUpdatesEnabled( bool enable );
     void	 update();
     void	 update( int x, int y, int w, int h);
+    void	 update( const QRect& );
     void	 repaint( bool erase=TRUE );
     void	 repaint( int x, int y, int w, int h, bool erase=TRUE );
     void	 repaint( const QRect &, bool erase=TRUE );
@@ -161,8 +187,10 @@ public:
     virtual void hide();
     void	 iconify();
 
+public:
     virtual bool close( bool forceKill=FALSE );
     bool	 isVisible()	const;
+    bool	 isVisibleToTLW() const;
 
     void	 raise();
     void	 lower();
@@ -190,6 +218,8 @@ public:
 
   // Misc. functions
 
+    QWidget     *focusWidget() const;
+
 public:
     QWidget	*parentWidget() const;
     bool	 testWFlags( WFlags n ) const;
@@ -242,6 +272,8 @@ protected:
     int		 metric( int )	const;
 
     void	 create( WId );
+    void	 create( WId, bool initializeWindow, bool destroyOldWindow );
+    void	 destroy( bool destroyWindow, bool destroySubWindows );
     WFlags	 getWFlags()	const;
     void	 setWFlags( WFlags );
     void	 clearWFlags( WFlags n );
@@ -258,10 +290,15 @@ protected:
     WId		 frm_wnd;
 #endif
 
+private slots:
+    void	 focusProxyDestroyed();
+
 private:
     void	 setWinId( WId );
     bool	 create();
     bool	 destroy();
+    void	 showWindow();
+    void	 hideWindow();
     void	 createExtra();
     void	 deleteExtra();
     void	 internalMove( int, int );
@@ -272,6 +309,12 @@ private:
     void	 cancelMove();
     void	 cancelResize();
     void	 sendDeferredEvents();
+    void 	 reparentFocusWidgets( QWidget *parent );
+    QFocusData  *focusData( bool create = FALSE );
+    void	 setBackgroundColorFromMode();
+    void	 setBackgroundColorDirect( const QColor & );
+    void	 setBackgroundModeDirect( BackgroundMode );
+    void	 setBackgroundEmpty();
 
     WId		 winid;
     WFlags	 flags;
@@ -282,9 +325,10 @@ private:
     QFont	 fnt;
     QCursor	 curs;
     QWExtra	*extra;
-    QWidget	*focusChild;
+    QWidget	*focusChild; // ### unused now
     static void	 createMapper();
     static void	 destroyMapper();
+    static QWidgetList   *wList();
     static QWidgetList   *tlwList();
     static QWidgetMapper *mapper;
     friend class QApplication;
@@ -384,6 +428,9 @@ inline QWidget::FocusPolicy QWidget::focusPolicy() const
 inline bool QWidget::isUpdatesEnabled() const
 { return !testWFlags(WState_BlockUpdates); }
 
+inline void QWidget::update( const QRect &r )
+{ update( r.x(), r.y(), r.width(), r.height() ); }
+
 inline void QWidget::repaint( bool erase )
 { repaint( 0, 0, crect.width(), crect.height(), erase ); }
 
