@@ -1,29 +1,31 @@
 /****************************************************************************
-** $Id: qwindow.cpp,v 2.4 1998/07/03 00:09:42 hanord Exp $
+** $Id: qwindow.cpp,v 2.1 1997/08/06 15:05:48 hanord Exp $
 **
 ** Implementation of QWindow class
 **
 ** Created : 931211
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qwindow.h"
 #include "qpixmap.h"
 
+RCSTAG("$Id: qwindow.cpp,v 2.1 1997/08/06 15:05:48 hanord Exp $");
+
+
 /*!
   \class QWindow qwindow.h
   \brief The QWindow class is reserved for future extensions.
@@ -46,7 +48,7 @@
 QWindow::QWindow( QWidget *parent, const char *name, WFlags f )
     : QWidget( parent, name, f )
 {
-    // nothing
+    initMetaObject();
 }
 
 /*!
