@@ -1,37 +1,40 @@
 /****************************************************************************
-** $Id: qbuffer.cpp,v 2.16 1998/07/03 00:09:43 hanord Exp $
+** $Id: qbuffer.cpp,v 2.12 1997/08/06 15:05:49 hanord Exp $
 **
 ** Implementation of QBuffer class
 **
 ** Created : 930812
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qbuffer.h"
 #include <stdlib.h>
 
+RCSTAG("$Id: qbuffer.cpp,v 2.12 1997/08/06 15:05:49 hanord Exp $");
+
+
 /*!
   \class QBuffer qbuffer.h
   \brief The QBuffer class is an I/O device that operates on a QByteArray
 
-  \ingroup io
+  \ingroup tools
+  \ingroup stream
 
   QBuffer is an I/O device for reading and writing a memory buffer. A
-  QBuffer may be used directly (readBlock() and writeBlock()) or more
+  QFile may be used directly (readBlock() and writeBlock()) or more
   conveniently via QDataStream or QTextStream.  Most of its behavior
   is inherited from QIODevice.
 
