@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qcollect.cpp,v 2.0 1996/09/24 10:16:08 agulbra Exp $
+** $Id: qcollect.cpp,v 2.1 1997/08/06 15:05:50 hanord Exp $
 **
 ** Implementation of base class for all collection classes
 **
@@ -7,19 +7,22 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qcollect.h"
 
-RCSTAG("$Id: qcollect.cpp,v 2.0 1996/09/24 10:16:08 agulbra Exp $");
+RCSTAG("$Id: qcollect.cpp,v 2.1 1997/08/06 15:05:50 hanord Exp $");
 
 
 /*!
