@@ -1,29 +1,28 @@
 /****************************************************************************
-** $Id: qdir.cpp,v 2.25 1998/07/03 00:09:43 hanord Exp $
+** $Id: qdir.cpp,v 2.14 1997/09/30 10:49:58 eiriken Exp $
 **
 ** Implementation of QDir class
 **
 ** Created : 950427
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qdir.h"
-#include "qfileinfo.h"
-#include "qfiledefs.h"
+#include "qfileinf.h"
+#include "qfiledef.h"
 #include "qregexp.h"
 #include <stdlib.h>
 #include <ctype.h>
@@ -37,6 +36,9 @@
 #endif
 #endif
 
+RCSTAG("$Id: qdir.cpp,v 2.14 1997/09/30 10:49:58 eiriken Exp $");
+
+
 #if defined(_OS_FATFS_) || defined(_OS_OS2EMX_)
 
 static void slashify( char *n )
@@ -62,10 +64,10 @@ static void slashify( char * )
 
 /*!
   \class QDir qdir.h
-  \brief Traverses directory structures and contents in a
-	    platform-independent way.
+  \brief QDir can traverse the directory structure and read the contents of
+	 directories in a platform-independent way.
 
-  \ingroup io
+  \ingroup tools
 
   A QDir can point to a file using either a relative or an absolute file
   path. Absolute file paths begin with the directory separator ('/') or a
@@ -291,9 +293,9 @@ QString QDir::absPath() const
 	if ( tmp.right(1) != "/" )
 	    tmp += '/';
 	tmp += dPath;
-	return cleanDirPath( tmp );
+	return cleanDirPath( tmp.data() );
     } else {
-	return cleanDirPath( dPath );
+	return dPath.copy();
     }
 }
 
@@ -955,8 +957,12 @@ bool QDir::remove( const char *fileName, bool acceptAbsPath )
 #endif
 	return FALSE;
     }
-    QString p = filePath( fileName, acceptAbsPath );
-    return QFile::remove( p );
+    QString tmp = filePath( fileName, acceptAbsPath );
+#if defined(UNIX)
+    return unlink( tmp ) == 0;			// unlink more common in UNIX
+#else
+    return ::remove( tmp ) == 0;		// use standard ANSI remove
+#endif
 }
 
 /*!
@@ -967,12 +973,6 @@ bool QDir::remove( const char *fileName, bool acceptAbsPath )
   any number of separators at the beginning of \e name will be removed.
 
   Returns TRUE if successful, otherwise FALSE.
-
-  On most file systems, rename() fails only if oldName does not exist
-  or if \a newName and \a oldName are not on the same partition, but
-  there are also other reasons why rename() can fail.  For example, on
-  at least one file system rename() fails if newName points to an open
-  file
 */
 
 bool QDir::rename( const char *name, const char *newName,
@@ -1070,7 +1070,7 @@ QDir QDir::home()
 
 /*!
   Returns the root directory.
-  \sa rootDirPath() drives()
+  \sa rootDirPath()
 */
 
 QDir QDir::root()
@@ -1089,7 +1089,7 @@ QString QDir::currentDirPath()
     static bool forcecwd = TRUE;
     static ino_t cINode;
     static dev_t cDevice;
-    QString currentName( PATH_MAX );
+    static QString currentName( PATH_MAX );
 
     STATBUF st;
 
@@ -1133,8 +1133,7 @@ QString QDir::homeDirPath()
 
 /*!
   Returns the absolute path for the root directory ("/" under UNIX).
-
-  \sa root() drives()
+  \sa root()
 */
 
 QString QDir::rootDirPath()
@@ -1573,42 +1572,3 @@ bool QDir::readDirEntries( const QString &nameFilter,
 	dirty = TRUE;
     return TRUE;
 }
-
-
-/*!  Returns a list if the root directories on this system.  On
-  win32, this returns a number of QFileInfo objects containing "C:/",
-  "D:/" etc.  On other operating systems, it returns a list containing
-  just one root directory (e.g. "/").
-*/
-
-const QFileInfoList * QDir::drives()
-{
-    // at most one instance of QFileInfoList is leaked, and this variable
-    // points to that list
-    static QFileInfoList * knownMemoryLeak = 0;
-
-    if ( !knownMemoryLeak ) {
-	knownMemoryLeak = new QFileInfoList;
-
-#if defined(_OS_WIN32_)
-
-	Q_UINT32 driveBits = (Q_UINT32) GetLogicalDrives() & 0x3ffffff;
-	char driveName[4];
-	qstrcpy( driveName, "a:/" );
-	while( driveBits ) {
-	    if ( driveBits & 1 )
-		knownMemoryLeak->append( new QFileInfo( driveName ) );
-	    driveName[0]++;
-	    driveBits = driveBits >> 1;
-	}
-
-#else
-
-	// non-win32 versions both use just one root directory
-	knownMemoryLeak->append( new QFileInfo( rootDirPath() ) );
-
-#endif
-    }
-
-    return knownMemoryLeak;
-}
