@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qdstream.cpp,v 2.2 1997/02/05 10:02:50 hanord Exp $
+** $Id: qdstream.cpp,v 2.10 1997/09/19 09:46:51 warwick Exp $
 **
 ** Implementation of QDataStream class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -23,12 +26,7 @@
 #include <ctype.h>
 #include <stdlib.h>
 
-#if defined(UNIX)
-#include <sys/types.h>				// htonl etc.
-#include <netinet/in.h>
-#endif
-
-RCSTAG("$Id: qdstream.cpp,v 2.2 1997/02/05 10:02:50 hanord Exp $");
+RCSTAG("$Id: qdstream.cpp,v 2.10 1997/09/19 09:46:51 warwick Exp $");
 
 
 /*!
@@ -136,12 +134,11 @@ QDataStream::QDataStream( QIODevice *d )
 
   Example:
   \code
-    static uchar bindata[] = { 231, 1, 44, ... };
+    static char bindata[] = { 231, 1, 44, ... };
     QByteArray	a;
     a.setRawData( bindata, sizeof(bindata) );	// a points to bindata
     QDataStream s( a, IO_ReadOnly );		// open on a's data
     s >> <something>;				// read raw bindata
-    s.close();
     a.resetRawData( bindata, sizeof(bindata) ); // finished
   \endcode
 
@@ -163,7 +160,8 @@ QDataStream::QDataStream( QByteArray a, int mode )
 /*!
   Destroys the data stream.
 
-  The destructor will not affect the current IO device.
+  The destructor will not affect the current IO device, unless it
+  is an internal IO device processing a QByteArray passed in the constructor.
 */
 
 QDataStream::~QDataStream()
@@ -267,7 +265,7 @@ void QDataStream::setByteOrder( int bo )
  *****************************************************************************/
 
 
-static INT32 read_int_ascii( QDataStream *s )
+static Q_INT32 read_int_ascii( QDataStream *s )
 {
     register int n = 0;
     char buf[40];
@@ -283,7 +281,7 @@ static INT32 read_int_ascii( QDataStream *s )
 
 
 /*!
-  \fn QDataStream &QDataStream::operator>>( UINT8 &i )
+  \fn QDataStream &QDataStream::operator>>( Q_UINT8 &i )
   Reads an unsigned byte from the stream and returns a reference to
   the stream.
 */
@@ -292,25 +290,25 @@ static INT32 read_int_ascii( QDataStream *s )
   Reads a signed byte from the stream.
 */
 
-QDataStream &QDataStream::operator>>( INT8 &i )
+QDataStream &QDataStream::operator>>( Q_INT8 &i )
 {
     CHECK_STREAM_PRECOND
     if ( printable ) {				// printable data
-	i = (INT8)dev->getch();
+	i = (Q_INT8)dev->getch();
 	if ( i == '\\' ) {			// read octal code
 	    char buf[4];
 	    dev->readBlock( buf, 3 );
 	    i = (buf[2] & 0x07)+((buf[1] & 0x07) << 3)+((buf[0] & 0x07) << 6);
 	}
     } else {					// data or text
-	i = (INT8)dev->getch();
+	i = (Q_INT8)dev->getch();
     }
     return *this;
 }
 
 
 /*!
-  \fn QDataStream &QDataStream::operator>>( UINT16 &i )
+  \fn QDataStream &QDataStream::operator>>( Q_UINT16 &i )
   Reads an unsigned 16-bit integer from the stream and returns a reference to
   the stream.
 */
@@ -320,13 +318,13 @@ QDataStream &QDataStream::operator>>( INT8 &i )
   the stream.
 */
 
-QDataStream &QDataStream::operator>>( INT16 &i )
+QDataStream &QDataStream::operator>>( Q_INT16 &i )
 {
     CHECK_STREAM_PRECOND
     if ( printable ) {				// printable data
-	i = (INT16)read_int_ascii( this );
+	i = (Q_INT16)read_int_ascii( this );
     } else if ( noswap ) {			// no conversion needed
-	dev->readBlock( (char *)&i, sizeof(INT16) );
+	dev->readBlock( (char *)&i, sizeof(Q_INT16) );
     } else {					// swap bytes
 	register uchar *p = (uchar *)(&i);
 	char b[2];
@@ -339,7 +337,7 @@ QDataStream &QDataStream::operator>>( INT16 &i )
 
 
 /*!
-  \fn QDataStream &QDataStream::operator>>( UINT32 &i )
+  \fn QDataStream &QDataStream::operator>>( Q_UINT32 &i )
   Reads an unsigned 32-bit integer from the stream and returns a reference to
   the stream.
 */
@@ -349,13 +347,13 @@ QDataStream &QDataStream::operator>>( INT16 &i )
   the stream.
 */
 
-QDataStream &QDataStream::operator>>( INT32 &i )
+QDataStream &QDataStream::operator>>( Q_INT32 &i )
 {
     CHECK_STREAM_PRECOND
     if ( printable ) {				// printable data
 	i = read_int_ascii( this );
     } else if ( noswap ) {			// no conversion needed
-	dev->readBlock( (char *)&i, sizeof(INT32) );
+	dev->readBlock( (char *)&i, sizeof(Q_INT32) );
     } else {					// swap bytes
 	register uchar *p = (uchar *)(&i);
 	char b[4];
@@ -458,11 +456,12 @@ QDataStream &QDataStream::operator>>( char *&s )
   stream.
 
   The buffer \e s is allocated using \c new. Destroy it with the \c delete
-  operator.  If \e s cannot be allocated, \e s is set to 0.
+  operator.  If the length is zero or \e s cannot be allocated, \e s is
+  set to 0.
 
   The \e l parameter will be set to the length of the buffer.
 
-  The serialization format is an UINT32 length specifier first, then the
+  The serialization format is an Q_UINT32 length specifier first, then the
   data (\e length bytes).
 
   \sa readRawBytes(), writeBytes()
@@ -471,14 +470,19 @@ QDataStream &QDataStream::operator>>( char *&s )
 QDataStream &QDataStream::readBytes( char *&s, uint &l )
 {
     CHECK_STREAM_PRECOND
-    UINT32 len;
+    Q_UINT32 len;
     *this >> len;				// first read length spec
     l = (uint)len;
-    s = new char[len];				// create char array
-    CHECK_PTR( s );
-    if ( !s )					// no memory
+    if ( len == 0 || eof() ) {
+	s = 0;
 	return *this;
-    return readRawBytes( s, (uint)len );
+    } else {
+	s = new char[len];			// create char array
+	CHECK_PTR( s );
+	if ( !s )				// no memory
+	    return *this;
+	return readRawBytes( s, (uint)len );
+    }
 }
 
 
@@ -511,7 +515,7 @@ QDataStream &QDataStream::readRawBytes( char *s, uint len )
 
 
 /*!
-  \fn QDataStream &QDataStream::operator<<( UINT8 i )
+  \fn QDataStream &QDataStream::operator<<( Q_UINT8 i )
   Writes an unsigned byte to the stream and returns a reference to
   the stream.
 */
@@ -520,7 +524,7 @@ QDataStream &QDataStream::readRawBytes( char *s, uint len )
   Writes a signed byte to the stream.
 */
 
-QDataStream &QDataStream::operator<<( INT8 i )
+QDataStream &QDataStream::operator<<( Q_INT8 i )
 {
     CHECK_STREAM_PRECOND
     if ( printable && (i == '\\' || !isprint(i)) ) {
@@ -539,7 +543,7 @@ QDataStream &QDataStream::operator<<( INT8 i )
 
 
 /*!
-  \fn QDataStream &QDataStream::operator<<( UINT16 i )
+  \fn QDataStream &QDataStream::operator<<( Q_UINT16 i )
   Writes an unsigned 16-bit integer to the stream and returns a reference
   to the stream.
 */
@@ -549,7 +553,7 @@ QDataStream &QDataStream::operator<<( INT8 i )
   the stream.
 */
 
-QDataStream &QDataStream::operator<<( INT16 i )
+QDataStream &QDataStream::operator<<( Q_INT16 i )
 {
     CHECK_STREAM_PRECOND
     if ( printable ) {				// printable data
@@ -557,7 +561,7 @@ QDataStream &QDataStream::operator<<( INT16 i )
 	sprintf( buf, "%d\n", i );
 	dev->writeBlock( buf, strlen(buf) );
     } else if ( noswap ) {			// no conversion needed
-	dev->writeBlock( (char *)&i, sizeof(INT16) );
+	dev->writeBlock( (char *)&i, sizeof(Q_INT16) );
     } else {					// swap bytes
 	register uchar *p = (uchar *)(&i);
 	char b[2];
@@ -570,7 +574,7 @@ QDataStream &QDataStream::operator<<( INT16 i )
 
 
 /*!
-  \fn QDataStream &QDataStream::operator<<( UINT32 i )
+  \fn QDataStream &QDataStream::operator<<( Q_UINT32 i )
   Writes an unsigned 32-bit integer to the stream and returns a reference to
   the stream.
 */
@@ -580,7 +584,7 @@ QDataStream &QDataStream::operator<<( INT16 i )
   the stream.
 */
 
-QDataStream &QDataStream::operator<<( INT32 i )
+QDataStream &QDataStream::operator<<( Q_INT32 i )
 {
     CHECK_STREAM_PRECOND
     if ( printable ) {				// printable data
@@ -588,7 +592,7 @@ QDataStream &QDataStream::operator<<( INT32 i )
 	sprintf( buf, "%d\n", i );
 	dev->writeBlock( buf, strlen(buf) );
     } else if ( noswap ) {			// no conversion needed
-	dev->writeBlock( (char *)&i, sizeof(INT32) );
+	dev->writeBlock( (char *)&i, sizeof(Q_INT32) );
     } else {					// swap bytes
 	register uchar *p = (uchar *)(&i);
 	char b[4];
@@ -605,13 +609,13 @@ QDataStream &QDataStream::operator<<( INT32 i )
 /*!
   \fn QDataStream &QDataStream::operator<<( uint i )
   Writes an unsigned integer to the stream as a 32-bit unsigned integer
-  (UINT32).
+  (Q_UINT32).
   Returns a reference to the stream.
 */
 
 /*!
   \fn QDataStream &QDataStream::operator<<( int i )
-  Writes a signed integer to the stream as a 32-bit signed integer (INT32).
+  Writes a signed integer to the stream as a 32-bit signed integer (Q_INT32).
   Returns a reference to the stream.
 */
 
@@ -686,8 +690,12 @@ QDataStream &QDataStream::operator<<( double f )
 
 QDataStream &QDataStream::operator<<( const char *s )
 {
+    if ( !s ) {
+	*this << (Q_UINT32)0;
+	return *this;
+    }
     uint len = strlen( s ) + 1;			// also write null terminator
-    *this << (UINT32)len;			// write length specifier
+    *this << (Q_UINT32)len;			// write length specifier
     return writeRawBytes( s, len );
 }
 
@@ -696,7 +704,7 @@ QDataStream &QDataStream::operator<<( const char *s )
   Writes the length specifier \e len and the buffer \e s to the stream and
   returns a reference to the stream.
 
-  The \e len is serialized as an UINT32, followed by \e len bytes from
+  The \e len is serialized as an Q_UINT32, followed by \e len bytes from
   \e s.
 
   \sa writeRawBytes(), readBytes()
@@ -705,7 +713,7 @@ QDataStream &QDataStream::operator<<( const char *s )
 QDataStream &QDataStream::writeBytes(const char *s, uint len)
 {
     CHECK_STREAM_PRECOND
-    *this << (UINT32)len;			// write length specifier
+    *this << (Q_UINT32)len;			// write length specifier
     if ( len )
 	writeRawBytes( s, len );
     return *this;
@@ -716,9 +724,6 @@ QDataStream &QDataStream::writeBytes(const char *s, uint len)
   Writes \e len bytes from \e s to the stream and returns a reference to the
   stream.
 
-  The \e len is serialized as an UINT32, followed by \e len bytes from
-  \e s.
-
   \sa writeBytes(), QIODevice::writeBlock(), readRawBytes()
 */
 
