@@ -1,34 +1,38 @@
 /****************************************************************************
-** $Id: qfile.cpp,v 2.23 1998/07/03 00:09:44 hanord Exp $
+** $Id: qfile.cpp,v 2.10 1997/08/29 10:45:21 paul Exp $
 **
 ** Implementation of QFile class
 **
 ** Created : 930812
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qfile.h"
-#include "qfiledefs.h"
+#include "qfiledef.h"
+
+RCSTAG("$Id: qfile.cpp,v 2.10 1997/08/29 10:45:21 paul Exp $");
+
 
 /*!
   \class QFile qfile.h
   \brief The QFile class is an I/O device that operates on files.
 
-  \ingroup io
+  \ingroup tools
+  \ingroup files
+  \ingroup stream
 
   QFile is an I/O device for reading and writing binary and text files.	A
   QFile may be used by itself (readBlock and writeBlock) or by more
@@ -175,40 +179,6 @@ bool QFile::exists( const char *fileName )
 }
 
 
-/*!
-  Removes the file specified by the file name currently set.
-  Returns TRUE if successful, otherwise FALSE.
-
-  The file is closed before it is removed.
-*/
-
-bool QFile::remove()
-{
-    close();
-    return remove( fn.data()  );
-}
-
-/*!
-  Removes the file \a fileName.
-  Returns TRUE if successful, otherwise FALSE.
-*/
-
-bool QFile::remove( const char *fileName )      // remove file
-{
-    if ( fileName == 0 || fileName[0] == '\0' ) {
-#if defined(CHECK_NULL)
-        warning( "QFile::remove: Empty or NULL file name" );
-#endif
-        return FALSE;
-    }
-#if defined(UNIX)
-    return unlink( fileName ) == 0;		// unlink more common in UNIX
-#else
-    return ::remove( fileName ) == 0;		// use standard ANSI remove
-#endif
-}
-
-
 #if defined(_OS_MAC_) || defined(_OS_MSDOS_) || defined(_OS_WIN32_) || defined(_OS_OS2_)
 # define HAS_TEXT_FILEMODE			// has translate/text filemode
 #endif
@@ -237,17 +207,17 @@ bool QFile::remove( const char *fileName )      // remove file
   file index manually using at() in append mode.
   <li>\c IO_Truncate truncates the file.
   <li>\c IO_Translate enables carriage returns and linefeed translation
-  for text files under MS-DOS, Windows and OS/2.
+  for text files under MS-DOS, Window, OS/2 and Macintosh.
   </ul>
 
   The raw access mode is best when I/O is block-operated using 4kB block size
   or greater. Buffered access works better when reading small portions of
   data at a time.
-
+  
   <strong>Important:</strong> When working with buffered files, data may
   not be written to the file at once. Call \link flush() flush\endlink
   to make sure the data is really written.
-
+  
   \warning We have experienced problems with some C libraries when a buffered
   file is opened for both reading and writing. If a read operation takes place
   immediately after a write operation, the read buffer contains garbage data.
@@ -323,7 +293,7 @@ bool QFile::open( int m )
 	if ( fd != -1 ) {			// open successful
 	    STATBUF st;
 	    FSTAT( fd, &st );
-	    length = (int)st.st_size;
+	    length = st.st_size;
 	    index  = (flags() & IO_Append) == 0 ? 0 : length;
 	} else {
 	    ok = FALSE;
@@ -364,7 +334,7 @@ bool QFile::open( int m )
 	if ( fh ) {
 	    STATBUF st;
 	    FSTAT( FILENO(fh), &st );
-	    length = (int)st.st_size;
+	    length = st.st_size;
 	    index  = (flags() & IO_Append) == 0 ? 0 : length;
 	} else {
 	    ok = FALSE;
@@ -403,9 +373,8 @@ bool QFile::open( int m )
   When a QFile is opened using this function, close() does not actually
   close the file, only flushes it.
 
-  \warning If \e f is \c stdin, \c stdout, \c stderr, you may not
-  be able to seek.  See QIODevice::isSequentialAccess() for more
-  information.
+  \warning If \e f is one of \c stdin, \c stdout or \c stderr, you may not
+  be able to seek. size() is set to \c INT_MAX (in limits.h).
 
   \sa close()
 */
@@ -419,19 +388,17 @@ bool QFile::open( int m, FILE *f )
 	return FALSE;
     }
     init();
-    setMode( m &~IO_Raw );
+    setMode( m );
     setState( IO_Open );
     fh = f;
     ext_f = TRUE;
-    STATBUF st;
-    FSTAT( FILENO(fh), &st );
-    index = (int)ftell( fh );
-    if ( (st.st_mode & STAT_MASK) != STAT_REG ) {
-	// non-seekable
-	setType( IO_Sequential );
+    if ( fh == stdin || fh == stdout || fh == stderr ) {
 	length = INT_MAX;
     } else {
-	length = (int)st.st_size;
+	STATBUF st;
+	FSTAT( FILENO(fh), &st );
+	length = st.st_size;
+	index = ftell( fh );
     }
     return TRUE;
 }
@@ -459,7 +426,7 @@ bool QFile::open( int m, int f )
 	return FALSE;
     }
     init();
-    setMode( m |IO_Raw );
+    setMode( m );
     setState( IO_Open );
     fd = f;
     ext_f = TRUE;
@@ -468,8 +435,8 @@ bool QFile::open( int m, int f )
     } else {
 	STATBUF st;
 	FSTAT( fd, &st );
-	length = (int)st.st_size;
-	index  = (int)LSEEK(fd, 0, SEEK_CUR);
+	length = st.st_size;
+	index  = LSEEK(fd, 0, SEEK_CUR);
     }
     return TRUE;
 }
@@ -568,7 +535,7 @@ bool QFile::at( int pos )
     }
     bool ok;
     if ( isRaw() ) {				// raw file
-	pos = (int)LSEEK(fd, pos, SEEK_SET);
+	pos = LSEEK(fd, pos, SEEK_SET);
 	ok = pos != -1;
     } else {					// buffered file
 	ok = fseek(fh, pos, SEEK_SET) == 0;
@@ -596,7 +563,13 @@ bool QFile::atEnd() const
 #endif
 	return FALSE;
     }
-    return QIODevice::atEnd();
+    bool end;
+    if ( isRaw() ) {				// raw file
+	end = (at() == (int)size());
+    } else {					// buffered file
+	end = feof( fh );
+    }
+    return end;
 }
 
 
@@ -617,11 +590,8 @@ bool QFile::atEnd() const
 
 int QFile::readBlock( char *p, uint len )
 {
-#if defined(CHECK_NULL)
-    if ( !p )
-	warning( "QFile::readBlock: Null pointer error" );
-#endif
 #if defined(CHECK_STATE)
+    CHECK_PTR( p );
     if ( !isOpen() ) {				// file not open
 	warning( "QFile::readBlock: File not open" );
 	return -1;
@@ -632,19 +602,10 @@ int QFile::readBlock( char *p, uint len )
     }
 #endif
     int nread;					// number of bytes read
-    if ( isRaw() ) {				// raw file
+    if ( isRaw() )				// raw file
 	nread = READ( fd, p, len );
-	if ( len && nread <= 0 ) {
-	    nread = 0;
-	    setStatus(IO_ReadError);
-	}
-    } else {					// buffered file
+    else					// buffered file
 	nread = fread( p, 1, len, fh );
-	if ( (uint)nread != len ) {
-	    if ( ferror( fh ) || nread==0 )
-		setStatus(IO_ReadError);
-	}
-    }
     index += nread;
     return nread;
 }
@@ -690,7 +651,7 @@ int QFile::writeBlock( const char *p, uint len )
 	else
 	    setStatus( IO_WriteError );
 	if ( isRaw() )				// recalc file position
-	    index = (int)LSEEK( fd, 0, SEEK_CUR );
+	    index = LSEEK( fd, 0, SEEK_CUR );
 	else
 	    index = fseek( fh, 0, SEEK_CUR );
     } else {
@@ -736,14 +697,9 @@ int QFile::readLine( char *p, uint maxlen )
 	nread = QIODevice::readLine( p, maxlen );
     } else {					// buffered file
 	p = fgets( p, maxlen, fh );
-	if ( p ) {
-	    nread = strlen( p );
-	    index += nread;
-	} else {
-	    nread = -1;
-	    setStatus(IO_ReadError);
-	}
+	nread = p ? strlen( p ) : 0;
     }
+    index += nread;
     return nread;
 }
 
@@ -776,8 +732,6 @@ int QFile::getch()
     } else {					// buffered file
 	if ( (ch = getc( fh )) != EOF )
 	    index++;
-	else
-	    setStatus(IO_ReadError);
     }
     return ch;
 }
@@ -811,8 +765,6 @@ int QFile::putch( int ch )
 	    index++;
 	    if ( index > length )		// update file length
 		length = index;
-	} else {
-	    setStatus(IO_WriteError);
 	}
     }
     return ch;
@@ -854,8 +806,6 @@ int QFile::ungetch( int ch )
     } else {					// buffered file
 	if ( (ch = ungetc(ch, fh)) != EOF )
 	    index--;
-	else
-	    setStatus( IO_ReadError );
     }
     return ch;
 }
