@@ -1,23 +1,22 @@
 /****************************************************************************
-** $Id: qgcache.cpp,v 2.8 1998/07/03 00:09:44 hanord Exp $
+** $Id: qgcache.cpp,v 2.2 1997/08/06 15:05:52 hanord Exp $
 **
 ** Implementation of QGCache and QGCacheIterator classes
 **
 ** Created : 950208
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -26,6 +25,9 @@
 #include "qdict.h"
 #include "qstring.h"				/* used for statistics */
 
+RCSTAG("$Id: qgcache.cpp,v 2.2 1997/08/06 15:05:52 hanord Exp $");
+
+
 /*!
   \class QGCache qgcache.h
 
@@ -216,7 +218,6 @@ QGCache::QGCache( int maxCost, uint size,
 */
 
 QGCache::QGCache( const QGCache & )
-    : QCollection()
 {
 #if defined(CHECK_NULL)
     fatal( "QGCache::QGCache(QGCache &): Cannot copy a cache" );
@@ -291,14 +292,6 @@ void QGCache::setMaxCost( int maxCost )
 /*!
   \internal
   Inserts an item into the cache.
-
-  \warning If this function returns FALSE, you must delete \a data
-  yourself.  Additionally, be very careful about using \a data after
-  calling this function, as any other insertions into the cache, from
-  anywhere in the application, or within Qt itself, could cause the
-  data to be discarded from the cache, and the pointer to become
-  invalid.
-
 */
 
 bool QGCache::insert( const char *key, GCI data, int cost, int priority )
@@ -320,10 +313,6 @@ bool QGCache::insert( const char *key, GCI data, int cost, int priority )
 #endif
     if ( copyK )
 	key = qstrdup( key );
-    if ( priority < -32768 )
-	priority = -32768;
-    else if ( priority > 32767 )
-	priority = 32677;
     QCacheItem *ci = new QCacheItem( key, newItem(data), cost,
 				     (short)priority );
     CHECK_PTR( ci );
@@ -504,7 +493,7 @@ void QGCache::statistics() const
 /*!
   \class QGCacheIterator qgcache.h
 
-  \brief An internal class for implementing QCacheIterator and QIntCacheIterator.
+  \brief The QGCacheIterator is an internal class for implementing QCacheIterator and QIntCacheIterator.
 
   QGCacheIterator is a strictly internal class that does the heavy work for
   QCacheIterator and QIntCacheIterator.
