@@ -1,29 +1,31 @@
 /****************************************************************************
-** $Id: qglist.cpp,v 2.6 1998/07/03 00:09:45 hanord Exp $
+** $Id: qglist.cpp,v 2.2 1997/08/06 15:05:52 hanord Exp $
 **
 ** Implementation of QGList and QGListIterator classes
 **
 ** Created : 920624
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qglist.h"
 #include "qgvector.h"
-#include "qdatastream.h"
+#include "qdstream.h"
+
+RCSTAG("$Id: qglist.cpp,v 2.2 1997/08/06 15:05:52 hanord Exp $");
+
 
 /*!
   \class QLNode qglist.h
@@ -164,7 +166,6 @@ QGList::QGList()
 */
 
 QGList::QGList( const QGList & list )
-    : QCollection( list )
 {
     firstNode = lastNode = curNode = 0;		// initialize list
     numNodes  = 0;
