@@ -1,23 +1,22 @@
 /****************************************************************************
-** $Id: qglobal.cpp,v 2.17 1998/07/03 00:09:45 hanord Exp $
+** $Id: qglobal.cpp,v 2.12 1997/09/30 10:11:23 hanord Exp $
 **
 ** Global functions
 **
 ** Created : 920604
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -28,6 +27,9 @@
 #include <stdarg.h>
 #include <stdlib.h>
 
+RCSTAG("$Id: qglobal.cpp,v 2.12 1997/09/30 10:11:23 hanord Exp $");
+
+
 /*!
   \relates QApplication
   Returns the Qt version number for the library, typically "1.30"
@@ -341,7 +343,7 @@ Q_DECLARE(QDictM,int);
 static bool firstObsoleteWarning(const char *obj, const char *oldfunc )
 {
     static bool firstWarning = TRUE;
-    static QDictM(int) * obsoleteDict;
+    static QDictM(int) obsoleteDict;
     if ( firstWarning ) {
 	firstWarning = FALSE;
 	debug(
@@ -352,13 +354,12 @@ static bool firstObsoleteWarning(const char *obj, const char *oldfunc )
     QString s( obj );
     s += "::";
     s += oldfunc;
-    if ( !obsoleteDict )
-	obsoleteDict = new QDictM(int);
-    if ( obsoleteDict->find(s) == 0 ) {
-	obsoleteDict->insert( s, (int*) 666 );	// anything different from 0.
+    if ( obsoleteDict.find(s) == 0 ) {
+	obsoleteDict.insert( s, (int*) 666 );	// anything different from 0.
 	return TRUE;
+    } else {
+	return FALSE;
     }
-    return FALSE;
 }
 
 static bool suppressObsolete = FALSE;
@@ -419,7 +420,7 @@ void qObsolete(	 const char *message )
 
   Example:
   \code
-    #include <qapplication.h>
+    #include <qapp.h>
     #include <stdio.h>
     #include <stdlib.h>
 
