@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qiodev.cpp,v 2.5 1997/02/27 20:14:41 agulbra Exp $
+** $Id: qiodev.cpp,v 2.7 1997/08/06 15:05:53 hanord Exp $
 **
 ** Implementation of QIODevice class
 **
@@ -7,19 +7,22 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qiodev.h"
 
-RCSTAG("$Id: qiodev.cpp,v 2.5 1997/02/27 20:14:41 agulbra Exp $");
+RCSTAG("$Id: qiodev.cpp,v 2.7 1997/08/06 15:05:53 hanord Exp $");
 
 
 /*!
@@ -446,6 +449,8 @@ bool QIODevice::atEnd() const
   Reads a line of text, up to \e maxlen bytes including a terminating
   \0.  If there is a newline at the end if the line, it is not stripped.
 
+  Returns the number of bytes read, or -1 in case of error.
+
   This virtual function can be reimplemented much more efficiently by
   the most subclasses.
 
