@@ -1,35 +1,37 @@
 /****************************************************************************
-** $Id: qstring.cpp,v 2.22 1998/07/03 00:09:47 hanord Exp $
+** $Id: qstring.cpp,v 2.14 1997/09/04 15:08:48 warwick Exp $
 **
 ** Implementation of extended char array operations, and QByteArray and
 ** QString classes
 **
 ** Created : 920722
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #define	 QSTRING_C
 #include "qstring.h"
-#include "qdatastream.h"
+#include "qdstream.h"
 #include <stdio.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <ctype.h>
 
+RCSTAG("$Id: qstring.cpp,v 2.14 1997/09/04 15:08:48 warwick Exp $");
+
+
 /*****************************************************************************
   Safe and portable C string functions; extensions to standard string.h
  *****************************************************************************/
@@ -324,32 +326,6 @@ UINT16 qchecksum( const char *data, uint len )
     return (~crc & 0xffff);
 }
 
-//#define QSTRING_COMPATIBILITY 200
-
-#ifndef QSTRING_COMPATIBILITY
-#define QSTRING_COMPATIBILITY QT_VERSION
-#endif
-
-#if QSTRING_COMPATIBILITY < 200
-//
-// Explicitly-shared QString (ie. QString before Qt 2.0):
-//
-# define QSTRING_DETACH(s)
-#else
-# if QSTRING_COMPATIBILITY == 200
-//
-// Implicitly-shared QString, with warnings on potentially buggy old calls.
-//
-#  define QSTRING_DETACH(s) { if ((s)->nrefs()>1) warning("Changed behavior of QString"); (s)->detach(); }
-# else
-//
-// Implicitly-shared QString, with warnings turned off, once it is safe
-// to rely on the new QString behavior.
-//
-#  define QSTRING_DETACH(s) (s)->detach();
-# endif
-#endif
-
 
 /*****************************************************************************
   QByteArray member functions
@@ -469,7 +445,7 @@ QString::QString( int size )
 {
     if ( size > 0 ) {
 	*data() = '\0';				// set terminator
-	*(data()+(size-1)) = '\0';
+	*(data()+size-1) = '\0';
     }
 }
 
@@ -505,11 +481,7 @@ QString::QString( const char *str )
 
 QString::QString( const char *str, uint maxlen )
 {
-    uint len; // index of last nul character
-    // cannot use memchr - we need the *index*, not void*
-    for (len=0; len<maxlen-1; len++) {
-	if (!str[len]) break;
-    }
+    uint len = QMIN(maxlen-1,strlen(str));
     duplicate( str, len+1 );
     data()[len] = 0;
 }
@@ -602,7 +574,6 @@ QString::QString( const char *str, uint maxlen )
 
 bool QString::resize( uint len )
 {
-    QSTRING_DETACH(this)
     if ( !QByteArray::resize(len) )
 	return FALSE;
     if ( len )
@@ -641,7 +612,6 @@ bool QString::resize( uint len )
 
 QString &QString::sprintf( const char *format, ... )
 {
-    QSTRING_DETACH(this)
     va_list ap;
     va_start( ap, format );
     if ( size() < 256 )
@@ -665,7 +635,6 @@ QString &QString::sprintf( const char *format, ... )
 
 bool QString::fill( char c, int len )
 {
-    QSTRING_DETACH(this)
     if ( len < 0 )
 	len = length();
     if ( !QByteArray::fill(c,len+1) )
@@ -1203,13 +1172,11 @@ QString &QString::insert( uint index, const char *s )
     uint olen = length();
     int nlen = olen + len;
     if ( index >= olen ) {			// insert after end of string
-	QSTRING_DETACH(this)
 	if ( QByteArray::resize(nlen+index-olen+1) ) {
 	    memset( data()+olen, ' ', index-olen );
 	    memcpy( data()+index, s, len+1 );
 	}
     } else if ( QByteArray::resize(nlen+1) ) {	// normal insert
-	QSTRING_DETACH(this)
 	memmove( data()+index+len, data()+index, olen-index+1 );
 	memcpy( data()+index, s, len );
     }
@@ -1256,9 +1223,8 @@ QString &QString::insert( uint index, char c )	// insert char
   \e len is too large, the rest of the string is removed.
 
   \code
-    QString s = "Montreal";
-    s.remove( 1, 4 );
-    // s == "Meal"
+    QString s = "neutrino";
+    s.remove( 1, 6 );				// s == "no"
   \endcode
 
   \sa insert(), replace()
@@ -1268,12 +1234,9 @@ QString &QString::remove( uint index, uint len )
 {
     uint olen = length();
     if ( index + len >= olen ) {		// range problems
-	if ( index < olen ) {			// index ok
-	    QSTRING_DETACH(this)
+	if ( index < olen )			// index ok
 	    resize( index+1 );
-	}
     } else if ( len != 0 ) {
-	QSTRING_DETACH(this)
 	memmove( data()+index, data()+index+len, olen-index-len+1 );
 	QByteArray::resize(olen-len+1);
     }
@@ -1456,9 +1419,13 @@ uint QString::toUInt( bool *ok ) const
 double QString::toDouble( bool *ok ) const
 {
     char *end;
-    double val = strtod( data() ? data() : "", &end );
-    if ( ok )
-	*ok = ( data() && *data() && ( end == 0 || *end == '\0' ) );
+    double val = strtod( data(), &end );
+    if ( ok ) {
+	if ( end == 0 || *end == '\0' )
+	    *ok = TRUE;
+	else
+	    *ok = FALSE;
+    }
     return val;
 }
 
@@ -1484,7 +1451,6 @@ float QString::toFloat( bool *ok ) const
 
 QString &QString::setStr( const char *str )
 {
-    QSTRING_DETACH(this)
     if ( str )					// valid string
 	store( str, strlen(str)+1 );
     else					// empty
@@ -1499,7 +1465,6 @@ QString &QString::setStr( const char *str )
 
 QString &QString::setNum( long n )
 {
-    QSTRING_DETACH(this)
     char buf[20];
     register char *p = &buf[19];
     bool neg;
@@ -1527,7 +1492,6 @@ QString &QString::setNum( long n )
 
 QString &QString::setNum( ulong n )
 {
-    QSTRING_DETACH(this)
     char buf[20];
     register char *p = &buf[19];
     *p = '\0';
@@ -1619,7 +1583,6 @@ QString &QString::setNum( double n, char f, int prec )
 
 bool QString::setExpand( uint index, char c )
 {
-    QSTRING_DETACH(this)
     uint oldlen = length();
     if ( index >= oldlen ) {
 	if ( !QByteArray::resize( index+2 ) )	// no memory
@@ -1658,7 +1621,6 @@ QString& QString::operator+=( const char *str )
 {
     if ( !str )
 	return *this;				// nothing to append
-    QSTRING_DETACH(this)
     uint len1 = length();
     uint len2 = strlen(str);
     if ( !QByteArray::resize( len1 + len2 + 1 ) )
@@ -1673,7 +1635,6 @@ QString& QString::operator+=( const char *str )
 
 QString &QString::operator+=( char c )
 {
-    QSTRING_DETACH(this)
     uint len = length();
     if ( !QByteArray::resize( len + 2 ) )
 	return *this;				// no memory
@@ -1706,7 +1667,6 @@ QDataStream &operator<<( QDataStream &s, const QString &str )
 
 QDataStream &operator>>( QDataStream &s, QString &str )
 {
-    QSTRING_DETACH(&str)
     Q_UINT32 len;
     s >> len;					// read size of string
     if ( len == 0 || s.eof() ) {		// end of file reached
