@@ -1,36 +1,38 @@
 /****************************************************************************
-** $Id: qbutton.cpp,v 2.64 1998/07/03 00:09:47 hanord Exp $
+** $Id: qbutton.cpp,v 2.40 1997/10/09 15:19:36 eiriken Exp $
 **
 ** Implementation of QButton widget class
 **
 ** Created : 940206
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qbutton.h"
-#include "qbuttongroup.h"
+#include "qbttngrp.h"
 #include "qbitmap.h"
 #include "qpainter.h"
 #include "qkeycode.h"
 #include "qtimer.h"
 #include "qaccel.h"
-#include "qpixmapcache.h"
+#include "qpmcache.h"
 #include <ctype.h>
 
+RCSTAG("$Id: qbutton.cpp,v 2.40 1997/10/09 15:19:36 eiriken Exp $");
+
+
 static const int autoRepeatDelay  = 300;
 static const int autoRepeatPeriod = 100;
 
@@ -38,36 +40,30 @@ static const int drawingPixWidth  = 300;
 static const int drawingPixHeight = 100;
 
 
-/*
+/* 
   Returns a pixmap of dimension (drawingPixWidth x drawingPixHeight). The
   pixmap is used by paintEvent for flicker-free drawing.
  */
 
-static QPixmap *drawpm = 0;
-static void cleanupButtonPm()
-{
-    delete drawpm;
-}
 static QPixmap *getDrawingPixmap()
 {
-    if ( !drawpm ) {
-	qAddPostRoutine( cleanupButtonPm );
-	drawpm = new QPixmap( drawingPixWidth, drawingPixHeight );
-	CHECK_PTR( drawpm );
+    QPixmap *pm = QPixmapCache::find( "$qt-button-drawpix" );
+    if ( !pm ) {
+	pm = new QPixmap( drawingPixWidth, drawingPixHeight );
+	CHECK_PTR( pm );
+	QPixmapCache::insert( "$qt-button-drawpix", pm );
     }
-    return drawpm;
+    return pm;
 }
 
 
 struct QButtonData
 {
-    QButtonData() : group(0), a(0) {}
+    QButtonData() : group(0) {}
     QButtonGroup *group;
-    QTimer timer;
-    QAccel *a;
+    QTimer	  timer;
 };
 
-
 void QButton::ensureData()
 {
     if ( !d ) {
@@ -77,20 +73,17 @@ void QButton::ensureData()
     }
 }
 
-
 QButtonGroup *QButton::group() const
 {
     return d ? d->group : 0;
 }
 
-
 void QButton::setGroup( QButtonGroup* g )
 {
     ensureData();
     d->group = g;
 }
 
-
 QTimer *QButton::timer()
 {
     ensureData();
@@ -118,7 +111,7 @@ static int shortcutChar( const char *str )
   \class QButton qbutton.h
 
   \brief The QButton class is the abstract base class of button
-  widgets, providing functionality common to buttons.
+  widgets, and provides functionality common to buttons.
 
   \ingroup abstractwidgets
 
@@ -126,8 +119,8 @@ static int shortcutChar( const char *str )
   specify how to reply to user actions and how to draw the button.
 
   QButton provides both push and toggle buttons.  The QRadioButton and
-  QCheckBox classes provide only toggle buttons, QPushButton and
-  QToolButton provide both toggle and push buttons.
+  QCheckBox classes provide only toggle buttons, QPushButton provides
+  both toggle and push buttons.
 
   Any button can have either a text or pixmap label.  setText() sets
   the button to be a text button and setPixmap() sets it to be a
@@ -182,17 +175,17 @@ static int shortcutChar( const char *str )
   \link animateClick() animate a click\endlink.
 
   You can also set a custom accelerator using the setAccel() function.
-  This is useful mostly for pixmap buttons since they have no
-  automatic accelerator.
-
+  This is very useful for pixmap buttons since they have no automatic
+  accelerator.
+ 
   \code
     QPushButton *p;
     p->setPixmap( QPixmap("print.gif") );
     p->setAccel( ALT+Key_F7 );
   \endcode
 
-  All of the buttons provided by Qt (\l QPushButton, \l QToolButton,
-  \l QCheckBox and \l QRadioButton) can display both text and pixmaps.
+  All of the buttons provided by Qt (\l QPushButton, \l QCheckBox and \l
+  QRadioButton) support both text and pixmap labels.
 
   To subclass QButton, you have to reimplement at least drawButton()
   (to draw the button's outskirts) and drawButtonLabel() (to draw its
@@ -205,7 +198,6 @@ static int shortcutChar( const char *str )
   for a subclass of QButton unless you want to take over all drawing.
 
   \sa QButtonGroup
-  <a href="http://www.microsoft.com/win32dev/uiguide/uigui161.htm">Microsoft Style Guide</a>
 */
 
 
@@ -219,6 +211,7 @@ static int shortcutChar( const char *str )
 QButton::QButton( QWidget *parent, const char *name )
     : QWidget( parent, name )
 {
+    initMetaObject();
     bpixmap    = 0;
     toggleBt   = FALSE;				// button is not on/off
     buttonDown = FALSE;				// button is up
@@ -290,14 +283,11 @@ QButton::~QButton()
 */
 
 /*!
-  Sets the button to display \e text and repaints
+  Sets the button contents to \e text and redraws the contents.
 
-  The button resizes itself if auto-resizing is enabled, changes its
-  minimum size if autoMinimumSize() is enabled, and sets the
-  appropriate accelerator.
+  The button resizes itself if auto-resizing is enabled.
 
-  \sa text(), setPixmap(), setAutoMinimumSize(), setAutoResize(),
-  setAccel(), QPixmap::mask()
+  \sa text(), setPixmap(), setAutoResize()
 */
 
 void QButton::setText( const char *text )
@@ -327,7 +317,7 @@ void QButton::setText( const char *text )
 */
 
 /*!
-  Sets the button to display \a pixmap and repaints at once.
+  Sets the button pixmap to \a pixmap and redraws the contents.
 
   If \a pixmap is monochrome (i.e. it is a QBitmap or its \link
   QPixmap::depth() depth\endlink is 1) and it does not have a mask,
@@ -335,32 +325,31 @@ void QButton::setText( const char *text )
   this is to draw transparent bitmaps, which is important for
   e.g. toggle buttons.
 
-  The button resizes itself if auto-resizing is enabled, changes its
-  minimum size if autoMinimumSize() is enabled, and always disables
-  any accelerator.
+  The button resizes itself if auto-resizing is enabled, and always
+  disables any accelerator.
 
-  \sa pixmap(), setText(), setAutoMinimumSize(), setAutoResize(),
-  setAccel(), QPixmap::mask()
+  \sa pixmap(), setText(), setAutoResize(), setAccel(), QPixmap::mask()
 */
 
 void QButton::setPixmap( const QPixmap &pixmap )
 {
-    bool newSize;
+    int  w, h;
     if ( bpixmap ) {
-	newSize = pixmap.width() != bpixmap->width() ||
-		  pixmap.height() != bpixmap->height();
-        *bpixmap = pixmap;
+	w = bpixmap->width();
+	h = bpixmap->height();
     } else {
-	newSize = TRUE;
-	bpixmap = new QPixmap( pixmap );
+	bpixmap = new QPixmap;
 	CHECK_PTR( bpixmap );
+	w = h = -1;
     }
+    bool sameSize = w == bpixmap->width() && h == bpixmap->height();
+    *bpixmap = pixmap;
     if ( bpixmap->depth() == 1 && !bpixmap->mask() )
 	bpixmap->setMask( *((QBitmap *)bpixmap) );
     int oldAccelChar = shortcutChar(btext);
     if ( !btext.isNull() )
 	btext.resize( 0 );
-    if ( autoresize && newSize )
+    if ( autoresize && !sameSize )
 	adjustSize();
     if ( oldAccelChar )
 	setAccel( 0 );
@@ -376,7 +365,8 @@ void QButton::setPixmap( const QPixmap &pixmap )
 
 int QButton::accel() const
 {
-    return d && d->a ? d->a->key(0) : 0;
+    QAccel *a = CHILD((QObject*)this,QAccel,"buttonAccel");
+    return a ? a->key(0) : 0;
 }
 
 /*!
@@ -409,17 +399,16 @@ void QButton::setAccel( int key )
 	key = c ? ALT+toupper(c) : 0;
     }
 
-    ensureData();
+    QAccel *a = CHILD(this,QAccel,"buttonAccel");
     if ( key == 0 ) {				// delete accel
-	delete d->a;
-	d->a = 0;
+	delete a;
     } else {
-	if ( d->a )
-	    d->a->clear();
+	if ( a )
+	    a->clear();
 	else
-	    d->a = new QAccel( this, "buttonAccel" );
-	d->a->connectItem( d->a->insertItem(key,0),
-			   this, SLOT(animateClick()) );
+	    a = new QAccel( this, "buttonAccel" );
+	a->connectItem( a->insertItem(key,0),
+			this, SLOT(animateClick()) );
     }
 }
 
@@ -495,7 +484,9 @@ void QButton::setAutoRepeat( bool enable )
   appropriate.
 
   This function does nothing if the button is \link setEnabled()
-  disabled. \endlink
+  disabled\endlink or if it is the enabled toggle button in an
+  exclusive button group (this means, in practice, a \link
+  QRadioButton radio button \endlink in a button group).
 
   \sa setAccel()
 */
@@ -504,6 +495,8 @@ void QButton::animateClick()
 {
     if ( !isEnabled() || animation )
 	return;
+    if ( isOn() && group() && group()->isExclusive() )
+	return;
     animation  = TRUE;
     buttonDown = TRUE;
     repaint( FALSE );
@@ -522,9 +515,9 @@ void QButton::animateClick()
   Sets the state of the button to pressed down if \e enable is TRUE
   or to standing up if \e enable is FALSE.
 
-  If the button is a toggle button, it is \e not toggled.  Call
-  toggle() as well if you need to do that.  The pressed() and
-  released() signals are not emitted by this function.
+  If the button is a toggle button, it is \e not toggled.
+  Call toggle() as well if you need to do that.
+  The pressed() and released() signals are not emitted by this function.
 
   This method is provided in case you need to override the mouse event
   handlers.
@@ -557,14 +550,10 @@ void QButton::setDown( bool enable )
 
 void QButton::setOn( bool enable )
 {
-    if ( !toggleBt ) {
 #if defined(CHECK_STATE)
-	warning( "QButton::setOn(): (%s) Only toggle buttons may be switched",
-		 name( "unnamed" ) );
+    if ( !toggleBt )
+	warning( "QButton::setOn: Only toggle buttons may be switched" );
 #endif
-	return;
-    }
-
     if ( (bool)buttonOn != enable ) {		// changed state
 	buttonOn = enable;
 	repaint( FALSE );
@@ -638,7 +627,7 @@ void QButton::drawButtonLabel( QPainter * )
   Space is the only key that has any effect; it calls animateClick().
 */
 
-void QButton::keyPressEvent( QKeyEvent *e )
+void QButton::keyPressEvent( QKeyEvent *e ) 
 {
     if ( e->key() == Key_Space )
 	animateClick();
@@ -677,15 +666,15 @@ void QButton::mouseReleaseEvent( QMouseEvent *e)
     if ( e->button() != LeftButton )
 	return;
     if ( d )
-        timer()->stop();
+	timer()->stop();
     mlbDown = FALSE;				// left mouse button up
+    bool hit = hitButton( e->pos() );
     buttonDown = FALSE;
-    if ( hitButton( e->pos() ) ) {		// mouse release on button
-	int t = toggleBt && ( !isOn() || !group() || !group()->isExclusive() );
-	if ( t )
+    if ( hit ) {				// mouse release on button
+	if ( toggleBt )
 	    buttonOn = !buttonOn;
 	repaint( FALSE );
-	if ( t )
+	if ( toggleBt )
 	    emit toggled( buttonOn );
 	emit released();
 	emit clicked();
@@ -704,7 +693,8 @@ void QButton::mouseMoveEvent( QMouseEvent *e )
 {
     if ( !((e->state() & LeftButton) && mlbDown) )
 	return;					// left mouse button is up
-    if ( hitButton( e->pos() ) ) {		// mouse move in button
+    bool hit = hitButton( e->pos() );
+    if ( hit ) {				// mouse move in button
 	if ( !buttonDown ) {
 	    buttonDown = TRUE;
 	    repaint( FALSE );
@@ -729,15 +719,14 @@ void QButton::mouseMoveEvent( QMouseEvent *e )
 void QButton::paintEvent( QPaintEvent *event )
 {
     QPainter paint;
-
-    if ( event &&
-	 width() <= drawingPixWidth &&
-	 height() <= drawingPixHeight ) {
+    
+    if ( event && width() <= drawingPixWidth && height() <= drawingPixHeight ) {
 	QPixmap *pm = getDrawingPixmap();
 	ASSERT( pm );
-	
+
+	paint.begin( pm );
+	paint.setFont( font() );
 	pm->fill( this, 0, 0 );
-	paint.begin( pm, this );
 	drawButton( &paint );
 	paint.end();
 	
@@ -810,19 +799,19 @@ void QButton::animateTimeout()
 	return;
     animation  = FALSE;
     buttonDown = FALSE;
-    bool t = isToggleButton() &&
-	     (!isOn() || !group() || !group()->isExclusive());
-    if ( t )
+    if ( toggleBt )
 	buttonOn = !buttonOn;
     repaint( FALSE );
-    if ( t )
+    if ( toggleBt )
 	emit toggled( buttonOn );
     emit released();
     emit clicked();
 }
 
 
-/*! \reimp */
+/*!
+  Reimplemented for implementational reasons.
+*/
 
 void QButton::enabledChange( bool e )
 {
@@ -830,12 +819,3 @@ void QButton::enabledChange( bool e )
 	setDown( FALSE );
     QWidget::enabledChange( e );
 }
-
-
-/*!  if this is a toggle button, toggles it. */
-
-void QButton::toggle()
-{
-    if ( isToggleButton() )
-	 setOn( !isOn() );
-}
