@@ -1,31 +1,33 @@
 /****************************************************************************
-** $Id: qframe.cpp,v 2.20 1998/07/03 00:09:48 hanord Exp $
+** $Id: qframe.cpp,v 2.7 1997/09/15 16:29:53 warwick Exp $
 **
 ** Implementation of QFrame widget class
 **
 ** Created : 950201
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qframe.h"
 #include "qpainter.h"
-#include "qdrawutil.h"
+#include "qdrawutl.h"
 #include "qframe.h"
 
+RCSTAG("$Id: qframe.cpp,v 2.7 1997/09/15 16:29:53 warwick Exp $");
+
+
 /*!
   \class QFrame qframe.h
   \brief The QFrame class is the base class of widgets that have an (optional)
@@ -82,7 +84,7 @@
 
   <img src=frames.gif height=422 width=520 alt="Table of frame styles">
 
-  For obvious reasons, \c NoFrame isn't shown.  The gray areas next to
+  For obvious reasons, \c NoFrame isn't shown.	The gray areas next to
   the \c VLine and \c HLine examples are there because the widgets are
   taller/wider than the natural width of the lines.  frameWidth()
   returns the natural width of the line.
@@ -107,6 +109,7 @@ QFrame::QFrame( QWidget *parent, const char *name, WFlags f,
 		bool allowLines )
     : QWidget( parent, name, f )
 {
+    initMetaObject();
     frect  = QRect( 0, 0, 0, 0 );
     fstyle = NoFrame;
     lwidth = 1;
@@ -141,14 +144,14 @@ QFrame::QFrame( QWidget *parent, const char *name, WFlags f,
   Sets the frame style to \e style.
 
   The \e style is the bitwise OR between a frame shape and a frame
-  shadow style.  See the <a href="#picture">illustration</a> in the
+  shadow style.	 See the <a href=#picture>illustration</a> in the
   class documentation.
 
   The frame shapes are:
   <ul>
-  <li> \c NoFrame draws nothing. Naturally, you should not specify a shadow
+  <li> \c NoFrame draws nothing. Naturally, you should not specify a shadow 
   style if you use this.
-  <li> \c Box draws a rectangular box.  The contents appear to be
+  <li> \c Box draws a rectangular box.	The contents appear to be
   level with the surrounding screen, but the border itself may be
   raised or sunken.
   <li> \c Panel draws a rectangular panel that can be raised or sunken.
@@ -178,7 +181,7 @@ QFrame::QFrame( QWidget *parent, const char *name, WFlags f,
   (with any shadow style) are disregarded unless line shapes are
   allowed.  Line shapes are allowed by default.
 
-  \sa <a href="#picture">Illustration</a>, frameStyle(), lineShapesOk(),
+  \sa <a href=#picture>Illustration</a>, frameStyle(), lineShapesOk(),
   colorGroup(), QColorGroup
 */
 
@@ -193,8 +196,7 @@ void QFrame::setFrameStyle( int style )
     bool shape	= (style & MShape)  != 0;
     bool shadow = (style & MShadow) != 0;
     if ( shape != shadow )
-	warning( "QFrame::setFrameStyle: (%s) Incomplete frame style",
-		 name( "unnamed" ) );
+	warning( "QFrame::setFrameStyle: Incomplete frame style" );
 #endif
     fstyle = (short)style;
     updateFrameWidth();
@@ -247,33 +249,7 @@ void QFrame::setLineWidth( int w )
 
 void QFrame::setMidLineWidth( int w )
 {
-    mwidth = (short) ( w & 0x00ff | mwidth & 0xff00 );
-    updateFrameWidth();
-}
-
-
-
-/*!
-  \fn int QFrame::margin() const
-  Returns the width of the margin. The margin is the distance between the
-  innermost pixel of the frame and the outermost pixel of contentsRect().
-  It is included in frameWidth().
-
-  The margin is filled according to backgroundMode().
-
-  The default value is 0.
-
-  \sa setMargin(), lineWidth(), frameWidth()
-*/
-
-/*!
-  Sets the width of the margin to \e w.
-  \sa margin(), setLineWidth()
-*/
-
-void QFrame::setMargin( int w )
-{
-    mwidth = (short) ( ((w & 0xff) << 8) | mwidth & 0x00ff );
+    mwidth = (short)w;
     updateFrameWidth();
 }
 
@@ -303,7 +279,7 @@ void QFrame::updateFrameWidth()
 		    break;
 		case Raised:
 		case Sunken:
-		    fwidth = (short)(lwidth*2 + midLineWidth() );
+		    fwidth = (short)(lwidth*2 + mwidth);
 		    break;
 	    }
 	    break;
@@ -336,7 +312,7 @@ void QFrame::updateFrameWidth()
 		    break;
 		case Raised:
 		case Sunken:
-		    fwidth = (short)(lwidth*2 + midLineWidth());
+		    fwidth = (short)(lwidth*2 + mwidth);
 		    break;
 	    }
 	    break;
@@ -345,8 +321,6 @@ void QFrame::updateFrameWidth()
     if ( fwidth == -1 )				// invalid style
 	fwidth = 0;
 
-    fwidth += margin();
-
     frameChanged();
 }
 
@@ -375,10 +349,7 @@ void QFrame::updateFrameWidth()
 
 QRect QFrame::frameRect() const
 {
-    if ( frect.isNull() )
-	return rect();
-    else
-	return frect;
+    return frect.isNull() ? rect() : frect;
 }
 
 
@@ -440,29 +411,18 @@ QSize QFrame::sizeHint() const
 void QFrame::paintEvent( QPaintEvent *event )
 {
     QPainter paint( this );
-
-#if QT_VERSION >= 200
-#error "remove this hack"
-#else
-    
-    QPaintEvent kscdhack( rect() );
-    if ( !event )
-	event = &kscdhack;
-
-#endif
-
-    if ( !contentsRect().contains( event->rect() ) ) {
-	paint.save();
-	QRect r( frameRect() );
-	paint.setClipRect( r.intersect( event->rect() ) );
-	drawFrame( &paint );
-	paint.restore();
-    }
-    if ( event->rect().intersects( contentsRect() ) &&
-	 (fstyle & MShape) != HLine && (fstyle & MShape) != VLine ) {
+    if ( event ) {
+	if ( !contentsRect().contains( event->rect() ) ) {
+	    paint.save();
+	    paint.setClipRect( event->rect() );
+	    drawFrame( &paint );
+	    if ( !event->rect().intersects( contentsRect() ) )
+		return;
+	    paint.restore();
+	}
 	paint.setClipRect( event->rect().intersect( contentsRect() ) );
-	drawContents( &paint );
     }
+    drawContents( &paint );
 }
 
 
@@ -472,7 +432,7 @@ void QFrame::paintEvent( QPaintEvent *event )
   Adjusts the frame rectangle for the resized widget.  The frame
   rectangle is elastic, the surrounding area is static.
 
-  The resulting frame rectangle may be null or invalid.  You can use
+  The resulting frame rectangle may be null or invalid.	 You can use
   setMinimumSize() to avoid that possibility.
 
   Nothing is done if the frame rectangle is a \link QRect::isNull()
@@ -516,7 +476,7 @@ void QFrame::drawFrame( QPainter *p )
 		qDrawPlainRect( p, r, g.foreground(), lwidth );
 	    else
 		qDrawShadeRect( p, r, g, style == Sunken, lwidth,
-				midLineWidth() );
+				mwidth );
 	    break;
 
 	case Panel:
@@ -551,7 +511,7 @@ void QFrame::drawFrame( QPainter *p )
 	    }
 	    else
 		qDrawShadeLine( p, p1, p2, g, style == Sunken,
-				lwidth, midLineWidth() );
+				lwidth, mwidth );
 	    break;
     }
 }
@@ -561,14 +521,15 @@ void QFrame::drawFrame( QPainter *p )
   Virtual function that draws the contents of the frame.
 
   The QPainter is already open when you get it, and you must leave it
-  open.  Painter \link QPainter::setWorldMatrix() transformations\endlink
+  open.	 Painter \link QPainter::setWorldMatrix() transformations\endlink
   are switched off on entry.  If you transform the painter, remember to
   take the frame into account and \link QPainter::resetXForm() reset
   transformation\endlink before returning.
 
-  This function is reimplemented by subclasses that draw something
-  inside the frame.  It should draw only inside contentsRect(). The
-  default function does nothing.
+  This function is reimplemented by subclasses that draw something inside
+  the frame.  It should draw only inside contentsRect().  QFrame does not
+  enable \link QPainter::setClipRect() clipping\endlink but you may want
+  to. The default function does nothing.
 
   \sa contentsRect(), QPainter::setClipRect()
 */
