@@ -1,32 +1,29 @@
 /****************************************************************************
-** $Id: qframe.h,v 2.7 1998/07/03 00:09:48 hanord Exp $
+** $Id: qframe.h,v 2.2 1997/08/06 15:05:57 hanord Exp $
 **
 ** Definition of QFrame widget class
 **
 ** Created : 950201
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #ifndef QFRAME_H
 #define QFRAME_H
 
-#ifndef QT_H
 #include "qwidget.h"
-#endif // QT_H
 
 
 class QFrame : public QWidget			// frame class
@@ -58,9 +55,6 @@ public:
     int		lineWidth()	const;
     void	setLineWidth( int );
 
-    int		margin()	const;
-    void	setMargin( int );
-
     int		midLineWidth()	const;
     void	setMidLineWidth( int );
 
@@ -88,8 +82,8 @@ private:
     short	lineok;
 
 private:	// Disabled copy constructor and operator=
-    QFrame( const QFrame & );
-    QFrame &operator=( const QFrame & );
+    QFrame( const QFrame & ) {}
+    QFrame &operator=( const QFrame & ) { return *this; }
 };
 
 
@@ -103,16 +97,13 @@ inline int QFrame::frameShadow() const
 { return fstyle & MShadow; }
 
 inline bool QFrame::lineShapesOk() const
-{ return lineok; }				// ### Qt 2.0: bool
+{ return lineok; }
 
 inline int QFrame::lineWidth() const
 { return lwidth; }
 
 inline int QFrame::midLineWidth() const
-{ return mwidth & 0x00ff; }
-
-inline int QFrame::margin() const
-{ return ((int)mwidth) >> 8; }
+{ return mwidth; }
 
 inline int QFrame::frameWidth() const
 { return fwidth; }
