@@ -1,40 +1,42 @@
 /**********************************************************************
-** $Id: qlabel.cpp,v 2.34 1998/07/04 13:48:11 agulbra Exp $
+** $Id: qlabel.cpp,v 2.20 1997/09/03 15:18:16 paul Exp $
 **
 ** Implementation of QLabel widget class
 **
 ** Created : 941215
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qlabel.h"
 #include "qpixmap.h"
 #include "qpainter.h"
-#include "qdrawutil.h"
+#include "qdrawutl.h"
 #include "qaccel.h"
 #include "qkeycode.h"
 #include "qmovie.h"
 #include <ctype.h>
 
+RCSTAG("$Id: qlabel.cpp,v 2.20 1997/09/03 15:18:16 paul Exp $");
+
+
 #if QT_VERSION == 200
 #error "Remove QLabel dict!"
 #endif
 
-#include "qintdict.h"
+#include <qintdict.h>
 
 struct QLabel_Private
 {
@@ -70,7 +72,7 @@ static void cleanupLabel()
 
   \code
      QLineEdit * phone = new QLineEdit( this, "phone number" );
-     QLabel * phoneLabel = new QLabel( phone, "&Phone", this );
+     QLabel * phoneLabel = new QLineEdit( phone, "&Phone", this );
   \endcode
 
   In this example, keyboard focus is transferred to the label's buddy
@@ -99,9 +101,6 @@ static void cleanupLabel()
   Both lines are flush with the right side of the label.
 
   <img src=qlabel-m.gif> <img src=qlabel-w.gif>
-
-  \sa QLineEdit QMovie
-  <a href="guibooks.html#fowler">GUI Design Handbook: Label</a>
 */
 
 
@@ -118,6 +117,7 @@ static void cleanupLabel()
 QLabel::QLabel( QWidget *parent, const char *name, WFlags f )
     : QFrame( parent, name, f )
 {
+    initMetaObject();
     lpixmap    = 0;
     align      = AlignLeft | AlignVCenter | ExpandTabs;
     extraMargin= -1;
@@ -138,6 +138,7 @@ QLabel::QLabel( QWidget *parent, const char *name, WFlags f )
 QLabel::QLabel( const char *text, QWidget *parent, const char *name, WFlags f )
 	: QFrame( parent, name, f ), ltext(text)
 {
+    initMetaObject();
     lpixmap    = 0;
     align      = AlignLeft | AlignVCenter | ExpandTabs;
     extraMargin= -1;
@@ -174,6 +175,7 @@ QLabel::QLabel( QWidget *buddy,  const char *text,
 		QWidget *parent, const char *name, WFlags f )
     : QFrame( parent, name, f ), ltext("")
 {
+    initMetaObject();
     lpixmap    = 0;
     align      = ShowPrefix | AlignLeft | AlignVCenter | ExpandTabs;
     extraMargin= -1;
@@ -246,18 +248,9 @@ void QLabel::setText( const char *text )
 	    repaint();
     } else {
 	updateLabel();
-    }
+    } 
 }
 
-
-/*!  Clears the label.  Equivalent with setText( "" ). */
-
-void QLabel::clear()
-{
-    setText( "" );
-}
-
-
 /*!
   \fn QPixmap *QLabel::pixmap() const
   Returns the label pixmap.
@@ -472,21 +465,18 @@ void QLabel::setAutoResize( bool enable )
 
 QSize QLabel::sizeHint() const
 {
-    QPainter p( this );
+    QPainter p;
+    p.begin( this );
     QRect br;
     QPixmap *pix = pixmap();
     QMovie *mov = movie();
-    if ( pix ) {
+    if ( pix )
 	br = QRect( 0, 0, pix->width(), pix->height() );
-    } else if ( mov ) {
+    else if ( mov )
 	br = QRect( 0, 0, mov->framePixmap().width(),
 		mov->framePixmap().height() );
-    } else {
+    else
 	br = p.boundingRect( 0,0, 1000,1000, alignment(), text() );
-	// adjust so "Yes" and "yes" will have the same height
-	int h = fontMetrics().lineSpacing();
-	br.setHeight( ((br.height() + h-1) / h)*h - fontMetrics().leading() );
-    }
     int m  = 2*margin();
     int fw = frameWidth();
     if ( m < 0 ) {
@@ -497,6 +487,7 @@ QSize QLabel::sizeHint() const
     }
     int w = br.width()	+ m + 2*fw;
     int h = br.height() + m + 2*fw;
+    p.end();
 
     return QSize( w, h );
 }
@@ -529,7 +520,7 @@ void QLabel::drawContents( QPainter *p )
 
     QMovie *mov = movie();
     if ( mov ) {
-	// ### should add movie to qDrawItem when this Dict workaround is gone
+	// ### should add movie to qDrawItem when this Dict hack is gone
 	QRect r = qItemRect( p, style(),
 			cr.x(), cr.y(), cr.width(), cr.height(),
 			align, isEnabled(), &(mov->framePixmap()), ltext );
@@ -551,12 +542,11 @@ void QLabel::drawContents( QPainter *p )
 
 void QLabel::updateLabel()
 {
-    // ##### perhaps we should just use repaint(contentsRect())
-
-    QPainter paint( this );
-    if ( backgroundMode() != NoBackground )
-	paint.eraseRect( contentsRect() );
+    QPainter paint;
+    paint.begin( this );
+    paint.eraseRect( contentsRect() );
     drawContents( &paint );
+    paint.end();
 }
 
 
@@ -570,12 +560,15 @@ void QLabel::acceleratorSlot()
 	return;
 
     QLabel_Private * that = qlabel_extraStuff->find( (long)this );
-    if ( that && that->buddy && 
-	 !that->buddy->hasFocus() &&
-	 that->buddy->isEnabledToTLW() &&
-	 that->buddy->isVisibleToTLW() &&
-	 that->buddy->focusPolicy() != NoFocus )
+    if ( that && that->buddy && !that->buddy->hasFocus() ) {
+	QWidget * w = that->buddy;
+	while ( w ) {
+	    if ( !w->isVisible() )
+		return;
+	    w = w->parentWidget();
+	}
 	that->buddy->setFocus();
+    }
 }
 
 
@@ -621,7 +614,7 @@ void QLabel::setBuddy( QWidget *buddy )
 	that->buddy = buddy;
 	that->accel = new QAccel( this, "accel label accel" );
     }
-		
+		    
     const char * p = ltext.isEmpty() ? 0 : strchr( ltext, '&' );
     while( p && *p && p[1] == '&' )
 	p = strchr( p+2, '&' );
@@ -682,7 +675,7 @@ void QLabel::movieResized(const QSize& size)
 */
 void QLabel::setMovie( const QMovie& movie )
 {
-    //## ugle private data.
+    // Foul and ugly private data hack.
     if ( !qlabel_extraStuff ) {
 	qlabel_extraStuff = new QIntDict<QLabel_Private>;
 	CHECK_PTR( qlabel_extraStuff );
