@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qlcdnum.cpp,v 2.2 1997/03/12 15:31:52 paul Exp $
+** $Id: qlcdnum.cpp,v 2.6 1997/09/30 10:46:58 eiriken Exp $
 **
 ** Implementation of QLCDNumber class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -22,7 +25,7 @@
 #include "qpainter.h"
 #include <stdio.h>
 
-RCSTAG("$Id: qlcdnum.cpp,v 2.2 1997/03/12 15:31:52 paul Exp $");
+RCSTAG("$Id: qlcdnum.cpp,v 2.6 1997/09/30 10:46:58 eiriken Exp $");
 
 
 /*!
@@ -62,6 +65,8 @@ RCSTAG("$Id: qlcdnum.cpp,v 2.2 1997/03/12 15:31:52 paul Exp $");
   to a BASIC program on the <a
   href=http://www.nvg.unit.no/spectrum/>Sinclair Spectrum</a>.
 
+  <img src=qlcdnum-m.gif> <img src=qlcdnum-w.gif>
+
   \sa QLabel, QFrame
 */
 
@@ -788,9 +793,9 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
     QColorGroup g = colorGroup();
     QColor lightColor,darkColor,fgColor;
     if ( erase ){
-	lightColor = g.background();
-	darkColor  = g.background();
-	fgColor    = g.background();
+	lightColor = backgroundColor();
+	darkColor  = lightColor;
+	fgColor    = lightColor;
     } else {
 	lightColor = g.light();
 	darkColor  = g.dark();
@@ -1082,9 +1087,12 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
   <li>\c Outline gives raised segments filled with the background color. 
   <li>\c Filled gives raised segments filled with the foreground color.
   <li>\c Flat gives flat segments filled with the foreground color.
-  
-  \sa segmentStyle()
-*/
+  </ul>
+
+  \c Outline and \c Filled will additionally use QColorGroup::light()
+  and QColorGroup::dark() for shadow effects.
+
+  \sa segmentStyle() */
 
 void QLCDNumber::setSegmentStyle( SegmentStyle s )
 {
