@@ -1,5 +1,5 @@
 /**********************************************************************
-** $Id: qlined.cpp,v 2.16 1997/04/07 12:50:58 agulbra Exp $
+** $Id: qlined.cpp,v 2.50 1997/10/06 13:02:56 agulbra Exp $
 **
 ** Implementation of QLineEdit widget class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -25,10 +28,11 @@
 #include "qkeycode.h"
 #include "qclipbrd.h"
 #include "qapp.h"
+#include "qvalidator.h"
 
 #include <ctype.h>
 
-RCSTAG("$Id: qlined.cpp,v 2.16 1997/04/07 12:50:58 agulbra Exp $");
+RCSTAG("$Id: qlined.cpp,v 2.50 1997/10/06 13:02:56 agulbra Exp $");
 
 //### How to provide new member variables while keeping binary compatibility:
 #if QT_VERSION == 200
@@ -41,6 +45,7 @@ RCSTAG("$Id: qlined.cpp,v 2.16 1997/04/07 12:50:58 agulbra Exp $");
 struct QLineEditExtra {
     bool frame;
     QLineEdit::EchoMode mode;
+    QValidator * validator;
 };
 
 
@@ -66,6 +71,7 @@ static QLineEditExtra * makeLEDict( QLineEdit * that )
 	x = new QLineEditExtra;
 	x->frame = TRUE;
 	x->mode = QLineEdit::Normal;
+	x->validator = 0;
 	qle_extraStuff->insert( (long)that, x );
     }
 
@@ -109,6 +115,8 @@ static QLineEditExtra * lookInLEDict( const QLineEdit * that )
 
   The default key bindings are described in keyPressEvent(); they cannot
   be customized except by inheriting the class.
+
+  <img src=qlined-m.gif> <img src=qlined-w.gif>
 */
 
 
@@ -167,7 +175,8 @@ QLineEdit::QLineEdit( QWidget *parent, const char *name )
     : QWidget( parent, name )
 {
     initMetaObject();
-    pm		  = 0;
+    pm = new QPixmap( width(), height() );   // used for flicker-free update
+    CHECK_PTR( pm );
     cursorPos	  = 0;
     offset	  = 0;
     maxLen	  = 32767;
@@ -179,6 +188,7 @@ QLineEdit::QLineEdit( QWidget *parent, const char *name )
     tbuf	  = "";
     setFocusPolicy( StrongFocus );
     setCursor( ibeamCursor );
+    setBackgroundMode( PaletteBase );
 }
 
 /*!
@@ -194,7 +204,8 @@ QLineEdit::~QLineEdit()
 
 
 /*!
-  Sets the line editor text to \e text.
+  Sets the line editor text to \e text, clears the selection and moves
+  the cursor to the end of the line.
 
   If necessary the text is truncated to fit maxLength().
 
@@ -203,22 +214,31 @@ QLineEdit::~QLineEdit()
 
 void QLineEdit::setText( const char *text )
 {
-    if ( tbuf == text )				// no change
-	return;
+    QString oldText( tbuf );
+    oldText.detach();
     tbuf = text ? text : "";
-    if ( (int)tbuf.length() > maxLen )
+    if ( (int)tbuf.length() > maxLen ) {
 	tbuf.resize( maxLen+1 );
-    cursorPos = 0;
+	tbuf[maxLen] = '\0';
+    }
     offset    = 0;
+    cursorPos = 0;
+    markAnchor = 0;
+    markDrag = 0;
+    end( FALSE );
+    if ( validator() )
+	(void)validator()->validate( tbuf, cursorPos );
     repaint( !hasFocus() );
-    emit textChanged( tbuf.data() );
+    if ( oldText != tbuf )
+	emit textChanged( tbuf );
 }
 
 
 /*!
-  Selects all text (i.e. marks it) and does an "end" operation. Useful
-  when a default value has been inserted. If the user types before
-  clicking on the widget the selected text will be erased.
+  Selects all text (i.e. marks it) and moves the cursor to the
+  end. Useful when a default value has been inserted. If the user
+  types before clicking on the widget the selected text will be
+  erased.
 */
 
 void QLineEdit::selectAll()
@@ -258,7 +278,7 @@ void QLineEdit::deselect()
 
 const char *QLineEdit::text() const
 {
-    return tbuf.data();
+    return tbuf;
 }
 
 /*!
@@ -308,8 +328,10 @@ void QLineEdit::setMaxLength( int m )
     maxLen = m;
     markAnchor = 0;
     markDrag   = 0;
-    if ( (int)tbuf.length() > maxLen )
+    if ( (int)tbuf.length() > maxLen ) {
 	tbuf.resize( maxLen + 1 );		// include \0
+	tbuf[maxLen] = '\0';
+    }
     offset    = 0;
     cursorPos = 0;
     repaint( !hasFocus() );
@@ -324,7 +346,9 @@ void QLineEdit::setMaxLength( int m )
   The key press event handler converts a key press to some line editor
   action.
 
-  If return or enter is pressed, the signal returnPressed will be emitted.
+  If return or enter is pressed and the current text is valid (or if
+  the validator can \link QValidator::fixup() make the text
+  valid\endlink), the signal returnPressed is emitted.
 
   Here are the default key bindings:
   <ul>
@@ -353,22 +377,42 @@ void QLineEdit::setMaxLength( int m )
 void QLineEdit::keyPressEvent( QKeyEvent *e )
 {
     if ( e->key() == Key_Enter || e->key() == Key_Return ) {
-	emit returnPressed();
+	QValidator * v = validator();
+	if ( !v || v->validate( tbuf, cursorPos ) == QValidator::Acceptable ) {
+	    emit returnPressed();
+	} else if ( v ) {
+	    v->fixup( tbuf );
+	    if ( v->validate( tbuf, cursorPos ) == QValidator::Acceptable )
+		 emit returnPressed();
+	}
+	e->ignore();
 	return;
     }
-    if ( e->ascii() >= 32 && e->key() != Key_Delete ) {
+    QValidator * v = validator();
+    if ( e->ascii() >= 32 &&
+	 e->key() != Key_Delete &&
+	 e->key() != Key_Backspace ) {
+	QString test( tbuf.copy() );
+	int cp = cursorPos;
 	if ( hasMarkedText() ) {
-	    tbuf.remove( minMark(), maxMark() - minMark() );
-	    cursorPos = minMark();
-	    if ( cursorPos < offset )
-		offset = cursorPos;
+	    test.remove( minMark(), maxMark() - minMark() );
+	    cp = minMark();
 	}
-	if ( (int)tbuf.length() < maxLen ) {
-	    tbuf.insert( cursorPos, e->ascii() );
-	    cursorRight( FALSE );			// will repaint
-	    emit textChanged( tbuf.data() );
+	if ( (int)test.length() < maxLen )
+	    test.insert( cp, e->ascii() );
+	if ( v &&
+	     v->validate( test, cp ) == QValidator::Invalid &&
+	     v->validate( tbuf, cursorPos ) != QValidator::Invalid ) {
+	    // add stuff to indicate the error here and suggest remedies
+	    return;
 	}
 
+	if ( test != tbuf ) {
+	    cursorPos = cp;
+	    tbuf = test;
+	    cursorRight( FALSE );
+	    emit textChanged( tbuf );
+	}
 	return;
     }
     int unknown = 0;
@@ -382,9 +426,8 @@ void QLineEdit::keyPressEvent( QKeyEvent *e )
 	    cursorLeft( e->state() & ShiftButton );
 	    break;
 	case Key_C:
-	    if ( hasMarkedText() ) {
+	    if ( hasMarkedText() )
 		copyText();
-	    }
 	    break;
 	case Key_D:
 	    del();
@@ -402,6 +445,7 @@ void QLineEdit::keyPressEvent( QKeyEvent *e )
 	case Key_V: {			// paste
 	    QString t = QApplication::clipboard()->text();
 	    if ( !t.isEmpty() ) {
+		t.detach();
 		int i = t.find( '\n' );	// no multiline text
 		if ( i >= 0 )
 		    t.truncate( i );
@@ -413,21 +457,38 @@ void QLineEdit::keyPressEvent( QKeyEvent *e )
 		}
 		int tlen = t.length();
 		int blen;
+		int cp = cursorPos;
+		// do a test run without hurting tbuf and stuf
+		QString test( tbuf.copy() );
 		if ( hasMarkedText() ) {
-		    tbuf.remove( minMark(), maxMark() - minMark() );
-		    cursorPos = minMark();
-		    if ( cursorPos < offset )
-			offset = cursorPos;
+		    test.remove( minMark(), maxMark() - minMark() );
+		    if ( cp > maxMark() )
+			cp -= (maxMark() - minMark());
+		    else if ( cp > minMark() )
+			cp = minMark();
 		}
-		blen = tbuf.length();
+		blen = test.length();
 		if ( tlen+blen >= maxLen ) {
 		    if ( blen >= maxLen )
 			break;
-		    t.truncate( maxLen-tlen );
+		    if ( tlen > maxLen )
+			tlen = maxLen;
+		    t.truncate( maxLen-blen+1 );
+		    t[maxLen-blen] = '\0';
 		}
-		tbuf.insert( cursorPos, t );
-		cursorRight( FALSE, tlen );
-		emit textChanged( tbuf.data() );
+		test.insert( cp, t );
+		cp += t.length();
+
+		if ( v &&
+		     v->validate( test, cp ) == QValidator::Invalid &&
+		     v->validate( tbuf, cursorPos ) != QValidator::Invalid )
+		    break;
+
+		// okay, it succeeded, so use those changes.
+		tbuf = test;
+		cursorPos = cp;
+		repaint( FALSE );
+		emit textChanged( tbuf );
 	    }
 	}
 	case Key_X:
@@ -477,8 +538,8 @@ void QLineEdit::keyPressEvent( QKeyEvent *e )
 
 void QLineEdit::focusInEvent( QFocusEvent * )
 {
-    pm = new QPixmap( width(), height() );   // used for flicker-free update
-    CHECK_PTR( pm );
+    if ( style() == WindowsStyle && markAnchor == markDrag )
+	selectAll(); // calls repaint
     killTimers();
     startTimer( blinkTime );
     cursorOn = TRUE;
@@ -492,9 +553,12 @@ void QLineEdit::focusInEvent( QFocusEvent * )
 
 void QLineEdit::focusOutEvent( QFocusEvent * )
 {
+    if ( style() == WindowsStyle && 
+	 ( focusWidget() != this ||
+	   qApp->focusWidget() == 0 ||
+	   qApp->focusWidget()->topLevelWidget() != topLevelWidget() ) )
+	deselect();
     killTimers();
-    delete pm;
-    pm		  = 0;
     cursorOn	  = FALSE;
     dragScrolling = FALSE;
     repaint( !hasFocus() );
@@ -505,9 +569,9 @@ void QLineEdit::focusOutEvent( QFocusEvent * )
   Handles paint events for the line editor.
 */  
 
-void QLineEdit::paintEvent( QPaintEvent * )
+void QLineEdit::paintEvent( QPaintEvent *event )
 {
-    paint( TRUE );
+    paint( event->rect(), TRUE /* remove for 2.0 */ );
 }
 
 
@@ -538,10 +602,7 @@ void QLineEdit::timerEvent( QTimerEvent * )
 
 void QLineEdit::resizeEvent( QResizeEvent *e )
 {
-    if ( hasFocus() ) {
-	delete pm;
-	pm = new QPixmap( e->size().width(), e->size().height() );
-    }
+    pm->resize( e->size() );
     int max = lastCharVisible();
     if ( cursorPos > max ) {
 	QFontMetrics fm = fontMetrics();
@@ -554,6 +615,11 @@ void QLineEdit::resizeEvent( QResizeEvent *e )
 	if ( w < 0 && i != cursorPos )
 	    i++;
 	offset = i;
+    } else if ( offset ) {
+	int i = showLastPartOffset( tbuf.data(), fontMetrics(),
+				    width() - (frame() ? 8 : 4) );
+	if ( i < offset )
+	    offset = i;
     }
     repaint( !hasFocus() );
 }
@@ -634,7 +700,8 @@ void QLineEdit::mouseMoveEvent( QMouseEvent *e )
 
 void QLineEdit::mouseReleaseEvent( QMouseEvent * )
 {
-    copyText();
+    if ( style() == MotifStyle )
+	copyText();
     if ( dragScrolling ) {
 	dragScrolling = FALSE;
 	killTimers();
@@ -664,14 +731,15 @@ void QLineEdit::mouseDoubleClickEvent( QMouseEvent * )
   but flickering drawing method is used.
 */
 
-void QLineEdit::paint( bool frame )
+void QLineEdit::paint( const QRect& clip, bool )
 {
     if ( hasFocus() ) {
-	pixmapPaint();
+	pixmapPaint( clip );
     } else {
 	QPainter p;
 	p.begin( this );
-	paintText( &p, size(), frame );
+	p.setClipRect( clip );
+	paintText( &p, size(), TRUE /* remove for 2.0 */ );
 	p.end();
     }
 }
@@ -681,14 +749,12 @@ void QLineEdit::paint( bool frame )
   Paints the line editor in a pixmap and then blts the pixmap onto the screen.
 */
 
-void QLineEdit::pixmapPaint()
+void QLineEdit::pixmapPaint( const QRect& clip )
 {
-    if ( pm == 0 ) // be paranoid
-	pm = new QPixmap( width(), height() );
-    QPainter p;
-    p.begin( pm );
+    QPainter p( pm );
+    p.setClipRect( clip );
     p.setFont( font() );
-    paintText( &p, pm->size() , TRUE );
+    paintText( &p, pm->size(), TRUE /* remove for 2.0 */ );
     p.end();
     bitBlt( this, 0, 0, pm, 0, 0, width(), height() );
 }
@@ -699,7 +765,7 @@ void QLineEdit::pixmapPaint()
   Paints the line editor.
 */
 
-void QLineEdit::paintText( QPainter *p, const QSize &s, bool frame )
+void QLineEdit::paintText( QPainter *p, const QSize &s, bool )
 {
     QColorGroup	 g    = colorGroup();
     QColor	 bg   = isEnabled() ? g.base() : g.background();
@@ -708,13 +774,11 @@ void QLineEdit::paintText( QPainter *p, const QSize &s, bool frame )
     int markEnd	      = maxMark();
     int margin	     =  this->frame() ? 4 : 2;
 
-    if ( frame ) {
+    if ( frame() ) {
 	QBrush fill( bg );
 	qDrawWinPanel( p, 0, 0, s.width(), s.height(), g, TRUE, &fill );
     } else {
-	p->fillRect( margin, margin,
-		     width()  - 2*margin,
-		     height() - 2*margin, bg );
+	p->fillRect( 0, 0, width(), height(), bg );
     }
 
     QString displayText;
@@ -763,14 +827,16 @@ void QLineEdit::paintText( QPainter *p, const QSize &s, bool frame )
 	    mark2 = 0;
 	}
 
-	// display code come here - a bit yucky but it works
+	// display code comes here - a bit yucky but it works
 	if ( mark1 != mark2 ) {
 	    QString marked( displayText.mid( mark1, mark2 - mark1 ) );
 	    int xpos1 =  margin + fm.width( displayText, mark1 );
 	    int xpos2 =  xpos1 + fm.width( marked ) - 1;
 	    p->fillRect( xpos1, ypos - fm.ascent(),
-			 xpos2 - xpos1, fm.height(), g.text() );
-	    p->setPen( g.base() );
+			 xpos2 - xpos1, fm.height(),
+			 style() == WindowsStyle ?
+			 QApplication::winStyleHighlightColor() : g.text() );
+	    p->setPen(  style() == WindowsStyle ? white : g.base() );
 	    p->drawText( xpos1, ypos, marked );
 	}
 	p->setPen( g.text() );
@@ -792,12 +858,13 @@ void QLineEdit::paintText( QPainter *p, const QSize &s, bool frame )
 	    curXPos += fm.width( displayText, cursorPos - offset ) - 1;
 	int curYPos   = ypos - fm.ascent();
 	if ( hasFocus() ) {
-	    p->drawLine( curXPos - 2, curYPos,
-			 curXPos + 2, curYPos );
-	    p->drawLine( curXPos    , curYPos,
-			 curXPos    , curYPos + fm.height() - 1);
-	    p->drawLine( curXPos - 2, curYPos + fm.height() - 1,
-			 curXPos + 2, curYPos + fm.height() - 1);
+	    p->drawLine( curXPos, curYPos, curXPos, curYPos + fm.height() - 1);
+	    if ( style() != WindowsStyle ) {
+		p->drawLine( curXPos - 2, curYPos,
+			     curXPos + 2, curYPos );
+		p->drawLine( curXPos - 2, curYPos + fm.height() - 1,
+			     curXPos + 2, curYPos + fm.height() - 1);
+	    }
 	}
     }
 }
@@ -901,24 +968,34 @@ void QLineEdit::backspace()
 void QLineEdit::del()
 {
     QString test( tbuf.copy() );
+    QValidator * v = validator();
 
     if ( hasMarkedText() ) {
 	test.remove( minMark(), maxMark() - minMark() );
+	int cp = minMark();
+	if ( v &&
+	     v->validate( test, cp ) == QValidator::Invalid &&
+	     v->validate( tbuf, cursorPos ) != QValidator::Invalid )
+	    return;
 	tbuf = test;
-	cursorPos  = minMark();
+	cursorPos = cp;
 	markAnchor = cursorPos;
 	markDrag   = cursorPos;
 	if ( cursorPos < offset )
 	    offset = cursorPos;
 	repaint( !hasFocus() );
-	emit textChanged( tbuf.data() );
-    } else {
-	if ( cursorPos != (int)strlen(tbuf) ) {
-	    test.remove( cursorPos, 1 );
-	    tbuf = test;
-	    repaint( !hasFocus() );
-	    emit textChanged( tbuf.data() );
-	}
+	emit textChanged( tbuf );
+    } else if ( cursorPos != (int)strlen(tbuf) ) {
+	test.remove( cursorPos, 1 );
+	int cp = cursorPos;
+	if ( v &&
+	     v->validate( test, cp ) == QValidator::Invalid &&
+	     v->validate( tbuf, cursorPos ) != QValidator::Invalid )
+	    return;
+	tbuf = test;
+	cursorPos = cp;
+	repaint( !hasFocus() );
+	emit textChanged( tbuf );
     }
 }
 
@@ -982,7 +1059,7 @@ void QLineEdit::newMark( int pos, bool copy )
 {
     markDrag  = pos;
     cursorPos = pos;
-    if ( copy )
+    if ( copy && style() == MotifStyle ) // ### ?
 	copyText();
 }
 
@@ -1014,7 +1091,8 @@ void QLineEdit::markWord( int pos )
     } else {
 	cursorPos = markBegin;
     }
-    copyText();
+    if ( style() == MotifStyle )
+	copyText();
 }
 
 
@@ -1026,14 +1104,10 @@ void QLineEdit::copyText()
 {
     QString t = markedText();
     if ( !t.isEmpty() ) {
-#if defined(_WS_X11_)
 	disconnect( QApplication::clipboard(), SIGNAL(dataChanged()), this, 0);
-#endif
 	QApplication::clipboard()->setText( t );
-#if defined(_WS_X11_)
 	connect( QApplication::clipboard(), SIGNAL(dataChanged()),
 		 this, SLOT(clipboardChanged()) );
-#endif
     }
 }
 
@@ -1074,7 +1148,7 @@ int QLineEdit::maxMark() const
 
 
 
-/*!  Sets the line edit to draw itself inside a two-pixel frame iff \a
+/*!  Sets the line edit to draw itself inside a two-pixel frame if \a
   enable is TRUE, and to draw itself without any frame if \a enable is
   FALSE.
 
@@ -1085,7 +1159,7 @@ int QLineEdit::maxMark() const
 
 void QLineEdit::setFrame( bool enable )
 {
-    if ( !enable && !qle_extraStuff )
+    if ( enable && !qle_extraStuff )
 	return;
 
     QLineEditExtra * x = makeLEDict( this );
@@ -1144,3 +1218,57 @@ QLineEdit::EchoMode QLineEdit::echoMode() const
     QLineEditExtra * x = lookInLEDict( this );
     return x ? x->mode : Normal;
 }
+
+
+/*!
+  Returns a size which fits the contents of the line edit.
+
+  The width returned tends to be enough for about 15-20 characters.
+*/
+
+QSize QLineEdit::sizeHint() const
+{
+    int h = fontMetrics().height();
+    int margin = frame() ? 8 : 4;
+    return QSize( 10*h + margin, h + margin );
+}
+
+
+/*!
+  Sets this line edit to accept input only as accepted by \a v.
+
+  If \a v == 0, remove the currently set input validator.  The default
+  is no input validator (ie. any input is accepted up to maxLength()).
+
+  \sa validator() QValidator
+*/
+
+
+void QLineEdit::setValidator( QValidator * v )
+{
+    if ( v == 0 && !qle_extraStuff )
+	return;
+
+    QLineEditExtra * x = makeLEDict( this );
+    x->validator = v;
+    repaint();
+}
+
+/*!
+  Returns a pointer to the current input validator, or 0 if no
+  validator has been set.
+*/
+
+QValidator * QLineEdit::validator() const
+{
+    QLineEditExtra * x = lookInLEDict( this );
+    return x ? x->validator : 0;
+}
+
+
+/*!  This slot is equivalent to setValidator( 0 ). */
+
+void QLineEdit::clearValidator()
+{
+    setValidator( 0 );
+}
