@@ -1,34 +1,34 @@
 /**********************************************************************
-** $Id: qlistbox.cpp,v 2.71 1998/07/03 00:09:49 hanord Exp $
+** $Id: qlistbox.cpp,v 2.49 1997/10/07 14:24:15 agulbra Exp $
 **
 ** Implementation of QListBox widget class
 **
 ** Created : 941121
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qlistbox.h"
-#include "qfontmetrics.h"
+#include "qfontmet.h"
 #include "qpainter.h"
 #include "qstrlist.h"
 #include "qkeycode.h"
-#include "qscrollbar.h"
 #include "qpixmap.h"
-#include "qapplication.h"
+#include "qapp.h"
+
+RCSTAG("$Id: qlistbox.cpp,v 2.49 1997/10/07 14:24:15 agulbra Exp $");
 
 Q_DECLARE(QListM, QListBoxItem);
 
@@ -47,28 +47,24 @@ int QLBItemList::compareItems( GCI i1, GCI i2)
 }
 
 
-static inline bool checkInsertIndex( const char *method, const char * name,
-				     int count, int *index)
+static inline bool checkInsertIndex( const char *method, int count, int *index)
 {
     bool range_err = (*index > count);
 #if defined(CHECK_RANGE)
     if ( *index > count )
-	warning( "QListBox::%s: (%s) Index %i out of range",
-		 method, name ? name : "<no name>", *index );
+	warning( "QListBox::%s: Index %i out of range", method, *index );
 #endif
     if ( *index < 0 )				// append
 	*index = count;
     return !range_err;
 }
 
-static inline bool checkIndex( const char *method, const char * name,
-			       int count, int index )
+static inline bool checkIndex( const char *method, int count, int index )
 {
     bool range_err = ((uint)index >= (uint)count);
 #if defined(CHECK_RANGE)
     if ( range_err )
-	warning( "QListBox::%s: (%s) Index %d out of range",
-		 method, name ? name : "<no name>", index );
+	warning( "QListBox::%s: Index %d out of range", method, index );
 #endif
     return !range_err;
 }
@@ -78,6 +74,8 @@ static inline bool checkIndex( const char *method, const char * name,
   \class QListBoxItem qlistbox.h
   \brief This is the base class of all list box items.
 
+  \ingroup abstract
+
   This class is the abstract base class of all list box items. If you
   need to insert customized items into a QListBox, you must inherit
   this class and reimplement paint(), height() and width().
@@ -200,7 +198,7 @@ QListBoxItem::~QListBoxItem()
 
 /*!
   \class QListBoxText qlistbox.h
-  \brief The QListBoxText class provides list box items with text.
+  \brief QListBoxText provides list box items with text.
 
   The text is drawn in the widget's current font. If you need several
   different fonts, you have to make your own subclass of QListBoxItem.
@@ -266,7 +264,7 @@ int QListBoxText::width( const QListBox *lb ) const
 
 /*!
   \class QListBoxPixmap qlistbox.h
-  \brief The QListBoxPixmap class provides list box items with a pixmap.
+  \brief QListBoxPixmap provides list box items with a pixmap.
 
   \sa QListBox, QListBoxItem
 */
@@ -344,7 +342,7 @@ int QListBoxPixmap::width( const QListBox * ) const
   If the user does not select anything, no signals are emitted and
   currentItem() returns -1.
 
-  A list box has \c StrongFocus as a default focusPolicy(), i.e. it can
+  A list box has \c StrongFocus as a default focusPolicy(), i.e. it can 
   get keyboard focus both by tabbing and clicking.
 
   New items may be inserted using either insertItem(), insertStrList()
@@ -376,10 +374,6 @@ int QListBoxPixmap::width( const QListBox * ) const
   and will delete them when they are not needed.
 
   <img src=qlistbox-m.gif> <img src=qlistbox-w.gif>
-
-  \sa QListView QComboBox QButtonGroup
-  <a href="guibooks.html#fowler">GUI Design Handbook: List Box (two
-  sections)</a>
 */
 
 
@@ -390,7 +384,7 @@ int QListBoxPixmap::width( const QListBox * ) const
 #error "Remove QListBox dict."
 #endif
 
-#include "qintdict.h"
+#include <qintdict.h>
 
 static QIntDict<int> *qlb_maxLenDict = 0;
 
@@ -414,6 +408,7 @@ static void cleanupListbox()
 QListBox::QListBox( QWidget *parent, const char *name, WFlags f )
     : QTableView( parent, name, f )
 {
+    initMetaObject();
     doDrag	  = TRUE;
     doAutoScroll  = TRUE;
     current	  = -1;
@@ -448,20 +443,12 @@ QListBox::QListBox( QWidget *parent, const char *name, WFlags f )
     }
 }
 
-#if QT_VERSION == 200
-#error "Remove QListBox pointer."
-#endif
-
-static QListBox * changedListBox = 0;
-
 /*!
   Destroys the list box.  Deletes all list box items.
 */
 
 QListBox::~QListBox()
 {
-    if ( changedListBox == this )
-	changedListBox = 0;
     goingDown = TRUE;
     clearList();
     if ( qlb_maxLenDict )
@@ -475,7 +462,7 @@ QListBox::~QListBox()
   listbox changes. If the user selects five items by drag-selecting,
   QListBox tries to emit just one selectionChanged() signal, so the
   signal can be connected to computationally expensive slots.
-
+  
   \sa selected() currentItem()
 */
 
@@ -491,7 +478,7 @@ QListBox::~QListBox()
 
   This signal is emitted when the user highlights a new current item
   and the new item is a string.  The argument is the text of the
-  new current item.
+  new current item.  
 
   \sa selected() currentItem() selectionChanged()
 */
@@ -516,8 +503,7 @@ QListBox::~QListBox()
 */
 
 /*!
-  Reimplements QWidget::setFont() to update the list box line height
-  and maxItemWidth().
+  Reimplements QWidget::setFont() to update the list box line height.
 */
 
 void QListBox::setFont( const QFont &font )
@@ -525,9 +511,6 @@ void QListBox::setFont( const QFont &font )
     QWidget::setFont( font );
     if ( stringsOnly )
 	setCellHeight( fontMetrics().lineSpacing() + 1 );
-    // else ...?
-
-    updateCellWidth();
 }
 
 
@@ -552,7 +535,7 @@ uint QListBox::count() const
 
 void QListBox::insertStrList( const QStrList *list, int index )
 {
-    if ( !checkInsertIndex( "insertStrList", name(), count(), &index ) )
+    if ( !checkInsertIndex( "insertStrList", count(), &index ) )
 	return;
     if ( !list ) {
 #if defined(CHECK_NULL)
@@ -587,7 +570,7 @@ void QListBox::insertStrList( const QStrList *list, int index )
 
 void QListBox::insertStrList( const char **strings, int numStrings, int index )
 {
-    if ( !checkInsertIndex( "insertStrList", name(), count(), &index ) )
+    if ( !checkInsertIndex( "insertStrList", count(), &index ) )
 	return;
     if ( !strings ) {
 #if defined(CHECK_NULL)
@@ -621,7 +604,7 @@ void QListBox::insertStrList( const char **strings, int numStrings, int index )
 
 void QListBox::insertItem( const QListBoxItem *lbi, int index )
 {
-    if ( !checkInsertIndex( "insertItem", name(), count(), &index ) )
+    if ( !checkInsertIndex( "insertItem", count(), &index ) )
 	return;
     if ( !lbi ) {
 #if defined ( CHECK_NULL )
@@ -651,7 +634,7 @@ void QListBox::insertItem( const QListBoxItem *lbi, int index )
 
 void QListBox::insertItem( const char *text, int index )
 {
-    if ( !checkInsertIndex( "insertItem", name(), count(), &index ) )
+    if ( !checkInsertIndex( "insertItem", count(), &index ) )
 	return;
     if ( !text ) {
 #if defined ( CHECK_NULL )
@@ -681,7 +664,7 @@ void QListBox::insertItem( const char *text, int index )
 
 void QListBox::insertItem( const QPixmap &pixmap, int index )
 {
-    if ( !checkInsertIndex( "insertItem", name(), count(), &index ) )
+    if ( !checkInsertIndex( "insertItem", count(), &index ) )
 	return;
     if ( stringsOnly ) {
 	stringsOnly = FALSE;
@@ -755,7 +738,7 @@ void QListBox::inSort( const char *text )
 
 void QListBox::removeItem( int index )
 {
-    if ( !checkIndex( "removeItem", name(), count(), index ) )
+    if ( !checkIndex( "removeItem", count(), index ) )
 	return;
     bool currentChanged = ( current == index );
 
@@ -830,7 +813,7 @@ const QPixmap *QListBox::pixmap( int index ) const
 
 void QListBox::changeItem( const char *text, int index )
 {
-    if ( !checkIndex( "changeItem", name(), count(), index ) )
+    if ( !checkIndex( "changeItem", count(), index ) )
 	return;
     change( new QListBoxText(text), index );
 }
@@ -845,7 +828,7 @@ void QListBox::changeItem( const char *text, int index )
 
 void QListBox::changeItem( const QPixmap &pixmap, int index )
 {
-    if ( !checkIndex( "changeItem", name(), count(), index ) )
+    if ( !checkIndex( "changeItem", count(), index ) )
 	return;
     change( new QListBoxPixmap(pixmap), index );
 }
@@ -860,7 +843,7 @@ void QListBox::changeItem( const QPixmap &pixmap, int index )
 
 void QListBox::changeItem( const QListBoxItem *lbi, int index )
 {
-    if ( !checkIndex( "changeItem", name(), count(), index ) )
+    if ( !checkIndex( "changeItem", count(), index ) )
 	return;
     change( lbi, index );
 }
@@ -904,8 +887,6 @@ void QListBox::setAutoUpdate( bool enable )
 /*!
   Returns the number of visible items.	This may change at any time
   since the user may resize the widget.
-
-  \sa setFixedVisibleLines()
 */
 
 int QListBox::numItemsVisible() const
@@ -935,7 +916,7 @@ void QListBox::setCurrentItem( int index )
 {
     if ( index == current )
 	return;
-    if ( !checkIndex( "setCurrentItem", name(), count(), index ) )
+    if ( !checkIndex( "setCurrentItem", count(), index ) )
 	return;
     int oldCurrent = current;
     current	   = index;
@@ -1196,7 +1177,7 @@ void QListBox::setSmoothScrolling( bool enable )
 
 QListBoxItem *QListBox::item( int index ) const
 {
-    if (!checkIndex( "item", name(), count(), index ) )
+    if (!checkIndex( "item", count(), index ) )
 	return 0;
     return itemList->at( index );
 }
@@ -1276,7 +1257,7 @@ void QListBox::paintCell( QPainter *p, int row, int col )
     lbi->paint( p );
     if ( current == row && hasFocus() ) {
 	if ( style() == WindowsStyle ) {
-	    p->drawWinFocusRect( 1, 1, cellWidth(col)-2 , cellHeight(row)-2,
+	    p->drawWinFocusRect( 1, 1, cellWidth(col)-2 , cellHeight(row)-2, 
 				 QApplication::winStyleHighlightColor() );
 	} else {
 	    if ( isSelected( row ) )
@@ -1352,19 +1333,15 @@ void QListBox::mouseMoveEvent( QMouseEvent *e )
 {
     if ( doDrag && (e->state() & (RightButton|LeftButton|MidButton)) != 0 ) {
 	int itemClicked = findItem( e->pos().y() );
-	if ( itemClicked >= 0 ) {
+	if ( itemClicked != -1 ) {
 	    if ( isTiming ) {
 		killTimer( itemList->timerId );
 		isTiming = FALSE;
 	    }
 	    if ( multiSelect ) {
-		bool s = currentItem() >= 0
-			 ? isSelected( currentItem() ) : TRUE;
 		int i = QMIN( itemClicked, currentItem() );
-		if ( i < 0 )
-		    i = 0;
 		while( i <= itemClicked || i <= currentItem() ) {
-		    setSelected( i, s );
+		    setSelected( i, isSelected( currentItem() ) );
 		    i++;
 		}
 	    }
@@ -1409,19 +1386,31 @@ void QListBox::keyPressEvent( QKeyEvent *e )
     switch ( e->key() ) {
     case Key_Up:
 	if ( currentItem() > 0 ) {
-	    ensureCurrentVisible( currentItem() - 1 );
+	    setCurrentItem( currentItem() - 1 );
+	    if ( currentItem() < topItem()	)
+		setTopItem( currentItem() );
 	    if ( e->state() & ShiftButton )
 		toggleCurrentItem();
 	}
-	e->accept();
 	break;
     case Key_Down:
 	if ( currentItem() < (int)count() - 1 ) {
-	    ensureCurrentVisible( currentItem()+1 );
+	    bool a = autoUpdate();
+	    bool u = FALSE;
+	    setAutoUpdate( FALSE );
+	    setCurrentItem( currentItem() + 1 );
+	    while ( currentItem() < (int)count() - 1
+		 && currentItem() >= lastRowVisible() )
+	    {
+		setTopItem( topItem() + currentItem() + 1 - lastRowVisible() );
+		u = TRUE;
+	    }
+	    setAutoUpdate( a );
 	    if ( e->state() & ShiftButton )
 		toggleCurrentItem();
+	    if ( u )
+		repaint();
 	}
-	e->accept();
 	break;
     case Key_Next:
 	if ( style() == MotifStyle ) {
@@ -1434,7 +1423,6 @@ void QListBox::keyPressEvent( QKeyEvent *e )
 	    setTopItem( QMIN(currentItem(),(int)count()-pageSize+1) );
 	    setCurrentItem( QMIN(lastRowVisible(), (int)count()-1) );
 	}
-	e->accept();
 	break;
     case Key_Prior:
 	if ( style() == MotifStyle ) {
@@ -1447,12 +1435,10 @@ void QListBox::keyPressEvent( QKeyEvent *e )
 	    setTopItem( QMAX(0,currentItem()-pageSize+1) );
 	    setCurrentItem( topItem() );
 	}
-	e->accept();
 	break;
 
     case Key_Space:
 	toggleCurrentItem();
-	e->accept();
 	break;
 
     case Key_Return:
@@ -1463,7 +1449,6 @@ void QListBox::keyPressEvent( QKeyEvent *e )
 	    if ( !tmp.isEmpty() )
 		emit selected( tmp );
 	}
-	// do NOT accept here.  qdialog.
 	break;
     default:
 	break;
@@ -1474,7 +1459,7 @@ void QListBox::keyPressEvent( QKeyEvent *e )
 
 /*!
   Handles focus events.  Repaints the current item (if not set,
-  topItem() is made current).
+  topItem() is made current).  
   \sa keyPressEvent(), focusOutEvent()
 */
 
@@ -1487,7 +1472,7 @@ void QListBox::focusInEvent( QFocusEvent * )
 }
 
 
-/*!
+/*! 
   Handles focus out events. Repaints the current item, if set.
   \sa keyPressEvent(), focusOutEvent()
 */
@@ -1508,7 +1493,7 @@ void QListBox::focusOutEvent( QFocusEvent * )
 void QListBox::resizeEvent( QResizeEvent *e )
 {
     QTableView::resizeEvent( e );
-    setCellWidth( QMAX((int)maxItemWidth(), viewWidth()) );
+    setCellWidth( QMAX(maxItemWidth(), viewWidth()) );
 }
 
 
@@ -1523,27 +1508,21 @@ void QListBox::timerEvent( QTimerEvent * )
 	    int y = QMAX(currentItem(),lastRowVisible())+1;
 	    if ( y >= (int)count() )
 		y = count() - 1;
-	    if ( currentItem() >= 0 && multiSelect ) {
-		bool s = isSelected( currentItem() );
-		int i = currentItem();
-		while( i <= y ) {
-		    setSelected( i, s );
-		    i++;
-		}
+	    int h = viewHeight();
+	    int i = y;
+	    while( i > 0 && h > 0 ) {
+		h -= cellHeight( i );
+		if ( h >= 0 )
+		    i--;
 	    }
-	    ensureCurrentVisible( y );
+	    if ( h <= 0 && i < y ) // make sure ALL of y is visible
+		i++;
+	    setTopItem( i );
+	    setCurrentItem( y );
 	}
     } else {
-	if ( topItem() > 0 ) {
+	if ( topItem() != 0 ) {
 	    setTopItem( topItem() - 1 );
-	    if ( currentItem() > 0 && multiSelect ) {
-		bool s = isSelected( currentItem() );
-		int i = currentItem();
-		while( i >= topItem() ) {
-		    setSelected( i, s );
-		    i--;
-		}
-	    }
 	    setCurrentItem( topItem() );
 	}
     }
@@ -1717,26 +1696,14 @@ void QListBox::updateNumRows( bool updateWidth )
 
 
 /*!
-  Returns the width in pixels of the widest item.
+  Returns the width in pixels of the longest item. 
 */
 
-long QListBox::maxItemWidth() const
-{
-    if ( !qlb_maxLenDict )
-	return 0;
-    return (long) qlb_maxLenDict->find( (long)this );
-}
-
-/*!
-  For binary compatibility.
-*/
 long QListBox::maxItemWidth()
 {
-    // This is only here for binary compatibility
     if ( !qlb_maxLenDict )
 	return 0;
     return (long) qlb_maxLenDict->find( (long)this );
-    // This is only here for binary compatibility
 }
 
 
@@ -1768,7 +1735,7 @@ void QListBox::setMaxItemWidth( int len )
   single-selection mode if \a enable is FALSE.
 
   Single- and multi-selections modes work the same, except that the
-  highlighted() and selected() signals are emitted at different times.
+  highlighed() and selected() signals are emitted at different times.
 
   \sa isMultiSelection()
 */
@@ -1811,7 +1778,7 @@ void QListBox::toggleCurrentItem()
   unselects it if \a select is FALSE.  May also repaint the item.
 
   If the listbox is a single-selection listbox and and \a select is TRUE,
-  setCurrentItem will be called.
+  setCurrentItem will be called. 
 
   If the listbox is a single-selection listbox and and \a select is FALSE,
   clearSelection() will be called if \a index is the currently selected
@@ -1846,7 +1813,7 @@ void QListBox::setSelected( int index, bool select )
 
 
 /*!
-  Returns TRUE if item \a i is selected. Returns FALSE if it is not
+  Returns TRUE if item \a i is selected. Returns FALSE if it is not 
   selected or if there is an error.
 */
 
@@ -1883,14 +1850,19 @@ void QListBox::clearSelection()
     }
 }
 
+    
+    
 
+#if QT_VERSION == 200
+#error "Remove QListBox pointer."
+#endif
 
-
+static QListBox * changedListBox = 0;
 
 /*!  If \a lazy is FALSE, maybe emit the changed() signal.  If \a lazy
   is TRUE, note that it's to be sent out at some later time.
 */
-
+ 
 void QListBox::emitChangedSignal( bool lazy ) {
     if ( !multiSelect )
 	return;
@@ -1900,63 +1872,3 @@ void QListBox::emitChangedSignal( bool lazy ) {
 
     changedListBox = lazy ? this : 0;
 }
-
-
-/*!  Make sure that all of currentItem() is on-screen */
-
-void QListBox::ensureCurrentVisible( int newCurrent )
-{
-    if ( newCurrent < 0 )
-	newCurrent = currentItem();
-    if ( newCurrent < topItem() ) {
-	setTopItem( newCurrent );
-    } else if ( newCurrent >= lastRowVisible() ) {
-	int h = viewHeight();
-	int i = newCurrent;
-	while( i > 0 && h > 0 ) {
-	    h -= cellHeight( i );
-	    if ( h >= 0 )
-		i--;
-	}
-	if ( h <= 0 && i < newCurrent ) // make sure ALL of y is visible
-	    i++;
-	setTopItem( i );
-    }
-    if ( newCurrent != currentItem() )
-	setCurrentItem( newCurrent );
-}
-
-/*!
-  Sets a \link QWidget::setFixedHeight() fixed height\endlink for the
-  widget, so that it shows the given number of lines of text for the
-  current font size.
-*/
-void QListBox::setFixedVisibleLines( int lines )
-{
-    int ls = fontMetrics().lineSpacing() + 1; // #### explain +1
-    // #### What about auto-scrollbars?
-    int sb = testTableFlags(Tbl_hScrollBar)
-		? horizontalScrollBar()->height() : 0;
-    setFixedHeight( frameWidth()*2 + ls*lines + sb );
-    return;
-}
-
-/*!
-  Returns a size mased on maxItemWidth() and any value set by
-  setFixedVisibleLines().
-*/
-QSize QListBox::sizeHint() const
-{
-    QSize sz = QTableView::sizeHint();
-
-    int w = (int)maxItemWidth() + 2*frameWidth();
-    if ( testTableFlags(Tbl_vScrollBar) )
-	w += verticalScrollBar()->width();
-    sz.setWidth(w);
-
-    // For when setFixedVisibleLines is used
-    int h = maximumSize().height();
-    if ( h<1000 ) sz.setHeight(h);
-
-    return sz;
-}
