@@ -1,23 +1,22 @@
 /****************************************************************************
-** $Id: qmenubar.cpp,v 2.55 1998/07/03 00:09:51 hanord Exp $
+** $Id: qmenubar.cpp,v 2.37 1997/10/09 14:11:53 agulbra Exp $
 **
 ** Implementation of QMenuBar class
 **
 ** Created : 941209
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -25,16 +24,18 @@
 #include "qmenubar.h"
 #include "qaccel.h"
 #include "qpainter.h"
-#include "qdrawutil.h"
-#include "qapplication.h"
+#include "qdrawutl.h"
+#include "qapp.h"
 #include <ctype.h>
 
+RCSTAG("$Id: qmenubar.cpp,v 2.37 1997/10/09 14:11:53 agulbra Exp $");
+
+
 /*!
   \class QMenuBar qmenubar.h
   \brief The QMenuBar class provides a horizontal menu bar.
 
   \ingroup realwidgets
-  \ingroup application
 
   It automatically sets its own geometry to the top of the parent
   widget and changes appropriately it when the parent widget is
@@ -45,7 +46,6 @@
   <img src=qmenubar-m.gif> <img src=qmenubar-w.gif>
 
   \sa QPopupMenu
-  <a href="guibooks.html#fowler">GUI Design Handbook: Menu Bar</a>
 */
 
 /*!
@@ -117,6 +117,7 @@ static const int motifItemVMargin	= 4;	// menu item ver text margin
 QMenuBar::QMenuBar( QWidget *parent, const char *name )
     : QFrame( parent, name, 0, FALSE )
 {
+    initMetaObject();
     isMenuBar = TRUE;
     autoaccel = 0;
     irects    = 0;
@@ -138,7 +139,7 @@ QMenuBar::QMenuBar( QWidget *parent, const char *name )
     if ( gs == WindowsStyle ) {
 	h = 2 + fm.height() + motifItemVMargin + 2*motifBarFrame;
     } else {
-	h =  motifBarFrame + motifBarVMargin + fm.height()
+	h =  motifBarFrame + motifBarVMargin + fm.height() 
 	    + motifItemVMargin + 2*motifBarFrame + 2*motifItemFrame;
     }
 
@@ -166,8 +167,7 @@ QMenuBar::QMenuBar( QWidget *parent, const char *name )
 QMenuBar::~QMenuBar()
 {
     delete autoaccel;
-    if ( irects )		// Avoid purify complaint.
-	delete [] irects;
+    delete [] irects;
 }
 
 /*!
@@ -176,7 +176,7 @@ QMenuBar::~QMenuBar()
 */
 void QMenuBar::updateItem( int )
 {
- //   repaint( FALSE ); //###H: avoid this until we get a better solution
+ //   repaint( FALSE ); !!!hanord: avoid this until we get a better solution
 }
 
 
@@ -190,7 +190,6 @@ void QMenuBar::updateItem( int )
 
 void QMenuBar::menuContentsChanged()
 {
-    setupAccelerators();
     badSize = TRUE;				// might change the size
     if ( isVisible() ) {
 	calculateRects();
@@ -208,7 +207,7 @@ void QMenuBar::menuContentsChanged()
 
 void QMenuBar::menuStateChanged()
 {
- //   repaint(); ###H: avoid this until we get a better solution
+ //   repaint(); !!!hanord: avoid this until we get a better solution
 }
 
 void QMenuBar::menuInsPopup( QPopupMenu *popup )
@@ -222,7 +221,7 @@ void QMenuBar::menuInsPopup( QPopupMenu *popup )
 
 void QMenuBar::menuDelPopup( QPopupMenu *popup )
 {
-    popup->parentMenu = 0;
+    popup->parentMenu = this;
     popup->disconnect( SIGNAL(activatedRedirect(int)), this,
 		       SLOT(subActivated(int)) );
     popup->disconnect( SIGNAL(highlightedRedirect(int)), this,
@@ -255,10 +254,10 @@ bool QMenuBar::eventFilter( QObject *object, QEvent *event )
 	return FALSE;
     }
 
-    if ( style() != WindowsStyle ||
+    if ( style() != WindowsStyle || 
 	 !isVisibleToTLW() ||
 	 !object->isWidgetType() ||
-	 !( event->type() == Event_Accel ||
+	 !( event->type() == Event_Accel || 
 	    event->type() == Event_KeyPress ||
 	    event->type() == Event_KeyRelease ) )
 	return FALSE;
@@ -282,9 +281,8 @@ bool QMenuBar::eventFilter( QObject *object, QEvent *event )
 	    }
 	}
 	// ### ! block all accelerator events when the menu bar is active
-	if ( qApp && qApp->focusWidget() == this ) {
+	if ( qApp && qApp->focusWidget() == this )
 	    return TRUE;
-	}
     }
 
     // look for Alt release
@@ -306,7 +304,6 @@ bool QMenuBar::eventFilter( QObject *object, QEvent *event )
 		tlw->removeEventFilter( this );
 		tlw->installEventFilter( this );
 	    }
-	    return TRUE;
 	} else if ( (event->type() == Event_KeyPress ||
 		     event->type() == Event_KeyRelease) &&
 		    !(((QKeyEvent *)event)->key() == Key_Alt ||
@@ -317,7 +314,7 @@ bool QMenuBar::eventFilter( QObject *object, QEvent *event )
 	}
     }
 
-    // look for top-level window focus changes
+    // watch for top-level window focus changes
     if ( windowsaltactive &&
 	 event->type() == Event_FocusOut &&
 	 qApp && qApp->focusWidget() != this ) {
@@ -433,7 +430,7 @@ void QMenuBar::openActPopup()
 	return;
     setWindowsAltMode( FALSE, actItem );
     QPopupMenu *popup = mitems->at(actItem)->popup();
-    if ( !popup || !popup->isEnabled() )
+    if ( !popup )
 	return;
 
     QRect  r = itemRect( actItem );
@@ -474,9 +471,37 @@ void QMenuBar::hidePopups()
 
 void QMenuBar::show()
 {
-    setupAccelerators();
-    if ( parentWidget() )
-	resize( parentWidget()->width(), height() );
+    QMenuItemListIt it(*mitems);
+    register QMenuItem *mi;
+    QWidget *w = parentWidget();
+    while ( (mi=it.current()) ) {
+	++it;
+	QString s = mi->text();
+	if ( !s.isEmpty() ) {
+	    int i = s.find( '&' );
+	    if ( i >= 0 && isalnum(s[i+1]) ) {
+		int k = s[i+1];
+		if ( isalpha(k) )
+		    k = toupper(k) - 'A' + Key_A;
+		if ( !autoaccel ) {
+		    autoaccel = new QAccel( this );
+		    CHECK_PTR( autoaccel );
+		    connect( autoaccel, SIGNAL(activated(int)),
+			     SLOT(accelActivated(int)) );
+		    connect( autoaccel, SIGNAL(destroyed()),
+			     SLOT(accelDestroyed()) );
+		}
+		autoaccel->insertItem( ALT+k, mi->id() );
+	    }
+	}
+	if ( mi->popup() ) {
+	    mi->popup()->updateAccel( this );
+	    if ( !mi->popup()->isEnabled() )
+		mi->popup()->enableAccel( FALSE );
+	}
+    }
+    if ( w )
+	resize( w->width(), height() );
     calculateRects();
     QWidget::show();
     raise();
@@ -522,10 +547,13 @@ int QMenuBar::calculateRects( int max_width )
     bool update = ( max_width < 0 );
 
     if ( update ) {
+        if (width() == 1) { // gcc Linux/Alpha
+	    show();
+	    return 0;
+	}
 	if ( !badSize )				// size was not changed
 	    return 0;
-	if ( irects )		// Avoid purify complaint.
-	    delete [] irects;
+	delete [] irects;
 	if ( mitems->isEmpty() ) {
 	    irects = 0;
 	    return 0;
@@ -542,7 +570,7 @@ int QMenuBar::calculateRects( int max_width )
     int y = motifBarFrame + motifBarVMargin;
     int i = 0;
     int separator = -1;
-    if ( gs == WindowsStyle )	//###H
+    if ( gs == WindowsStyle )	// !!!hanord
 	x = y = 2;
     while ( i < (int)mitems->count() ) {	// for each menu item...
 	QMenuItem *mi = mitems->at(i);
@@ -689,7 +717,7 @@ void QMenuBar::drawContents( QPainter *p )
 		g = palette().normal();
 	    else
 		g = palette().disabled();
-		
+		     
 	    if ( gs == WindowsStyle ) {
 		p->fillRect( r, palette().normal().background() );
 		if ( i == actItem ) {
@@ -909,7 +937,7 @@ void QMenuBar::keyPressEvent( QKeyEvent *e )
 	    else if ( i < 0 )
 		i = c - 1;
 	    mi = mitems->at( i );
-	    // ### fix windows-style traversal - currently broken due to
+	    // ### fix windows-style traversal - currently broken due to 
 	    // QMenuBar's reliance on QPopupMenu
 	    if ( /* (style() == WindowsStyle || */ mi->isEnabled() /* ) */
 		 && !mi->isSeparator() )
@@ -968,9 +996,14 @@ void QMenuBar::keyPressEvent( QKeyEvent *e )
   Handles resize events for the menu bar.
 */
 
-void QMenuBar::resizeEvent( QResizeEvent * )
+void QMenuBar::resizeEvent( QResizeEvent *e )
 {
+    QSize oldSize = e->oldSize();
+    QSize newSize = e->size();
+    if(oldSize == newSize) return;
+  
     badSize = TRUE;
+    calculateRects();
 }
 
 
@@ -1022,7 +1055,7 @@ void QMenuBar::setWindowsAltMode( bool enable, int index )
     if ( enable ) {
 	if ( !windowsaltactive ) {
 	    windowsaltactive = 1;
-	    if ( style() == WindowsStyle )
+	    if ( style() == WindowsStyle ) 
 		setFocus();
 	}
 	if ( index == actItem ) // work around setActItem overoptimization
@@ -1039,40 +1072,3 @@ void QMenuBar::setWindowsAltMode( bool enable, int index )
 	setActItem( index, FALSE );
     }
 }
-
-
-/*!  Sets up keyboard accelerators for the menu bar. */
-
-void QMenuBar::setupAccelerators()
-{
-    delete autoaccel;
-
-    QMenuItemListIt it(*mitems);
-    register QMenuItem *mi;
-    while ( (mi=it.current()) ) {
-	++it;
-	QString s = mi->text();
-	if ( !s.isEmpty() ) {
-	    int i = s.find( '&' );
-	    if ( i >= 0 && isalnum(s[i+1]) ) {
-		int k = s[i+1];
-		if ( isalpha(k) )
-		    k = toupper(k) - 'A' + Key_A;
-		if ( !autoaccel ) {
-		    autoaccel = new QAccel( this );
-		    CHECK_PTR( autoaccel );
-		    connect( autoaccel, SIGNAL(activated(int)),
-			     SLOT(accelActivated(int)) );
-		    connect( autoaccel, SIGNAL(destroyed()),
-			     SLOT(accelDestroyed()) );
-		}
-		autoaccel->insertItem( ALT+k, mi->id() );
-	    }
-	}
-	if ( mi->popup() ) {
-	    mi->popup()->updateAccel( this );
-	    if ( !mi->popup()->isEnabled() )
-		mi->popup()->enableAccel( FALSE );
-	}
-    }
-}
