@@ -1,5 +1,5 @@
 /**********************************************************************
-** $Id: qmlined.cpp,v 2.75.2.5 1997/12/19 16:33:47 warwick Exp $
+** $Id: qmlined.cpp,v 2.75 1997/10/06 13:03:03 agulbra Exp $
 **
 ** Definition of QMultiLineEdit widget class
 **
@@ -7,7 +7,7 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
@@ -469,7 +469,7 @@ void QMultiLineEdit::timerEvent( QTimerEvent *t )
     if ( hasFocus() && t->timerId() == blinkTimer ) {
 	cursorOn = !cursorOn;
 	updateCell( cursorY, 0, FALSE );
-    } else if ( dragScrolling && t->timerId() == scrollTimer ) {
+    } else if ( t->timerId() == scrollTimer ) {
 	QPoint p = mapFromGlobal( QCursor::pos() );
 	if ( p.y() < 0 )
 	    cursorUp( TRUE );
@@ -552,7 +552,7 @@ QString QMultiLineEdit::markedText() const
 	QString tmp( len + 1 );
 	int idx = 0;
 
-	if ( firstS && firstS->data() ) {
+	if ( firstS ) {
 	    const char *p = firstS->data() + markBeginX;
 	    while (( tmp[idx++] = *p++ ))
 		;
@@ -856,6 +856,7 @@ void QMultiLineEdit::keyPressEvent( QKeyEvent *e )
 	case Key_Return:
 	    newLine();
 	    emit returnPressed();
+	    e->ignore();
 	    break;
 	default:
 	    unknown++;
