@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qpopmenu.cpp,v 2.47.2.2 1997/11/19 05:14:29 agulbra Exp $
+** $Id: qpopmenu.cpp,v 2.47 1997/10/03 21:28:23 agulbra Exp $
 **
 ** Implementation of QPopupMenu class
 **
@@ -7,7 +7,7 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
@@ -31,7 +31,7 @@
 #include "qpmcache.h"
 #include <ctype.h>
 
-RCSTAG("$Id: qpopmenu.cpp,v 2.47.2.2 1997/11/19 05:14:29 agulbra Exp $");
+RCSTAG("$Id: qpopmenu.cpp,v 2.47 1997/10/03 21:28:23 agulbra Exp $");
 
 
 // Motif style parameters
@@ -385,11 +385,6 @@ QPopupMenu::QPopupMenu( QWidget *parent, const char *name )
 
 QPopupMenu::~QPopupMenu()
 {
-    if ( syncMenu == this ) {
-	qApp->exit_loop();
-	syncMenu = 0;
-    }
-	
     if ( qpm_extraStuff )
 	qpm_extraStuff->remove( this );
     delete autoaccel;
@@ -519,9 +514,8 @@ void QPopupMenu::frameChanged()
 
 
 /*!
-  Opens the popup menu so that the item number \a indexAtPoint will be
-  at the specified \e global position \a pos.  To translate a widget's
-  local coordinates into global coordinates, use QWidget::mapToGlobal().
+  Opens the popup menu so that the item number \e indexAtPoint
+  will be at the specified global position \e pos.
 */
 
 void QPopupMenu::popup( const QPoint &pos, int indexAtPoint )
@@ -949,10 +943,8 @@ void QPopupMenu::hide()
     hidePopups();
     killTimers();
     QWidget::hide();
-    if ( syncMenu == this && qApp ) {
-	syncMenu = 0;
+    if ( syncMenu == this && qApp )
 	qApp->exit_loop();
-    }
 }
 
 #if 0
@@ -1574,7 +1566,6 @@ int QPopupMenu::exec()
     qApp->enter_loop();
     disconnect( this, SIGNAL(activated(int)),
 		this, SLOT(modalActivation(int)) );
-    syncMenu = 0;
 
     return syncMenuId;
 }
