@@ -1,5 +1,5 @@
 /**********************************************************************
-** $Id: qtablevw.cpp,v 2.18 1997/03/16 13:36:29 paul Exp $
+** $Id: qtablevw.cpp,v 2.36 1997/09/30 14:15:57 warwick Exp $
 **
 ** Implementation of QTableView class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -23,7 +26,7 @@
 #include "qdrawutl.h"
 #include <limits.h>
 
-RCSTAG("$Id: qtablevw.cpp,v 2.18 1997/03/16 13:36:29 paul Exp $");
+RCSTAG("$Id: qtablevw.cpp,v 2.36 1997/09/30 14:15:57 warwick Exp $");
 
 
 const int sbDim = 16;
@@ -56,13 +59,6 @@ CornerSquare::CornerSquare( QWidget *parent, const char *name )
 
 void CornerSquare::paintEvent( QPaintEvent * )
 {
-    if ( style() == MotifStyle ) {
-	QPainter p;
-	QColorGroup g = colorGroup();
-	p.begin( this );
-	qDrawShadePanel( &p, rect(), colorGroup() );
-	p.end();
-    }
 }
 
 
@@ -82,7 +78,8 @@ void CornerSquare::paintEvent( QPaintEvent * )
   call to setTableFlags(), some table content manipulation, and an
   implementation of paintCell().  Subclasses that need cells with
   variable width or height must reimplement cellHeight() and/or
-  cellWidth().
+  cellWidth(). Use updateTableSize() to tell QTableView when the
+  width or height has changed. 
 
   When you read this documentation, it is important to understand the
   distinctions between the four coordinate systems involved.
@@ -107,6 +104,15 @@ void CornerSquare::paintEvent( QPaintEvent * )
   coordinate systems, but if we were to provide a flexible and
   powerful base class, there wasn't any way around it.  
 
+  <img src=qtablevw-m.gif> <img src=qtablevw-w.gif>
+
+  \warning in the current (and previous) version of the library, the 
+  functions setNumRows(), setNumCols(), setCellHeight(),
+  setCellWidth(), setTableFlags() and clearTableFlags() may cause
+  virtual functions like cellWidth() and cellHeight() to be called, even
+  if autoUpdate() is FALSE.
+  This may cause errors if relevant state variables are not initialized.
+
 */
 
 
@@ -256,6 +262,7 @@ void QTableView::repaint( int x, int y, int w, int h, bool erase )
 
 /*!
   Sets the number of rows of the table to \e rows (must be non-negative).
+  Does not change topCell().
 
   The table repaints itself automatically if autoUpdate() is set.
 
@@ -276,10 +283,16 @@ void QTableView::setNumRows( int rows )
     nRows = rows;
     if ( autoUpdate() && isVisible() ) {
 	int maxRow  = lastRowVisible();
-	if ( maxRow < oldRows || maxRow < nRows )
+	if ( maxRow >= oldRows || maxRow >= nRows )
 	    repaint();
     }
     updateScrollBars( verRange );
+    if ( yOffset() > 0 && testTableFlags( Tbl_autoVScrollBar ) &&
+	 !testTableFlags( Tbl_vScrollBar ) ) {
+	setYOffset( 0 );
+	if ( autoUpdate() )
+	    update();
+    }
 }
 
 /*!
@@ -290,6 +303,7 @@ void QTableView::setNumRows( int rows )
 
 /*!
   Sets the number of columns of the table to \e cols (must be non-negative).
+  Does not change leftCell().
 
   The table repaints itself automatically if autoUpdate() is set.
 
@@ -310,10 +324,16 @@ void QTableView::setNumCols( int cols )
     nCols = cols;
     if ( autoUpdate() && isVisible() ) {
 	int maxCol = lastColVisible();
-	if ( maxCol < oldCols || maxCol < nCols )
+	if ( maxCol >= oldCols || maxCol >= nCols )
 	    repaint();
     }
     updateScrollBars( horRange );
+    if ( xOffset() > 0 && testTableFlags( Tbl_autoHScrollBar ) &&
+	 !testTableFlags( Tbl_hScrollBar ) ) {
+	setXOffset( 0 );
+	if ( autoUpdate() )
+	    update();
+    }
 }
 
 
@@ -406,8 +426,7 @@ void QTableView::setTopLeftCell( int row, int col )
   The \e x parameter is in \e table coordinates.
 
   The interaction with \link setTableFlags() Tbl_snapToHGrid
-  \endlink is tricky.  If \e updateScrBars is TRUE, the scroll bars are
-  updated.
+  \endlink is tricky. 
 
   \sa xOffset(), setYOffset(), setOffset(), setLeftCell()
 */
@@ -432,8 +451,7 @@ void QTableView::setXOffset( int x )
   The \e y parameter is in \e table coordinates.
 
   The interaction with \link setTableFlags() Tbl_snapToVGrid
-  \endlink is tricky.  If \e updateScrBars is TRUE, the scroll bars are
-  updated.
+  \endlink is tricky. 
 
   \sa yOffset(), setXOffset(), setOffset(), setTopCell()
 */
@@ -539,9 +557,10 @@ void QTableView::setOffset( int x, int y, bool updateScrBars )
   Returns the width of column \e col, in pixels.
 
   This function is virtual and must be reimplemented by subclasses that
-  have variable cell widths.
+  have variable cell widths. Note that if the total table width 
+  changes, updateTableSize() must be called.
 
-  \sa setCellWidth(), cellHeight(), totalWidth()
+  \sa setCellWidth(), cellHeight(), totalWidth(), updateTableSize()
 */
 
 int QTableView::cellWidth( int )
@@ -590,7 +609,8 @@ void QTableView::setCellWidth( int cellWidth )
   Returns the height of row \e row, in pixels.
 
   This function is virtual and must be reimplemented by subclasses that
-  have variable cell heights.
+  have variable cell heights.  Note that if the total table height
+  changes, updateTableSize() must be called.
 
   \sa setCellHeight(), cellWidth(), totalHeight()
 */
@@ -908,6 +928,9 @@ void QTableView::clearTableFlags( uint f )
 
 void QTableView::setAutoUpdate( bool enable )
 {
+#if QT_VERSION == 200
+#error "Remove unused flag obsoleteUpd"
+#endif
     obsoleteUpd = enable; //### binary compatibility contortions
     if ( isUpdatesEnabled() == enable )
 	return;
@@ -1052,8 +1075,8 @@ void QTableView::coverCornerSquare( bool enable )
     if ( !cornerSquare && enable ) {
 	cornerSquare = new CornerSquare( this );
 	CHECK_PTR( cornerSquare );
-	cornerSquare->setGeometry( maxViewX() + frameWidth() + 2 ,
-				   maxViewY() + frameWidth() + 2,
+	cornerSquare->setGeometry( maxViewX() + frameWidth() + 1,
+				   maxViewY() + frameWidth() + 1,
 				   sbDim, sbDim );
     }
     if ( autoUpdate() && cornerSquare ) {
@@ -1097,7 +1120,7 @@ void QTableView::snapToGrid( bool horizontal, bool vertical )
 	else
 	    newYCell = yCellOffs;
     }
-    setTopLeftCell( newXCell, newYCell );
+    setTopLeftCell( newYCell, newXCell );  //row,column
 }
 
 /*!
@@ -1267,7 +1290,7 @@ void QTableView::paintEvent( QPaintEvent *e )
 
     if ( !contentsRect().contains( updateR ) ) {// update frame ?
 	drawFrame( &paint );
-	if ( updateR.left() < frameWidth() )
+	if ( updateR.left() < frameWidth() ) 		//###
 	    updateR.setLeft( frameWidth() );
 	if ( updateR.top() < frameWidth() )
 	    updateR.setTop( frameWidth() );
@@ -1317,23 +1340,25 @@ void QTableView::paintEvent( QPaintEvent *e )
 	    cellR.setRect( xPos, yPos, cellW ? cellW : cellWidth(col),
 				       cellH ? cellH : cellHeight(row) );
 	    cellUR = cellR.intersect( updateR );
-	    cellUpdateR = cellUR;
-	    cellUpdateR.moveBy( -xPos, -yPos ); // cell coordinates
-	    if ( eraseInPaint )
-		paint.eraseRect( cellUR );
+	    if ( cellUR.isValid() ) {
+		cellUpdateR = cellUR;
+		cellUpdateR.moveBy( -xPos, -yPos ); // cell coordinates
+		if ( eraseInPaint )
+		    paint.eraseRect( cellUR );
 
-	    matrix.translate( xPos, yPos );
-	    paint.setWorldMatrix( matrix );
-	    if ( testTableFlags(Tbl_clipCellPainting) ||
-		 frameWidth() > 0 && !winR.contains( cellR ) ) {
-		paint.setClipRect( cellUR );
-		paintCell( &paint, row, col );
-		paint.setClipping( FALSE );
-	    } else {
-		paintCell( &paint, row, col );
+		matrix.translate( xPos, yPos );
+		paint.setWorldMatrix( matrix );
+		if ( testTableFlags(Tbl_clipCellPainting) ||
+		     frameWidth() > 0 && !winR.contains( cellR ) ) { //##arnt
+		    paint.setClipRect( cellUR );
+		    paintCell( &paint, row, col );
+		    paint.setClipping( FALSE );
+		} else {
+		    paintCell( &paint, row, col );
+		}
+		matrix.reset();
+		paint.setWorldMatrix( matrix );
 	    }
-	    matrix.reset();
-	    paint.setWorldMatrix( matrix );
 	    col++;
 	    xPos = nextX;
 	}
@@ -1390,7 +1415,7 @@ void QTableView::resizeEvent( QResizeEvent * )
 
 void QTableView::updateView()
 {
-    repaint( frameWidth(), frameWidth(), viewWidth(), viewHeight() );
+    repaint( viewRect() );
 }
 
 /*!
@@ -1471,29 +1496,26 @@ void QTableView::setHorScrollBar( bool on, bool update )
 	if ( update )
 	    updateScrollBars( horMask | verMask );
 	else
-	    sbDirty |= horMask | verMask;
+	    sbDirty = sbDirty | (horMask | verMask);
 	if ( testTableFlags( Tbl_vScrollBar ) )
 	    coverCornerSquare( TRUE );
-	if ( autoUpdate() ) {
-	    sbDirty |= horMask;		// so updateScrollBars will show()
-	    if ( isVisible() )
-		erase( frameWidth(), height() - sbDim - 1,
-		       viewWidth(), 1 );
-	}
+	if ( autoUpdate() )
+	    sbDirty = sbDirty | horMask;
     } else {
 	tFlags &= ~Tbl_hScrollBar;
 	if ( !hScrollBar )
 	    return;
 	coverCornerSquare( FALSE );
-	if ( autoUpdate() )
+	bool hideScrollBar = autoUpdate() && hScrollBar->isVisible();
+	if ( hideScrollBar )
 	    hScrollBar->hide();
 	if ( update )
 	    updateScrollBars( verMask );
 	else
-	    sbDirty |= verMask;
-	if ( autoUpdate() && isVisible() )
-	    repaint( frameWidth(), height() - frameWidth() - sbDim - 1,
-		     viewWidth(), sbDim + frameWidth() );
+	    sbDirty = sbDirty | verMask;
+	if ( hideScrollBar && isVisible() )
+	    repaint( hScrollBar->x(), hScrollBar->y(), 
+		     width() - hScrollBar->x(), hScrollBar->height() );
     }
     updateFrameSize();
 }
@@ -1523,29 +1545,26 @@ void QTableView::setVerScrollBar( bool on, bool update )
 	if ( update )
 	    updateScrollBars( verMask | horMask );
 	else
-	    sbDirty |= horMask | verMask;
+	    sbDirty = sbDirty | (horMask | verMask);
 	if ( testTableFlags( Tbl_hScrollBar ) )
 	    coverCornerSquare( TRUE );
-	if ( autoUpdate() ) {
-	    sbDirty |= verMask;		// so updateScrollBars will show()
-	    if ( isVisible() )
-		erase( width() - sbDim - 1, frameWidth(),
-		       1, viewHeight() );
-	}
+	if ( autoUpdate() )
+	    sbDirty = sbDirty | verMask;
     } else {
 	tFlags &= ~Tbl_vScrollBar;
 	if ( !vScrollBar )
 	    return;
 	coverCornerSquare( FALSE );
-	if ( autoUpdate() )
+	bool hideScrollBar = autoUpdate() && vScrollBar->isVisible();
+	if ( hideScrollBar )
 	    vScrollBar->hide();
 	if ( update )
 	    updateScrollBars( horMask );
 	else
-	    sbDirty |= horMask;
-	if ( autoUpdate() && isVisible() )
-	    repaint( width() - frameWidth() - sbDim - 1, frameWidth(),
-		     sbDim + frameWidth(), viewHeight() );
+	    sbDirty = sbDirty | horMask;
+	if ( hideScrollBar && isVisible() )
+	    repaint( vScrollBar->x(), vScrollBar->y(), 
+		     vScrollBar->width(), height() - vScrollBar->y() );
     }
     updateFrameSize();
 }
@@ -1559,26 +1578,26 @@ int QTableView::findRawRow( int yPos, int *cellMaxY, int *cellMinY,
     int r = -1;
     if ( nRows == 0 )
 	return r;
-    if ( goOutsideView || yPos >= frameWidth() && yPos <= maxViewY() ) {
-	if ( yPos < frameWidth() ) {
+    if ( goOutsideView || yPos >= minViewY() && yPos <= maxViewY() ) {
+	if ( yPos < minViewY() ) {
 #if defined(CHECK_RANGE)
 	    warning( "QTableView::findRawRow: internal error: "
-		     "yPos < frameWidth() && goOutsideView "
+		     "yPos < minViewY() && goOutsideView "
 		     "not supported. (%d,%d)", yPos, yOffs );
 #endif
 	    return -1;
 	}
 	if ( cellH ) {				     // uniform cell height
-	    r = (yPos - frameWidth() + yCellDelta)/cellH; // cell offs from top
+	    r = (yPos - minViewY() + yCellDelta)/cellH; // cell offs from top
 	    if ( cellMaxY )
-		*cellMaxY = (r + 1)*cellH + frameWidth() - yCellDelta - 1;
+		*cellMaxY = (r + 1)*cellH + minViewY() - yCellDelta - 1;
 	    if ( cellMinY )
-		*cellMinY = r*cellH + frameWidth() - yCellDelta;
+		*cellMinY = r*cellH + minViewY() - yCellDelta;
 	    r += yCellOffs;			     // absolute cell index
 	} else {				     // variable cell height
 	    QTableView *tw = (QTableView *)this;
 	    r	     = yCellOffs;
-	    int h    = frameWidth() - yCellDelta;
+	    int h    = minViewY() - yCellDelta; //##arnt3
 	    int oldH = h;
 	    ASSERT( r < nRows );
 	    while ( r < nRows ) {
@@ -1605,26 +1624,26 @@ int QTableView::findRawCol( int xPos, int *cellMaxX, int *cellMinX ,
     int c = -1;
     if ( nCols == 0 )
 	return c;
-    if ( goOutsideView || xPos >= frameWidth() && xPos <= maxViewX() ) {
-	if ( xPos < frameWidth() ) {
+    if ( goOutsideView || xPos >= minViewX() && xPos <= maxViewX() ) {
+	if ( xPos < minViewX() ) {
 #if defined(CHECK_RANGE)
 	    warning( "QTableView::findRawCol: intermal error: "
-		     "xPos < frameWidth() && goOutsideView "
+		     "xPos < minViewX() && goOutsideView "
 		     "not supported. (%d,%d)", xPos, xOffs );
 #endif
 	    return -1;
 	}
 	if ( cellW ) {				// uniform cell width
-	    c = (xPos - frameWidth() + xCellDelta)/cellW; //cell offs from left
+	    c = (xPos - minViewX() + xCellDelta)/cellW; //cell offs from left
 	    if ( cellMaxX )
-		*cellMaxX = (c + 1)*cellW + frameWidth() - xCellDelta - 1;
+		*cellMaxX = (c + 1)*cellW + minViewX() - xCellDelta - 1;
 	    if ( cellMinX )
-		*cellMinX = c*cellW + frameWidth() - xCellDelta;
+		*cellMinX = c*cellW + minViewX() - xCellDelta;
 	    c += xCellOffs;			// absolute cell index
 	} else {				// variable cell width
 	    QTableView *tw = (QTableView *)this;
 	    c	     = xCellOffs;
-	    int w    = frameWidth() - xCellDelta;
+	    int w    = minViewX() - xCellDelta; //##arnt3
 	    int oldW = w;
 	    ASSERT( c < nCols );
 	    while ( c < nCols ) {
@@ -1702,9 +1721,10 @@ bool QTableView::rowYPos( int row, int *yPos ) const
 	    int lastVisible = lastRowVisible();
 	    if ( row > lastVisible || lastVisible == -1 )
 		return FALSE;
-	    y = (row - yCellOffs)*cellH + frameWidth() - yCellDelta;
+	    y = (row - yCellOffs)*cellH + minViewY() - yCellDelta;
 	} else {
-	    y = frameWidth() - yCellDelta;	// y of leftmost cell in view
+	    //##arnt3
+	    y = minViewY() - yCellDelta;	// y of leftmost cell in view
 	    int r = yCellOffs;
 	    QTableView *tw = (QTableView *)this;
 	    int maxY = maxViewY();
@@ -1741,9 +1761,10 @@ bool QTableView::colXPos( int col, int *xPos ) const
 	    int lastVisible = lastColVisible();
 	    if ( col > lastVisible || lastVisible == -1 )
 		return FALSE;
-	    x = (col - xCellOffs)*cellW + frameWidth() - xCellDelta;
+	    x = (col - xCellOffs)*cellW + minViewX() - xCellDelta;
 	} else {
-	    x = frameWidth() - xCellDelta;	// x of uppermost cell in view
+	    //##arnt3
+	    x = minViewX() - xCellDelta;	// x of uppermost cell in view
 	    int c = xCellOffs;
 	    QTableView *tw = (QTableView *)this;
 	    int maxX = maxViewX();
@@ -1765,7 +1786,7 @@ bool QTableView::colXPos( int col, int *xPos ) const
   Moves the visible area of the table rightwards by \e xPixels and
   downwards by \e yPixels pixels.  Both may be negative.
 
-  This is an reimplementation of QWidget::scroll() to reduce flicker.
+  This function is \e not the same as QWidget::scroll().
 
   \sa setXOffset(), setYOffset(), setOffset(), setTopCell(),
   setLeftCell(), setTopLeftOffset()
@@ -1784,9 +1805,9 @@ void QTableView::scroll( int xPixels, int yPixels )
     int xStart, yStart, width, height;
 
     if ( xPixels != 0 ) {
-	yStart = frameWidth();
+	yStart = minViewY();
 	height = viewHeight();
-	xStart = xPixels < 0 ? frameWidth() : frameWidth() + xPixels;
+	xStart = xPixels < 0 ? minViewX() : minViewX() + xPixels;
 
 	width = viewWidth() - QABS(xPixels);
 
@@ -1819,24 +1840,24 @@ void QTableView::scroll( int xPixels, int yPixels )
 		    this, xStart, yStart, width, height );
 
 	    if ( xPixels < 0 )
-		repaint( frameWidth(), yStart, -xPixels, height );
+		repaint( minViewX(), yStart, -xPixels, height );
 	    else
-		repaint( frameWidth() + width, yStart,
-			 maxX - frameWidth() - width, height );
+		repaint( minViewX() + width, yStart,
+			 maxX - minViewX() - width, height );
 	} else {
 	    bitBlt( this, xStart - xPixels, yStart,
 		    this, xStart, yStart, width, height );
 	    if ( xPixels < 0 )
-		repaint( frameWidth(), yStart, -xPixels, height );
+		repaint( minViewX(), yStart, -xPixels, height );
 	    else
-		repaint( frameWidth() + width, yStart, xPixels, height );
+		repaint( minViewX() + width, yStart, xPixels, height );
 	}
     }
 
     if ( yPixels != 0 ) {
-	xStart = frameWidth();
+	xStart = minViewY();
 	width  = viewWidth();
-	yStart = yPixels < 0 ? frameWidth() : frameWidth() + yPixels;
+	yStart = yPixels < 0 ? minViewY() : minViewY() + yPixels;
 
 	height = viewHeight() - QABS(yPixels);
 
@@ -1869,25 +1890,51 @@ void QTableView::scroll( int xPixels, int yPixels )
 		    this, xStart, yStart, width, height );
 
 	    if ( yPixels < 0 ) {
-		repaint( xStart, frameWidth(), width, -yPixels );
+		repaint( xStart, minViewY(), width, -yPixels );
 	    } else {
-		repaint( xStart, frameWidth() + height,
-			 width, maxY - frameWidth() - height );
+		repaint( xStart, minViewY() + height,
+			 width, maxY - minViewY() - height );
 	    }
 	} else {
 	    bitBlt( this, xStart, yStart - yPixels,
 		    this, xStart, yStart, width, height );
 
 	    if ( yPixels < 0 ) {
-		repaint( xStart, frameWidth(), width, -yPixels );
+		repaint( xStart, minViewY(), width, -yPixels );
 	    } else {
-		repaint( xStart, frameWidth() + height, width, yPixels );
+		repaint( xStart, minViewY() + height, width, yPixels );
 	    }
 	}
     }
 }
 
 
+/*!
+  Returns the leftmost pixel of the table view in \e view
+  coordinates.	This excludes the frame and any header.
+
+  \sa maxViewY(), viewWidth(), contentsRect()
+*/
+
+int QTableView::minViewX() const
+{
+    return frameWidth();
+}
+
+
+/*!
+  Returns the top pixel of the table view in \e view
+  coordinates.	This excludes the frame and any header.
+
+  \sa maxViewX(), viewHeight(), contentsRect()
+*/
+
+int QTableView::minViewY() const
+{
+    return frameWidth();
+}
+
+
 /*!
   Returns the rightmost pixel of the table view in \e view
   coordinates.	This excludes the frame and any scroll bar, but
@@ -1899,7 +1946,7 @@ void QTableView::scroll( int xPixels, int yPixels )
 int QTableView::maxViewX() const
 {
     return width() - 1 - frameWidth()
-		   - (tFlags & Tbl_vScrollBar ? sbDim + 1 : 0);
+		   - (tFlags & Tbl_vScrollBar ? sbDim : 0);
 }
 
 
@@ -1914,42 +1961,42 @@ int QTableView::maxViewX() const
 int QTableView::maxViewY() const
 {
     return height() - 1 - frameWidth()
-		    - (tFlags & Tbl_hScrollBar ? sbDim + 1 : 0);
+		    - (tFlags & Tbl_hScrollBar ? sbDim : 0);
 }
 
 
 /*!  
-  Returns the width of the table view, as such, in \e view coordinates.
-  This does not include any scroll bar or frame, but does include
-  background pixels to the right of the table data.
+  Returns the width of the table view, as such, in \e view
+  coordinates.  This does not include any header, scroll bar or frame,
+  but does include background pixels to the right of the table data.
 
-  \sa maxViewX(), viewHeight(), contentsRect() 
+  \sa minViewX() maxViewX(), viewHeight(), contentsRect() viewRect()
 */
 
 int QTableView::viewWidth() const
 {
-    return maxViewX() - frameWidth() + 1;
+    return maxViewX() - minViewX() + 1;
 }
 
 
 /*!  
   Returns the height of the table view, as such, in \e view
-  coordinates.  This does not include any scroll bar or frame, but does
-  include background pixels below the table data.  
+  coordinates.  This does not include any header, scroll bar or frame,
+  but does include background pixels below the table data.
 
-  \sa maxViewY() viewWidth() contentsRect() 
+  \sa minViewY() maxViewY() viewWidth() contentsRect() viewRect()
 */
 
 int QTableView::viewHeight() const
 {
-    return maxViewY() - frameWidth() + 1;
+    return maxViewY() - minViewY() + 1;
 }
 
 
 void QTableView::doAutoScrollBars()
 {
-    int viewW = width()	 - frameWidth()*2;
-    int viewH = height() - frameWidth()*2;
+    int viewW = width()	 - frameWidth() - minViewX();
+    int viewH = height() - frameWidth() - minViewY();
     bool vScrollOn = testTableFlags(Tbl_vScrollBar);
     bool hScrollOn = testTableFlags(Tbl_hScrollBar);
     int w = 0;
@@ -1986,15 +2033,15 @@ void QTableView::doAutoScrollBars()
     }
 
     if ( testTableFlags(Tbl_autoHScrollBar) && vScrollOn && !hScrollOn )
-	if ( w > viewW - sbDim - 1 )
+	if ( w > viewW - sbDim )
 	    hScrollOn = TRUE;
 
     if ( testTableFlags(Tbl_autoVScrollBar) && hScrollOn && !vScrollOn )
-	if ( h > viewH - sbDim - 1 )
+	if ( h > viewH - sbDim )
 	    vScrollOn = TRUE;
 
-    setHorScrollBar( hScrollOn , FALSE );
-    setVerScrollBar( vScrollOn , FALSE );
+    setHorScrollBar( hScrollOn, FALSE );
+    setVerScrollBar( vScrollOn, FALSE );
 }
 
 
@@ -2032,7 +2079,7 @@ void QTableView::updateScrollBars( uint f )
     if ( testTableFlags(Tbl_hScrollBar) && (sbDirty & horMask) != 0 ) {
 	if ( sbDirty & horGeometry )
 	    hScrollBar->setGeometry( 0,height() - sbDim,
-				     viewWidth() + frameWidth()*2+1, sbDim );
+				     viewWidth() + frameWidth()*2, sbDim );
 
 	if ( sbDirty & horSteps ) {
 	    if ( cellW )
@@ -2055,7 +2102,7 @@ void QTableView::updateScrollBars( uint f )
     if ( testTableFlags(Tbl_vScrollBar) && (sbDirty & verMask) != 0 ) {
 	if ( sbDirty & verGeometry )
 	    vScrollBar->setGeometry( width() - sbDim, 0,
-				     sbDim, viewHeight() + frameWidth()*2+1 );
+				     sbDim, viewHeight() + frameWidth()*2 );
 
 	if ( sbDirty & verSteps ) {
 	    if ( cellH )
@@ -2076,8 +2123,8 @@ void QTableView::updateScrollBars( uint f )
     }
     if ( coveringCornerSquare &&
 	 ( (sbDirty & verGeometry ) || (sbDirty & horGeometry)) )
-	cornerSquare->move( maxViewX() + frameWidth() + 2,
-			    maxViewY() + frameWidth() + 2 );
+	cornerSquare->move( maxViewX() + frameWidth() + 1,
+			    maxViewY() + frameWidth() + 1 );
 
     sbDirty = 0;
     inSbUpdate = FALSE;
@@ -2086,8 +2133,8 @@ void QTableView::updateScrollBars( uint f )
 
 void QTableView::updateFrameSize()
 {
-    int rw = width()  - ( testTableFlags(Tbl_vScrollBar) ? sbDim + 1 : 0 );
-    int rh = height() - ( testTableFlags(Tbl_hScrollBar) ? sbDim + 1 : 0 );
+    int rw = width()  - ( testTableFlags(Tbl_vScrollBar) ? sbDim : 0 );
+    int rh = height() - ( testTableFlags(Tbl_hScrollBar) ? sbDim : 0 );
     if ( rw < 0 )
 	rw = 0;
     if ( rh < 0 )
@@ -2247,7 +2294,7 @@ void QTableView::showOrHideScrollBars()
     if ( vScrollBar ) {
 	if ( testTableFlags(Tbl_vScrollBar) ) {
 	    if ( !vScrollBar->isVisible() )
-		sbDirty |= verMask;
+		sbDirty = sbDirty | verMask;
 	} else {
 	    if ( vScrollBar->isVisible() )
 	       vScrollBar->hide();
@@ -2256,7 +2303,7 @@ void QTableView::showOrHideScrollBars()
     if ( hScrollBar ) {
 	if ( testTableFlags(Tbl_hScrollBar) ) {
 	    if ( !hScrollBar->isVisible() )
-		sbDirty |= horMask;
+		sbDirty = sbDirty | horMask;
 	} else {
 	    if ( hScrollBar->isVisible() )
 		hScrollBar->hide();
@@ -2273,3 +2320,20 @@ void QTableView::showOrHideScrollBars()
 	}
     }
 }
+
+
+/*!
+  Updates the scroll bars and internal state.
+
+  Call this function when the table view's total size is changed;
+  typically because the result of cellHeight() or cellWidth() have changed.
+  
+  This function does not repaint the widget.
+*/
+
+void QTableView::updateTableSize()
+{
+    updateScrollBars( horSteps |  horRange |
+		      verSteps |  verRange );
+    showOrHideScrollBars();    
+}
