@@ -1,28 +1,29 @@
 /****************************************************************************
-** $Id: qtooltip.cpp,v 2.51 1998/07/03 00:09:54 hanord Exp $
+** $Id: qtooltip.cpp,v 2.33 1997/08/26 16:45:03 agulbra Exp $
 **
 ** Tool Tips (or Balloon Help) for any widget or rectangle
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qtooltip.h"
 #include "qlabel.h"
 #include "qptrdict.h"
-#include "qapplication.h"
+#include "qapp.h"
+
+RCSTAG("$Id: qtooltip.cpp,v 2.33 1997/08/26 16:45:03 agulbra Exp $");
 
 // Magic value meaning an entire widget - if someone tries to insert a
 // tool tip on this part of a widget it will be interpreted as the
@@ -56,7 +57,7 @@ public:
     };
 
     bool    eventFilter( QObject * o, QEvent * e );
-    void    add( QWidget *, const QRect &, const char *,
+    void    add( QWidget *, const QRect &, const char *, 
 		 QToolTipGroup *, const char *, QToolTip *, bool );
     void    remove( QWidget *, const QRect & );
     void    remove( QWidget * );
@@ -81,8 +82,8 @@ private:
     QLabel	     *label;
     QPoint	      pos;
     QWidget	     *widget;
-    Tip *currentTip;
-    Tip *previousTip;
+    QTipManager::Tip *currentTip;
+    QTipManager::Tip *previousTip;
     bool isApplicationFilter;
 };
 
@@ -93,7 +94,7 @@ private:
 ** QTipManager meta object code from reading C++ file 'qtooltip.cpp'
 **
 ** Created: Mon Mar 17 12:39:34 1997
-**      by: The Qt Meta Object Compiler ($Revision: 2.51 $)
+**      by: The Qt Meta Object Compiler ($Revision: 2.33 $)
 **
 ** WARNING! All changes made in this file will be lost!
 *****************************************************************************/
@@ -104,7 +105,7 @@ private:
 #error Moc format conflict - please regenerate all moc files
 #endif
 
-#include <qmetaobject.h>
+#include <qmetaobj.h>
 
 
 const char *QTipManager::className() const
@@ -175,6 +176,7 @@ static void initTipManager()
 QTipManager::QTipManager()
     : QObject( 0, "toolTipManager" )
 {
+    initMetaObject();
     tips = new QPtrDict<QTipManager::Tip>( 313 );
     currentTip = 0;
     previousTip = 0;
@@ -231,10 +233,8 @@ void QTipManager::add( QWidget *w, const QRect &r, const char *s,
     t->group = g;
 
     tips->insert( w, t );
-    connect( w, SIGNAL(destroyed()), this, SLOT(clientWidgetDestroyed()) );
     if ( a ) {
-	if ( t->rect.contains( pos ) )
-	    showTip();
+	showTip();
 	tips->take( w );
 	if( t->next )
 	    tips->insert( w, t->next );
@@ -348,11 +348,8 @@ void QTipManager::removeFromGroup( QToolTipGroup *g )
 
 bool QTipManager::eventFilter( QObject *obj, QEvent *e )
 {
-    // avoid dumping core in case of application madness, and return
-    // quickly for some common but irrelevant events
-    if ( !tips || !e || !obj || !obj->isWidgetType() ||
-	 e->type() == Event_Timer || e->type() == Event_Paint ||
-	 e->type() == Event_SockAct )
+    // avoid dumping core in case of application madness
+    if ( !tips || !e || !obj || !obj->isWidgetType() )
 	return FALSE;
     QWidget *w = (QWidget *)obj;
 
@@ -360,7 +357,7 @@ bool QTipManager::eventFilter( QObject *obj, QEvent *e )
     while( w && !t ) {
 	t = (*tips)[ w ];
 	if ( !t )
-	    w = w->testWFlags(WType_TopLevel) ? 0 : w->parentWidget();
+	    w = w->parentWidget();
     }
 
     if ( !t ) {
@@ -373,6 +370,10 @@ bool QTipManager::eventFilter( QObject *obj, QEvent *e )
     // with that out of the way, let's get down to action
 
     switch( e->type() ) {
+    case Event_Timer: // fall through
+    case Event_Paint:
+	// no processing at all
+	break;
     case Event_MouseButtonPress:
     case Event_MouseButtonRelease:
     case Event_MouseButtonDblClick:
@@ -405,7 +406,7 @@ bool QTipManager::eventFilter( QObject *obj, QEvent *e )
 		    wakeUp.start( 100, TRUE );
 		else {
 		    previousTip = 0;
-		    wakeUp.start( 700, TRUE );
+		    wakeUp.start( 1000, TRUE );
 		}
 		widget = w;
 		pos = mousePos;
@@ -482,7 +483,7 @@ void QTipManager::showTip()
     label->show();
     label->raise();
 
-    fallAsleep.start( 5000, TRUE );
+    fallAsleep.start( 4000, TRUE );
     leaveWindow.stop();
 
     if ( t->group && !t->groupText.isEmpty() )
@@ -496,7 +497,7 @@ void QTipManager::hideTip()
 {
     if ( label && label->isVisible() ) {
 	label->hide();
-	fallAsleep.start( 10000, TRUE );
+	fallAsleep.start( 5000, TRUE );
 	if ( currentTip && currentTip->group )
 	    emit currentTip->group->removeTip();
     } else if ( wakeUp.isActive() ) {
@@ -519,8 +520,6 @@ void QTipManager::hideTip()
   \brief The QToolTip class provides tool tips (sometimes called
   balloon help) for any widget or rectangular part of a widget.
 
-  \ingroup application
-
   The tip is a short, one-line text reminding the user of the widget's
   or rectangle's function.  It is drawn immediately below the region,
   in a distinctive black on yellow combination.  In Motif style, Qt's
@@ -540,10 +539,10 @@ void QTipManager::hideTip()
   a key, lets the mouse rest for five seconds, or moves the mouse
   outside \e all tip-equpped regions for at least a second.
 
-  The QToolTip class can be used in three different ways: <ol> <li>
-  Adding a tip to an entire widget. <li> Adding a tip to a fixed
-  rectangle within a widget. <li> Adding a tip to a dynamic rectangle
-  within a widget. </ol>
+  There are no less than three APIs for QToolTip: <ol> <li> Adding a
+  tip to an entire widget. <li> Adding a tip to a fixed rectangle
+  within a widget. <li> Adding a tip to a dynamic rectangle within a
+  widget. </ol>
 
   To add a tip to a widget, call the static function QToolTip::add()
   with the widget and tip as arguments:
@@ -560,10 +559,10 @@ void QTipManager::hideTip()
     QToolTip::remove( quitButton );
   \endcode
 
-  You can also display another text (typically in a \link QStatusBar
-  status bar),\endlink courtesy of QToolTipGroup.  This example
-  assumes that \e g is a <code>QToolTipGroup *</code> and already
-  connected to the appropriate status bar:
+  You can also display another text (typically in a status bar),
+  courtesy of QToolTipGroup.  This example assumes that \e g is a
+  <code>QToolTipGroup *</code> and already connected to the
+  appropriate status bar:
 
   \code
     QToolTip::add( quitButton, "Leave the application", g,
@@ -574,29 +573,25 @@ void QTipManager::hideTip()
 
   To add a tip to a fixed rectangle within a widget, call the static
   function QToolTip::add() with the widget, rectangle and tip as
-  arguments.  (See the tooltip/tooltip.cpp example.)  Again, you can supply a
+  arguments.  (See the tellme/tellme.cpp example.)  Again, you can supply a
   QToolTipGroup * and another text if you want.
 
   Both of the above are one-liners and cover the vast majority of
-  cases.  The third and most general way to use QToolTip uses a pure
-  virtual function to decide whether to pop up a tool tip.  The
-  tooltip/tooltip.cpp example demonstrates this too.  This mode can be
-  used to implement e.g. tips for text that can move as the user
-  scrolls.
+  cases.  The third and most general API uses a pure virtual function
+  to decide whether to pop up a tool tip.  The tellme/tellme.cpp
+  example demonstrates this too.  This mode can be used to implement
+  e.g. tips for text that can move as the user scrolls.
 
-  To use QToolTip like this, you need to subclass QToolTip and
-  reimplement maybeTip().  maybeTip() will be called when there's a
-  chance that a tip should pop up.  It must decide whether to show a
-  tip, and possibly call add() with the rectangle the tip applies to,
-  the tip's text and optionally the QToolTipGroup details.  The tip
-  will disappear once the mouse moves outside the rectangle you
-  supply, and \e not \e reappear - maybeTip() will be called again if
-  the user lets the mouse rest within the same rectangle again.  You
-  can forcibly remove the tip by calling remove() with no arguments.
-  This is handy if the widget scrolls.
-
-  \sa QStatusBar QWhatsThis QToolTipGroup
-  <a href="guibooks.html#fowler">GUI Design Handbook: Tool Tip</a>
+  To use this API, you need to subclass QToolTip and reimplement
+  maybeTip().  maybeTip() will be called when there's a chance that a
+  tip should pop up.  It must decide whether to show a tip, and
+  possibly call add() with the rectangle the tip applies to, the tip's
+  text and optionally the QToolTipGroup details.  The tip will
+  disappear once the mouse moves outside the rectangle you supply, and
+  \e not \e reappear - maybeTip() will be called again if the user
+  lets the mouse rest within the same rectangle again.  You can
+  forcibly remove the tip by calling remove() with no arguments.  This
+  is handy if the widget scrolls.
 */
 
 
@@ -751,7 +746,7 @@ void QToolTip::remove( QWidget * widget )
 	tipManager->remove( widget, entireWidget() );
 }
 
-/*!
+/*!  
   Adds a tool tip to a fixed rectangle within \a widget.  \a text is
   the text shown in the tool tip.  QToolTip makes a deep copy of this
   string.
@@ -805,29 +800,14 @@ void QToolTip::remove( QWidget * widget, const QRect & rect )
   QToolTip offers.
 
   It is called when there is a chance that a tool tip should be shown,
-  and must decide whether there is a tool tip for the point \a p in
-  the widget this QToolTip object relates to.
-
-  \p is given in that widget's local coordinates.  Most maybeTip()
-  implementation will be of the form:
-  
-  \code
-    if ( \<something\> ) {
-	tip( \<something\>, \<something\> );
-    }
-  \endcode
-
-  The first argument to tip() (a rectangle) should include the \a p,
-  or QToolTip, the user or both can be confused.
-
-  \sa tip()
+  and must decide whether there is a tool tip for the point \a p and
+  what rectangle
 */
 
 
 /*!
-  Pop up a tip saying \a text right now, and remove that tip once the
-  cursor moves out of rectangle \a rect (which is given in the
-  coordinate system of the widget this QToolTip relates to).
+  Pop up a tip saying \a text right now, and remove that tip once
+  the cursor moves out of rectangle \a rect.
 
   The tip will not come back if the cursor moves back; your maybeTip()
   has to reinstate it each time.
@@ -896,7 +876,8 @@ void QToolTip::clear()
 /*!
   \class QToolTipGroup qtooltip.h
 
-  \brief The QToolTipGroup class collects tool tips into natural groups.
+  \brief The QToolTipGroup class provides a way to group tool tips
+  into natural groups.
 
   Tool tips can display \e two texts, the one in the tip and
   optionally another one, typically in a status bar.  QToolTipGroup
@@ -950,7 +931,7 @@ void QToolTip::clear()
 QToolTipGroup::QToolTipGroup( QObject *parent, const char *name )
     : QObject( parent, name )
 {
-    // nothing
+    initMetaObject();
 }
 
 
