@@ -1,46 +1,44 @@
 /****************************************************************************
-** $Id: qvalidator.cpp,v 2.21 1998/07/03 00:09:54 hanord Exp $
+** $Id: qvalidator.cpp,v 2.11 1997/10/08 12:28:50 eiriken Exp $
 **
 ** Implementation of validator classes.
 **
 ** Created : 970610
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the free edition of Qt 1.31.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** and distribution terms, or http://www.troll.no/qt/license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qvalidator.h"
 #include "qwidget.h"
-#include "qregexp.h"
 
 #include <limits.h> // *_MIN, *_MAX
-#include <ctype.h> // isdigit
+
+RCSTAG("$Id: qvalidator.cpp,v 2.11 1997/10/08 12:28:50 eiriken Exp $");
+
 
 /*!
   \class QValidator qvalidator.h
 
-  \brief The QValidator class provides validation of input text.
-
-  \ingroup misc
+  \brief The QValidator class provides ways to validate an input text.
 
   The class itself is abstract; two subclasses provide rudimentary
   numeric range checking.
 
-  The class includes two virtual functions, validate() and fixup().
+  The class includes two virtual functions, isValid() and fixup().
 
-  validate() is pure virtual, so it must be implemented by every
+  isValid() is pure virtual, so it must be implemented by every
   subclass.  It returns Invalid, Valid or Acceptable depending on
   whether its argument is valid (for the class' definition of valid).
 
@@ -76,10 +74,10 @@ QValidator::~QValidator()
 /*!
   \fn QValidator::State QValidator::validate( QString & input, int & pos )
 
-  This pure virtual function returns \c Invalid if \a input is invalid
+  This pure virtual function returns \c Invalid if \a input is valid
   according to this validator's rules, \c Valid if it is likely that a
   little more editing will make the input acceptable (e.g. the user
-  types '4' into a widget which accepts 10-99) and \c Acceptable if
+  types '4' into a widget which accepts 10-99) and \a Acceptable if
   the input is completely acceptable.
 
   The function can change \a input and \a pos (the cursor position) if
@@ -121,8 +119,6 @@ void QValidator::fixup( QString & input )
 
   \brief The QIntValidator class provides range checking of integers.
 
-  \ingroup misc
-
   QIntValidator provides a lower and an upper bound.  It does not
   provide a fixup() function.
 
@@ -168,15 +164,12 @@ QIntValidator::~QIntValidator()
 
 /*!
   Returns \a Acceptable if \a input contains a number in the legal
-  range, \a Valid if it contains another integer or is empty, and \a
-  Invalid if \a input is not an integer.
+  range, \a Valid if it contains another integer, and \a Invalid if \a
+  input is not an integer.
 */
 
 QValidator::State QIntValidator::validate( QString & input, int & )
 {
-    QRegExp empty( "^ *-? *$" );
-    if ( empty.match( input ) >= 0 )
-	return QValidator::Valid;
     bool ok;
     long int tmp = input.toLong( &ok );
     if ( !ok )
@@ -221,10 +214,7 @@ void QIntValidator::setRange( int bottom, int top )
 /*!
   \class QDoubleValidator qvalidator.h
 
-  \brief The QDoubleValidator class provides range checking of
-  floating-point numbers.
-
-  \ingroup misc
+  \brief The QDoubleValidator class provides range checking of integers.
 
   QDoubleValidator provides an upper bound, a lower bound, and a limit
   on the number of digits after the decimal point.  It does not
@@ -280,33 +270,18 @@ QDoubleValidator::~QDoubleValidator()
 
 /*!
   Returns \a Acceptable if \a input contains a number in the legal
-  range and format, \a Valid if it contains another number, a number
-  with too many digits after the decimal point or is empty, and \a
-  Invalid if \a input is not a number.
+  range and format, \a Valid if it contains another, and \a Invalid if
+  \a input is not a number.
 */
 
 QValidator::State QDoubleValidator::validate( QString & input, int & )
 {
-    QRegExp empty( "^ *-? *$" );
-    if ( empty.match( input ) >= 0 )
-	return QValidator::Valid;
-    bool ok = TRUE;
+    bool ok;
     double tmp = input.toDouble( &ok );
+    // check the number of decimals here!
     if ( !ok )
 	return QValidator::Invalid;
-
-    int i = input.find( '.' );
-    if ( i >= 0 ) {
-	// has decimal point, now count digits after that
-	i++;
-	int j = i;
-	while( isdigit( input[j] ) )
-	    j++;
-	if ( j - i > d )
-	    return QValidator::Valid;
-    }
-
-    if ( tmp < b || tmp > t )
+    else if ( tmp < b || tmp > t )
 	return QValidator::Valid;
     else
 	return QValidator::Acceptable;
